/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: Config.cpp,v 1.4 2000/01/04 14:10:33 icostanzo Exp $
 *-----------------------------------------------------------------------------
 */


// Config.cpp: implementation of the CConfig class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Config.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

// static initializers
// This is our hard-coded config file name
char *CConfig::filename = "C:\\temp\\enhydraConn.conf";

CConfig* CConfig::m_pConfig = NULL;
char CConfig::hostname[64];
char CConfig::urlPrefix[128];
int CConfig::port = 0;

int CConfig::m_activeConnections = 0;        // total current active connections
int CConfig::m_loadBalanceThreshold = 10;    // active connection threshold where we actively load balance
int CConfig::m_failThreshold = 99;           // consecutive failures before taking connection offline
char CConfig::m_adminPrefix[64];   // URL prefix for admin requests
char CConfig::m_adminUser[64];     // user id for admin requests
char CConfig::m_adminPassword[64]; // password for admin requests


BOOL CConfig::m_fCacheInitialized = FALSE;

char *CConfig::m_portString = "port";
char *CConfig::m_addressString = "address";
char *CConfig::m_URLPrefix = "URL-prefix";
char *CConfig::m_hostString = "host";

char *CConfig::m_loadBalanceThreaholdString = "loadbalance_threshold";
char *CConfig::m_failThresholdString  = "failure_threshold";
char *CConfig::m_adminPrefixString    = "admin_url_prefix";
char *CConfig::m_adminUserString      = "admin_user_name";
char *CConfig::m_adminPasswordString  = "admin_password";

// DEBUG LOG FILE
FILE *CConfig::m_fp = fopen("C:\\temp\\enhydralog.wri", "w");;

///////////////////////////////////////////////////////////
// config static members
DWORD CConfig::m_cHostCount = 0;		 // Number of hosts
//LIST_ENTRY CConfig::m_HostHead;		     // Circular double linked list of Javalin hosts
JAVALIN_HOST* CConfig::m_HostNext = NULL;
CRITICAL_SECTION CConfig::m_csCacheLock; // Critical section protects host list


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CConfig::CConfig()
{
}

// Log message to the file
void CConfig::debug(char *msg) 
{
	if(NULL != m_fp) {
		fputs(msg, m_fp);
		fflush(m_fp);
	}
}

CConfig::~CConfig()
{ 
	if(NULL != m_fp) {
		fflush(m_fp);
		fclose(m_fp);
		m_fp = NULL;
	}
}

CConfig* CConfig::getInstance() 
{
	if(NULL == m_pConfig) {
		m_pConfig = new CConfig;

		InitializeCriticalSection( &m_csCacheLock );
		//m_HostHead.Blink = m_HostHead.Flink = &m_HostHead;

		// If there was an error reading the config file
		if(CConfig::readConfigFile() < 0) {
			return NULL;
		}

		m_fCacheInitialized = TRUE;
	}

	return m_pConfig;
}

void CConfig::destroy() {
	if(NULL != m_pConfig) {
		delete m_pConfig;
	}
}

/**
 * Reads user configurable variables from the config file.  
 * @return config   This is a pointer to a struct containing all the config 
 * settings which were read from the config file and is of type configHandle.
 */
int CConfig::readConfigFile() {
    FILE *configFile;
    char line[BUFSIZE];
    char *fieldAndEntry;
    
	CConfig::m_activeConnections = 0;        // total current active connections
	memset(CConfig::m_adminPrefix, 0, 64);   // URL prefix for admin requests
	memset(CConfig::m_adminUser, 0, 64);     // user id for admin requests
	memset(CConfig::m_adminPassword, 0, 64); // password for admin requests

    configFile = fopen(filename, "r");
    
    if(configFile == NULL) {
		CConfig::debug("Config file not found");
		return -1;
    }
    
    /* get a NULL terminated string (line) from the config file */
    while(fgets(line, BUFSIZE, configFile)!= NULL) {
		if(strchr(line, '#') != NULL) {
			continue; /*this is a comment line, skip it */
		}
		
		if((fieldAndEntry = (char *)strstr(line, m_portString))!= NULL) {
			readEntry(m_portString, fieldAndEntry);
		} else if((fieldAndEntry = (char *)strstr(line, m_addressString)) != NULL) {
			readEntry(m_addressString, fieldAndEntry);
		} else if((fieldAndEntry = (char *)strstr(line, m_URLPrefix)) != NULL) {
			readEntry(m_URLPrefix, fieldAndEntry);
		} else if((fieldAndEntry = (char *)strstr(line, m_hostString)) != NULL) {
			readEntry(m_hostString, fieldAndEntry);

		} else if((fieldAndEntry = (char *)strstr(line, m_loadBalanceThreaholdString)) != NULL) {
			readEntry(m_loadBalanceThreaholdString, fieldAndEntry);
		} else if((fieldAndEntry = (char *)strstr(line, m_failThresholdString)) != NULL) {
			readEntry(m_failThresholdString, fieldAndEntry);
		} else if((fieldAndEntry = (char *)strstr(line, m_adminPrefixString)) != NULL) {
			readEntry(m_adminPrefixString, fieldAndEntry);
		} else if((fieldAndEntry = (char *)strstr(line, m_adminUserString)) != NULL) {
			readEntry(m_adminUserString, fieldAndEntry);
		} else if((fieldAndEntry = (char *)strstr(line, m_adminPasswordString)) != NULL) {
			readEntry(m_adminPasswordString, fieldAndEntry);
		}
    }  
    
    fclose(configFile);

	if( strlen(hostname) > 0 && 0 < port ) 
	{
		// add to host list
		addJavalinHost(hostname, port, 1);
	}

	if (0 == m_cHostCount || strlen(urlPrefix) == 0)
	{
		CConfig::debug("Incomplete config file data");
		return -1;
	} else {	
		return 0;
	}
}

/**
 * Processes a line from the config file and stores it in a config structure.
 * @param fieldAndEntry   This is line from the config file consisting of a field and an entry.
 * @param config   This is a pointer to a struct containing all the config 
 * settings which were read from the config file and is of type configHandle.
 * @return config   This is a pointer to a struct containing all the config 
 * settings which were read from the config file and is of type configHandle.
 */
int CConfig::readEntry(char* field, char* fieldAndEntry) {
   
    char entry[BUFSIZE];
    char entryBuffer[BUFSIZE];
    int entryIndex = 0;
    int entryBufferIndex = 0;
    
    if(field != NULL) {
		/* In fieldAndEntry, go end of field +one character('=') and copy into entryBuffer */
		strcpy(entryBuffer,  (fieldAndEntry+strlen(field)+1) );
    } else {
		return -1;
	}
    
    /* read entry */
    while((entryBuffer[entryBufferIndex] != '\0') 
		&& (!isspace(entryBuffer[entryBufferIndex]))) 
	{
		entry[entryIndex] = entryBuffer[entryBufferIndex];    
		entryIndex++; 
		entryBufferIndex++;
    }  
	
	/* go to last character of entry */
    entry[entryIndex] = '\0';                 /* and terminate with NULL character */
    
    if(strcmp(field, m_addressString) == 0) {
		strcpy(hostname, entry);
    } else if(strcmp(field, m_portString) == 0) {
		port = atoi(entry);
    } else if(strcmp(field, m_URLPrefix) == 0) {
		strcpy(urlPrefix, entry);
    } else if(strcmp(field, m_hostString) == 0) {
		addJavalinHost(entry);

    } else if(strcmp(field, m_loadBalanceThreaholdString) == 0) {
		CConfig::m_loadBalanceThreshold = atoi(entry);
    } else if(strcmp(field, m_failThresholdString) == 0) {
		CConfig::m_failThreshold = atoi(entry);
    } else if(strcmp(field, m_adminPrefixString) == 0) {
		strcpy(CConfig::m_adminPrefix, entry);
    } else if(strcmp(field, m_adminUserString) == 0) {
		strcpy(CConfig::m_adminUser, entry);
    } else if(strcmp(field, m_adminPasswordString) == 0) {
		strcpy(CConfig::m_adminPassword, entry);

	} else {
		return -1;
	}

	return 0;
}

void CConfig::addJavalinHost(char* entry)
{
	// parse "entry" into "host:port,weight"
	char pszHostName[64];
	char pszPort[64];
	int iPort;
	int iWeight;

	CHAR* pchColon = strchr( entry, ':' );

	DWORD cchh = pchColon - entry;
	memcpy( pszHostName, entry, cchh );
	pszHostName[cchh] = '\0';
	entry = pchColon + 1;

	CHAR* pchComma = strchr( entry, ',' );

	DWORD cchp = pchComma - entry;
	memcpy( pszPort, entry, cchp );
	pszPort[cchp] = '\0';
	iPort = atoi(pszPort);
	entry = pchComma + 1;

	iWeight = atoi(entry);

	addJavalinHost(pszHostName, iPort, iWeight);
}

void CConfig::addJavalinHost(char* pszHostName, int iPort, int iWeight)
{
	// allocate new host info struct and add to our list
	EnterCriticalSection( &m_csCacheLock );

	JAVALIN_HOST* pHost = (JAVALIN_HOST *) LocalAlloc( LPTR, sizeof( JAVALIN_HOST ));
	if ( !pHost )
	{
		LeaveCriticalSection( &m_csCacheLock );
		return;
	}
	if (0 == m_cHostCount)
	{
		pHost->ListEntry.Flink = &(pHost->ListEntry);
		pHost->ListEntry.Blink = &(pHost->ListEntry);
	}
	else
	{
		pHost->ListEntry.Flink = m_HostNext->ListEntry.Flink;
		m_HostNext->ListEntry.Flink = &(pHost->ListEntry);
		pHost->ListEntry.Blink = &(m_HostNext->ListEntry);
	}

	//  Set the various fields
	pHost->index = m_cHostCount;
	strcpy( pHost->achHostName, pszHostName );
	pHost->port = iPort;
	pHost->weight = iWeight;
	pHost->accumWeight = 0;
	pHost->activeConnect = 0;
	pHost->totalConnect = 0;
	pHost->consecutiveFail = 0;
	pHost->totalFail = 0;
	pHost->totalComplete = 0;
	pHost->totalTime = 0;
	pHost->totalError = 0;
	pHost->consecutiveComplete = 0;
	pHost->consecutiveTime = 0;
	pHost->consecutiveError = 0;
	pHost->active = true;

	m_cHostCount++;
	m_HostNext = pHost;

	LeaveCriticalSection( &m_csCacheLock );
}

