/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: Config.h,v 1.4 2000/01/04 14:10:33 icostanzo Exp $
 *-----------------------------------------------------------------------------
 */

// Config.h: interface for the CConfig class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CONFIG_H__6D8DFFD1_3630_11D3_9070_0050045D7A4E__INCLUDED_)
#define AFX_CONFIG_H__6D8DFFD1_3630_11D3_9070_0050045D7A4E__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/**
Cached Javalin host ientification structure
*/
typedef struct _JAVALIN_HOST
{
	LIST_ENTRY  ListEntry;   // Double linked list entry
	int   index;             // index of this entry in the list (zero-based)

	char  achHostName[64];   // Host name and port
	int   port;

	int   weight;            // per-request weighting
	int   accumWeight;

	int   activeConnect;     // successful connections
	long  totalConnect;

	int   consecutiveFail;   // failed connections
	int   totalFail;

	int    totalComplete;    // total successful completed requests
	double totalTime;        // total request processing time;
	int    totalError;       // processing errors (totalComplete + totalError = totalConnect)

	int    consecutiveComplete; // total successful completed requests
	double consecutiveTime;     // total request processing time;
	int    consecutiveError;    // processing errors (totalComplete + totalError = totalConnect)

	bool   active;           // whether this connection is currently active

} JAVALIN_HOST, *PJAVALIN_HOST;


/**
This class contains the configuration info read from the config file hard-coded at
the top of Config.cpp.

This class is a singleton
*/

static const int BUFSIZE = 1024;

class CConfig  
{
public:	
	// Public static variables used by other objects
	static char hostname[64];
	static char urlPrefix[128];
	static int port;
	
	static DWORD m_cHostCount;			// Number of hosts
	//static LIST_ENTRY m_HostHead;		// Circular double linked list of Javalin hosts
	static JAVALIN_HOST* m_HostNext;
	static CRITICAL_SECTION m_csCacheLock; // Critical section protects host list

	static int m_activeConnections;     // total current active connections
	static int m_loadBalanceThreshold;  // active connection threshold where we actively load balance
	static int m_failThreshold;			// consecutive failed connections before taking a connection offline
	static char m_adminPrefix[64];      // URL prefix for admin requests
	static char m_adminUser[64];        // user id for admin requests
	static char m_adminPassword[64];    // password for admin requests

	// Static singleton accessor methods
	static CConfig* getInstance();
	static void destroy();

	// This is a debug method and should be disabled for production use if desired
	static void debug(char *msg); // Log to file method

protected:
	CConfig();
	virtual ~CConfig();
	static int readConfigFile();
	static int readEntry(char * field, char * fieldAndEntry);

	static void addJavalinHost(char* entry);
	static void addJavalinHost(char* pszHostName, int iPort, int iWeight);

	static CConfig *m_pConfig;
	static char *filename;
	static char *m_portString;
	static char *m_addressString;
	static char *m_URLPrefix;
	static char *m_hostString;
	static char *m_loadBalanceThreaholdString;
	static char *m_failThresholdString;
	static char *m_adminPrefixString;
	static char *m_adminUserString;
	static char *m_adminPasswordString;

	static BOOL  m_fCacheInitialized;	// Indicates whether the cache is initialized

	static FILE *m_fp;
};

#endif // !defined(AFX_CONFIG_H__6D8DFFD1_3630_11D3_9070_0050045D7A4E__INCLUDED_)
