package com.lutris.xml;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import java.util.Enumeration;
import java.util.Vector;
import java.util.NoSuchElementException;

/*
 * @author Hani Suleiman
 */
public class DOMUtil
{
  public static Element findElementByID(Element root, String id)
  {
    BreadthFirstEnumeration enum = new BreadthFirstEnumeration(root);
    while(enum.hasMoreElements())
    {
      Object o = enum.nextElement();
      if(o instanceof Element)
      {
        Element e = (Element)o;
        if (id.equals(e.getAttribute("ID"))
          || id.equals(e.getAttribute("id")))
        {
          return e;
        }
      }
    }
    return null;
  }
  
}

final class ElementEnumerator implements Enumeration
{
  int count = 0;
  private Vector list = new Vector();
  
  public ElementEnumerator(Element e)
  {
    NodeList elems = e.getChildNodes();
    for(int i=0;i<elems.getLength();i++)
    {
      Node n = elems.item(i);
      if(n instanceof Element)
      {
        list.addElement(n);
      }
    }
  }
  
  public boolean hasMoreElements()
  {
    if(count<list.size()) return true;
    return false;
  }
  
  public Object nextElement()
  {
    return list.get(count++);
  }
}  

final class BreadthFirstEnumeration implements Enumeration 
{
  protected Queue queue;
  
  public BreadthFirstEnumeration(Element rootNode)
  {
    super();
    Vector v = new Vector(1);
    v.addElement(rootNode);
    queue = new Queue();
    queue.enqueue(new ElementEnumerator(rootNode));
  }
  
  public boolean hasMoreElements() 
  {
    return (!queue.isEmpty() &&
      ((Enumeration)queue.firstObject()).hasMoreElements());
  }
  
  public Object nextElement() 
  {
    Enumeration enumer = (Enumeration)queue.firstObject();
    Element  node = (Element)enumer.nextElement();
    Enumeration children = new ElementEnumerator(node);
    
    if (!enumer.hasMoreElements()) 
    {
      queue.dequeue();
    }
    if (children.hasMoreElements()) 
    {
      queue.enqueue(children);
    }
    return node;
  }
  
  
  // A simple queue with a linked list data structure.
  final class Queue 
  {
    QNode head; // null if empty
    QNode tail;
    
    final class QNode 
    {
      public Object object;
      public QNode  next; // null if end
      public QNode(Object object, QNode next) 
      {
        this.object = object;
        this.next = next;
      }
    }
    
    public void enqueue(Object anObject) 
    {
      if (head == null) 
      {
        head = tail = new QNode(anObject, null);
      }
      else 
      {
        tail.next = new QNode(anObject, null);
        tail = tail.next;
      }
    }
    
    public Object dequeue() 
    {
      if (head == null) 
      {
        throw new NoSuchElementException("No more elements");
      }
      
      Object retval = head.object;
      QNode oldHead = head;
      head = head.next;
      if (head == null) 
      {
        tail = null;
      }
      else 
      {
        oldHead.next = null;
      }
      return retval;
    }
    
    public Object firstObject() 
    {
      if (head == null) 
      {
        throw new NoSuchElementException("No more elements");
      }
      return head.object;
    }
    
    public boolean isEmpty() 
    {
      return head == null;
    }
  }
}

