/*-----------------------------------------------------------------------------
 * Dogen -- the Data Object GENerator
 * Copyright 1999 Mindgarden.com, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *
 *      This product includes Dogen software developed by Mindgarden.com, Inc.
 *
 * 4. The name of Mindgarden.com, Inc. may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MINDGARDEN.COM, INC. ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL MINDGARDEN.COM BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: CodeGenerator.java,v 1.1.1.1 1999/09/23 13:42:20 andy Exp $
 *-----------------------------------------------------------------------------
 */

package dogen.generator;

import java.io.*;

public abstract class CodeGenerator {
    protected static final String DECLSUFFIX = "DECL";
    protected String declarationName;
    protected String qualifiedName;
    protected String pkg;
    protected String name;

    protected CodeGenerator(String declarationName) throws MetaException {
	if(!declarationName.endsWith(DECLSUFFIX)) {
	    throw error(declarationName + " doesn't end with " + DECLSUFFIX);
	}
	    
	this.declarationName = declarationName;

	int dot = declarationName.lastIndexOf((int) '.');

	if(dot > -1) {
	    name = declarationName.substring(dot + 1);
	    pkg = declarationName.substring(0, dot);
	} else {
	    name = declarationName;
	    pkg = null;
	}

	name = name.substring(0, name.length() - DECLSUFFIX.length());

	if(pkg != null) {
	    qualifiedName = pkg + "." + name;
	} else {
	    qualifiedName = name;
	}
    }

    public String getName() {
	return name;
    }

    public File getSourceFile(String suffix) {
	return new File(qualifiedName.replace('.', File.separatorChar) +
			suffix + ".java");
    }

    public File getSourceFile() {
	return getSourceFile("");
    }

    public String getFullyQualifiedName() {
	return qualifiedName;
    }
    
    protected static MetaException error(String msg) {
	return new MetaException(msg);
    }

    protected void emitPackage(CodeWriter cw) {
	if(pkg != null) {
	    cw.println("package " + pkg + ";\n");
	}
    }

    protected void createDirectoryFor(File path) {
	File parent = new File(path.getParent());

	if(!parent.isDirectory()) {
	    if(!parent.mkdirs()) {
		error("can't create directory: " + parent);
	    }
	}
    }

    public CodeWriter outputTo(File file) throws IOException {
	createDirectoryFor(file);
	return new CodeWriter(new FileOutputStream(file));
    }

    
    public void generateJavaCode() throws IOException {
	CodeWriter cw = outputTo(getSourceFile());

	emitJavaCode(cw);

	cw.close();
    }

    public abstract void emitJavaCode(CodeWriter cw);
}
