/*-----------------------------------------------------------------------------
 * Dogen -- the Data Object GENerator
 * Copyright 1999 Mindgarden.com, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *
 *      This product includes Dogen software developed by Mindgarden.com, Inc.
 *
 * 4. The name of Mindgarden.com, Inc. may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MINDGARDEN.COM, INC. ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL MINDGARDEN.COM BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: Dogen.java,v 1.1.1.1 1999/09/23 13:42:20 andy Exp $
 *-----------------------------------------------------------------------------
 */

package dogen.generator;

import java.util.*;
import java.io.*;

public class Dogen {
    protected static final String SCHEMA = "-schema";

    protected String argv[];
    protected MetaSchema metaSchema;

    public static void main(String argv[]) {
	Dogen instance = new Dogen(argv);
	System.exit(instance.run());
    }

    protected static void debug(String msg) {
	System.out.println("Dogen: " + msg);
    }

    protected void error(String msg) {
	System.out.println("dogen: " + msg);
	System.exit(1);
    }

    protected void usage() {
	System.out.println("java dogen.generator.Dogen options\n");
	System.out.println("where options are:");
	System.out.println("    -schema <classname>     read specified schema; "+
			   "required exactly once");
	System.out.println("    -emit <classname>       emit Java code for " +
			   "specified class");
	System.exit(1);
    }

    public Dogen(String argv[]) {
	this.argv = argv;
	metaSchema = null;
    }

    public int run() {
	for(int i = 0; i < argv.length; ++i) {
	    if(argv[i].equals(SCHEMA) && (i < (argv.length - 1))) {
		if(metaSchema != null) {
		    usage();
		} else {
		    buildSchema(argv[++i]);
		}
	    } else {
		usage();
	    }
	}

	return 0;
    }

    public void buildSchema(String schemaToUse) {
	try {
	    Class metaSchemaClass = Class.forName(schemaToUse);
	    Object schemaDecl = metaSchemaClass.newInstance();
	    metaSchema = new MetaSchema(schemaDecl);

	    metaSchema.generateJavaCode();

	    for(Enumeration e = MetaClass.getMetaClasses();
		e.hasMoreElements();)
	    {
		MetaClass c = (MetaClass) e.nextElement();
		c.generateJavaCode();
	    }
	} catch (Exception ex) {
	    ex.printStackTrace();
	    error("caught " + ex.toString() + " while processing " +
		  schemaToUse);
	}
    }
}
