/*-----------------------------------------------------------------------------
 * Dogen -- the Data Object GENerator
 * Copyright 1999 Mindgarden.com, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *
 *      This product includes Dogen software developed by Mindgarden.com, Inc.
 *
 * 4. The name of Mindgarden.com, Inc. may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MINDGARDEN.COM, INC. ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL MINDGARDEN.COM BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: DataObjectQuery.java,v 1.1.1.1 1999/09/23 13:42:20 andy Exp $
 *-----------------------------------------------------------------------------
 */

package dogen.runtime;

import java.sql.*;
import java.util.*;

import com.lutris.util.*;
import com.lutris.appserver.server.sql.*;

public abstract class DataObjectQuery implements Query {
    public static boolean trace = false;

    public DataObjectQuery() {
    }

    protected abstract Schema getSchema();
    protected abstract String getSqlTemplate();
    protected abstract DataObject buildResult(ResultSet rs, int offset)
	throws DataObjectException;

    protected Object buildResult(ResultSet rs) throws DataObjectException {
	return buildResult(rs, 0);
    }

    protected void debug(String msg) {
	System.out.println("DataObjectQuery: " + msg);
    }

    protected void setStatementParameters(PreparedStatement s)
	throws SQLException
    {
	// override for useful work
    }

    public ResultSet executeQuery(DBConnection connection) {
	try {
	    String template = getSqlTemplate();

	    if(trace) {
		debug(getClass().getName() + " -> " + template);
	    }

	    PreparedStatement stmt = connection.prepareStatement(template);

	    if(trace) {
		debug("created PreparedStatement");
	    }

      	    setStatementParameters(stmt);

	    if(trace) {
		debug("parameters are set");
	    }

	    ResultSet retval = connection.executeQuery(stmt, getClass().getName());

	    if(trace) {
		debug("result set created");
		ResultSetMetaData md = retval.getMetaData();

		debug("there are " + md.getColumnCount() + " columns in the result");
	    }

	    return retval;
	} catch (SQLException e) {
	    throw new FatalExceptionError(e);
	}
    }

    public Object getResult() throws DatabaseManagerException {
	Object retval = null;

	DBQuery dbQuery = null;
	try {
	    dbQuery = getSchema().getDatabaseManager().createQuery();
	    dbQuery.query(this);

	    retval = dbQuery.next();

	    if(dbQuery.next() != null) {
		System.err.println("Error: multiple results from "
				   + getClass().getName());
	    }
	} catch (ObjectIdException oidExp) {
	    throw new FatalExceptionError(oidExp);
	} catch (SQLException sqlExp) {
	    throw new FatalExceptionError(sqlExp);
	} finally {
	    if (dbQuery != null)
		dbQuery.release();
	}

	return retval;
    }

    public Vector getResults() throws DatabaseManagerException {
	Vector result = new Vector();

	Object resultElement;
	DBQuery dbQuery = null;
	try {
	    dbQuery = getSchema().getDatabaseManager().createQuery();
	    dbQuery.query(this);

	    int resultCount = 1;
	    while ((resultElement = dbQuery.next()) != null) {
		result.addElement(resultElement);
		if(trace && (resultElement != null)) {
		    debug("got result " + resultCount + ", of class " + resultElement.getClass().getName());
		}
		resultCount++;
	    }
	} catch (SQLException sqlExp) {
	    throw new FatalExceptionError(sqlExp);
	} catch (ObjectIdException oidExp) {
	    throw new FatalExceptionError(oidExp);
	} finally {
	    if (dbQuery != null)
		dbQuery.release();
	}

	return result;
    }

    public final Object next(ResultSet rs) throws SQLException {
	try {
	    if(rs.next()) {
		return buildResult(rs);
	    } else {
		return null;
	    }
	} catch (SQLException e) {
	    while(e != null) {
		debug("caught: " + e);
		debug("error code = " + e.getErrorCode());
		debug("state = " + e.getSQLState());

		e = e.getNextException();
	    }
	    return null;
	} catch (DataObjectException e) {
	    if(e.getWrappedException() != null) {
		debug("caught: " + e.getWrappedException());
	    } else {
		debug("caught: " + e);
	    }
	    return null;
	} catch (Exception e) {
	    debug("caught: " + e);
	    return null;
	}
    }
}



