/*-----------------------------------------------------------------------------
 * Dogen -- the Data Object GENerator
 * Copyright 1999 Mindgarden.com, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *
 *      This product includes Dogen software developed by Mindgarden.com, Inc.
 *
 * 4. The name of Mindgarden.com, Inc. may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MINDGARDEN.COM, INC. ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL MINDGARDEN.COM BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: DefaultTypeMapping.java,v 1.1.1.1 1999/09/23 13:42:20 andy Exp $
 *-----------------------------------------------------------------------------
 */

package dogen.types;

import dogen.generator.*;
import dogen.runtime.*;

import java.util.*;

/*
 * This class defines a mapping of types that can be used in a data object
 * declaration to MetaType classes used within the code generator. You may
 * want to customize the default type mapping to add new types to the code
 * generator or to change the behavior of an existing type. For example,
 * the default mapping uses NUMBER(19,0) to represent object references.
 * That won't work with Postres because it doesn't have enough precision.
 * So, there's a PosgresTypeMapping which uses INT8 instead.
 *
 * Each pair of fields represents a single mapping. The type of the first
 * field is "from" type. The second of the pair is the "to" MetaType. Always
 * make sure there are an even number of fields.
 *
 * If you redefine a "from" type the last definition is used. Use Java
 * inheritance to load up an existing mapping and then change only what you
 * need to.
 */
public class DefaultTypeMapping {
    public DefaultTypeMapping() {
    }

    public byte      defaultType1;
    public T_byte    defaultType1m;
    public short     defaultType2;
    public T_short   defaultType2m;
    public int       defaultType3;
    public T_int     defaultType3m;
    public long      defaultType4;
    public T_long    defaultType4m;
    public char      defaultType5;
    public T_char    defaultType5m;
    public boolean   defaultType6;
    public T_boolean defaultType6m;
    public float     defaultType7;
    public T_float   defaultType7m;
    public double    defaultType8;
    public T_double  defaultType8m;
    public String    defaultType9;
    public T_String  defaultType9m;
    public Date      defaultType10;
    public T_Date    defaultType10m;
    public OID       defaultType11;
    public T_OID     defaultType11m;

    // MetaClass wants to be able to look up a T_Reference to see if it should
    // be mapped to something else.

    public T_Reference defaultType12;
    public T_Reference defaultType12m;
}
