/*-----------------------------------------------------------------------------
 * Dogen -- the Data Object GENerator
 * Copyright 1999 Mindgarden.com, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *
 *      This product includes Dogen software developed by Mindgarden.com, Inc.
 *
 * 4. The name of Mindgarden.com, Inc. may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MINDGARDEN.COM, INC. ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL MINDGARDEN.COM BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: Main.java,v 1.1.1.1 1999/09/23 13:42:20 andy Exp $
 *-----------------------------------------------------------------------------
 */

package dogenDemo.presentation;

import java.util.*;
import java.sql.*;
import java.io.*;

import com.lutris.util.*;
import com.lutris.xml.xmlc.*;
import com.lutris.appserver.server.Enhydra;
import com.lutris.logging.Logger;
import com.lutris.appserver.server.httpPresentation.*;

import org.w3c.dom.*;
import org.w3c.dom.html.*;

import dogen.runtime.*;
import dogenDemo.data.*;

public class Main implements HttpPresentation {
    protected void debug(String msg) {
	Enhydra.getLogChannel().write(Logger.INFO, "Main: " + msg);
    }

    protected String get(HttpPresentationComms comms, String name)
	throws HttpPresentationException
    {
	return comms.request.getParameter(name);
    }

    protected void performOperation(HttpPresentationComms comms,
				    StringBuffer message)
	throws HttpPresentationException, DataObjectException
    {
	String operation = get(comms, "operation");

	try {
	    if(operation != null) {
		if(operation.equals("Create Tables")) {
		    DogenDemoSchema.create();

		    DogenDemoSchema.begin();
		    Teacher t1 = new Teacher();
		    Teacher t2 = new Teacher();
		    Student s1 = new Student();

		    t1.setFirstName("John");
		    t1.setLastName("Barlycorn");
		    t1.setAge(45);
		    t1.setFavoriteStudent(s1);
		    t1.setSalary(10000);

		    t2.setFirstName("Ralph");
		    t2.setLastName("Zithernose");
		    t2.setAge(99);
		    t2.setSalary(100000);

		    s1.setFirstName("Steve");
		    s1.setLastName("Studyhard");
		    s1.setAge(17);
		    s1.setMajor("Apple Polishing");

		    DogenDemoSchema.end();
		    message.append("The database was formatted.\n");
		} else if(operation.equals("Drop Tables")) {
		    DogenDemoSchema.destroy();
		    message.append("The database was destroyed.\n");
		} else if(operation.equals("Create Student")) {
		    String firstName = get(comms, "firstName");
		    String lastName = get(comms, "lastName");
		    String age = get(comms, "age");
		    String major = get(comms, "major");

		    try {
			DogenDemoSchema.begin();
			Student s = new Student();

			s.setFirstName(firstName);
			s.setLastName(lastName);
			s.setAge(Integer.parseInt(age));
			s.setMajor(major);

			DogenDemoSchema.end();
		    } catch (Exception e) {
			message.append("caught: " + e.toString() +
				       "\naborting transaction");
			DogenDemoSchema.abort();
		    }
		}
	    }
	} catch (Exception e) {
	    StringWriter sw = new StringWriter();
	    PrintWriter pw = new PrintWriter(sw);
	    
	    e.printStackTrace(pw);
	    message.append(sw.toString());
	}
    }

    protected void setupForSchema(MainHTML page, StringBuffer message) {
	try {
	    if(DogenDemoSchema.exists()) {
		HTMLUtil.removeXMLNode(page.getElementUnformatted());
	    } else {
		HTMLUtil.removeXMLNode(page.getElementFormatted());
	    }
	} catch (Exception e) {
	    StringWriter sw = new StringWriter();
	    PrintWriter pw = new PrintWriter(sw);
	    
	    e.printStackTrace(pw);
	    message.append(sw.toString());
	}

    }

    protected void formatTeacherTable(MainHTML page, StringBuffer message) {
	try {
	    TableFormatter f =
		new TableFormatter(page.getElementTeacher());

	    if(!DogenDemoSchema.exists()) {
		f.newRow("teacherEmpty");
		f.completeFormatting();
		return;
	    }

	    Vector results;

	    TeacherQuery q = new TeacherQuery();
	    results = q.getResults();

	    if((results != null) && (results.size() > 0)) {
		f.newRow("teacherHeader");

		for(int i = 0; i < results.size(); ++i) {
		    Teacher t = (Teacher) results.elementAt(i);

		    f.newRow("teacherRow" + (i % 2));

		    f.setTemplateText("objid", t.get$OID().toString());
		    f.setTemplateText("version", "" + t.getVersion());
		    f.setTemplateText("firstName", t.getFirstName());
		    f.setTemplateText("lastName", t.getLastName());
		    f.setTemplateText("age", "" + t.getAge());

		    if(t.getFavoriteStudent() != null) {
			f.setTemplateText("favoriteStudent",
					  t.getFavoriteStudent().
					  get$OID().toString());
		    } else {
			f.setTemplateText("favoriteStudent", "null");
		    }

		    f.setTemplateText("salary", "" + t.getSalary());
		}
	    } else {
		f.newRow("teacherEmpty");
	    }
	    
	    f.completeFormatting();
	} catch (Exception e) {
	    StringWriter sw = new StringWriter();
	    PrintWriter pw = new PrintWriter(sw);
	    
	    e.printStackTrace(pw);
	    message.append(sw.toString());
	}
    }

    protected void formatStudentTable(MainHTML page, StringBuffer message) {
	try {
	    TableFormatter f =
		new TableFormatter(page.getElementStudent());

	    if(!DogenDemoSchema.exists()) {
		f.newRow("studentEmpty");
		f.completeFormatting();
		return;
	    }

	    Vector results;

	    StudentQuery q = new StudentQuery();
	    results = q.getResults();

	    f.newRow("studentHeader");

	    if((results != null) && results.size() > 0) {

		for(int i = 0; i < results.size(); ++i) {
		    Student s = (Student) results.elementAt(i);

		    f.newRow("studentRow" + (i % 2));

		    f.setTemplateText("objid", s.get$OID().toString());
		    f.setTemplateText("version", "" + s.getVersion());
		    f.setTemplateText("firstName", s.getFirstName());
		    f.setTemplateText("lastName", s.getLastName());
		    f.setTemplateText("age", "" + s.getAge());
		    f.setTemplateText("major", s.getMajor());
		}
	    } else {
		f.newRow("studentEmpty");
	    }
	    
	    f.newRow("studentInput");
	    f.completeFormatting();
	} catch (Exception e) {
	    StringWriter sw = new StringWriter();
	    PrintWriter pw = new PrintWriter(sw);
	    
	    e.printStackTrace(pw);
	    message.append(sw.toString());
	}
    }

    public void run(HttpPresentationComms comms) 
        throws HttpPresentationException
    {
	StringBuffer message = new StringBuffer();
	MainHTML page = new MainHTML();

	try {
	    if(DogenDemoSchema.getInstance() == null) {
		new DogenDemoSchema(Enhydra.getApplication().getConfig(),
				    null); // null means the default db
	    }

	    performOperation(comms, message);
	    setupForSchema(page, message);
	    formatTeacherTable(page, message);
	    formatStudentTable(page, message);

	    if(message.length() > 0) {
		page.setTextErrorMsg(message.toString());
	    } else {
		Node resultContainer = page.getElementErrorBox();
		Node parent = resultContainer.getParentNode();
		parent.removeChild(resultContainer);
	    }

	    comms.response.writeHTML(page.toDocument());
	} catch (Exception e) {
	    throw new FatalExceptionError(e);
	}
    }

}
