/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: EnhydraConn.cpp,v 1.5 2000/01/04 14:10:33 icostanzo Exp $
 *-----------------------------------------------------------------------------
 */

// HttpStream.cpp: implementation of the HttpStream class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "EnhydraConn.h"
#include <winsock2.h>


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CEnhydraConn::CEnhydraConn()
{
	m_enhydraSocketHandle = -1;
	m_javalinHost = NULL;
}

CEnhydraConn::~CEnhydraConn()
{
	if(m_enhydraSocketHandle > 0)
	{
		EnterCriticalSection( &(CConfig::m_csCacheLock) );

		closesocket(m_enhydraSocketHandle);

		m_javalinHost->activeConnect --;
		m_javalinHost->accumWeight -= m_javalinHost->weight;
		CConfig::m_activeConnections--;

		LeaveCriticalSection( &(CConfig::m_csCacheLock) );
	}
}

CHttpResponse* CEnhydraConn::getResponse()
{
	return new CHttpResponse(m_enhydraSocketHandle);
}

/**
 * Opens socket to LBS and returns socketID (socketHandle).
 * @param config   This is a pointer to a struct containing all the config 
 * settings which were read from the config file and is of type configHandle.
 * @return SocketID   This int is the file descriptor for the socket created. 
 */
int CEnhydraConn::connectToEnhydra() 
{                  
    int  returnCode = 0;
    struct sockaddr_in sockAddr; 
    struct hostent * hostAddress;
    char  ipAddress[1024];
	static int connectingCount = 0;
	JAVALIN_HOST* tmp = NULL;
	DWORD i;

	EnterCriticalSection( &(CConfig::m_csCacheLock) );

	connectingCount++;
	m_javalinHost = CConfig::m_HostNext;
	CConfig::m_HostNext = CONTAINING_RECORD( CConfig::m_HostNext->ListEntry.Flink, JAVALIN_HOST, ListEntry );

	if (CConfig::m_loadBalanceThreshold < (connectingCount + CConfig::m_activeConnections))
	{
		// find the active connection with the lowest usage count
		tmp = m_javalinHost;
		for (i=1; i<CConfig::m_cHostCount; i++)
		{
			tmp = CONTAINING_RECORD( tmp->ListEntry.Flink, JAVALIN_HOST, ListEntry );
			if (tmp->active && tmp->activeConnect < m_javalinHost->activeConnect)
			{
				m_javalinHost = tmp;
			}
		}
	}

	LeaveCriticalSection( &(CConfig::m_csCacheLock) );

	bool connected = false;
	for (i=0; i<CConfig::m_cHostCount && !connected; i++)
	{
		if (m_javalinHost->active)
		{
			returnCode = 0;

			//char foo[1024];
			//sprintf(foo, "%d %s:%d", i, m_javalinHost->achHostName, m_javalinHost->port);
			//CConfig::debug(foo);
			//CConfig::debug("\r\n");

			/* if address is of the form "ra.lutris.com" convert to dotted decimal format */
			if(isalpha(m_javalinHost->achHostName[0]) != 0) {
				hostAddress = gethostbyname(m_javalinHost->achHostName);
	
				if(hostAddress == 0) {
					returnCode = - 1;
				} else {
					strcpy(ipAddress,(char *) inet_ntoa(* (struct in_addr *)hostAddress->h_addr));
				}
		    } else {
				strcpy(ipAddress, m_javalinHost->achHostName);
		    }
    
			if (0 <= returnCode)
			{
				m_enhydraSocketHandle = socket(AF_INET, SOCK_STREAM, 0);
    
			    if(m_enhydraSocketHandle < 0) {
					returnCode = m_enhydraSocketHandle;
				}
			}

			if (0 <= returnCode)
			{
			    memset(&sockAddr, 0, sizeof(struct sockaddr_in));
				sockAddr.sin_family = AF_INET;
				sockAddr.sin_port = htons(m_javalinHost->port);
				sockAddr.sin_addr.s_addr = inet_addr(ipAddress); 
    
			    returnCode = connect(m_enhydraSocketHandle, (struct sockaddr *) &sockAddr,
									 sizeof(struct sockaddr_in));
			}

			EnterCriticalSection( &(CConfig::m_csCacheLock) );

			connectingCount--;

			if (0 <= returnCode)
			{
				// successful connection
				connected = true;
				m_javalinHost->accumWeight += m_javalinHost->weight;
				m_javalinHost->activeConnect ++;
				m_javalinHost->totalConnect ++;
				m_javalinHost->consecutiveFail = 0;
				CConfig::m_activeConnections++;
				//CConfig::debug("Success\r\n");
			}
			else
			{
				// failed connection
				m_javalinHost->consecutiveFail ++;
				m_javalinHost->totalFail ++;

				if (CConfig::m_failThreshold < m_javalinHost->consecutiveFail)
				{
					m_javalinHost->active = false;
				}

				char foo[1024];
				sprintf(foo, "Failed to connect to %s:%d\r\n", m_javalinHost->achHostName, m_javalinHost->port);
				CConfig::debug(foo);

				m_javalinHost = CONTAINING_RECORD( m_javalinHost->ListEntry.Flink, JAVALIN_HOST, ListEntry );
			}

			LeaveCriticalSection( &(CConfig::m_csCacheLock) );
		}
	}

	return returnCode;
}

void CEnhydraConn::completeRequest(bool success, double duration)
{
	EnterCriticalSection( &(CConfig::m_csCacheLock) );

	if (success)
	{
		m_javalinHost->totalComplete ++;
		m_javalinHost->totalTime += duration;
		m_javalinHost->consecutiveComplete ++;
		m_javalinHost->consecutiveTime += duration;
	}
	else
	{
		m_javalinHost->totalError ++;
		m_javalinHost->consecutiveError ++;
	}

	LeaveCriticalSection( &(CConfig::m_csCacheLock) );
}

int CEnhydraConn::writeToEnhydra(char *buf, int len) 
{
	int totalBytesWritten = 0;
	int bytesWritten = 0;

	while((bytesWritten = send(m_enhydraSocketHandle, (char *) buf, len, 0)) > 0) { 
		totalBytesWritten += bytesWritten;
		if(totalBytesWritten >= len) {
			break;
		}
	}

#ifdef _DEBUG
	CConfig::debug(">>");
	CConfig::debug(buf);
#endif

	return totalBytesWritten;
}
