/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: HttpRequest.cpp,v 1.5 1999/12/31 22:08:57 icostanzo Exp $
 *-----------------------------------------------------------------------------
 */

// EnvVariables.cpp: implementation of the CEnvVariables class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "EnhydraConn.h"
#include "HttpRequest.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CHttpRequest::CHttpRequest(CHttpServerContext *pContext)
{
    memset(RequestMethod, 0, HEADER_VAR_SIZE);  
    memset(ScriptName , 0, HEADER_VAR_SIZE);
    memset(QueryString , 0, HEADER_VAR_SIZE);
    memset(PathInfo , 0, HEADER_VAR_SIZE);
    memset(ServerProtocol , 0, HEADER_VAR_SIZE);    
    memset(ContentLength , 0, HEADER_VAR_SIZE);
    memset(ContentType , 0, HEADER_VAR_SIZE);
	memset(AuthPass , 0, HEADER_VAR_SIZE);
	memset(AuthType , 0, HEADER_VAR_SIZE);
	memset(GatewayInterface , 0, HEADER_VAR_SIZE);
	memset(PathTranslated , 0, HEADER_VAR_SIZE);
	memset(RemoteAddr , 0, HEADER_VAR_SIZE);
	memset(RemoteHost , 0, HEADER_VAR_SIZE);
	memset(RemoteUser , 0, HEADER_VAR_SIZE);
	memset(ServerName , 0, HEADER_VAR_SIZE);
	memset(ServerPort , 0, HEADER_VAR_SIZE);
	memset(ServerSoftware , 0, HEADER_VAR_SIZE);
	memset(RequestLine , 0, HEADER_VAR_SIZE);
	memset(Accept , 0, HEADER_VAR_SIZE);
	memset(AcceptEncoding , 0, HEADER_VAR_SIZE);
	memset(AcceptLanguage , 0, HEADER_VAR_SIZE);
	memset(Referer , 0, HEADER_VAR_SIZE);
	memset(Cookie , 0, HEADER_VAR_SIZE);
	memset(UserAgent , 0, HEADER_VAR_SIZE);
	memset(AllHttp, 0, HTTP_VAR_SIZE);

	memset(m_headerList, 0, sizeof(m_headerList));

   	init(pContext);
}

// Protected default constructor
CHttpRequest::CHttpRequest()
{
}

CHttpRequest::~CHttpRequest()
{
}

/**
 * Creates a struct of pointers to environment variables (envVars).
 * @return  cgiArgsHandle   This argument is of type cgiArgsHandle, and is a pointer to a struct 
 * containing all pertinent envVars (Environment Variables) for the script process.
 */
void CHttpRequest::init(CHttpServerContext *pContext) 
{
	DWORD bufSize = HEADER_VAR_SIZE;
	pContext->GetServerVariable("REQUEST_METHOD", RequestMethod, &bufSize);
	bufSize = HEADER_VAR_SIZE;
	pContext->GetServerVariable("SCRIPT_NAME", ScriptName, &bufSize);
	bufSize = HEADER_VAR_SIZE;
	pContext->GetServerVariable("QUERY_STRING", QueryString, &bufSize);
	bufSize = HEADER_VAR_SIZE;
	pContext->GetServerVariable("SERVER_PROTOCOL", ServerProtocol, &bufSize);
	bufSize = HEADER_VAR_SIZE;
	pContext->GetServerVariable("PATH_INFO", PathInfo, &bufSize);
	bufSize = HEADER_VAR_SIZE;
	pContext->GetServerVariable("CONTENT_TYPE", ContentType, &bufSize);
	bufSize = HEADER_VAR_SIZE;
	pContext->GetServerVariable("CONTENT_LENGTH", ContentLength, &bufSize);	
	bufSize = HEADER_VAR_SIZE;
	pContext->GetServerVariable("AUTH_PASS", AuthPass, &bufSize);
	bufSize = HEADER_VAR_SIZE;
	pContext->GetServerVariable("AUTH_TYPE", AuthType, &bufSize);
	bufSize = HEADER_VAR_SIZE;
	pContext->GetServerVariable("GATEWAY_INTERFACE", GatewayInterface, &bufSize);
	bufSize = HEADER_VAR_SIZE;
	pContext->GetServerVariable("PATH_TRANSLATED", PathTranslated, &bufSize);
	bufSize = HEADER_VAR_SIZE;
	pContext->GetServerVariable("REMOTE_ADDR", RemoteAddr, &bufSize);
	bufSize = HEADER_VAR_SIZE;
	pContext->GetServerVariable("REMOTE_HOST", RemoteHost, &bufSize);
	bufSize = HEADER_VAR_SIZE;
	pContext->GetServerVariable("REMOTE_USER", RemoteUser, &bufSize);
	bufSize = HEADER_VAR_SIZE;
	pContext->GetServerVariable("SERVER_NAME", ServerName, &bufSize);
	bufSize = HEADER_VAR_SIZE;
	pContext->GetServerVariable("SERVER_PORT", ServerPort, &bufSize);
	bufSize = HEADER_VAR_SIZE;
	pContext->GetServerVariable("SERVER_SOFTWARE", ServerSoftware, &bufSize);
	bufSize = HEADER_VAR_SIZE;
	pContext->GetServerVariable("HTTP_ACCEPT", Accept, &bufSize);
	bufSize = HTTP_VAR_SIZE;
	pContext->GetServerVariable("HTTP_ACCEPT_ENCODING", AcceptEncoding, &bufSize);
	bufSize = HTTP_VAR_SIZE;
	pContext->GetServerVariable("HTTP_ACCEPT_LANGUAGE", AcceptLanguage, &bufSize);
	bufSize = HTTP_VAR_SIZE;
	pContext->GetServerVariable("HTTP_USER_AGENT", UserAgent, &bufSize);
	bufSize = HTTP_VAR_SIZE;
	pContext->GetServerVariable("HTTP_REFERER", Referer, &bufSize);
	bufSize = HTTP_VAR_SIZE;
	pContext->GetServerVariable("HTTP_Cookie", Cookie, &bufSize);
	bufSize = HTTP_VAR_SIZE;
	pContext->GetServerVariable("ALL_HTTP", AllHttp, &bufSize);

	buildRequestLine();
	buildHeaders();
}

void CHttpRequest::buildHeaders()
{
    if(strlen(RequestMethod) > 0) {
		appendHeader("Request-method",  RequestMethod, m_headerList);
	}
    if(strlen(PathInfo) > 0) {
		appendHeader("Path-info",  PathInfo, m_headerList);
	}
    if(strlen(QueryString) > 0) {
		appendHeader("Query-string",  QueryString, m_headerList);
	}
    if(strlen(ServerProtocol) > 0) {
		appendHeader("Server-protocol",  ServerProtocol, m_headerList);
	}
    if(strlen(ContentType) > 0) {
		appendHeader("Content-Type",  ContentType, m_headerList);
	}
	if(strlen(ContentLength) > 0) {
	    appendHeader("Content-Length",  ContentLength, m_headerList);
	}
	if(strlen(ScriptName) > 0) {
		appendHeader("Script-Name",  ScriptName, m_headerList);
	}
	if(strlen(AuthPass) > 0) {
		appendHeader("Auth-Pass",  AuthPass, m_headerList);
	}
	if(strlen(AuthType) > 0) {
		appendHeader("Auth-Type",  AuthType, m_headerList);
	}
	if(strlen(GatewayInterface) > 0) {
		appendHeader("Gateway-Interface",  GatewayInterface, m_headerList);
	}
	if(strlen(PathTranslated) > 0) {
		appendHeader("Path-Translated",  PathTranslated, m_headerList);
	}
	if(strlen(RemoteAddr) > 0) {
		appendHeader("Remote-Addr",  RemoteAddr, m_headerList);
	}
	if(strlen(RemoteHost) > 0) {
		appendHeader("Remote-Host",  RemoteHost, m_headerList);
	}
	if(strlen(RemoteUser) > 0) {
		appendHeader("Remote-User",  RemoteUser, m_headerList);
	}
	if(strlen(ServerName) > 0) {
		appendHeader("Server-Name",  ServerName, m_headerList);
	}
	if(strlen(ServerPort) > 0) {
		appendHeader("Server-Port",  ServerPort, m_headerList);
	}
	if(strlen(ServerSoftware) > 0) {
		appendHeader("Server-Software",  ServerSoftware, m_headerList);
	}
	if(strlen(Accept) > 0) {
		appendHeader("Accept", Accept, m_headerList);
	}
	if(strlen(AcceptEncoding) > 0) {
		appendHeader("Accept-Encoding", AcceptEncoding, m_headerList);
	}
	if(strlen(AcceptLanguage) > 0) {
		appendHeader("Accept-Language", AcceptLanguage, m_headerList);
	}
	if(strlen(Referer) > 0) {
		appendHeader("Referer", Referer, m_headerList);
	}
	if(strlen(Cookie) > 0) {
		appendHeader("Cookie", Cookie, m_headerList);
	}
	if(strlen(UserAgent) > 0) {
		appendHeader("User-Agent", UserAgent, m_headerList);
	}
    if(strlen(AllHttp) > 0) {
		strcat(m_headerList, AllHttp);
		strcat(m_headerList, "\r\n");
    }

	strcat(m_headerList, "\r\n");
}

void CHttpRequest::buildRequestLine()
{
	// This method builds
	// RequestLine = RequestMethod + PathInfo + ServerProtocol
	// e.g. "GET /foo.html HTTP/1.0 \r\n";
	// From http://MyIISServer/blah/enhydra.dll/foo.html
	// Where foo.html is the Path Info

	if(NULL == PathInfo || strlen(PathInfo) == 0) {
		strcpy(PathInfo, "/");
	}

	strcpy(m_requestLine, RequestMethod);
	strcat(m_requestLine, " ");
	strcat(m_requestLine, PathInfo);
	if (0 < lstrlen(QueryString))
	{
		strcat(m_requestLine, "?");
		strcat(m_requestLine, QueryString);
	}
	strcat(m_requestLine, " ");
	strcat(m_requestLine, ServerProtocol);
	strcat(m_requestLine, "\r\n");
}

char* CHttpRequest::getHeaders()
{
	return m_headerList;
}

int CHttpRequest:: getHeaderLength()
{
	return strlen(m_headerList);
}

char* CHttpRequest::getRequestLine()
{
	//eg. "GET /foo.html HTTP/1.0 \r\n";

	return m_requestLine;
}

void CHttpRequest::appendHeader(char *headerName, char *headerValue, char *buf)
{
    //strcpy(buf, headerName);
    strcat(buf, headerName);
    strcat(buf, ": ");
    strcat(buf, headerValue);
    strcat(buf, "\r\n");
}

/**
 * Determines if HTTP request is in a valid format.
 * @param config   This is a pointer to a struct containing all the configuration 
 * settings which were read from the configuration file and is of type configHandle.
 * @return TRUE (1) or FALSE (0)
 */
int CHttpRequest::isValidHttpRequest() {
   
    if(NULL == RequestMethod || strlen(RequestMethod) == 0) {
	    return  0;
    }
    
    if(NULL == ServerProtocol || strlen(ServerProtocol) == 0) {
    	return 0;
    }

    return 1;
}

int CHttpRequest::isPostRequest() {
	return (RequestMethod != NULL &&
			strcmp(RequestMethod,"POST") == 0);
}