/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: HttpResponse.cpp,v 1.4 1999/12/20 03:32:38 icostanzo Exp $
 *-----------------------------------------------------------------------------
 */

// HttpResponse.cpp: implementation of the CHttpResponse class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "JavalinIIS.h"
#include "HttpResponse.h"
#include "Config.h"
#include <winsock2.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CHttpResponse::CHttpResponse(int socketHandle)
{
	memset(m_buffer, 0, sizeof(m_buffer));
	memset(m_headerBuf, 0, sizeof(m_headerBuf));
	m_iStatus = 200;
	m_headerSize = 0;
	m_bufferSize = 0;
	m_dataStartIndex = 0;
	m_enhydraSocketHandle = socketHandle;
	readFromEnhydra();
}

CHttpResponse::~CHttpResponse()
{
}

void CHttpResponse::readFromEnhydra() 
{
	//int bytesRead = 0;	
	//int totalBytesRead = 0;

    /* set up socket stream with NO buffering */
	//int bufSize = 0;
    //if(setsockopt(m_enhydraSocketHandle, SOL_SOCKET, SO_RCVBUF, 
	//		(const char *) bufSize, sizeof(bufSize) ) != 0) 
	//{
		// Handle error
    //}
   
	// while buffer is not full and we haven't reached EOF then read more data
	/*while((bytesRead = recv(m_enhydraSocketHandle, rawLine, BUFSIZE, 0)) > 0 
		&& totalBytesRead < sizeof(m_buffer)) 
	{
		char rawLine[BUFSIZE];
		char *bufPtr = &(m_buffer)[totalBytesRead];
		totalBytesRead += bytesRead;
		memcpy(bufPtr, rawLine, bytesRead);
	}*/

	m_bufferSize = recv(m_enhydraSocketHandle, m_buffer, sizeof(m_buffer), 0);

	parseHeaders();
}

// Parse cookie: and location: headers to reflect the correct path
void CHttpResponse::parseHeaders()
{
	if(strncmp(m_buffer, "HTTP/", 5) == 0 ) {
#ifdef _DEBUG
		CConfig::debug(m_buffer);
#endif

		// keep the http status code - HTTP/1.x<sp>nnn<sp>message
		char* tmpStat = m_buffer + 9;
		char stat[10];
		memset(stat, 0, 10);
		strncpy(stat, tmpStat, 3);
		m_iStatus = atoi(stat);
#ifdef _DEBUG
		char foob[1024];
		sprintf(foob, "\r\n>>> %s = %d \r\n", stat, m_iStatus);
		CConfig::debug(foob);
#endif

	   char *tempPtr = strstr(m_buffer, "\r\n\r\n");
	   // Some simple pointer arithmetic to get the header string length
	   m_headerSize = (tempPtr - m_buffer) + 4; // 4 is the length of "\r\n\r\n"
	   m_dataStartIndex = m_headerSize;
	   editHeaders();
	}
}

char* CHttpResponse::getHeaders()
{
	return m_headerBuf;
}

int CHttpResponse:: getHeaderLength()
{
	return m_headerSize;
}

int CHttpResponse::getData(char *buf, int len)
{
	int totalBytesRead = 0;

	// If there is data left in our buffer copy it,
	// else read directly from the enhydra socket
	if(m_dataStartIndex < m_bufferSize) {
		int dataRemaining = m_bufferSize - m_dataStartIndex;

		if(len >= dataRemaining) {
			memcpy(buf, &m_buffer[m_dataStartIndex], dataRemaining);
			m_dataStartIndex = m_bufferSize;
			totalBytesRead = dataRemaining;
		} else {
			memcpy(buf, &m_buffer[m_dataStartIndex], len);
			m_dataStartIndex += len;
			return len;
		}
	} 

	int bytesRead = 0;

	// Block send - just copy the data
	while((bytesRead = recv(m_enhydraSocketHandle, &buf[totalBytesRead], 
				len - totalBytesRead, 0)) > 0) 
	{
		totalBytesRead += bytesRead;
		if(totalBytesRead >= len) {
			break;
		}
	}

	return totalBytesRead;
}

int CHttpResponse::getStatus()
{
	return m_iStatus;
}

void CHttpResponse::editHeaders()
{
	char line[BUFSIZE];
	char tempBuffer[sizeof(m_headerBuf)];
	char *startPtr = m_buffer;					   // Mark the start of the headers
	char *nextPtr  = strstr(startPtr, "\r\n");	   // Find the end of the first header line
	char *endPtr   = strstr(m_buffer, "\r\n\r\n"); // Find the end of all the headers

	memset(tempBuffer, 0, sizeof(tempBuffer));

	while(nextPtr <= endPtr) {				 // While we haven't reached the end of the headers
		nextPtr += 2;						 // 2 is the length of "\r\n"
		int lineLen = nextPtr - startPtr;
		memset(line, 0, sizeof(line));

		memcpy(line, startPtr, lineLen);	 // Copy the next header line into the buffer
		char *newLine = editHeaderLine(line);// Get back a pointer to an edited header line
		strcat(tempBuffer, newLine);		 // Copy that string to the end of our buffer
		
		startPtr = nextPtr;
		nextPtr = strstr(startPtr, "\r\n");

		if(newLine != line) {  // If the pointers are not equal then we got a new pointer back, 
			delete [] newLine; // so delete it now
		}
	}

	strcat(tempBuffer, "\r\n");				// Add the last trailing newline
	m_headerSize = strlen(tempBuffer);		// Update the header length
	memcpy(m_headerBuf, tempBuffer, m_headerSize);
}

/**
 * Modifies "Location:" and "Set-Cookie:" Header Lines with absolute paths to allow redirects.
 * @param Line   This char * is a header line extracted from the LBS response
 * @param config   This is a pointer to a struct containing all the config 
 * settings which were read from the config file and is of type configHandle.
 * @return Line   This char * is a header line extracted from the LBS response which has been 
 * modified with absolute path information if it contains URL information used in a redirect.
 */
char* CHttpResponse::editHeaderLine(char *line) 
{    
    char oldLineBuf[BUFSIZE];
    char * targetLocation;                   /* pointer to location of string being searched for.  */
    char * modification;                     /* string to be inserted following the target string. */
    char * tail;                             /* portion of data stream after target.               */
    int index = 0; 
	char *newLineBuf = NULL;

    modification = CConfig::urlPrefix;        /* get path prefix from config file */

    /* put first word of line into a buffer */
    while((line[index]!= ' ') && (line[index]!= '\0')) { 
		oldLineBuf[index] = line[index];
		index++;
    }
    oldLineBuf[index] = '\0';
  
    /* case where header field "Location:" is found */
    if(_stricmp(oldLineBuf, "Location:") == 0) { 
		newLineBuf = new char[strlen(line) + strlen(modification) + 32]; // The 32 is arbitrarily chose for wiggle room in the buffer
 	    strcpy(newLineBuf, oldLineBuf);               /* start building new line */
 	    strcat(newLineBuf, " ");    
	    targetLocation = (char *)strstr(line, oldLineBuf); /* find part of line following target */
 	    tail = targetLocation+10;                     /* portion of data stream after target (tail) */  	

	/* case where header field "Set-Cookie:" is found */
	// NOTE - the following is Enhydra code which was REMOVED from JavalinIIS
	// - sets cookie prefix based on URL-prefix in config file

	//} else if(_stricmp(oldLineBuf, "Set-Cookie:") == 0) { 
	//	newLineBuf = new char[strlen(line) + strlen(modification) + 32]; // The 32 is arbitrarily chose for wiggle room in the buffer
 	//    targetLocation = (char *)strstr(line, "path=");       /* find "path=" field in "Set-Cookie:" header */  
	//    if(targetLocation == NULL) {                  /* no "path=" field in "Set-Cookie:" header   */ 
	//        return line;
	//    }
	//    
    //    tail = targetLocation + 5;                      /* portion of data stream after target (tail) */   
	//    index = 0;
	//    while((&line[index])!= tail){  
	//        newLineBuf[index] = line[index];          /* start building new line */    
	//        index++;
	//    }
	//
	//    newLineBuf[index] = '\0';
    } else {
    	return line;
    }
    
    strcat(newLineBuf, modification);                 /* insert modification */
    strcat(newLineBuf, tail);                         /* copy remaining part of data stream (tail) */  
    return newLineBuf;
}