/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: HttpResponse.h,v 1.3 1999/12/20 03:32:38 icostanzo Exp $
 *-----------------------------------------------------------------------------
 */

// HttpResponse.h: interface for the CHttpResponse class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_HTTPRESPONSE_H__86AF39D4_4470_11D3_907B_0050045D7A4E__INCLUDED_)
#define AFX_HTTPRESPONSE_H__86AF39D4_4470_11D3_907B_0050045D7A4E__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "Config.h"

/**
This class encapsulates an enhydra response. It encapsulates the headers. It also may contain the 
start of any returned data, since it is doing some internal buffering (see getData() for more info).
  */

class CHttpResponse  
{
public:
	CHttpResponse(int enhydraSocketHandle);
	virtual ~CHttpResponse();

	char *getHeaders();
	int getHeaderLength();
	int getData(char *buf, int len);
	int getStatus();

protected:
	// The enhydra socket handle
	int m_enhydraSocketHandle;
	// The pointer to the temporary header/data buffer
	char m_buffer[BUFSIZE * 40];
	// The size of the header
	int m_headerSize;
	// The total size of the data buffer
	int m_bufferSize;
	// The index of the start of the data in the data buffer
	int m_dataStartIndex;
	// Permanent header Buffer
	char m_headerBuf[BUFSIZE * 40];
	// status of the request (200, 404, etc)
	int m_iStatus;

	// Hide the default constructor
	CHttpResponse();
	void readFromEnhydra();
	char* editHeaderLine(char *line);
	void parseHeaders();
	void editHeaders();
};

#endif // !defined(AFX_HTTPRESPONSE_H__86AF39D4_4470_11D3_907B_0050045D7A4E__INCLUDED_)
