// This file contains code from the Enhydra IIS extension, which carries the following copyright:
/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: Enhydra.cpp,v 1.1.2.1 1999/08/18 03:49:44 shawn Exp $
 *-----------------------------------------------------------------------------
 */

// JAVALINIIS.CPP - Implementation file for your Internet Server
//    JavalinIIS Extension


#include "stdafx.h"
#include "JavalinIIS.h"
#include "EnhydraConn.h"
#include "Config.h"
#include "HttpRequest.h"
#include "HttpResponse.h"
#include "JavalinIISAdmin.h"
#include <time.h>
#include <winsock2.h> // For WSA calls (windows sockets initialization)

static const TCHAR szGet[] = _T("GET");
static const TCHAR szPost[] = _T("POST");
static const TCHAR szDecimalFormat[] = _T("%d");
static const TCHAR szFloatFormat[] = _T("%f");

CJavalinIISAdmin *CJavalinIISExtension::jAdmin = NULL;


///////////////////////////////////////////////////////////////////////
// command-parsing map

BEGIN_PARSE_MAP(CJavalinIISExtension, CHttpServer)
	// TODO: insert your ON_PARSE_COMMAND() and 
	// ON_PARSE_COMMAND_PARAMS() here to hook up your commands.
	// For example:

	ON_PARSE_COMMAND(Default, CJavalinIISExtension, ITS_EMPTY)
	DEFAULT_PARSE_COMMAND(Default, CJavalinIISExtension)
END_PARSE_MAP(CJavalinIISExtension)


///////////////////////////////////////////////////////////////////////
// The one and only CJavalinIISExtension object

CJavalinIISExtension theExtension;


///////////////////////////////////////////////////////////////////////
// CJavalinIISExtension implementation

CJavalinIISExtension::CJavalinIISExtension()
{
	// Load the windows sockets libraries.
	WSADATA wsData;
	WSAStartup(MAKEWORD(1,1), &wsData);

	jAdmin = new CJavalinIISAdmin();
}

CJavalinIISExtension::~CJavalinIISExtension()
{
	// Unload the windows sockets libraries.
	WSACleanup();

	delete jAdmin;
}

BOOL CJavalinIISExtension::GetExtensionVersion(HSE_VERSION_INFO* pVer)
{
	// Call default implementation for initialization
	CHttpServer::GetExtensionVersion(pVer);

	// Load description string
	TCHAR sz[HSE_MAX_EXT_DLL_NAME_LEN+1];
	ISAPIVERIFY(::LoadString(AfxGetResourceHandle(),
			IDS_SERVER, sz, HSE_MAX_EXT_DLL_NAME_LEN));
	_tcscpy(pVer->lpszExtensionDesc, sz);
	return TRUE;
}

BOOL CJavalinIISExtension::TerminateExtension(DWORD dwFlags)
{
	// extension is being terminated
	//TODO: Clean up any per-instance resources
	CConfig::destroy();
	return TRUE;
}

///////////////////////////////////////////////////////////////////////
// CJavalinIISExtension command handlers

// This is the root HTTP processing function, typically not over-ridden by ISAPI
// implementations, but over-ridden here to work around bugs observed when trying
// to pass HTTP parameters to an Enhydra-generated application.

// Most code in this method has been COPIED from the "HttpServer" class, which is
// the super-class of "JavalinIISExtension".

DWORD CJavalinIISExtension::HttpExtensionProc(EXTENSION_CONTROL_BLOCK *pECB) 
{
	DWORD dwRet = HSE_STATUS_SUCCESS;
	LPTSTR pszPostBuffer = NULL;
	LPTSTR pszQuery;
	LPTSTR pszCommand = NULL;
	LPTSTR pstrLastChar;
	DWORD cbStream = 0;
	BYTE* pbStream = NULL;
	CHttpServerContext ctxtCall(pECB);
	int iRet = 0;

	// Initialize the config object if necessary
#ifdef _DEBUG
	CConfig *pConfig = CConfig::getInstance();
	if(NULL != pConfig) {
		CConfig::debug(" ============== \r\n");
	}
#endif

	if (lstrcmpi(pECB->lpszMethod, szGet) == 0)
	{
		pszQuery = pECB->lpszQueryString;
		ctxtCall.m_dwBytesReceived = lstrlen(pszQuery);
	}
	else if (lstrcmpi(pECB->lpszMethod, szPost) == 0)
	{
		pszCommand = pECB->lpszQueryString;
		pszPostBuffer = new TCHAR[pECB->cbTotalBytes + 1];
		pszQuery =  GetQuery(&ctxtCall, pszPostBuffer);
		if (pszQuery == NULL)
		{
			dwRet = HSE_STATUS_ERROR;
			goto CleanUp;
		}
	}
	else
	{
		CConfig::debug("Error: Unrecognized method: ");
		CConfig::debug(pECB->lpszMethod);
		CConfig::debug("\r\n");
		dwRet = HSE_STATUS_ERROR;
		goto CleanUp;
	}

	// trim junk that some browsers put at the very end
	pstrLastChar = pszQuery + ctxtCall.m_dwBytesReceived -1;
	while ((*pstrLastChar == ' ' || *pstrLastChar == '\n' ||
		   *pstrLastChar == '\r') && pstrLastChar > pszQuery)
	{
		*pstrLastChar-- = '\0';
	}

	ctxtCall.m_pStream = ConstructStream();
	if (ctxtCall.m_pStream == NULL)
	{
		dwRet = HSE_STATUS_ERROR;
		goto CleanUp;
	}

	// do something about it
	pECB->dwHttpStatusCode = HTTP_STATUS_OK;

	// do the call!!!
	//int iRet = CJavalinIISExtension::jDefault(&ctxtCall, pszQuery);
	iRet = 0;
	if (CJavalinIISExtension::jAdmin->isAdminRequest(&ctxtCall, pszQuery))
		Default(&ctxtCall);
	else
		iRet = jDefault(&ctxtCall, pszQuery);

	// figure out the response status
	if (iRet != 0)
	{
		dwRet = HSE_STATUS_ERROR;
		switch (iRet)
		{
		case (-2):
			pECB->dwHttpStatusCode = HTTP_STATUS_BAD_REQUEST;
			break;

		default:
			pECB->dwHttpStatusCode = HTTP_STATUS_SERVER_ERROR;
			break;
		}
	}

	if (iRet == 0)
	{
		cbStream = ctxtCall.m_pStream->GetStreamSize();
		pbStream = ctxtCall.m_pStream->Detach();
	}
	else if (OnParseError(&ctxtCall, iRet))
	{
		cbStream = ctxtCall.m_pStream->GetStreamSize();
		pbStream = ctxtCall.m_pStream->Detach();
	}

CleanUp:
	// if there was an error, return an appropriate status
	TCHAR szResponse[64];
	BuildStatusCode(szResponse, pECB->dwHttpStatusCode);

	DWORD dwSize = cbStream - ctxtCall.m_dwEndOfHeaders;
	LPBYTE pbContent = NULL;
	BYTE cSaved = 0;

	if (pbStream != NULL && ctxtCall.m_bSendHeaders)
	{
		cSaved = pbStream[ctxtCall.m_dwEndOfHeaders];
		pbStream[ctxtCall.m_dwEndOfHeaders] = '\0';
		pbContent = &pbStream[ctxtCall.m_dwEndOfHeaders];
	}

	if (ctxtCall.m_bSendHeaders &&
		!ctxtCall.ServerSupportFunction(HSE_REQ_SEND_RESPONSE_HEADER,
			szResponse, 0, (LPDWORD) pbStream) &&
		::GetLastError() != 10054)  // WSAECONNRESET
	{
		pECB->dwHttpStatusCode = HTTP_STATUS_SERVER_ERROR;
		dwRet = HSE_STATUS_ERROR;
	}
	else
	{
		if (pbContent != NULL)
		{
			BOOL bWorked = TRUE;

			// write a newline to separate content from headers
			if (ctxtCall.m_bSendHeaders)
			{
				*pbContent = cSaved;
				DWORD dwNewLineSize = 2;
				bWorked = ctxtCall.WriteClient(_T("\r\n"), &dwNewLineSize, 0);
			}

			if (!bWorked || !OnWriteBody(&ctxtCall, pbContent, dwSize))
			{
				dwRet = HSE_STATUS_ERROR;
				pECB->dwHttpStatusCode = HTTP_STATUS_SERVER_ERROR;
			}
		}
	}

	if (pbStream != NULL)
		ctxtCall.m_pStream->Free(pbStream);

	if (ctxtCall.m_dwStatusCode != DWORD(-1))
		dwRet = ctxtCall.m_dwStatusCode;
	if (dwRet == HSE_STATUS_SUCCESS)
		pECB->dwHttpStatusCode = HTTP_STATUS_OK;

	if (pszPostBuffer != NULL)
		delete [] pszPostBuffer;

#ifdef _DEBUG
	if(NULL != pConfig) {
		char foo[30];
		sprintf(foo, " result --> %d\r\n", pECB->dwHttpStatusCode);
		CConfig::debug(foo);
		CConfig::debug(" ============== \r\n");
	}
#endif

	return dwRet;
}

void CJavalinIISExtension::Default(CHttpServerContext* pCtxt)
{
	// this gets called if the request is for a status report
	StartContent(pCtxt);
	WriteTitle(pCtxt);

	CJavalinIISExtension::jAdmin->Default(pCtxt);

	EndContent(pCtxt);
}

//void CJavalinIISExtension::Default(CHttpServerContext* pCtxt)
int CJavalinIISExtension::jDefault(CHttpServerContext* pCtxt, LPTSTR pszQuery)
{
	// Don't let IIS send any of its own headers
	pCtxt->m_bSendHeaders = FALSE;

	// Initialize the config object if necessary
	CConfig *pConfig = CConfig::getInstance();
	if(NULL == pConfig) {
		handleError("Invalid config file", pCtxt);
		return -1;
	}

#ifdef _DEBUG
	CConfig::debug("Start request\r\n");
#endif

   	CHttpRequest req(pCtxt);

    if(!req.isValidHttpRequest()) {
		handleError("Invalid HTTP request", pCtxt);
		return -2;
    }

	// Initialize Enhydra connection with host, port, and configuration data
	CEnhydraConn enhydraConn;	
	int retCode = enhydraConn.connectToEnhydra();	// Open socket connection to Enhydra and send headers
	if(retCode < 0) {
		handleError("Couldn't connect to enhydra", pCtxt);
		return -3;
	}


	clock_t start, finish;
	double  duration;

	// capture the start time of the request
	start = clock();

	int iRet = enhydraConn.writeToEnhydra(req.getRequestLine(), strlen(req.getRequestLine()));
	if(iRet < 0) {
		enhydraConn.completeRequest(false, 0);
		handleError("Couldn't write request to enhydra", pCtxt);
		return -3;
	}

#ifdef _DEBUG
	CConfig::debug(req.getRequestLine());
#endif

	iRet = enhydraConn.writeToEnhydra(req.getHeaders(), req.getHeaderLength());
	if(iRet < 0) {
		enhydraConn.completeRequest(false, 0);
		handleError("Couldn't write headers to enhydra", pCtxt);
		return -3;
	}
#ifdef _DEBUG
	CConfig::debug(req.getHeaders());
#endif

	DWORD bufSize = BUFSIZE;
	char buffer [BUFSIZE];

    // If request method is POST, also send the post data to Enhydra
	if(req.isPostRequest()) {
#ifdef _DEBUG
		CConfig::debug("Request is a POST\r\n");
#endif

		bufSize = lstrlen(pszQuery);
		iRet = enhydraConn.writeToEnhydra(pszQuery, bufSize);
		if(iRet < 0) {
			enhydraConn.completeRequest(false, 0);
			handleError("Couldn't write POST parameters to enhydra", pCtxt);
			return -3;
		}

#ifdef _DEBUG
		CConfig::debug(pszQuery);
		CConfig::debug("\r\n");
#endif
    }

	// Now get the response from Enhydra and return it to the client
	int reqStatus;
	CHttpResponse *pRes = enhydraConn.getResponse();

	if(NULL != pRes) {
#ifdef _DEBUG
		CConfig::debug("Got response\r\n");
#endif

		// Write out the headers
		int responseSize = pRes->getHeaderLength();
		retCode = pCtxt->WriteClient(pRes->getHeaders(), (LPDWORD) &responseSize);
		if (retCode < 0)
		{
			enhydraConn.completeRequest(false, 0);
			handleError("Error writing response headers to web server", pCtxt);
			return -4;
		}
#ifdef _DEBUG
		CConfig::debug(pRes->getHeaders());
		CConfig::debug("\r\n");
#endif
		
		// Try to get data from enhydra
		bufSize = BUFSIZE;
		responseSize = pRes->getData(buffer, bufSize);
	
		// If we have data write it to the web client
		while(responseSize > 0) {
			retCode = pCtxt->WriteClient(buffer, (LPDWORD) &responseSize);
			if (retCode < 0)
			{
				enhydraConn.completeRequest(false, 0);
				handleError("Error writing response page to web server", pCtxt);
				return -4;
			}

			// bufSize is changed by calling WriteClient() so re-initialize it here
			bufSize = BUFSIZE;
			//memset(buffer, 0, bufSize);
			responseSize = pRes->getData(buffer, bufSize);
		}

#ifdef _DEBUG
		CConfig::debug("Get final status\r\n");
#endif
		reqStatus = pRes->getStatus();

		delete pRes;
	}
	else
	{
		enhydraConn.completeRequest(false, 0);
		handleError("Error null response received from enhydra", pCtxt);
		return -5;
	}

	// get end time and calculate processing interval
	finish = clock();
	duration = (double)(finish - start) / CLOCKS_PER_SEC;

#ifdef _DEBUG
	char foo[1024];
	sprintf(foo, ">>> request status = %d\r\n", reqStatus);
	CConfig::debug(foo);
#endif
	bool reqSuccess = (200 <= reqStatus && reqStatus < 400);
	enhydraConn.completeRequest(reqSuccess, duration);

#ifdef _DEBUG
	CConfig::debug("Done request\r\n");
#endif

	return 0;
}

// Do not edit the following lines, which are needed by ClassWizard.
#if 0
BEGIN_MESSAGE_MAP(CJavalinIISExtension, CHttpServer)
	//{{AFX_MSG_MAP(CJavalinIISExtension)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
#endif	// 0


// Method to log errors and write error out to client
void CJavalinIISExtension::handleError(char *msg, CHttpServerContext* pCtxt)
{
	CConfig::debug("handleError(): ");
	CConfig::debug(msg);
	CConfig::debug("\r\n");

	// Let IIS send all of its own headers
	pCtxt->m_bSendHeaders = TRUE;
	*pCtxt << msg;
	strcat(pCtxt->m_pECB->lpszLogData, msg);
	CConfig::debug(msg);
}


///////////////////////////////////////////////////////////////////////
// If your extension will not use MFC, you'll need this code to make
// sure the extension objects can find the resource handle for the
// module.  If you convert your extension to not be dependent on MFC,
// remove the comments arounn the following AfxGetResourceHandle()
// and DllMain() functions, as well as the g_hInstance global.

/****

static HINSTANCE g_hInstance;

HINSTANCE AFXISAPI AfxGetResourceHandle()
{
	return g_hInstance;
}

BOOL WINAPI DllMain(HINSTANCE hInst, ULONG ulReason,
					LPVOID lpReserved)
{
	if (ulReason == DLL_PROCESS_ATTACH)
	{
		g_hInstance = hInst;
	}

	return TRUE;
}

****/
