/*
 * Larry and Daryl's EnhyDraw! Poker
 * A sample Enhydra Application
 * 
 * EnhyDraw!, beta 4, 5/21/99
 *
 * Copyright 1999, Larry Wolcot & Daryl Tempesta
 * ALL rights reserved. Not for commercial use
 * without written permission from both authors.
 *
 *
 */

package poker;

import java.io.*;
import java.math.BigDecimal;
import java.util.Enumeration;
import com.lutris.util.*;
import com.lutris.logging.Logger;
import com.lutris.logging.LogChannel;
import com.lutris.appserver.server.LBS;
import com.lutris.appserver.server.session.*;
import com.lutris.appserver.server.user.*;
import com.lutris.appserver.server.*;
import com.lutris.appserver.server.httpPresentation.*;
import poker.business.*;

/**
 * This is the main application object
 * that handles the appication wide data,
 * like the GameManager.
 */

public class Poker extends StandardApplication {

    private static GameManager gameManager = new GameManager();
    private static FortuneManager fortuneManager = new FortuneManager();

    // This is where you should enable the use of DODS_Objects
    // instead of Vector Objects for persistance
    private boolean useDB = false;

    public void startup(Config appConfig) throws ApplicationException {
        super.startup(appConfig);

	//Yea, this needs to be finished
        fortuneManager.addFortune("Who's there?!");
        fortuneManager.addFortune("What do you want?!");
        fortuneManager.addFortune("Don't come in here with loco motives!");
        fortuneManager.addFortune("Whats the secret knock?!");
        fortuneManager.addFortune("I don't like your looks?!");
        fortuneManager.addFortune("Come to play poker or tiddly winks?");
        fortuneManager.addFortune("BEA sure your app runs this nice!");
        fortuneManager.addFortune("Watch your step, pal...");
        fortuneManager.addFortune("Are you talking to me?");
        fortuneManager.addFortune("I'll J-Run your ass right outta town!");
        fortuneManager.addFortune("Feeling lucky?    I hope so..");

    	//This is tells the gameManager to use the DB instead of memory.
	//FIX: Still need to add the useDB key into the
	// Poker.conf configuration file..

    	try { gameManager.setUseDB(useDB); }
           catch (Exception e) { 
		logChannel.write(Logger.INFO, "EnhyDraw! DB errror:"
							+ e.toString());
		//There was a problem with the DB init 
	   }

        if (logChannel != null)
            logChannel.write(Logger.INFO, "EnhyDraw Poker now starting!");
        state = RUNNING;  
  }


    /**
     * This is an optional function, used only by the Multiserver's graphical
     * administration. This bit of HTML appears in the status page for this
     * application. In this case, it displays the House Statistics
     * which include total earnings, total players, etc.
     *
     * @return HTML that is displayed in the status page of the Multiserver.
     */
    public String toHtml() {
	GameManager g = gameManager;
	String n = new Integer(gameManager.getCount()).toString();
	String m = "Welcome to Larry and Daryl's EnhyDraw Poker!<br>";
      m = "<table border=\"1\" cellpadding=\"3\" cellspacing=\"0\"";
      m += "width=\"271\"><tr><th width=\"171\">Total Players Cataloged";
      m += "</th><th width=\"84\">" + g.getTotalPlayers() +"</th></tr><tr>";
      m += "<th width=\"171\">Total Hands Dealt</th>";
      m += "<th width=\"84\">"+ g.getTotalHandsDealt() +"</th></tr><tr>";
      m += "<th width=\"171\">Total Hands Won</th>";
      m += "<th width=\"84\">"+ g.getTotalHandsWon()+"</th></tr><tr>";
      m += "<th width=\"171\">Total Hands Lost</th>";
      m += "<th width=\"84\">"+ g.getTotalHandsLost()+"</th></tr><tr>";
      m += "<th width=\"171\">Total Server Profit</th>";
      m += "<th width=\"84\">"+ g.getHouseProfit() +"</th></tr><tr>";
      m += "<th width=\"171\">Total Players Bankrupt</th>";
      m += "<th width=\"84\">"+ g.getTotalBankrupt() +"</th></tr></table>";
        return m;
    }

    /**
     * Return the game manager for this application.
     */
    public GameManager getGameManager(){
	return gameManager;
    }

    public FortuneManager getFortuneManager(){
	return fortuneManager;
    }

    /**
     * Shutdown the application.
     */
    public void shutdown() {
        super.shutdown();
    }

    /**
     * This method pre processes all requests
     */
    public boolean requestPreprocessor(HttpPresentationComms comms)
        throws Exception {
        super.requestPreprocessor(comms);
 	comms.session.getSessionData().set("app", this);
        String pp = comms.request.getApplicationPath();
        if (!pp.endsWith("/"))
            pp += "/";
        String loginDir = pp + "login/";
        String mediaDir = pp + "media/";
        String page = comms.request.getRequestURI();
        if (page.startsWith(loginDir) || page.startsWith(mediaDir)) {
            /*
             * Let them go to the login and logout areas whether or not
             * they are logged in.  Also let them get media on welcomepage.
             */
            return false;
        } else {
            /*
             * All other pages require the user to be logged in.
             */
            if (comms.session.getUser() != null)
                // It's ok, they are logged in already.
                return false;
            else {
                // They are not logged in! Redirect to login page.
                String url = pp + "login/Welcome.po";
                throw new ClientPageRedirectException(url);
            }
        }
    }
}

