/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * /root/test/poker/data/DODS_GameData/GameDataDataStruct.java
 *-----------------------------------------------------------------------------
 */

package poker.data.DODS_GameData;

import com.lutris.appserver.server.sql.*;
import java.sql.*;
import java.math.*;


/**
 * Data structure for DO class.
 * A container for data members of a DO class.
 * A DO class contains a reference to a DataStruct class.  This reference
 * can be null (a DO whose data has not yet been retrieved from the database),
 * allowing a DO object to be a lightweight placeholder until its data is needed.
 *
 * @version $Revision: 1.2 $
 * @author  root
 * @since   poker
 */
public class GameDataDataStruct implements Cloneable {

    /**
     * A DO refers to this DataStruct.
     * readOnly is set to true when the DO is stored in its class cache.
     */
    public boolean readOnly = false;

    private byte[] copyByteArray( byte[] source ) {
	byte[] dest = new byte[ source.length ];
	System.arraycopy( source, 0, dest, 0, source.length );
	return dest;
    }

/**
 * Total cash available to the player
 */
   public int Cash = 0;

/**
 * Players deck image name
 */
   public String Deck = null;

/**
 * Players email address
 */
   public String Email = null;

/**
 * Largest bet placed by player
 */
   public int LargestBet = 0;

/**
 * Player Name
 */
   public String Name = null;

/**
 * Player password
 */
   public String Password = null;

/**
 * Smallest bet placed by the player
 */
   public int SmallestBet = 0;

/**
 * Total hands played
 */
   public int TotalPlayed = 0;

/**
 * Total hands won
 */
   public int TotalWon = 0;
    /**
     * Create a copy of the guts of a DO.
     *
     * @exception DatabaseManagerException 
     *       if createExisting() fails for a contained DO
     * @exception ObjectIdException 
     *       if GenericDO has trouble obtaining a valid id.
     */
    public GameDataDataStruct duplicate() 
    throws DatabaseManagerException, ObjectIdException {
        GameDataDataStruct ret = new GameDataDataStruct ();
 
	// writeMemberStuff uses the cloning.template
	// to build up the value for this tag:
	// the value is a series of assignments that copy
	// values from the DataStruct instance to the new instance 'ret'.
		/* The value of the CLONE_VALUE tag is either just
	 *     MEMBER_NAME
	 * or
	 *     MEMBER_NAME.clone()
	 * depending on the type of the data member.
	 * 
	 * The output of this cloning.template is stored in a SourceBuffer
	 * whose final contents is assigned to the tag CLONING_ASSIGNMENTS
	 * which is expanded inside the clone() method that is defined
	 * in the writeFileFooter.template file.
	 * The variable 'ret' is used in that clone() method.
	 */
	
	ret.Cash = Cash;
	
	/* The value of the CLONE_VALUE tag is either just
	 *     MEMBER_NAME
	 * or
	 *     MEMBER_NAME.clone()
	 * depending on the type of the data member.
	 * 
	 * The output of this cloning.template is stored in a SourceBuffer
	 * whose final contents is assigned to the tag CLONING_ASSIGNMENTS
	 * which is expanded inside the clone() method that is defined
	 * in the writeFileFooter.template file.
	 * The variable 'ret' is used in that clone() method.
	 */
	
	ret.Deck = Deck + "";
	
	/* The value of the CLONE_VALUE tag is either just
	 *     MEMBER_NAME
	 * or
	 *     MEMBER_NAME.clone()
	 * depending on the type of the data member.
	 * 
	 * The output of this cloning.template is stored in a SourceBuffer
	 * whose final contents is assigned to the tag CLONING_ASSIGNMENTS
	 * which is expanded inside the clone() method that is defined
	 * in the writeFileFooter.template file.
	 * The variable 'ret' is used in that clone() method.
	 */
	
	ret.Email = Email + "";
	
	/* The value of the CLONE_VALUE tag is either just
	 *     MEMBER_NAME
	 * or
	 *     MEMBER_NAME.clone()
	 * depending on the type of the data member.
	 * 
	 * The output of this cloning.template is stored in a SourceBuffer
	 * whose final contents is assigned to the tag CLONING_ASSIGNMENTS
	 * which is expanded inside the clone() method that is defined
	 * in the writeFileFooter.template file.
	 * The variable 'ret' is used in that clone() method.
	 */
	
	ret.LargestBet = LargestBet;
	
	/* The value of the CLONE_VALUE tag is either just
	 *     MEMBER_NAME
	 * or
	 *     MEMBER_NAME.clone()
	 * depending on the type of the data member.
	 * 
	 * The output of this cloning.template is stored in a SourceBuffer
	 * whose final contents is assigned to the tag CLONING_ASSIGNMENTS
	 * which is expanded inside the clone() method that is defined
	 * in the writeFileFooter.template file.
	 * The variable 'ret' is used in that clone() method.
	 */
	
	ret.Name = Name + "";
	
	/* The value of the CLONE_VALUE tag is either just
	 *     MEMBER_NAME
	 * or
	 *     MEMBER_NAME.clone()
	 * depending on the type of the data member.
	 * 
	 * The output of this cloning.template is stored in a SourceBuffer
	 * whose final contents is assigned to the tag CLONING_ASSIGNMENTS
	 * which is expanded inside the clone() method that is defined
	 * in the writeFileFooter.template file.
	 * The variable 'ret' is used in that clone() method.
	 */
	
	ret.Password = Password + "";
	
	/* The value of the CLONE_VALUE tag is either just
	 *     MEMBER_NAME
	 * or
	 *     MEMBER_NAME.clone()
	 * depending on the type of the data member.
	 * 
	 * The output of this cloning.template is stored in a SourceBuffer
	 * whose final contents is assigned to the tag CLONING_ASSIGNMENTS
	 * which is expanded inside the clone() method that is defined
	 * in the writeFileFooter.template file.
	 * The variable 'ret' is used in that clone() method.
	 */
	
	ret.SmallestBet = SmallestBet;
	
	/* The value of the CLONE_VALUE tag is either just
	 *     MEMBER_NAME
	 * or
	 *     MEMBER_NAME.clone()
	 * depending on the type of the data member.
	 * 
	 * The output of this cloning.template is stored in a SourceBuffer
	 * whose final contents is assigned to the tag CLONING_ASSIGNMENTS
	 * which is expanded inside the clone() method that is defined
	 * in the writeFileFooter.template file.
	 * The variable 'ret' is used in that clone() method.
	 */
	
	ret.TotalPlayed = TotalPlayed;
	
	/* The value of the CLONE_VALUE tag is either just
	 *     MEMBER_NAME
	 * or
	 *     MEMBER_NAME.clone()
	 * depending on the type of the data member.
	 * 
	 * The output of this cloning.template is stored in a SourceBuffer
	 * whose final contents is assigned to the tag CLONING_ASSIGNMENTS
	 * which is expanded inside the clone() method that is defined
	 * in the writeFileFooter.template file.
	 * The variable 'ret' is used in that clone() method.
	 */
	
	ret.TotalWon = TotalWon;
	


	return ret;
    }
}
