/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * /root/test/poker/data/DODS_PitBoss/PitBossDO.java
 *-----------------------------------------------------------------------------
 */

package poker.data.DODS_PitBoss;

import com.lutris.appserver.server.sql.*;
import com.lutris.appserver.server.LBS;
import com.lutris.util.Config;
import com.lutris.util.KeywordValueTable;
import dods.builder.sourceGenerators.Query.NonUniqueQueryException;
import dods.builder.sourceGenerators.Query.DataObjectException;
import java.sql.*;
import java.math.*;
import java.util.Hashtable;
import java.util.Enumeration;


/**
 * Data core class, used to set, retrieve the PitBossDO information.
 *
 * @version $Revision: 1.9 $
 * @author  root
 * @since   poker
 */
 public class PitBossDO extends dods.builder.sourceGenerators.DO.GenericDO {

    /**
     * The ObjectId is the unique key for the DO in the database.
     */

    /**
     * A DO class contains a reference to a DataStruct class.
     * This reference can be null (when the data for the DO 
     * has not yet been retrieved from the database),
     * allowing a DO object to be a lightweight placeholder
     * until its data is needed.
     */
    private PitBossDataStruct data = null;

    /**
     * Protected constructor.  Only derived classes should call it.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     */
    protected PitBossDO ()
    throws ObjectIdException, DatabaseManagerException {
        super();
    }

    /**
     * loadData() 
     * Load the fields for the DO from the database.
     *
     * @exception DataObjectException
     *   If an object id can't be allocated for this object.
     * @exception SQLException
     *   If an object id can't be allocated for this object.
     */
    public void loadData() 
    throws SQLException, ObjectIdException, DataObjectException 
    {
	if ( null == data ) {
	    
	    data = new PitBossDataStruct ();
	}

	

	ObjectId id = getOId();
	if ( null == id )
	    return;

	// an id means we complain if no record in database
	PitBossQuery query = new PitBossQuery ();
	query.setQueryOId( id );
	query.requireUniqueInstance();
	PitBossDO obj;
	try {
	    while ( null != ( obj = query.getNextDO() ) ) {
	        data = obj.data;
	        // obj.data.readOnly = true;
	        cache.put( obj.getOId().toString(), obj );
	    }
	} catch ( NonUniqueQueryException e ) { 
	    throw new ObjectIdException( "Duplicate ObjectId" );
	}

	
    }

    

    /**
     * A DO class that contains unchanging (static) data from the database
     * will have a cache of DOs representing the entire contents of the table.
     */
    static { 
        if ( false ) {
    	       try {
    		    
    		    refreshCache();
    		    
    	       } catch ( Exception e ) { 
    		    // cannot throw from static block
    	       }
    	   }
    }

    /**
     * The cache is used automatically by the class.
     * Callers of the create() methods do not know whether
     * the returned DO instance is shared or not.
     * This prevents the existence of multiple instances in memory 
     * of the same database object.
     * The cache is instantiated when the first DO instance is created.
     * After the cache is instantiated, the Query class for this DO
     * will search only the cache.
     */
    
    private static Hashtable cache = null;
    

    /**
     * Queries all rows in table, and for each row
     * creates a DO instance in the cache.  
     * For these DOs, data.readOnly = true,
     * which causes set methods to throw an exception.
     *
     * @exception java.sql.SQLException
     *   If an unrecoverable error occurs.
     */
    
    public static void refreshCache()
    throws java.sql.SQLException, DatabaseManagerException, ObjectIdException,
	   DataObjectException {
	    cache = new Hashtable();
	    PitBossQuery query = new PitBossQuery ();
	    PitBossDO obj;
	    try {
		while ( null != ( obj = query.getNextDO() ) ) {
		    obj.data.readOnly = true;
		    cache.put( obj.getOId().toString(), obj );
		}
	    } catch ( NonUniqueQueryException e ) { 
		// Since we do not call query.requireUniqueInstance()
		// this should never happen.
		throw new ObjectIdException( "Duplicate ObjectId" );
	    }
    }
    

    /**
     * Retrieve contents of cache as an enumeration of DO objects.
     */
    public static Enumeration getCacheContents() {
	Enumeration ret = null;
	Hashtable h = null;
	
	h = cache;
	
	if ( null != h ) 
	    ret = h.elements();
        return ret;
    }

    /**
     * createVirgin() 
     * Creates a DO that has no ObjectId or data.
     * Such a DO is used to insert a new database entry
     * after its data has been set.
     */
    
    public static PitBossDO createVirgin()
    throws DatabaseManagerException, ObjectIdException {
	return new PitBossDO ();
    }
    

    /**
     * createExisting() 
     * Creates a DO that represents an existing entry in the database.
     * Such a DO is used to examine and possibly update such an entry.
     * createExisting() is called only from the code that retrieves
     * an ObjectId from a ResultSet (database query result).
     * createExisting() is protected because no other DO or BO should ever
     * need to call it.
     * FIX unfortunately the createExisting(BigDecimal) form *does* need 
     * to be public because it is called by the public ctors of other DOs.
     * For example,
     *    AaaDO contains a ref to a BbbDO,
     *    so there is a method AaaDO.setBbb(BbbDO).
     *    In the ctor AaaDO(ResultSet), we have the call
     *       setBbb( BbbDO.createExisting( rs.getBigDecimal( "bbb", 0 )));
     * Since AaaDO is not in the same package as BbbDO,
     * BbbDO.createExisting(BigDecimal) must be public, not protected.
     * Java needs the C++ 'friend' idea.
     */
    
    public static PitBossDO createExisting( BigDecimal bd ) 
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException
    {
	return createExisting( new ObjectId( bd ) );
    }
    protected static PitBossDO createExisting( ObjectId id ) 
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException
    {
	PitBossDO ret = new PitBossDO ();
	  ret.setOId( id );
	if ( ! false )
	    ret.loadData();
	return ret;
    }

    protected static PitBossDO createExisting( ResultSet rs )
    throws SQLException, ObjectIdException, DataObjectException
		, DatabaseManagerException
    {
	PitBossDO ret = new PitBossDO ( rs );
	return ret;
    }
    

    /**
     * createCopy() 
     * Creates a DO that has no ObjectId
     * but has a copy of an existing DO's data.
     * Such a DO is used to insert a new database entry
     * that is largely similar to an existing entry.
     */
    
    public static PitBossDO createCopy( PitBossDataStruct data )
    throws DatabaseManagerException, ObjectIdException {
	PitBossDO ret = new PitBossDO ();
	ret.data = ( PitBossDataStruct ) data.duplicate();
	return ret;
    }
    

    /**
     * createCopy() 
     * Creates a DO that has no ObjectId
     * but has a copy of an existing DO's data.
     * Such a DO is used to insert a new database entry
     * that is largely similar to an existing entry.
     */
    
    public static PitBossDO createCopy( PitBossDO orig )
    throws DatabaseManagerException, ObjectIdException {
	PitBossDO ret = new PitBossDO ();
	if ( null != orig.data ) {
	    ret.data = ( PitBossDataStruct ) orig.data.duplicate();
	}
	return ret;
    }
    

    /**
     * reload() 
     * Causes the DO to refresh itself from the database
     * the next time a set or get method is called.
     */
    
    public void reload() {
	data = null;
    }
    

    /** 
     * The methods <CODE>
     *     getHandle
     *     hasMatchingHandle
     *     findCachedObjectByHandle
     * </CODE> are used by Presentation Objects that need to populate
     * HTML select lists with Data Objects as options.
     * The <CODE>getHandle()</CODE> method is used
     * to set the value for each option,
     * and the <CODE>hasMatchingHandle()<CODE>
     * and <CODE>findCachedObjectByHandle()<CODE>
     * methods are used to lookup the Data Object when the selection has
     * been made.
     *
     * @return id of this DO as a string
     *   If an object id can't be allocated for this object.
     */
    
    public        String  getHandle() 
    throws DatabaseManagerException {
	String ret = null;
	    if ( null == getOId() )
		   throw new DatabaseManagerException( "ID not set" );
	    ret = getOId().toString();
        return ret;
    }
    

   /**
     * hasMatchingHandle
     *
     * @param handle
     *   <CODE>String</CODE> version of DO id
     * @return boolean
     *   True if the string version of the id of this DO matches passed handle
     * @see getHandle
     */
    
    public        boolean hasMatchingHandle( String handle ) {
	boolean ret = false;
	    if ( null == getOId() )
		   ret = false;
	    else
		   ret = getOId().toString().equals( handle );
        return ret;
    }
    

   /**
     * findCachedObjectByHandle
     *
     * @param handle
     *   <CODE>String</CODE> version of DO id
     * @return <CODE>PitBossDO</CODE>
     *   Object if one is found in cache, otherwise null.
     * @see getHandle
     */
    
    public static PitBossDO findCachedObjectByHandle( String handle ){
        if ( null == handle ) 
            return null;
        if ( null == cache ) 
            return null;
        return ( PitBossDO ) cache.get( handle );
    }
    

    


   /**
    * Get GameName of the PitBoss
    *
    * @return GameName of the PitBoss
    */
   public String getGameName () 
   throws DataObjectException {
      if ( null == data )
 	try {
 	  loadData();
 	} catch ( Exception e ) {
 	  throw new DataObjectException( 
 	    "Unable to load data for PitBossDO id=" + getOId() +
 	    ", error = " + e.getMessage() );
        }
      return data.GameName;
   }

   /**
    * Set GameName of the PitBoss
    *
    * @param GameName of the PitBoss
    */
   
   public void setGameName ( String GameName )
   throws DataObjectException {
      if ( null == data )
	try {
	  loadData();
	} catch ( Exception e ) {
	  throw new DataObjectException( 
	    "Unable to load data for PitBossDO id=" + getOId() +
	    ", error = " + e.getMessage() );
	}
      data.GameName =  markNewValue(
	data.GameName, GameName , 0, 32, false );
   }
   

   /**
    * Get TotalDollars of the PitBoss
    *
    * @return TotalDollars of the PitBoss
    */
   public int getTotalDollars () 
   throws DataObjectException {
      if ( null == data )
 	try {
 	  loadData();
 	} catch ( Exception e ) {
 	  throw new DataObjectException( 
 	    "Unable to load data for PitBossDO id=" + getOId() +
 	    ", error = " + e.getMessage() );
        }
      return data.TotalDollars;
   }

   /**
    * Set TotalDollars of the PitBoss
    *
    * @param TotalDollars of the PitBoss
    */
   
   public void setTotalDollars ( int TotalDollars )
   throws DataObjectException {
      if ( null == data )
	try {
	  loadData();
	} catch ( Exception e ) {
	  throw new DataObjectException( 
	    "Unable to load data for PitBossDO id=" + getOId() +
	    ", error = " + e.getMessage() );
	}
      data.TotalDollars =  markNewValue(
	data.TotalDollars, TotalDollars  );
   }
   

   /**
    * Get TotalBankrupt of the PitBoss
    *
    * @return TotalBankrupt of the PitBoss
    */
   public int getTotalBankrupt () 
   throws DataObjectException {
      if ( null == data )
 	try {
 	  loadData();
 	} catch ( Exception e ) {
 	  throw new DataObjectException( 
 	    "Unable to load data for PitBossDO id=" + getOId() +
 	    ", error = " + e.getMessage() );
        }
      return data.TotalBankrupt;
   }

   /**
    * Set TotalBankrupt of the PitBoss
    *
    * @param TotalBankrupt of the PitBoss
    */
   
   public void setTotalBankrupt ( int TotalBankrupt )
   throws DataObjectException {
      if ( null == data )
	try {
	  loadData();
	} catch ( Exception e ) {
	  throw new DataObjectException( 
	    "Unable to load data for PitBossDO id=" + getOId() +
	    ", error = " + e.getMessage() );
	}
      data.TotalBankrupt =  markNewValue(
	data.TotalBankrupt, TotalBankrupt  );
   }
   

   /**
    * Get TotalHandsDealt of the PitBoss
    *
    * @return TotalHandsDealt of the PitBoss
    */
   public int getTotalHandsDealt () 
   throws DataObjectException {
      if ( null == data )
 	try {
 	  loadData();
 	} catch ( Exception e ) {
 	  throw new DataObjectException( 
 	    "Unable to load data for PitBossDO id=" + getOId() +
 	    ", error = " + e.getMessage() );
        }
      return data.TotalHandsDealt;
   }

   /**
    * Set TotalHandsDealt of the PitBoss
    *
    * @param TotalHandsDealt of the PitBoss
    */
   
   public void setTotalHandsDealt ( int TotalHandsDealt )
   throws DataObjectException {
      if ( null == data )
	try {
	  loadData();
	} catch ( Exception e ) {
	  throw new DataObjectException( 
	    "Unable to load data for PitBossDO id=" + getOId() +
	    ", error = " + e.getMessage() );
	}
      data.TotalHandsDealt =  markNewValue(
	data.TotalHandsDealt, TotalHandsDealt  );
   }
   

   /**
    * Get TotalHandsWon of the PitBoss
    *
    * @return TotalHandsWon of the PitBoss
    */
   public int getTotalHandsWon () 
   throws DataObjectException {
      if ( null == data )
 	try {
 	  loadData();
 	} catch ( Exception e ) {
 	  throw new DataObjectException( 
 	    "Unable to load data for PitBossDO id=" + getOId() +
 	    ", error = " + e.getMessage() );
        }
      return data.TotalHandsWon;
   }

   /**
    * Set TotalHandsWon of the PitBoss
    *
    * @param TotalHandsWon of the PitBoss
    */
   
   public void setTotalHandsWon ( int TotalHandsWon )
   throws DataObjectException {
      if ( null == data )
	try {
	  loadData();
	} catch ( Exception e ) {
	  throw new DataObjectException( 
	    "Unable to load data for PitBossDO id=" + getOId() +
	    ", error = " + e.getMessage() );
	}
      data.TotalHandsWon =  markNewValue(
	data.TotalHandsWon, TotalHandsWon  );
   }
   
    /**
     * Protected constructor.
     *
     * @param rs
     *   Result set from which to obtain product data.
     * @exception java.sql.SQLException
     *   If a database connection error occurs.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If the database contains an invalid object id.
     */
    protected PitBossDO(ResultSet rs)
            throws SQLException, ObjectIdException, DataObjectException
 	    , DatabaseManagerException
/*  	      */
    {
        super(rs);

	// Constructing a DO from a ResultSet means we definitely need the 
	// DataStruct ready for the setXxx methods invoked below.
	data = new PitBossDataStruct ();
 
	// writeMemberStuff uses the ResultSetExtraction.template
	// to build up the value for this tag:
	// the value is a series of calls to the DO set methods.
		
	setGameName( 
	    
		rs.getString( 
			"GameName"  )
	    
	);
	
	
	setTotalDollars( 
	    
		rs.getInt( 
			"TotalDollars"  )
	    
	);
	
	
	setTotalBankrupt( 
	    
		rs.getInt( 
			"TotalBankrupt"  )
	    
	);
	
	
	setTotalHandsDealt( 
	    
		rs.getInt( 
			"TotalHandsDealt"  )
	    
	);
	
	
	setTotalHandsWon( 
	    
		rs.getInt( 
			"TotalHandsWon"  )
	    
	);
	

 
        markClean();
    }

    

    

    private int[] param = null;

    /**
     * Prepares the statement used to insert this object
     * into the database.
     * @param conn the database connection.
     * @return the insert statement.
     * @exception java.sql.SQLException if an error occurs.
     */
    public PreparedStatement getInsertStatement(DBConnection conn)
            throws SQLException {

        ObjectId oid;

        PreparedStatement stmt = conn.prepareStatement( 
	    "insert into PitBoss values ( ?, ?, ?, ?, ?, ?, ? )" );

	param = new int[1]; param[0] = 1;
	// writeMemberStuff uses the JDBCsetCalls.template
	// to build up the value for this tag:
	// the value is a series of calls to setPrepStmtParam_TYPE methods.
	// Those methods are defined below.
	try {
	    	setPrepStmtParam_String( stmt, param,
		getGameName() );
	setPrepStmtParam_int( stmt, param,
		getTotalDollars() );
	setPrepStmtParam_int( stmt, param,
		getTotalBankrupt() );
	setPrepStmtParam_int( stmt, param,
		getTotalHandsDealt() );
	setPrepStmtParam_int( stmt, param,
		getTotalHandsWon() );


	    /* The order of the values being inserted must match
	     * the order of the columns listed in the CREATE TABLE command
	     * that appears in the .sql file for this DO.
	     * Since the id and version number are always the last columns
	     * listed in the CREATE TABLE command, their values are added
	     * to the PreparedStatement after all other values.
	     */
	    setPrepStmtParam_BigDecimal( stmt, param, getOId().toBigDecimal() );
	    setPrepStmtParam_int(        stmt, param, getNewVersion() );

	} catch ( Exception e ) {
	    throw new SQLException( "Data Object error: " + e.getMessage() );
	}

        return stmt;
    }

    /**
     * Prepares the statement used to update this object
     * in the database.
     * @param conn the database connection
     * @return the update statement.
     * @exception java.sql.SQLException if an error occurs.
     */
    public PreparedStatement getUpdateStatement(DBConnection conn)
            throws SQLException {

        ObjectId oid;

        PreparedStatement stmt = conn.prepareStatement(
	    "update PitBoss set GameName = ?, TotalDollars = ?, TotalBankrupt = ?, TotalHandsDealt = ?, TotalHandsWon = ? " +
	    "where oId = ? and version = ?" );

	param = new int[1]; param[0] = 1;
	//int[] param = {1};
	// writeMemberStuff builds up the value for this tag:
	// the value is a series of calls to setPrepStmtParam_TYPE methods.
	// Those methods are defined below.
	try {
	    	setPrepStmtParam_String( stmt, param,
		getGameName() );
	setPrepStmtParam_int( stmt, param,
		getTotalDollars() );
	setPrepStmtParam_int( stmt, param,
		getTotalBankrupt() );
	setPrepStmtParam_int( stmt, param,
		getTotalHandsDealt() );
	setPrepStmtParam_int( stmt, param,
		getTotalHandsWon() );


	    /* When updating a persistent object, the UPDATE_WHERE_CLAUSE tag
	     * used to build the SQL WHERE clause (above) uses the 
	     * DO's id and version number to locate the correct row in
	     * the database table.
	     */
	    setPrepStmtParam_BigDecimal( stmt, param, getOId().toBigDecimal() );
	    setPrepStmtParam_int(        stmt, param, getVersion() );

	} catch ( Exception e ) {
	    throw new SQLException( "Data Object error: " + e.getMessage() );
	}

        return stmt;
    }

    /**
     * Prepares the statement used to delete this object
     * from the database.
     * @param conn the database connection
     * @return the delete statement.
     * @exception java.sql.SQLException if an error occurs.
     */
    public PreparedStatement getDeleteStatement(DBConnection conn)
            throws SQLException {
        String sql =
            "delete from PitBoss \n" +
            "where oId = ?";
        PreparedStatement stmt = conn.prepareStatement(sql);
        stmt.setBigDecimal(1, getOId().toBigDecimal());
        return stmt;
    }

    

    /*
     * toString - for debugging
     */
    public String toString(){
        return "PitBossDO dump: " + " GameName=" + data.GameName
+ " TotalDollars=" + data.TotalDollars
+ " TotalBankrupt=" + data.TotalBankrupt
+ " TotalHandsDealt=" + data.TotalHandsDealt
+ " TotalHandsWon=" + data.TotalHandsWon
 + ".";
    }

}
