/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: GeneratorInfo.java,v 1.5 1999/02/12 12:17:57 andy Exp $
 *-----------------------------------------------------------------------------
 */



package dods.builder.engine;

import dods.data.DataObject;
import dods.data.Attribute;

import  com.lutris.util.Config;

import java.lang.reflect.*;
import java.io.*;
import java.io.FileNotFoundException;
import java.util.*;

public class GeneratorInfo {

    public Config config;
    public boolean runForAbstract;
    public boolean handlesArrays;
    public String outputDir;
    public String outputFileFlavor;
    public String outputFileSuffix;
    public String outputClassParent;
    public String extraClasspath;
    public Class  processor;
    public String templateDir;
    public TemplateReader[] templates;

    public static String WRITE_FILE_HEADER    = "writeFileHeader";
    public static String WRITE_FILE_FOOTER    = "writeFileFooter";
    public static String WRITE_MEMBER_STUFF   = "writeMemberStuff";

    private String[] methodNames = 
	{ WRITE_FILE_HEADER, WRITE_FILE_FOOTER, WRITE_MEMBER_STUFF };

    public TemplateReader getTemplateReader( String key ) {
	for ( int i = 0; i < methodNames.length; i++ ) {
	    if ( methodNames[i].equals( key ) ) {
/* 		return templates[i]; */
		/* I'd like to reuse the 3 main TemplateReaders.
		   I wanted to open them only once, and reset() them
		   for each DataObject.
		   But for some reason, reset does not work for them.
		   Reset() does work for the 2 extra templates used 
		   by SourceGenerator_DO, so maybe I just need to 
		   figure out what makes them work correctly.
		   FIX remove the 'return' below in favor of the one above.
		*/
		try {
//DBG
// System.err.println("getTemplateReader: <"+templateDir + File.separator + methodNames[i] + ".template>");
	    	return new TemplateReader( 
	    	templateDir + File.separator + methodNames[i] + ".template" );
		} catch ( Exception e ) { 
		}
	    }
	}
	return null;
    }

    public GeneratorInfo(	
				Config cfg,
				boolean genAbstract,
				boolean doArrays,
				String outDir,
				String flavor,
				String suffix,
				String parent,
				String extraCP,
				Class  generator,
				String tempDir )
    throws Exception {
	config            = cfg;
	runForAbstract    = genAbstract;
	handlesArrays     = doArrays;
	outputDir         = outDir;
	outputFileFlavor  = flavor;
	outputFileSuffix  = suffix;
	outputClassParent = parent;
	extraClasspath    = extraCP;
	templateDir       = tempDir;
	processor         = generator;
	Method m[]        = null;
	try {
	    m = generator.getMethods();
	} catch ( Exception e ) {
	    throw e;
	}
	int found = 0;
	for ( int i = 0; i < m.length; i++ ) {
//System.err.println("m[i].getName()="+m[i].getName());
	    Class[] parms = m[i].getParameterTypes();
	    if ( 	m[i].getName().equals( WRITE_FILE_HEADER ) ||
	    	 	m[i].getName().equals( WRITE_FILE_FOOTER ) ) {
	    	Class[] expect = {DataObject.class};
		checkTypeOfParms( processor, m[i], parms, expect );
		found++;
	    } else if (	m[i].getName().equals( WRITE_MEMBER_STUFF ) ) {
	    	Class[] expect = {Attribute.class};
		checkTypeOfParms( processor, m[i], parms, expect );
		found++;
	    }
	}
//System.err.println("methodNames.length="+methodNames.length);
	if ( found != methodNames.length )
	    throw new Exception( "Class " + generator.getName() +
		", is missing at least 1 method." );
	
	/* see FIX above
	templates = new TemplateReader[methodNames.length];
	for ( int i = 0; i < methodNames.length; i++ ) {
	    templates[i] = new TemplateReader( 
	    	templateDir + File.separator + methodNames[i] + ".template" );
	}
	*/
    }

    private void checkTypeOfParms( Class c, Method m, Class[] parms,
					Class[] correctTypes ) 
    throws Exception {
	if ( parms.length != correctTypes.length ) 
	    throw new Exception( "Class " + c.getName() +
		", method " + m.getName() + 
		" has " + parms.length + 
		" parameters, but should take " + correctTypes.length );

	for ( int i = 0; i < parms.length; i++ ) {
	    if ( parms[i] != correctTypes[i] )
		throw new Exception( "Class " + c.getName() +
		    ", method " + m.getName() + 
		    ", parm #" + i + 
		    " is type " + parms[i].getName() + 
		    " but should be type " + correctTypes[i].getName() );
    	}
    }
}
