/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: TemplateFilter.java,v 1.9 1999/06/30 02:31:32 jay Exp $
 *-----------------------------------------------------------------------------
 */



//------------------------------------------------------------------------------
// TemplateFilter

package dods.builder.engine;

import java.io.*;
import java.util.Enumeration;

public class TemplateFilter
{
    public static String skipStart = "__START_SKIPPING__";
    public static String skipStop  = "__STOP_SKIPPING__";

    private TemplateReader input;
    private Writer         output;

    //--------------------------------------------------------------------------
    /**
     *  TemplateFilter
     *    constructor
     *
     *  @param reader
     *    The input template with embedded tags to be translated.
     *
     *  @param writer
     *    The output text with tags translated to values.
     */
    public TemplateFilter( TemplateReader reader, Writer writer ) throws Exception {
        input  = reader;
	input.mark( 100000 );
        output = writer;
    }

    //--------------------------------------------------------------------------
    /**
     *  process
     *    Reads from input, writing to output,
     *    replacing the tags with values from the TagSet.
     *
     *    This method is called by the implementations of the abstract methods
     *       writeFileHeader
     *       writeFileFooter
     *       writeMemberStuff
     *    which are defined in classes derived from class SourceGenerator.
     *
     *  @param tagSet
     *    The TagSet containing a values for various tags.
     */
    public void process( TagSet tagSet ) throws Exception {
	String line;
	String tag;
	int    tagLen, index, skipDepth = 0;
//System.err.println("input.markSupported()="+input.markSupported());
	while ( null != ( line = input.readLine() ) ) {
/* System.err.println("line ="+line ); */
/* System.err.println("line.length()="+line.length()); */
	    Enumeration tags = tagSet.getTags();
	    while ( tags.hasMoreElements() ) {
	        tag = (String) tags.nextElement();
/* System.err.println("tag ="+tag ); */
		while ( -1 != ( index = line.indexOf( tag ) ) ) {
/* System.err.println("index ="+index ); */
		    tagLen = tag.length();
		    line = line.substring( 0, index )
			    + tagSet.getTagValue( tag )
			    + line.substring( index + tagLen, line.length() );
		}
/* System.err.println("line ="+line ); */
	    }
	    if ( -1 != line.indexOf( skipStart ) ) {
		skipDepth++;
/* System.err.println("skipDepth="+skipDepth); */
	    } else if ( -1 != line.indexOf( skipStop ) ) {
		if ( skipDepth > 0 )
		    skipDepth--;
/* System.err.println("skipDepth="+skipDepth); */
	    } else if ( skipDepth == 0 ) {
	        output.write( line + "\n" );
	    }
	}
/*         output.close(); */
/* System.err.println("PAST CLOSE" ); */
    }

    public void reset() throws Exception {
//System.err.println("reset: input.markSupported()="+input.markSupported());
	input.reset();
    }

    public void close() throws Exception {
	// This line fixes a problem a Linux user was seeing,
	// but caused other problems on Windows.
	input.close();
    }

}   // End of class TemplateFilter
