/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: SourceGenerator_DO.java,v 1.17 1999/06/30 02:31:34 jay Exp $
 *-----------------------------------------------------------------------------
 */




package dods.builder.sourceGenerators.DO;

import dods.data.*;
import dods.util.*;

import dods.builder.engine.*;
import dods.builder.sourceGenerators.*;

import com.lutris.appserver.server.sql.*;

/*
import java.io.*;
import java.io.IOException;
*/
import java.io.File;
import java.io.StringWriter;
import java.util.Vector;

public class SourceGenerator_DO extends SourceGenerator {

  final String DATA_STRUCT      = "DATA_STRUCT";
  final String TO_STRING      = "TO_STRING";
  final String ABSTRACT      = "ABSTRACT";
  final String BEGIN_CONCRETE      = "BEGIN_CONCRETE";
  final String END_CONCRETE      = "END_CONCRETE";
  final String BEGIN_LEAF      = "BEGIN_LEAF";
  final String END_LEAF      = "END_LEAF";
  final String LEAF_ONLY      = "LEAF_ONLY";
  final String NOT_LOADING      = "NOT_LOADING";
  final String BEGIN_NONCONSTANT    = "BEGIN_NONCONSTANT";
  final String END_NONCONSTANT    = "END_NONCONSTANT";
  final String MARK_NEW_VALUE_EXTRAS    = "MARK_NEW_VALUE_EXTRAS";
  final String BIG_DECIMAL_SCALE    = "BIG_DECIMAL_SCALE";

  //final String IMPORT_PARENT      = "IMPORT_PARENT";
  final String INSERT_QUESTION_MARKS    = "INSERT_QUESTION_MARKS";
  final String UPDATE_QUAL_QUESTION_MARKS  = "UPDATE_QUAL_QUESTION_MARKS";
  final String UPDATE_WHERE_CLAUSE    = "UPDATE_WHERE_CLAUSE";
  final String RESULT_SET_EXTRACTION_CALLS  = "RESULT_SET_EXTRACTION_CALLS";
  final String PARAM_SET_OID      = "PARAM_SET_OID";
  final String UPDATE_PARAM_SET_CALLS    = "UPDATE_PARAM_SET_CALLS";
  final String PREP_STMT_PARAM_CALLS    = "PREP_STMT_PARAM_CALLS";

  final String CAST_TO_DO      = "CAST_TO_DO";
  final String REFERENCED_DO_EXCEPTIONS  = "REFERENCED_DO_EXCEPTIONS";
  final String REFERENCE_OID_TYPE    = "REFERENCE_OID_TYPE";
  final String BEGIN_CREATE_EXISTING    = "BEGIN_CREATE_EXISTING";
  final String END_CREATE_EXISTING    = "END_CREATE_EXISTING";
  final String ENABLE_CACHE      = "ENABLE_CACHE";
  final String LAZY_LOADING      = "LAZY_LOADING";

  final protected String ARRAY_DO_NAME        = "ARRAY_DO_NAME";
  final protected String ARRAY_QUERY_NAME     = "ARRAY_QUERY_NAME";
  final protected String ARRAY_GET_METHODS    = "ARRAY_GET_METHODS";

  final protected String SUPER_LOADDATA       = "SUPER_LOADDATA";

  final protected String DUMP_METHOD          = "DUMP_METHOD";

  final protected String GET_REFERRERS        = "GET_REFERRERS";
  final protected String REFERRER             = "REFERRER";
  final protected String FULL_REFERRER             = "FULL_REFERRER";
  final protected String GET_ARRAY_METHOD_NAME = "GET_ARRAY_METHOD_NAME";
  final protected String GET_SINGLE_METHOD_NAME = "GET_SINGLE_METHOD_NAME";


  final protected String JAVA_TO_JDBC = "JAVA_TO_JDBC";
  final protected String JDBC_TO_JAVA = "JDBC_TO_JAVA";


  final String defaultStringLength    = "32";

  // template reader and filter for JDBCsetCall.template file.
  private TemplateFilter filterJDBCsetCalls;
  private StringWriter   JDBCsetCalls;

  // template reader and filter for ResultSetExtraction.template file.
  private TemplateFilter filterResultSetExtractions;
  private StringWriter   ResultSetExtractions;

  // template reader and filter for toString.template file.
  private TemplateFilter filterMemberDump;
  private StringWriter   memberDump;

  // template reader and filter for HandleArray.template file.
  private TemplateFilter filterHandleArray;
  private StringWriter   handleArray;

  // template reader and filter for getReferrers.template file.
  private TemplateFilter filterGetReferrers;
  private StringWriter   getReferrers;

  private DODSTypes dodsTypes = null;

  public SourceGenerator_DO() throws Exception {
    dodsTypes = DODSTypes.getInstance();
  }


  /**
   * Adds tag values to the <CODE>TagSet</CODE>.
   * These methods should invoke
   *    <CODE>TemplateFilter.process( tagSet )</CODE>
   * which will create the output source file.
   *
   * @param sgi      Generator information
   * @param project  The DODS project
   * @param o        
   *   DataObject for which source code will be generated by
   *    <CODE>writeFileHeader</CODE> and
   *    <CODE>writeFileFooter</CODE>
   * @exception Exception
   *   If an unrecoverable error occurs.
   */
  public void objectPrep(  GeneratorInfo sgi, DODSProject project, DataObject o ) throws Exception {

    super.objectPrep( sgi, project, o );

    TemplateReader reader = new TemplateReader( info.templateDir + File.separator + "JDBCsetCall.template" );

    JDBCsetCalls = new StringWriter();
    filterJDBCsetCalls = new TemplateFilter( reader, JDBCsetCalls );

    reader = new TemplateReader( info.templateDir + File.separator + "ResultSetExtraction.template" );
    ResultSetExtractions = new StringWriter();
    filterResultSetExtractions = new TemplateFilter( reader, ResultSetExtractions );

    reader = new TemplateReader( info.templateDir + File.separator + "toString.template" );
    memberDump = new StringWriter();
    filterMemberDump = new TemplateFilter( reader, memberDump );

    reader = new TemplateReader( info.templateDir + File.separator + "HandleArray.template" );
    handleArray = new StringWriter();
    filterHandleArray = new TemplateFilter( reader, handleArray );

    reader = new TemplateReader( info.templateDir + File.separator + "getReferrers.template" );
    getReferrers = new StringWriter();
    filterGetReferrers = new TemplateFilter( reader, getReferrers );

    getReferrers( o, project );

    boolean isAbstract = o.isAbstract();

    tagSet.setTagValue( DATA_STRUCT,    o.getName() + "DataStruct" );
    tagSet.setTagValue( ENABLE_CACHE,   o.isFullCaching() ? "true":"false");
    tagSet.setTagValue( LAZY_LOADING,   o.isLazyLoading() ? "true":"false");
    tagSet.setTagValue( ABSTRACT,       isAbstract ? "abstract" : "" );
    tagSet.setTagValue( BEGIN_CONCRETE, isAbstract ? "/*" : "" );
    tagSet.setTagValue( END_CONCRETE,   isAbstract ? "*/" : "" );
    Vector v = o.getDescendents();
    boolean isLeaf = ( null == v || 0 == v.size() );

    /* FIX (see "concrete-non-leaf" note in Builder.java)
       It really doesn't make sense
       to derive a concrete DO from another concrete DO.
       To do so would raise a number of implementation problems:
         1)  If each DO has its own table, then
           you'd get tables with redundant data.
         2)  If both DOs reside in the same table,
           A) you'd have wasted columns for the parent class
           B) you'd need an extra column to indicate the class 
              that a given row represents.
       So, Philip is changing the GUI so that when class B is 
       derived from class A, class A is automatically made 'abstract'.
       Until this is done, we force it to be so.
    */
    if ( o.isAbstract() || ! isLeaf )
      tagSet.setTagValue( NOT_LOADING, "true" );
    else
      tagSet.setTagValue( NOT_LOADING, "false" );

    tagSet.setTagValue( BEGIN_LEAF,       isLeaf ? "" : "/*" );
    tagSet.setTagValue( END_LEAF,         isLeaf ? "" : "*/" );
    tagSet.setTagValue( LEAF_ONLY,        isLeaf ? "" : "// " );

    // These tags must have values even if there are no Attributes
    // (a stupid situation for a Data Object to be in.)
    tagSet.setTagValue( CAST_TO_DO, "" );
    tagSet.setTagValue( REFERENCED_DO_EXCEPTIONS, "" );
    tagSet.setTagValue( REFERENCE_OID_TYPE, "");
    tagSet.setTagValue( BEGIN_CREATE_EXISTING, "");
    tagSet.setTagValue( END_CREATE_EXISTING, "");

    DataObject parent = o.getExtensionOf();
    String parentName = null;
    if ( null != parent )
      parentName = parent.getName();
    if ( null == parentName || -1 != parentName.indexOf( "GenericDO" ) )
      tagSet.setTagValue( SUPER_LOADDATA, "");
    else
      tagSet.setTagValue( SUPER_LOADDATA, "super.loadData();");

  }


  /**
   *
   *  This algorithm generates short or long method names.
   *  Consider the following example Data Objects used by a telephone company:
   *      Data Object  Attribute type,  name
   *      -----------  --------------  -----------
   *      Phone  int    phoneNumber
   *      Person  PhoneDO,  phone
   *      Business  PhoneDO,  mainPhone
   *      Business  PhoneDO,  faxPhone
   *   Multiple Persons can share the same Phone,
   *   and (in this example) multiple Businesses can share the same Phone.
   *   We generate the following method names for the PhoneDO class:
   *      return type  method name
   *      -----------  ------------------------------------
   *      PersonDO[]  Phone.getPersonDOArray()
   *      PersonDO[]  Phone.getBusinessDOArray_mainPhone()
   *      PersonDO[]  Phone.getBusinessDOArray_faxPhone()
   *  The telephone company would find all the Persons
   *  with the same home phone number by invoking
   *      Phone.getPersonDOArray()
   *          (a short method name, since Person has
   *           just 1 Attribute referring to Phone)
   *  and find all the Businesses using a given fax line by invoking
   *      Phone.getBusinessDOArray_faxPhone()
   *          (a long method name, since Business has
   *           more than 1 Attribute referring to Phone)
   */
  private void getReferrers( DataObject obj, DODSProject project ) throws Exception {

    //System.err.println("getReferrers: obj.getName()="+obj.getName());
    filterGetReferrers.reset();

    DataObject[] all = Builder.getAllDataObjects( project );

    for ( int i = 0; i < all.length; i++ ) {

      DataObject DO = all[ i ];
      Vector attrs = DO.getAttributes();

      if ( null != attrs ) {

        Vector refs = new Vector();

        for ( int j = 0; j < attrs.size(); j++ ) {
          Attribute attr = (Attribute) attrs.elementAt( j );
          if ( attr.isObjectRef()
               && attr.getObjectRefDO() == obj
               && attr.isUsedForQuery() ) {

            //System.err.println("getReferrers: DO.getName()="+DO.getName() + ", attr="+attr.getName());
            refs.addElement( attr );

          }

        }

        for ( int k = 0; k < refs.size(); k++ ) {

          String DOname = DO.getName();
          Attribute attr = (Attribute) refs.elementAt( k );
          String get_single = "get" + DOname + "DO";
          String get_array  = "get" + DOname + "DOArray";
          if ( 1 < refs.size() ) {
            get_single += "_" + attr.getName();
            get_array  += "_" + attr.getName();
          }
          memberPrep( attr );
          tagSet.setTagValue( GET_SINGLE_METHOD_NAME, get_single );
          tagSet.setTagValue( GET_ARRAY_METHOD_NAME,  get_array );
          tagSet.setTagValue( REFERRER, DOname );
          tagSet.setTagValue( FULL_REFERRER,
                              DO.getPackageName() + "." + DOname );
          filterGetReferrers.reset();
          filterGetReferrers.process( tagSet );
        }

      }

    }

  }


  /**
   * Prepares 'member' for code generation
   *
   * @param a  The attribute or member currently worked on
   */
  public void memberPrep(  Attribute a ) throws Exception {

    super.memberPrep( a );

    String dodsType = a.getDODSType();
    String jdbcType = dodsTypes.getJDBCType(dodsType);
    String javaType = dodsTypes.getJavaType(dodsType);
    String strSize = a.getSize();
    String jdbc2Java = dodsTypes.getJDBCToJava(dodsType);
    String java2JDBC = dodsTypes.getJavaToJDBC(dodsType);

    /* check for nulls.
     * This will (probably?) occur if the attribute references
     * another table
     */
    if (java2JDBC == null)
      java2JDBC = dodsTypes.getJavaToJDBC("ObjectId");

    if (jdbc2Java == null)
      jdbc2Java = dodsTypes.getJDBCToJava("ObjectId");

    if (javaType == null)
      javaType = dodsType;

    tagSet.setTagValue( BEGIN_NONCONSTANT, a.isConstant() ? "/*" : "" );
    tagSet.setTagValue( END_NONCONSTANT,   a.isConstant() ? "*/" : "" );

    if (jdbcType != null) {
      tagSet.setTagValue( MEMBER_JDBC_TYPE_SAFE_NAME, jdbcType );
    } else {
      tagSet.setTagValue( MEMBER_JDBC_TYPE_SAFE_NAME, "DO" );
      jdbcType = dodsTypes.getJDBCType("ObjectId");
    }

    /* For a BigDecimal field, the JDBC getBigDecimal method
     * takes an extra 'scale' parameter.
     * FIX the GUI should prompt for this scale value,
     * and the Attribute class should store it.
     * For now we use the Attribute size.
     */
    if ( jdbcType != null && jdbcType.equals( "BigDecimal" ) ) {
      if ( null == strSize || 0 == strSize.length() )
        tagSet.setTagValue( BIG_DECIMAL_SCALE, ", 0" );
      else
        tagSet.setTagValue( BIG_DECIMAL_SCALE, ", " + strSize );
    } else {
      tagSet.setTagValue( BIG_DECIMAL_SCALE, "" );
    }

    /* For a field that is an object reference to another Data Object,
     * the type of the field will be a class derived from GenericDO.
     * The GenericDO.markNewValue() method that handles such fields
     * takes GenericDO objects as parameters and returns a GenericDO.
     * The call to this form of markNewValue() must cast the return value
     * from GenericDO to the derived class.
     */
    if ( a.isObjectRef()) {
      tagSet.setTagValue( CAST_TO_DO, "(" + dodsType + ")" );
      tagSet.setTagValue( REFERENCED_DO_EXCEPTIONS, ", DatabaseManagerException");
      tagSet.setTagValue( BEGIN_CREATE_EXISTING, dodsType + ".createExisting( ");
      tagSet.setTagValue( END_CREATE_EXISTING,   " )");

      //tagSet.setTagValue( BIG_DECIMAL_SCALE, ", 0" );

      tagSet.setTagValue( MEMBER_JDBC_TYPE, oidJdbcType );
      //      tagSet.setTagValue( REFERENCE_OID_TYPE, ".getOId().toBigDecimal()");
      tagSet.setTagValue( REFERENCE_OID_TYPE, "");
      tagSet.setTagValue( JDBC_TO_JAVA, "");

      /* DUMP_METHOD was used to prevent recursion in the
       * DO.toString() method that would occur when
       * DO1 had a ref to DO2 and DO2 had a ref back to DO1.
       * The recursion was caused by printing
       *    DO1.data.DO2
       * To avoid this, DUMP_METHOD was used to print
       *    DO1.data.DO2.getOId()
       * But this type of recursion is too infrequent,
       * and it is too valuable to see the entire dump of ref'd objects.
       * So, DUMP_METHOD is not being used.
       */
      //tagSet.setTagValue( DUMP_METHOD, ".getOId()" );
      tagSet.setTagValue( DUMP_METHOD, "" );

    } else if (a.isRefConstraint()) {

      tagSet.setTagValue( CAST_TO_DO, "(" + dodsType + ")" );
      tagSet.setTagValue( REFERENCED_DO_EXCEPTIONS, ", DatabaseManagerException");
      tagSet.setTagValue( BEGIN_CREATE_EXISTING, dodsType + ".createExisting( ");
      tagSet.setTagValue( END_CREATE_EXISTING,   " )");

      tagSet.setTagValue( REFERENCE_OID_TYPE, "");
      tagSet.setTagValue( JDBC_TO_JAVA, "");

    } else {

      // field is not a reference to another DO...

      tagSet.setTagValue( CAST_TO_DO, "" );
      tagSet.setTagValue( REFERENCED_DO_EXCEPTIONS, "" );
      tagSet.setTagValue( REFERENCE_OID_TYPE, "");
      tagSet.setTagValue( BEGIN_CREATE_EXISTING, "/*");
      tagSet.setTagValue( END_CREATE_EXISTING, "*/");
      tagSet.setTagValue( DUMP_METHOD, "" );


      // Set the JDBC to Java conversion details
      String jdbcGet = "rs.get"+tagSet.getTagValue(MEMBER_JDBC_TYPE)+"(\""+tagSet.getTagValue(MEMBER_NAME) +"\" "+tagSet.getTagValue(BIG_DECIMAL_SCALE) + ")";
  
      // special conversion available?
      if (jdbc2Java != null) {

        jdbcGet = Util.replace(jdbc2Java, DODSTypes.JDBC_VAR, jdbcGet);
        jdbcGet = Util.replace(jdbcGet, DODSTypes.JAVA_TYPE, javaType);

      }

      // set the JDBC to Java conversion tag
      tagSet.setTagValue( JDBC_TO_JAVA, jdbcGet );

    }

    // Set the Java to JDBC conversion details
    String javaGet = "get"+tagSet.getTagValue(UPPER_MEMBER_NAME)+"()";

    // special conversion available?
    if (java2JDBC != null) {

      javaGet = Util.replace(java2JDBC, DODSTypes.JAVA_VAR, javaGet);
      javaGet = Util.replace(javaGet, DODSTypes.JDBC_TYPE, jdbcType);

    }

    // set the Java to JDBC conversion tag
    tagSet.setTagValue( JAVA_TO_JDBC, javaGet );

    /* For a String field, the GenericDO.markNewValue() method
     * takes some extra parameters that validate max length
     * and whether null strings are allowed.
     */
    String markNewValue_extraParms = "";

    if (dodsTypes.isSizeAllowed(dodsType)) {

      if ( null == strSize || 0 == strSize.length() ) {
        Builder.print("WARNING: Attribute '" + a.getName() + "' size defaulting to " + defaultStringLength );
        strSize = defaultStringLength;
      }
      markNewValue_extraParms = ", 0, " + strSize + ", " + ( a.canBeNull() ? "true" : "false" );
    }
    tagSet.setTagValue( MARK_NEW_VALUE_EXTRAS, markNewValue_extraParms );

    /* For an array Attribute, we use a separate template. */
    if ( a.isArray() ) {
      tagSet.setTagValue( ARRAY_QUERY_NAME,
                          currentObj.getAdjustedPackageName() + "." + currentObj.getName() + "_" + a.getName() + "Query" );
      tagSet.setTagValue( ARRAY_DO_NAME, currentObj.getName() + "_" + a.getName() + "DO" );
    }

  }

  private String sql_insert_question_marks = "";
  private String sql_update_question_marks = "";

  public void writeFileHeader(  DataObject o ) throws Exception {
    /*
    if ( null != info.extraClasspath )
      tagSet.setTagValue( IMPORT_PARENT, info.extraClasspath );
    */
    defaultWriteFileHeader( o );
    sql_insert_question_marks = "?, ?";  // oid and version
    sql_update_question_marks = "";

    Vector ancestors = o.getAncestors();
    /* System.err.println("ancestors.size()="+ancestors.size()); */
    if ( null != ancestors ) {
      for ( int i = 0; i < ancestors.size(); i++ ) {
        DataObject p = (DataObject) ancestors.elementAt( i );
        if ( null != p ) {
          Vector members = p.getAttributes();
          for ( int j = 0; j < members.size(); j++ ) {
            Attribute attr = (Attribute) members.elementAt( j );
            memberPrep( attr );
            sqlMarks(   attr );
          }
        }
      }
    }
  }

  public void writeMemberStuff( Attribute  a ) throws Exception {
    /* System.err.println("--- writeMemberStuff: a="+a.getName()); */
    /* For an array Attribute, we use a separate template. */
    if ( a.isArray() ) {
      filterHandleArray.reset();
      filterHandleArray.process( tagSet );
      /* Arrays are stored in separate tables,
       * so the DO does not process them 
       * the way it does other Attributes.
       */
      return;
    }
    defaultWriteMemberStuff( a );
    filterResultSetExtractions.reset();
    filterResultSetExtractions.process( tagSet );
    filterMemberDump.reset();
    filterMemberDump.process( tagSet );
    sqlMarks( a );
  }

  private void sqlMarks( Attribute  a ) throws Exception {
    filterJDBCsetCalls.reset();
    filterJDBCsetCalls.process( tagSet );
    /* System.err.println("JDBCsetCalls.toString()="+JDBCsetCalls.toString()); */

    // We could be using 2 more templates to generate the lists of qmarks.
    // But those templates would contain no tags, so it seems best to
    // build the lists of qmarks like this.
    sql_insert_question_marks += ", ?";
    if ( 0 == sql_update_question_marks.length() ) {
      sql_update_question_marks = a.getName() + " = ?";
    } else {
      sql_update_question_marks += ", " + a.getName() + " = ?";
    }
  }

  public void writeFileFooter(  DataObject o ) throws Exception {
    /* System.err.println("--- writeFileFooter"); */

    tagSet.setTagValue( INSERT_QUESTION_MARKS,
                        sql_insert_question_marks );
    tagSet.setTagValue( UPDATE_QUAL_QUESTION_MARKS,
                        sql_update_question_marks );
    tagSet.setTagValue( UPDATE_WHERE_CLAUSE,
                        "where " + CoreDO.NAME_OBJECTID + "= ? and "+ CoreDO.NAME_VERSION + " = ?" );
    if ( null != JDBCsetCalls )
      tagSet.setTagValue( PREP_STMT_PARAM_CALLS,
                          JDBCsetCalls.toString() );
    if ( null != ResultSetExtractions )
      tagSet.setTagValue( RESULT_SET_EXTRACTION_CALLS,
                          ResultSetExtractions.toString() );
    if ( null != memberDump )
      tagSet.setTagValue( TO_STRING, memberDump.toString() );
    if ( null != handleArray )
      tagSet.setTagValue( ARRAY_GET_METHODS, handleArray.toString() );
    if ( null != getReferrers )
      tagSet.setTagValue( GET_REFERRERS, getReferrers.toString() );
    defaultWriteFileFooter( o );
    filterGetReferrers.close();
    filterHandleArray.close();
    filterResultSetExtractions.close();
    filterMemberDump.close();
    filterJDBCsetCalls.close();
  }

}
