/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: SourceGenerator_Query.java,v 1.8 1999/07/16 22:25:57 jay Exp $
 *-----------------------------------------------------------------------------
 */




package dods.builder.sourceGenerators.Query;

import dods.data.*;

import dods.builder.engine.*;
import dods.builder.sourceGenerators.*;

import java.io.File;
import java.io.StringWriter;
import java.util.Vector;

public class SourceGenerator_Query extends SourceGenerator {

/*     final String DO_CLASS_NAME		= "DO_CLASS_NAME"; */
    final String NULL_FLAG		= "NULL_FLAG";
    final String CAN_QUERY		= "CAN_QUERY";

    private static final String EXACT_MATCH = "EXACT_MATCH";
    private static final String INEXACT_MATCH = "INEXACT_MATCH";
    private static final String BEGIN_TYPE_PRIMITIVE = "BEGIN_TYPE_PRIMITIVE";
    private static final String END_TYPE_PRIMITIVE = "END_TYPE_PRIMITIVE";
    private static final String BEGIN_TYPE_OTHER = "BEGIN_TYPE_OTHER";
    private static final String END_TYPE_OTHER = "END_TYPE_OTHER";

    private DODSTypes dodsTypes = null;

    public SourceGenerator_Query() throws Exception {
      dodsTypes = DODSTypes.getInstance();
    }

   /**
     * Adds tag values to the <CODE>TagSet</CODE>.
     * This method invokes 
     *    <CODE>TemplateFilter.process( tagSet )</CODE>
     * which will create the output source file.
     *
     * @param o
     *   DataObject for which source code will be generated by
     *    <CODE>writeFileHeader</CODE> and
     *    <CODE>writeFileFooter</CODE>
     * @exception Exception
     *   If an unrecoverable error occurs.
     *
     */
    public void objectPrep(  GeneratorInfo sgi, DODSProject project, DataObject o ) throws Exception {

      super.objectPrep( sgi, project, o );

/*   tagSet.setTagValue( DO_CLASS_NAME, o.getName() + "DO" ); */
    }


    public void memberPrep(  Attribute a ) throws Exception {

      if ( ! a.isUsedForQuery() ) return;
        super.memberPrep( a );
      tagSet.setTagValue( NULL_FLAG, a.canBeNull() ? "NULL_OK" : "NOT_NULL" );
      tagSet.setTagValue( CAN_QUERY, a.isUsedForQuery() ? "public" : "private" );

      tagSet.setTagValue( BEGIN_TYPE_PRIMITIVE,  TemplateFilter.skipStart );
      tagSet.setTagValue( END_TYPE_PRIMITIVE,     TemplateFilter.skipStop );
      tagSet.setTagValue( BEGIN_TYPE_OTHER,  TemplateFilter.skipStart );
      tagSet.setTagValue( END_TYPE_OTHER,     TemplateFilter.skipStop );

      String type = dodsTypes.getJavaType(a.getDODSType()); 
      boolean primitive = dodsTypes.isJavaPrimitive(type); 

      if (primitive) {

        tagSet.setTagValue( BEGIN_TYPE_PRIMITIVE, "");
        tagSet.setTagValue( END_TYPE_PRIMITIVE, "");

      } else {

        tagSet.setTagValue( BEGIN_TYPE_OTHER, "");
        tagSet.setTagValue( END_TYPE_OTHER, "");

      }

      String match_exact = dodsTypes.getJavaExactCompare(a.getDODSType()); 
      String match_inexact = dodsTypes.getJavaInexactCompare(a.getDODSType()); 

      // set the exact match code
      if (match_exact != null) {

        tagSet.setTagValue(EXACT_MATCH, match_exact);

      } else {

        tagSet.setTagValue(EXACT_MATCH, "(x == y)");

      }

      // set the inexact match code
      if (match_inexact != null) {

        tagSet.setTagValue(INEXACT_MATCH, match_inexact);

      } else {

        // use the exact match code since we don't have inexact match code 
        if (match_exact != null) {

          tagSet.setTagValue(INEXACT_MATCH, match_exact);

        } else {

          tagSet.setTagValue(INEXACT_MATCH, "(x == y)");

        }

      }

    }

    public void writeFileHeader(  DataObject o ) throws Exception {
	defaultWriteFileHeader( o );

        Vector ancestors = o.getAncestors();
	if ( null != ancestors ) {
	    for ( int i = 0; i < ancestors.size(); i++ ) {
		DataObject p = (DataObject) ancestors.elementAt( i );
		if ( null != p ) {
		    Vector members = p.getAttributes();
		    for ( int j = 0; j < members.size(); j++ ) {
			Attribute attr = (Attribute) members.elementAt( j );
			memberPrep( attr );
			writeMemberStuff(   attr );
		    }
		}
	    }
	}
    }

    public void writeMemberStuff( Attribute  a ) throws Exception {
	if ( ! a.isUsedForQuery() ) return;
	defaultWriteMemberStuff( a );
    }

    public void writeFileFooter(  DataObject o ) throws Exception {
	defaultWriteFileFooter( o );
    }
}
