/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: SourceGenerator.java,v 1.23 1999/07/23 22:02:06 jay Exp $
 *-----------------------------------------------------------------------------
 */




package dods.builder.sourceGenerators;

import dods.data.*;
import dods.util.*;
import dods.exceptions.*;
import dods.builder.engine.*;

import com.lutris.util.Config;
import com.lutris.appserver.server.sql.*;

import java.io.*;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Vector;
import java.text.*;

abstract public class SourceGenerator {

  /*
  String    templateDir;
  TemplateReader  templateFileHeader;
  TemplateReader  templateFileFooter;
  TemplateReader  templateMemberStuff;
  */

  protected GeneratorInfo  info;
  TemplateFilter    filterFileHeader;
  TemplateFilter    filterFileFooter;
  TemplateFilter    filterMemberStuff;

  protected TagSet    tagSet;
  final String TEMPLATE_BEGIN_REMARK         = "TEMPLATE_BEGIN_REMARK";
  final String TEMPLATE_END_REMARK           = "TEMPLATE_END_REMARK";
  final String AUTHOR        = "AUTHOR";
  final String PROJECT_NAME      = "PROJECT_NAME";
  final String CATEGORY      = "CATEGORY";
  final String SUFFIX        = "SUFFIX";
  final String PACKAGE      = "PACKAGE";
  final String CLASS_NAME      = "CLASS_NAME";
  final String EXTENDS      = "EXTENDS";
  final String TABLE_NAME      = "TABLE_NAME";
  final String DB_VENDOR      = "DB_VENDOR";
  final String FILE_NAME      = "FILE_NAME";
  final protected String MEMBER_NAME      = "MEMBER_NAME";
  final protected String UPPER_MEMBER_NAME    = "UPPER_MEMBER_NAME";
  final String MEMBER_JAVA_TYPE    = "MEMBER_JAVA_TYPE";
  final protected String MEMBER_JDBC_TYPE_SAFE_NAME  = "MEMBER_JDBC_TYPE_SAFE_NAME";
  final String MEMBER_JAVA_DEFAULT_VALUE  = "MEMBER_JAVA_DEFAULT_VALUE";
  final protected String MEMBER_JDBC_TYPE    = "MEMBER_JDBC_TYPE";
  final protected String MEMBER_DB_TYPE  = "MEMBER_DB_TYPE";
  //final String MEMBER_DB_DEFAULT_VALUE  = "MEMBER_DB_DEFAULT_VALUE";
  final String MEMBER_JAVADOC      = "MEMBER_JAVADOC";
  final protected String DO_CLASS_NAME        = "DO_CLASS_NAME";
  final protected String BDO_CLASS_NAME        = "BDO_CLASS_NAME";
  final protected String QUERY_CLASS_NAME     = "QUERY_CLASS_NAME";
  final protected String BASE_NAME     = "BASE_NAME";

  final protected String COLUMN_NAME_OBJECTID     = "COLUMN_NAME_OBJECTID";
  final protected String COLUMN_NAME_VERSION     = "COLUMN_NAME_VERSION";

  final protected String START_CONCRETE       = "START_CONCRETE";
  final protected String STOP_CONCRETE        = "STOP_CONCRETE";

  final protected String START_CACHING  = "START_CACHING";
  final protected String STOP_CACHING    = "STOP_CACHING";
  final protected String START_FULL_CACHING  = "START_FULL_CACHING";
  final protected String STOP_FULL_CACHING  = "STOP_FULL_CACHING";
  final protected String TRUE_IF_PARTIAL_CACHING= "TRUE_IF_PARTIAL_CACHING";
  final protected String TRUE_IF_FULL_CACHING  = "TRUE_IF_FULL_CACHING";
  final protected String TRUE_IF_NO_CACHING  = "TRUE_IF_NO_CACHING";

  //static final String STRING_QUOTE_CHARACTER_SECTION = "StringQuoteCharacter";
  //static final String STRING_QUOTE_CHARACTER       = "STRING_QUOTE_CHARACTER";

  //protected String stringQuoteCharacter = null;

  static final String STRING_MATCH_SECTION = "StringMatch";
  static final String STRING_MATCH       = "STRING_MATCH";
  protected String stringMatch = null;

  static final String STRING_WILDCARD_SECTION = "StringWildcard";
  static final String STRING_WILDCARD       = "STRING_WILDCARD";
  protected String stringWildcard = null;

  static final String DB_COMMENT_BEGIN  = "CommentStart";
  static final String DB_COMMENT_END    = "CommentEnd";
  protected static final String BEGIN_COMMENT    = "BEGIN_COMMENT";
  protected static final String END_COMMENT    = "END_COMMENT";

  static final String BEGIN_IS_REF    = "BEGIN_IS_REF";
  static final String END_IS_REF    = "END_IS_REF";

  protected String dbCommentBegin = null;
  protected String dbCommentEnd = null;

  protected String    db_vendor;

  protected DataObject  currentObj;

  protected SourceWriter  output;

  static final String FILE_HEADER_TEMPLATE_NAME  = "FileHeader";
  static final String FILE_FOOTER_TEMPLATE_NAME  = "FileFooter";
  static final String MEMBER_STUFF_TEMPLATE_NAME = "MemberStuff";

  static final String DATABASE_SECTION  = "Database";
  //static final String OID_DB_SECTION    = "OidDbType";
  //static final String INT_DB_SECTION    = "IntegerType";
  //static final String BIT_DB_SECTION    = "BitType";
  //static final String STR_DB_SECTION    = "StringType";
  static final String ON_DEL_SECTION    = "OnCascadeDelete";
  //static final String OID_JDBC_SECTION  = "OidJdbcType";
  static final String OID_DB_TYPE    = "OID_DB_TYPE";
  static final String OID_JDBC_TYPE    = "OID_JDBCB_TYPE";
  static final String INT_DB_TYPE    = "INT_DB_TYPE";

  /* Builder.generateSQLcommands() uses oidDbType */
  public    static String oidDbType = null;

  protected        String oidJdbcType = null;
  protected        String intDbType = null;
  //protected        String bitDbType = null;
  //protected        String strDbType = null;
  protected boolean cascadeDelete = false;


  // Holds the DODS types and assorted information
  //private DODSTypes dodsTypes = DODSTypes.getInstance();
  private DODSTypes dodsTypes = null;


/**
 * Creates a Template filter
 *
 * @param templateKey The file to create filter from
 */
  protected TemplateFilter createFilter( String templateKey ) throws Exception {

    TemplateReader reader = info.getTemplateReader( templateKey );

    // DBG
/*
    try {
      reader.mark( 100000 );
      reader.reset();
    } catch ( Exception e ) { 
      Builder.print("CANNOT RESET READER: " + e.getMessage() );
      System.err.println("CANNOT RESET READER: " + e.getMessage() );
    }
*/

    // DBG
    // System.err.println("reader=" + reader );

    return new TemplateFilter( reader, output );

  }


  static private Hashtable fixedDefaults = null;


/**
 * Resets the 'fixed' defaults hashtable
 */
  static public void reset() {
    fixedDefaults = new Hashtable();
  }


/**
 * Adds tag values to the <CODE>TagSet</CODE>.
 * A subclass of <CODE>SourceGenerator</CODE> can override this method
 * to set additional tag values.  
 * An overrided <CODE>objectPrep()</CODE>
 * should invoke <CODE>super.objectPrep()</CODE>
 * to get the basic tag settings performed by this method.
 * A subclass of <CODE>SourceGenerator</CODE>
 * will call <CODE>objectPrep()</CODE>
 * and then call its implementation of one of the following methods:
 *    <CODE>
 *    writeFileHeader
 *    writeFileFooter
 *    writeMemberStuff
 *    </CODE>
 * These methods should invoke
 *    <CODE>TemplateFilter.process( tagSet )</CODE>
 * which will create the output source file.
 *
 * @param o
 *   DataObject for which source code will be generated by
 *    <CODE>writeFileHeader</CODE> and
 *    <CODE>writeFileFooter</CODE>
 * @exception Exception
 *   If an unrecoverable error occurs.
 *
 * @author    Jay Gunter
 * @since     LBS1.8
 * @version   $Revision: 1.23 $
 *
 */
  public void objectPrep(  GeneratorInfo  sgi, DODSProject  project, DataObject  o ) throws Exception {

    if (dodsTypes == null)
      dodsTypes = DODSTypes.getInstance();

    // DBG
    /* System.err.println("SUPER objectPrep o.getName()="+o.getName()); */

    info = sgi;
    currentObj = o;
    String flavor = info.outputFileFlavor;
    String suffix = info.outputFileSuffix;

    tagSet = new TagSet();
    tagSet.setTagValue( TEMPLATE_BEGIN_REMARK, TemplateFilter.skipStart );
    tagSet.setTagValue( TEMPLATE_END_REMARK,   TemplateFilter.skipStop );
    tagSet.setTagValue( CATEGORY,  flavor );
    tagSet.setTagValue( SUFFIX,    suffix );
    tagSet.setTagValue( BASE_NAME, o.getName() );
    tagSet.setTagValue( CLASS_NAME, getOutputClassName( o, flavor ) );
    DataObject parentClass = o.getExtensionOf();

    // The column names of the object id & version columns in each table
    tagSet.setTagValue(COLUMN_NAME_OBJECTID, CoreDO.NAME_OBJECTID);
    tagSet.setTagValue(COLUMN_NAME_VERSION, CoreDO.NAME_VERSION);

    /*
    System.err.println("o.isConcrete()="+o.isConcrete());
    System.err.println("o.isCaching()="+o.isCaching());
    System.err.println("o.isFullCaching()="+o.isFullCaching());
    */

    tagSet.setTagValue( TRUE_IF_PARTIAL_CACHING, "false" );
    tagSet.setTagValue( TRUE_IF_FULL_CACHING,    "false" );
    tagSet.setTagValue( TRUE_IF_NO_CACHING,      "true" );

    if ( o.isCaching() && o.isConcrete() ) {

      tagSet.setTagValue( TRUE_IF_NO_CACHING,      "false" );
      tagSet.setTagValue( TRUE_IF_PARTIAL_CACHING, "true" );
      tagSet.setTagValue( START_CACHING,     "" );
      tagSet.setTagValue( STOP_CACHING,      "" );

    } else {

      tagSet.setTagValue( START_CACHING,     TemplateFilter.skipStart );
      tagSet.setTagValue( STOP_CACHING,      TemplateFilter.skipStop );

    }

    if ( o.isFullCaching() && o.isConcrete() ) {

      tagSet.setTagValue( TRUE_IF_NO_CACHING,      "false" );
      tagSet.setTagValue( TRUE_IF_PARTIAL_CACHING, "false" );
      tagSet.setTagValue( TRUE_IF_FULL_CACHING,    "true" );
      tagSet.setTagValue( START_FULL_CACHING,     "" );
      tagSet.setTagValue( STOP_FULL_CACHING,      "" );

    } else {

      tagSet.setTagValue( START_FULL_CACHING,     TemplateFilter.skipStart );
      tagSet.setTagValue( STOP_FULL_CACHING,      TemplateFilter.skipStop );

    }

    if ( o.isConcrete() ) {

      /* System.err.println("concrete"); */
      tagSet.setTagValue( START_CONCRETE,     "" );
      tagSet.setTagValue( STOP_CONCRETE,      "" );

    } else {

      /* System.err.println("NOT concrete"); */
      tagSet.setTagValue( START_CONCRETE,     TemplateFilter.skipStart );

      tagSet.setTagValue( STOP_CONCRETE,      TemplateFilter.skipStop );
    }

    // If the Data Object does not derive from another Data Object,
    // and the OutputClassParent setting is not set for this SourceGenerator
    // then the output class does not need an "extends" clause.
    tagSet.setTagValue( EXTENDS, "" );
    if ( null != info.outputClassParent ) {
      // If a Data Object does not derive from another Data Object,
      // but an OutputClassParent is specified in the config file,
      // the extends clause names that parent class.
      tagSet.setTagValue( EXTENDS, "extends " + info.outputClassParent );
    } 

    if ( null != parentClass ) {
      // If a Data Object Y derives from another Data Object X,
      // the extends clause must reflect this.
      tagSet.setTagValue( EXTENDS, "extends " + parentClass.getAdjustedPackageName() + "." +
        getOutputClassName( parentClass, flavor ) );
    }

    db_vendor = project.getDatabase();
    tagSet.setTagValue( DB_VENDOR,    db_vendor );
    tagSet.setTagValue( TABLE_NAME,   getTableName( o ) );
    tagSet.setTagValue( PACKAGE,      o.getAdjustedPackageName() );
    tagSet.setTagValue( AUTHOR,       System.getProperty( "user.name" ) );
    tagSet.setTagValue( PROJECT_NAME, project.getName() );

    // FIX the strings "DO" and "Query" should really come from 
    // the dods.conf file.  These suffixes should not be hard-coded here.
    tagSet.setTagValue( DO_CLASS_NAME,  o.getName() + "DO" );
    tagSet.setTagValue( BDO_CLASS_NAME,  o.getName() + "BDO" );
    tagSet.setTagValue( QUERY_CLASS_NAME,  o.getName() + "Query" );

    Config db = (Config) info.config.getSection(  DATABASE_SECTION );
    //Config oidDbSec =   (Config) db.getSection(  OID_DB_SECTION );
    //Config oidJdbcSec = (Config) db.getSection(  OID_JDBC_SECTION );
    //Config intDbSec =   (Config) db.getSection(  INT_DB_SECTION );
    //Config bitDbSec =   (Config) db.getSection(  BIT_DB_SECTION );
    //Config strDbSec =   (Config) db.getSection(  STR_DB_SECTION );
    Config delDbSec =   (Config) db.getSection(  ON_DEL_SECTION );
    String vendor_key = db_vendor.replace( ' ', '_' );

    //Config quoteSec = (Config) db.getSection( STRING_QUOTE_CHARACTER_SECTION );  // see dods.conf
    //stringQuoteCharacter = quoteSec.getString(   vendor_key );
    //tagSet.setTagValue( STRING_QUOTE_CHARACTER, stringQuoteCharacter );

    Config matchSec = (Config) db.getSection( STRING_MATCH_SECTION );  // see dods.conf
    stringMatch = matchSec.getString(   vendor_key );
    tagSet.setTagValue( STRING_MATCH, stringMatch );

    Config wildcardSec = (Config) db.getSection( STRING_WILDCARD_SECTION );  // see dods.conf
    stringWildcard = wildcardSec.getString(   vendor_key );
    tagSet.setTagValue( STRING_WILDCARD, stringWildcard );

    Config commentSec = (Config) db.getSection( DB_COMMENT_BEGIN );
    tagSet.setTagValue( BEGIN_COMMENT, commentSec.getString( vendor_key ) );
    commentSec = (Config) db.getSection(        DB_COMMENT_END );
    tagSet.setTagValue( END_COMMENT,   commentSec.getString( vendor_key ) );

    //oidDbType =  oidDbSec.getString(  vendor_key );
    //oidJdbcType =  oidJdbcSec.getString(  vendor_key );
    oidDbType =  dodsTypes.getDBType("ObjectId");
    oidJdbcType =  dodsTypes.getJDBCType("ObjectId");
    intDbType =  dodsTypes.getDBType("Integer");
    //bitDbType =  bitDbSec.getString(  vendor_key );
    //strDbType =  strDbSec.getString(  vendor_key );
    tagSet.setTagValue( OID_DB_TYPE,   oidDbType );
    tagSet.setTagValue( OID_JDBC_TYPE, oidJdbcType );
    tagSet.setTagValue( INT_DB_TYPE,   intDbType );

    cascadeDelete =  false;
    String cascadingDeleteAllowed = delDbSec.getString( vendor_key );
    if ( null != cascadingDeleteAllowed && cascadingDeleteAllowed.equals( "true" ) )
      cascadeDelete = true;

    String outputFilename =  info.outputDir + File.separator + o.getName() + flavor + "." + suffix;

    Builder.print("Generating "+outputFilename );

    output = new SourceWriter( outputFilename );
    tagSet.setTagValue( FILE_NAME, fixFileName( outputFilename ) );
// DBG
// System.err.println("creating filter for: "+GeneratorInfo.WRITE_FILE_HEADER);
    filterFileHeader  = createFilter( GeneratorInfo.WRITE_FILE_HEADER );
    filterFileFooter  = createFilter( GeneratorInfo.WRITE_FILE_FOOTER );
    filterMemberStuff = createFilter( GeneratorInfo.WRITE_MEMBER_STUFF );

  }

/**
 * This method is used to fix this problem:
 *
 * On Windows platforms, it is common to get filenames like
 *  C:/usr/...
 *    except the forward slashes are backslashes. 
 *    When javacc sees the pattern \ followed by u
 *    it tries to interpret it as a Unicode character
 *    (even if it occurs in a comment)
 *    and because it was not valid Unicode, you got an error.
 *
 * @param s The filename string to fix
 * @return The modified filename
 */
  private String fixFileName( String s ) {
    return s.replace( '\\', '/' );
  }

/**
 *
 * @param o
 * @param flavor
 * @return The output class name
 */
  private String getOutputClassName( DataObject o, String flavor ) {
    return o.getName() + flavor;
  }


  static Hashtable tableNames = new Hashtable();

/**
 * Gets the table name for this Data Object
 *
 * @param o The Data Object
 * @return see comment
 */
  private String getTableName( DataObject o ) throws Exception {

    String name  = o.getDBTableName();
    return name;

  }


/**
 * Prepares the 'member' for code generation
 *
 * @param a The Attribute or 'member' currently worked on
 */
  public void memberPrep(  Attribute a ) throws Exception {

    if (dodsTypes == null)
      dodsTypes = DODSTypes.getInstance();

    String dodsType = a.getDODSType();
    String name = a.getName();
    String type = dodsTypes.getJavaType(dodsType);

// DBG
// System.err.println("Name is: <"+name+">, type is: <"+type+">, dodsType: <"+dodsType+">");

    tagSet.setTagValue( MEMBER_NAME,       name );
    String firstChar = name.substring( 0, 1 ).toUpperCase();
    name = firstChar + name.substring( 1 );
    tagSet.setTagValue( UPPER_MEMBER_NAME,    name );

    if ( a.isObjectRef()) {
      type = dodsTypes.getJavaType("ObjectId");
      tagSet.setTagValue( MEMBER_JAVA_TYPE,    type);
      tagSet.setTagValue( MEMBER_JAVA_DEFAULT_VALUE,  "null" );
      tagSet.setTagValue( MEMBER_JDBC_TYPE,    dodsTypes.getJDBCType("ObjectId" ) );
      tagSet.setTagValue( MEMBER_DB_TYPE,    oidDbType );
    } else if (a.isRefConstraint()) {
      tagSet.setTagValue( MEMBER_JAVA_TYPE,    dodsType);
      tagSet.setTagValue( MEMBER_JAVA_DEFAULT_VALUE,  "null" );
      tagSet.setTagValue( MEMBER_JDBC_TYPE,    dodsTypes.getJDBCType("ObjectId" ) );
      tagSet.setTagValue( MEMBER_DB_TYPE,    oidDbType );
    } else {
      tagSet.setTagValue( MEMBER_JAVA_TYPE,    type );
      tagSet.setTagValue( MEMBER_JAVA_DEFAULT_VALUE,  fixDefault( a ) );
      tagSet.setTagValue( MEMBER_JDBC_TYPE,    dodsTypes.getJDBCType(dodsType ) );
      tagSet.setTagValue( MEMBER_DB_TYPE, dodsTypes.getDBType(dodsType) );
    }


    tagSet.setTagValue( BEGIN_IS_REF, beginNoCodeIf( ! a.isObjectRef() ));
    tagSet.setTagValue( END_IS_REF,   endNoCodeIf(   ! a.isObjectRef() ));

    // Chris Ryan's comment: I think the following tag isn't used -
    // can it be deleted?  I believe in SQL/templates, DEF_VALUE is
    // used instead of this: 
    // tagSet.setTagValue( MEMBER_DB_DEFAULT_VALUE,  a.getInitialValue() );

    tagSet.setTagValue( MEMBER_JAVADOC,    a.getJavadoc() );

  }


/**
 * Fixes the default java value
 *
 * @param a The attribute to fix
 * @return The code containing the default java value for this attribute
 */
  private String fixDefault( Attribute a ) {

    if (dodsTypes == null)
      dodsTypes = DODSTypes.getInstance();

    String dodsType = a.getDODSType();
    String type  = dodsTypes.getJavaType(dodsType);
    String value = a.getInitialJavaValue();

    String javaInitial = dodsTypes.getJavaInitial(dodsType);

    // is there any code for the java initial value?
    if (javaInitial != null) {

      // replace the special variable where needed
      value = Util.replace(javaInitial, DODSTypes.JAVA_VALUE, value);
      value = Util.replace(value, DODSTypes.JAVA_TYPE, type);

    }

    // the java value, potentially modified
    return value;

  }


// no longer used...
    private java.util.Date str2date( String s ) {
  String[] formats = { 
      "yyyy/MM/dd H:mm:ss a",    "yyyy/MM/dd K:mm:ss",
      "d/M/yyyy H:mm:ss a",    "d/M/yyyy K:mm:ss",
      "MM/dd/yyyy H:mm:ss a",    "MM/dd/yyyy K:mm:ss",
      "yyyy-MM-dd H:mm:ss a",    "yyyy MM dd H:mm:ss a",
      "yyyy-MM-dd H:mm:ss",    "yyyy MM dd H:mm:ss",
      "yyyy-MM-dd H:mm a",    "yyyy MM dd H:mm a",
      "yyyy-MM-dd H:mm",    "yyyy MM dd H:mm",
      "yyyy/MM/dd",      "dd/MM/yyyy",
      "yyyy-MM-dd",      "yyyy MM dd",
      "MMM dd yyyy h:mm:ss a",  "MMM dd, yyyy h:mm:ss a",
      "MMM dd yyyy h:mm:ss",    "MMM dd, yyyy h:mm:ss",
      "MMM dd yyyy h:mm a",    "MMM dd, yyyy h:mm a",
      "MMM dd yyyy h:mm",    "MMM dd, yyyy h:mm" };
  java.util.Date ret = null;
  for ( int i = 0; i < formats.length; i++ ) {
      SimpleDateFormat df = new SimpleDateFormat( formats[ i ] );
      try { 
    ret = df.parse( s );
    break;
      } catch ( ParseException e ) { 
      }
  }
  return ret;
    }


/**
 * If the type passed is an array, then the
 * returned name is fixed to remove the '[]'
 * and add an 's' (simple pluralise).
 *
 * @param type  The type to check/convert
 * @return see comment
 */
  private String convertArrayType( String type ) {

    String at = type;
    int i = type.indexOf( "[]" );
    if ( -1 != i )
      at = type.substring( 0, i ) + "s";
    return at;

  }


/**
 * Makes a safe Java name from the type.  The
 * '.' character is replace with an underscore.
 * Also a check is made to determine if the
 * type is an array.
 *
 * @param type  The Java type to convert
 * @return see comment
 */
  private String JavaType( String type ) {

    String jt = type.replace( '.', '_' );
    return convertArrayType( jt );

  }

// no longer used
/*
    private String JdbcType( String javaType ) {
  String type = javaType;
  int dot = type.lastIndexOf( '.' );
  if ( -1 != dot ) 
    type = type.substring( dot + 1 );
  String firstChar = type.substring( 0, 1 ).toUpperCase();
  return convertArrayType( firstChar + type.substring( 1 ) );
    }
*/

    public void adjustOutputFile() throws Exception {
  if ( null == output ) {
      // The SourceGenerator subclass decided not to generate
      // an output file.  For example, there is no 
      // SQL output file needed for an abstract class.
      return;
  }
  output.close();

  String[] argv;
        int status;

    }



    public void defaultWriteFileHeader(  DataObject o ) throws Exception {
/* System.err.println("*** defaultWriteFileHeader"); */
  filterFileHeader.reset();
  filterFileHeader.process( tagSet );
  filterFileHeader.close();
    }
    public void defaultWriteFileFooter(  DataObject o ) throws Exception {
/* System.err.println("*** defaultWriteFileFooter"); */
  filterFileFooter.reset();
  filterFileFooter.process( tagSet );
  filterFileFooter.close();
  filterMemberStuff.close();
    }
    public void defaultWriteMemberStuff( Attribute  a ) throws Exception {
/* System.err.println("*** defaultWriteMemberStuff"); */
  filterMemberStuff.reset();
  filterMemberStuff.process( tagSet );
/*   filterMemberStuff.close(); */
    }

    abstract public void writeFileHeader(  DataObject o ) throws Exception;
    abstract public void writeFileFooter(  DataObject o ) throws Exception;
    abstract public void writeMemberStuff( Attribute  a ) throws Exception;

    /*
    public void openTemplates( String templateDirFromConfig ) {
  templateDir = templateDirFromConfig;
  File templateFile;
  templateFile = File( templateDir + "/" + FILE_HEADER_TEMPLATE_NAME );
  templateFileHeader = new TemplateReader( templateFile.getPath() );
  templateFile = File( templateDir + "/" + FILE_FOOTER_TEMPLATE_NAME );
  templateFileFooter = new TemplateReader( templateFile.getPath() );
  templateFile = File( templateDir + "/" + MEMBER_STUFF_TEMPLATE_NAME );
  templateMemberStuff = new TemplateReader( templateFile.getPath() );
    }
    */


/** Array impl #2 (see $qdb/todo)
 *  If DataObject XXX has an array Attribute AAA,
 *  AAA is converted to a new DataObject that refers to XXX.
 *  That new DataObject is processed normally by the Builder,
 *  generating all necessary .java and schema files.
 */
  public static DataObject handleArray( DataObject o, Attribute a ) {

    String tableName = o.getName() + "_" + a.getName();
    DataObject ao   = new DataObject( tableName );
    ao.setAbstract( false );
    ao.setCaching( false );

    try {
      ao.setDBTableName( tableName );
    } catch ( DataObjectNameException e ) { 
      Builder.print( "ERROR!! DataObjectNameException for tableName="+tableName);
    }

    try {
      ao.setDBTableType( DataObject.CONCRETE );
    } catch ( DataObjectTypeException e ) { 
      Builder.print( "ERROR!! DataObjectTypeException for tableName="+tableName);
    }

    ao.setFinal( true );
    ao.setFullCaching( false );
    ao.setIndex( false );
    ao.setLazyLoading( false );

    try {
      ao.setPackageName( o.getAdjustedPackageName() );
    } catch ( DataObjectNameException e ) { 
      Builder.print( "ERROR!! DataObjectNameException for tableName="+tableName);
    }

    Attribute index = new Attribute( "index" ); 
    index.setArray( false );
    index.setCanBeNull( false );
    index.setConstant( false );
    index.setDODSType( "Integer" );
    index.setIndex( false );
    index.setJavadoc( "/* index for array table " + tableName + " */" );
    index.setObjectRef( false );
    index.setRefConstraint( false );
    index.setUsedForQuery( false );
    ao.addAttribute( index );

    Attribute  value = new Attribute( "value" );
    value.setArray( false );
    value.setCanBeNull( true );
    value.setConstant( false );
    if ( -1 == o.getName().indexOf( "DO" ) ) 
      value.setDODSType( "ObjectId" );
    else
      value.setDODSType( a.getDODSType() );
    value.setIndex( false );
    value.setJavadoc( a.getJavadoc() );
    value.setObjectRef( a.isObjectRef() );
    value.setRefConstraint( false );
    value.setUsedForQuery( false );
    ao.addAttribute( value );

    return ao;

  }

    /**
     * If condition is true, return tag to temporarily suspend code generation.
     * This allows conditionally processed sections within a .template file.
     */
    public String beginNoCodeIf( boolean condition ) {
  return condition ? TemplateFilter.skipStart : "";
    }

    /**
     * If condition is true, return tag to resume code generation.
     */
    public String endNoCodeIf( boolean condition ) {
  return condition ? TemplateFilter.skipStop : "";
    }

}
