/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: MySqlConnectionAllocator.java,v 1.6 1999/02/12 12:26:19 andy Exp $
 *-----------------------------------------------------------------------------
 */





package	com.lutris.appserver.server.sql.mysql;

import com.lutris.appserver.server.sql.standard.*;
import com.lutris.appserver.server.sql.*;
import com.lutris.util.Config;
import com.lutris.util.ConfigException;
import java.sql.*;

/**
 * A MySql database connection allocator used by the MySql logical
 * database.
 *
 * @see		StandardConnectionAllocator
 * @author	Paul Morgan
 * @since	LBS1.8
 * @version	$Revision: 1.6 $
 */
public class MySqlConnectionAllocator extends StandardConnectionAllocator {

    /**
     * Creates an MySql database connection allocator.
     *
     * @see StandardConnectionAllocator#StandardConnectionAllocator
     * @param logicalDatabase
     *   the logical MySql database for which connections are pooled.
     * @param connectionConfig
     *   configuration parameters for the database connections.  There
     *   are no Informix specific configuration data.
     * @exception ConfigException
     *   If bad configuration information is given in the config file.
     */
    protected MySqlConnectionAllocator(MySqlLogicalDatabase logicalDatabase,
                                        Config connectionConfig)
        throws ConfigException {

        super(logicalDatabase, connectionConfig);
    }


    /**
     * Creates a new MySql connection to be added to the connection pool.
     * 
     * @exception SQLException If a SQL error occures.
     */
    protected DBConnection createConnection ()
        throws SQLException {

        return new MySqlDBConnection (this, url, user, password,
                                       maxPreparedStatements,
                                       sqlLogging, generation);
    }
}










