/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: MySqlDBConnection.java,v 1.6 1999/02/12 12:26:20 andy Exp $
 *-----------------------------------------------------------------------------
 */





package	com.lutris.appserver.server.sql.mysql;

import com.lutris.appserver.server.sql.standard.*;
import com.lutris.appserver.server.sql.*;
import java.sql.*;


/**
 * A MySql database connection object used by the MySql logical database.
 *
 * @see		StandardDBConnection
 * @author	Paul Morgan
 * @since	LBS1.8
 * @version	$Revision: 1.6 $
 */
public class MySqlDBConnection extends StandardDBConnection {

    /**
     * Create a connection to a JDBC database with oracle specific symantics.
     *
     * @see StandardDBConnection#StandardDBConnection
     * @param url
     *   JDBC URL of database.
     * @param user
     *   SQL user name.
     * @param password
     *   SQL password.
     * @param maxPreparedStatements 
     *   maximum number of preparse statements. a value of less than zero
     *   queries JDBC for the value.
     * @param connectionAllocator
     *   the connection allocator that this connection belongs to.
     * @param logging
     *   specifying <CODE>true</CODE> enables SQL logging.
     * @param generation
     *   a generation number used to drop old connection when they
     *   are released.
     * @exception SQLException
     *   If a connection can't be established.
     */
    protected MySqlDBConnection (MySqlConnectionAllocator connectionAllocator,
				  String url, String user, String password,
                                  int maxPreparedStatements,
				  boolean logging, int generation)
        throws SQLException {

        super(connectionAllocator, url, user, password, maxPreparedStatements,
              logging, generation);
    }


    /*
     * Here we override the following methods since MySql does not
     * support Transaction
     */
    /**
     * Autocommit on/off.
     *
     * @param on - False to disable auto commit mode.  True to enable.
     * @exception java.sql.SQLException If a database access error occurs. 
     */
    public void setAutoCommit(boolean on) throws SQLException {
        validate();
        logDebug("set auto commit: " + on);
	/* SBDT
        connection.setAutoCommit(on);
	*/
    }

    /**
     * Commit a transaction.
     *
     * @exception java.sql.SQLException If a database access error occurs. 
     */
    public void commit() throws SQLException {
        validate();
        logDebug("commit");
	/* SBDT
        connection.commit();
	*/
    }


    /**
     * Rollback a transaction.  Should only be used when
     * <A HREF=#setAutoCommit>auto-commit</A> mode
     * has been disabled.
     *
     * @exception java.sql.SQLException If a database access error occurs. 
     */
    public void rollback() throws SQLException {
        validate();
        logDebug("rollback");
	/* SBDT
        connection.rollback();
	*/
    }

}










