/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: MySqlLogicalDatabase.java,v 1.5 1999/02/12 12:26:20 andy Exp $
 *-----------------------------------------------------------------------------
 */





package	com.lutris.appserver.server.sql.mysql;

import com.lutris.appserver.server.sql.standard.*;
import com.lutris.appserver.server.sql.*;
import com.lutris.util.*;
import java.util.Date;	// Must be before java.sql.Date
import java.sql.*;


/**
 * Represents a logical database with MySql specific capabilities.
 *
 * @see		StandardLogicalDatabase
 * @author	Paul Morgan
 * @since	LBS1.8
 * @version	$Revision: 1.5 $
 */
public class MySqlLogicalDatabase extends StandardLogicalDatabase {

    /**
     * Creates and configures a single logical database with mysql
     * specific enhancements.
     *
     * @see StandardLogicalDatabase#StandardLogicalDatabase
     * @param dbName
     *   The logical name of the database.
     * @param dbConfig
     *   The configuration object for this logical database.
     * @exception ConfigException
     *   If there is an error in the configuration file.
     * @exception SQLException
     *   If a SQL error occurs.
     */
    public MySqlLogicalDatabase (String dbName, Config dbConfig)
        throws ConfigException, SQLException {
	    
	    super.init(dbName, dbConfig);
    }
    

    /**
     * Default constructor to configure a single logical database.
     * Note that the <CODE>init()</CODE> method must be called to
     * configure the database.
     * 
     * @see StandardLogicalDatabase#StandardLogicalDatabase
     */
    public MySqlLogicalDatabase () {
        super();
    }
    

    /**
     * Return the connection allocator.
     *
     * @param connectionConfig
     *   The configuration object for the connection allocator.
     * @exception ConfigException
     *   If there is an error in the configuration file.
     * @return
     *   The connection allocator.
     */
    public ConnectionAllocator
        loadConnectionAllocator(Config connectionConfig)
        throws ConfigException {

	return (ConnectionAllocator)
	    new MySqlConnectionAllocator (this, connectionConfig);
    }


    /**
     * Return the object id allocator.
     *
     * @param objIdConfig
     *   The configuration object for the object id allocator.
     * @exception ConfigException
     *   If there is an error in the configuration file.
     * @return
     *   The object id allocator.
     */
    public ObjectIdAllocator
        loadObjectIdAllocator (Config objIdConfig)
        throws ConfigException {

	return (ObjectIdAllocator)
	    new MySqlObjectIdAllocator (this, objIdConfig);
    }


    /**
     * Return a transaction for use on this mysql logical database.
     *
     * @return The transaction object.
     * @exception SQLException
     *   if a SQL error occurs.
     */
    public DBTransaction createTransaction ()
	throws SQLException {

	return new MySqlDBTransaction
	    ((MySqlDBConnection)connectionAllocator.allocate());
    }


    /**
     * Return a query for use on this logical database.
     *
     * @return The query object.
     * @exception SQLException
     *   if a SQL error occurs.
     */
    public DBQuery createQuery ()
	throws SQLException {

	return new MySqlDBQuery (connectionAllocator.allocate());
    }


    /**
     * Return a description of the logical database type.
     *
     * @return The type.
     */
    public String getType () {
	return "MySql JDBC";
    }
}






