/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: DODSDirectoryChooser.java,v 1.6 1999/02/12 12:18:06 andy Exp $
 *-----------------------------------------------------------------------------
 */



package dods.gui;

import dods.data.*;
import javax.swing.JFileChooser;
import javax.swing.filechooser.*;
import java.io.File;


/**
 * This class is used to choose the directory to build everything in.
 */
public class DODSDirectoryChooser extends JFileChooser {

    public DODSDirectoryChooser( File dir ) {
	super( getDirHead( dir ) );
/* System.err.println("dir ="+dir ); */
	setSelectedFile( dir );

	this.setDialogTitle("Destroy and Create New DODS Directory");

	/* If we do this, the "approve" button will
	 * not cause the window to close.
	 * So instead, we use a FileFilter.
	 */
        //setFileSelectionMode(DIRECTORIES_ONLY);

        setFileFilter(new InternalFilter());
    }

    private static String getDirHead( File dir ) {
	if ( null == dir ) return null;
	String s = dir.getParent();
/* System.err.println("s ="+s ); */
	return s;
    }


    public boolean isDirectorySelectionEnabled() {
	return(true);
    }


    class InternalFilter extends FileFilter {
        public boolean accept(File f) {
	    if(f != null) {
	        if(f.isDirectory()) {
                    return true;
                }
	    }
	    return false;
	}
        public String getDescription() {
            return "Directory";
        }
    }
}
