/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: DODSFileChooser.java,v 1.7 1999/03/09 09:23:55 andy Exp $
 *-----------------------------------------------------------------------------
 * Constructor to set working directory added by David Wood (dwood@plugged.net.au)
 */



package dods.gui;

import dods.data.*;
import javax.swing.ImageIcon;
import javax.swing.Icon;
import javax.swing.JFileChooser;
import javax.swing.filechooser.*;
import java.io.File;
/**
 * This class is used to save and open files for DODS
 */
public class DODSFileChooser extends JFileChooser {

    public DODSFileChooser(File file) {
        super(file);
        setFileFilter(new InternalFilter());
        //setFileView(new InternalFileView());
        setFileSelectionMode(FILES_ONLY);
    }
    public DODSFileChooser() {
        this( null );
    }
    class InternalFilter extends FileFilter {
        String extension;
        InternalFilter() {
	    extension = "doa";
        }
        public boolean accept(File f) {
	    if(f != null) {
	        if(f.isDirectory()) {
                    return true;
                }
                String exten = getExtension(f);
                if(exten != null && exten.equals("doa")) {
                    return true;
		}
	    }
	    return false;
	}
        public String getDescription() {
            return "DODS files (*.doa)";
        }
        public String getExtension(File f) {
            if(f != null) {
	        String filename = f.getName();
	        int i = filename.lastIndexOf('.');

                if(i>0 && i<filename.length()-1) {
		    return filename.substring(i+1).toLowerCase();
	        }
	    }
	return null;
        }
    }
    class InternalFileView extends FileView {
        InternalFileView() {}
        public Icon getIcon(File f) {
  	    if(f.isDirectory()) {
	        return Icons.getIcon("folder.gif");
	    }
            return Icons.getIcon("object_ref.gif");
        }
        public String getName(File f) {
	    String fileName = null;
	    if(f != null) {
		fileName = f.getName();
		if (fileName.equals("")) {
		    fileName = f.getPath();
		}
	    }
	    return fileName;
	}


	public String getDescription(File f) {
	    return f.getName();
	}
        public String getTypeDescription(File f) {
	    if (f.isDirectory()) {
		return "";
	    } else {
		return "";
	    }
	}
        public Boolean isTraversable(File f) {
	    if (f.isDirectory()) {
		return Boolean.TRUE;
	    } else {
		return Boolean.FALSE;
	    }
	}
    }
}


