/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: DataObjectApplication.java,v 1.24 1999/07/23 22:02:06 jay Exp $
 *-----------------------------------------------------------------------------
 * Changes to support DODS Types made by David Wood (dwood@pisoftware.com)
 */



   package dods.gui;

   import dods.gui.importer.*;
   import dods.gui.tree.*;
   import dods.gui.table.*;
   import dods.gui.editors.*;
   import dods.data.*;
   import dods.exceptions.*;

   import javax.swing.*;
   /*
   import javax.swing.JFrame;
   import javax.swing.JTextArea;
   import javax.swing.JPanel;
   import javax.swing.JTextField;
   import javax.swing.JTree;
   import javax.swing.JTable;
   import javax.swing.JMenuItem;
   import javax.swing.JCheckBoxMenuItem;
   import javax.swing.JDialog;
   import javax.swing.JMenu;
   import javax.swing.JLabel;
   import javax.swing.ButtonGroup;
   import javax.swing.JScrollPane;
   import javax.swing.ImageIcon;
   import javax.swing.UIManager;
   import javax.swing.SwingUtilities;
   import javax.swing.FocusManager;
   import javax.swing.JSplitPane;
   import javax.swing.JMenuBar;
   import javax.swing.JToolBar;
   import javax.swing.JButton;
   import javax.swing.JOptionPane;
   */
   import javax.swing.border.EmptyBorder;
   import javax.swing.border.LineBorder;
   import javax.swing.table.TableColumnModel;
   import javax.swing.table.TableColumn;
   import javax.swing.table.TableCellRenderer;
   import javax.swing.table.DefaultTableCellRenderer;
   import javax.swing.tree.TreePath;
   import javax.swing.tree.TreeSelectionModel;

   import java.beans.PropertyChangeListener; 
   import java.beans.PropertyChangeEvent; 

   import java.awt.*;
   import java.awt.event.*;
   /*
   import java.awt.Color;
   import java.awt.Cursor;
   import java.awt.FileDialog;
   import java.awt.BorderLayout;
   import java.awt.GridBagLayout;
   import java.awt.GridBagConstraints;
   import java.awt.FlowLayout;
   import java.awt.Font;
   import java.awt.Insets;
   import java.awt.Dimension;
   import java.awt.Component;
   import java.awt.Point;
   import java.awt.event.WindowAdapter;
   import java.awt.event.ActionListener;
   import java.awt.event.ComponentListener;
   import java.awt.event.ItemListener;
   import java.awt.event.ActionEvent;
   import java.awt.event.WindowEvent;
   import java.awt.event.ItemEvent;
   import java.awt.event.ComponentEvent;
   */
   import java.io.File;
   import java.io.FileOutputStream;
   import java.io.ObjectOutputStream;
   import java.io.FileInputStream;
   import java.io.ObjectInputStream;
   import java.util.Vector;

   import dods.builder.engine.Builder;

/**
 * This class is the heart of the Data Object Design Studio.  This frame
 * contains all of the panels and menus that make up DODS.
 *
 * @author  Philip Martin
 * @version $Revision: 1.24 $
 */
   public final class DataObjectApplication extends JFrame {
    /**
     * This constructor initializes all of the visible parts of the Frame.
     */
      public DataObjectApplication() {
        //Setting the look and feel.
        /*
        String cn="javax.swing.plaf.windows.WindowsLookAndFeel";
        try {
            UIManager.setLookAndFeel(cn);
            SwingUtilities.updateComponentTreeUI(this);
        }
        catch(Exception e) {
            System.out.println(e.toString());
        }
        */
      
        //General Frame initial values.
      
         setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
         getContentPane().setLayout(new BorderLayout(0,0));
         setSize(800,600);
         setBackground(Color.lightGray);
         setTitle("Enhydra Data Object Design Studio");
      
        //File Dialog initializations.
      
         fileChooser = new DODSFileChooser( new File (System.getProperty("user.dir")) );
      
        //Toolbar Initializations.
      
         toolBar = new JToolBar();
         toolBar.setForeground(new Color(0));
         toolBar.setBackground(Color.lightGray);
         newButton = new JButton(Icons.getIcon("new.gif"));
         newButton.setMargin(new Insets(0,0,0,0));
         newButton.setToolTipText("New");
         newButton.setFocusPainted(false);
         toolBar.add(newButton);
         openButton = new JButton(Icons.getIcon("open.gif"));
         openButton.setMargin(new Insets(0,0,0,0));
         openButton.setToolTipText("Open");
         openButton.setFocusPainted(false);
         toolBar.add(openButton);
         saveButton = new JButton(Icons.getIcon("save.gif"));
         saveButton.setDisabledIcon(Icons.getIcon("disabled_save.gif"));
         saveButton.setMargin(new Insets(0,0,0,0));
         saveButton.setToolTipText("Save");
         saveButton.setFocusPainted(false);
         toolBar.add(saveButton);
         toolBar.addSeparator();
         insertPackageButton = new JButton(Icons.getIcon("insert_package.gif"));
         insertPackageButton.setDisabledIcon(Icons.getIcon("disabled_insert_package.gif"));
         insertPackageButton.setMargin(new Insets(0,0,0,0));
         insertPackageButton.setToolTipText("Insert Package");
         insertPackageButton.setFocusPainted(false);
         toolBar.add(insertPackageButton);
         insertDataObjectButton = new JButton(Icons.getIcon("insert_dataobject.gif"));
         insertDataObjectButton.setDisabledIcon(Icons.getIcon("disabled_insert_dataobject.gif"));
         insertDataObjectButton.setMargin(new Insets(0,0,0,0));
         insertDataObjectButton.setToolTipText("Insert Data Object");
         insertDataObjectButton.setFocusPainted(false);
         toolBar.add(insertDataObjectButton);
      
         //****opensource contribution by Larry D'Wolcoto
         insertImportedObjectButton = new JButton(Icons.getIcon("import_schema.gif"));
         insertImportedObjectButton.setDisabledIcon(Icons.getIcon("disabled_insert_dataobject.gif"));
         insertImportedObjectButton.setMargin(new Insets(0,0,0,0));
         insertImportedObjectButton.setToolTipText("Insert Imported Object");
         insertImportedObjectButton.setFocusPainted(false);
         toolBar.add(insertImportedObjectButton);
         //****end
      
         insertAttributeButton = new JButton(Icons.getIcon("insert_attribute.gif"));
         insertAttributeButton.setDisabledIcon(Icons.getIcon("disabled_insert_attribute.gif"));
         insertAttributeButton.setMargin(new Insets(0,0,0,0));
         insertAttributeButton.setToolTipText("Insert Attribute");
         insertAttributeButton.setFocusPainted(false);
         toolBar.add(insertAttributeButton);
         toolBar.addSeparator();
         editPackageButton = new JButton(Icons.getIcon("edit_package.gif"));
         editPackageButton.setDisabledIcon(Icons.getIcon("disabled_edit_package.gif"));
         editPackageButton.setMargin(new Insets(0,0,0,0));
         editPackageButton.setToolTipText("Edit Package");
         editPackageButton.setFocusPainted(false);
         toolBar.add(editPackageButton);
         editDataObjectButton = new JButton(Icons.getIcon("edit_dataobject.gif"));
         editDataObjectButton.setDisabledIcon(Icons.getIcon("disabled_edit_dataobject.gif"));
         editDataObjectButton.setMargin(new Insets(0,0,0,0));
         editDataObjectButton.setToolTipText("Edit Data Object");
         editDataObjectButton.setFocusPainted(false);
         toolBar.add(editDataObjectButton);
         editAttributeButton = new JButton(Icons.getIcon("edit_attribute.gif"));
         editAttributeButton.setDisabledIcon(Icons.getIcon("disabled_edit_attribute.gif"));
         editAttributeButton.setMargin(new Insets(0,0,0,0));
         editAttributeButton.setToolTipText("Edit Attribute");
         editAttributeButton.setFocusPainted(false);
         toolBar.add(editAttributeButton);
         toolBar.addSeparator();
         findErrorsButton = new JButton(Icons.getIcon("error_button.gif"));
         findErrorsButton.setDisabledIcon(Icons.getIcon("disabled_error_button.gif"));
         findErrorsButton.setMargin(new Insets(0,0,0,0));
         findErrorsButton.setToolTipText("Find An Error");
         findErrorsButton.setFocusPainted(false);
         toolBar.add(findErrorsButton);
         toolBar.addSeparator();
         buildAllButton = new JButton(Icons.getIcon("build_all.gif"));
         buildAllButton.setDisabledIcon(Icons.getIcon("disabled_build_all.gif"));
         buildAllButton.setMargin(new Insets(0,0,0,0));
         buildAllButton.setToolTipText("Build All");
         buildAllButton.setFocusPainted(false);
         toolBar.add(buildAllButton);
         toolBar.setFloatable(false);
         toolBar.setBorder(new EmptyBorder(3,7,3,0));
         toolBar.setBorderPainted(true);
         getContentPane().add("North", toolBar);
      
        //Initializing the tree elements;
      
         tree = new JTree();
         DOTreeMouseListener dotreeml = new DOTreeMouseListener(this, table);
         tree.addMouseListener(dotreeml);
         tree.addMouseMotionListener(dotreeml);
         tree.addTreeExpansionListener(new DOTreeExpansionListener());
         dodsProject = new DODSProject("root");
         dodsProject.setDatabase("Standard_JDBC");
         treeRoot = new DOTreeNode(dodsProject, true);
         updateTree();
         ToolTipListener tipListener = new ToolTipListener();
         tree.setToolTipText("Package Tree");
      
        //Initializing the Graphical Pane.
      
         graphicalView = new GraphicalPanel(dodsProject, tree);
         graphicalView.setBackground(Color.white);
         top = new JScrollPane();
         top.setBackground(Color.white);
         top.setViewport(graphicalView);
         top.setMinimumSize(new Dimension(0,0));
      
         doTreeSelectionListener = new DOTreeSelectionListener(this, tree, graphicalView);
         tree.addTreeSelectionListener(doTreeSelectionListener);
         TreeSelectionModel selModel = tree.getSelectionModel();
         selModel.setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
         tree.setEditable(false);
         DOTreeRenderer dotr = new DOTreeRenderer();
         tree.setCellRenderer(dotr);
         JScrollPane left = new JScrollPane(tree);
         left.setMinimumSize(new Dimension(0,0));
         left.setPreferredSize(new Dimension(150, 0));
      
        //Initializing the table.
      
         table = new JTable();
         doTableModel = new DOTableModel(new DataObject(), this, table);

         // Allow the table to resize within the scrollpane.
         table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

         table.setBackground(Color.white);
         table.setModel(doTableModel);
         table.getTableHeader().setReorderingAllowed(false);
         TableMouseListener tableMouseListener = new TableMouseListener(table, this);
         table.addMouseListener(tableMouseListener);
         table.addMouseMotionListener(tipListener);
         table.addMouseListener(tipListener);
         table.getTableHeader().addMouseListener(new TableHeaderListener(this));
         TableColumnModel tcm = table.getColumnModel();
         JScrollPane right = new JScrollPane(table);
         right.setViewportBorder(new LineBorder(Color.gray, 1));
         right.setMinimumSize(new Dimension(0,0));
        //Initializing the TableColumns.
      
         for(int i = 0; i < tcm.getColumnCount(); i++) {
            TableColumn tc = tcm.getColumn(i);
            switch(i) {
               case 0:
                  tc.setMaxWidth(16);
                  tc.setMinWidth(16);
                  tc.setResizable(false);
                  tc.setCellRenderer(new ErrorColumnRenderer(false));
                  ErrorColumnRenderer ecr = new ErrorColumnRenderer(true);
                  ecr.setToolTipText("Errors");
                  tc.setHeaderRenderer(ecr);
                  break;
               case 1:
                  tc.setMinWidth(16);
                  tc.setMaxWidth(16);
                  tc.setResizable(false);
                  tc.setCellRenderer(new ObjectRefColumnRenderer(false));
                  ObjectRefColumnRenderer orcr = new ObjectRefColumnRenderer(true);
                  orcr.setToolTipText("Is an Object Reference");
                  tc.setHeaderRenderer(orcr);
                  break;
               case 2:
                  tc.setMaxWidth(16);
                  tc.setMinWidth(16);
                  tc.setResizable(false);
                  tc.setCellRenderer(new CanBeNullColumnRenderer(false));
                  CanBeNullColumnRenderer cbncr = new CanBeNullColumnRenderer(true);
                  cbncr.setToolTipText("Can Be Null");
                  tc.setHeaderRenderer(cbncr);
                  break;
/*
               case 3:
                  tc.setMaxWidth(16);
                  tc.setMinWidth(16);
                  tc.setResizable(false);
                  tc.setCellRenderer(new ArrayColumnRenderer(false));
                  ArrayColumnRenderer acr = new ArrayColumnRenderer(true);
                  acr.setToolTipText("Is an Array");
                  tc.setHeaderRenderer(acr);
                  break;
*/
               case 3:
                  tc.setMaxWidth(16);
                  tc.setMinWidth(16);
                  tc.setResizable(false);
                  tc.setCellRenderer(new IndexColumnRenderer(false));
                  IndexColumnRenderer icr = new IndexColumnRenderer(true);
                  icr.setToolTipText("Is an Index");
                  tc.setHeaderRenderer(icr);
                  break;
               case 4:
                  tc.setMaxWidth(16);
                  tc.setMinWidth(16);
                  tc.setResizable(false);
                  tc.setCellRenderer(new QueryColumnRenderer(false));
                  QueryColumnRenderer qcr = new QueryColumnRenderer(true);
                  qcr.setToolTipText("Can be used in a query");
                  tc.setHeaderRenderer(qcr);
                  break;
               case 5:
                  tc.setMaxWidth(16);
                  tc.setMinWidth(16);
                  tc.setResizable(false);
                  tc.setCellRenderer(new RefConstraintColumnRenderer(false));
                  RefConstraintColumnRenderer rccr = new RefConstraintColumnRenderer(true);
                  rccr.setToolTipText("Is a referential constraint for the database");
                  tc.setHeaderRenderer(rccr);
                  break;
               case 6:
                  tc.setMaxWidth(16);
                  tc.setMinWidth(16);
                  tc.setResizable(false);
                  tc.setCellRenderer(new UniqueColumnRenderer(false));
                  UniqueColumnRenderer ucr = new UniqueColumnRenderer(true);
                  ucr.setToolTipText("Is unique");
                  tc.setHeaderRenderer(ucr);
                  break;
               case 7:
                  tc.setMaxWidth(16);
                  tc.setMinWidth(16);
                  tc.setResizable(false);
                  tc.setCellRenderer(new ConstantColumnRenderer(false));
                  ConstantColumnRenderer ccr = new ConstantColumnRenderer(true);
                  ccr.setToolTipText("Has a constant value");
                  tc.setHeaderRenderer(ccr);
                  break;
               case 8:
                  tc.setPreferredWidth(100);
                  break;
               case 9:
                  tc.setPreferredWidth(80);
/*
                  DOTableComboEditor dotce = new DOTableComboEditor(this, tree, table, 6);
                  dotce.getEditor().addActionListener(dotce);
                  tc.setCellEditor(dotce);
*/
                  tc.setCellEditor(null);
                  break;
               case 10:
                  tc.setPreferredWidth(50);
                  break;
               case 11:
                  tc.setPreferredWidth(120);
                  break;
               case 12:
                  tc.setPreferredWidth(120);
                  break;
               default:
                  break;
            }
         }

        //Initializing the SplitPanes.
      
         verticalBar = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, left, right);
         horizontalBar = new JSplitPane(JSplitPane.VERTICAL_SPLIT, top, verticalBar);
         verticalBar.setBackground(Color.white);
         horizontalBar.setBackground(Color.white);
         getContentPane().add("Center", horizontalBar);
      
        //Initializing the Menus.
      
         dodsMenuBar = new JMenuBar();
         fileMenu = new JMenu("File");
         newMenuItem = new JMenuItem("New...");
         fileMenu.add(newMenuItem);
         openMenuItem = new JMenuItem("Open");
         fileMenu.add(openMenuItem);
         closeMenuItem = new JMenuItem("Close");
         fileMenu.add(closeMenuItem);
         fileMenu.addSeparator();
         buildAllMenuItem = new JMenuItem("Build All");
         fileMenu.add(buildAllMenuItem);
         fileMenu.addSeparator();
         saveMenuItem = new JMenuItem("Save");
         fileMenu.add(saveMenuItem);
         saveAsMenuItem = new JMenuItem("Save As...");
         fileMenu.add(saveAsMenuItem);
         fileMenu.addSeparator();
         exitMenuItem = new JMenuItem("Exit");
         fileMenu.add(exitMenuItem);
         dodsMenuBar.add(fileMenu);
         editMenu = new JMenu("Edit");
         undoMenuItem = new JMenuItem("Undo Delete");
         editMenu.add(undoMenuItem);
         editMenu.addSeparator();
         deleteMenu = new JMenu("Delete...");
         deletePackageMenuItem = new JMenuItem("Package");
         deleteMenu.add(deletePackageMenuItem);
         deleteDataObjectMenuItem = new JMenuItem("Data Object");
         deleteMenu.add(deleteDataObjectMenuItem);
         deleteAttributeMenuItem = new JMenuItem("Attribute(s)");
         deleteMenu.add(deleteAttributeMenuItem);
         editMenu.add(deleteMenu);
         editMenu.addSeparator();
         findErrorsMenuItem = new JMenuItem("Find Next Error");
         editMenu.add(findErrorsMenuItem);
         editMenu.addSeparator();
         editPackageMenuItem = new JMenuItem("Package...");
         editMenu.add(editPackageMenuItem);
         editDOMenuItem = new JMenuItem("Data Object...");
         editMenu.add(editDOMenuItem);
         editAttributeMenuItem = new JMenuItem("Attribute...");
         editMenu.add(editAttributeMenuItem);
         dodsMenuBar.add(editMenu);
         viewMenu = new JMenu("View");
         toolbarCheckBoxMenuItem = new JCheckBoxMenuItem("Toolbar");
         toolbarCheckBoxMenuItem.setState(true);
         viewMenu.add(toolbarCheckBoxMenuItem);
         viewMenu.addSeparator();
         refreshMenuItem = new JMenuItem("Refresh");
         viewMenu.add(refreshMenuItem);
         dodsMenuBar.add(viewMenu);
         insertMenu = new JMenu("Insert");
         insertPackageMenuItem = new JMenuItem("Package...");
         insertMenu.add(insertPackageMenuItem);
         insertDOMenuItem = new JMenuItem("Data Object...");
         insertMenu.add(insertDOMenuItem);
      
      	//****opensource contribution by Larry D'Wolcoto
         insertImportedObjectMenuItem = new JMenuItem("Imported Object...");
         insertMenu.add(insertImportedObjectMenuItem);
         //****end
      
         insertAttributeMenuItem = new JMenuItem("Attribute...");
         insertMenu.add(insertAttributeMenuItem);
         dodsMenuBar.add(insertMenu);
         databaseMenu = new JMenu("Database");
         standardJDBCCheckBoxMenuItem = new JCheckBoxMenuItem("Standard JDBC");
         standardJDBCCheckBoxMenuItem.setState(true);
         databaseMenu.add(standardJDBCCheckBoxMenuItem);
         oracleCheckBoxMenuItem = new JCheckBoxMenuItem("Oracle");
         databaseMenu.add(oracleCheckBoxMenuItem);
         informixCheckBoxMenuItem = new JCheckBoxMenuItem("Informix");
         databaseMenu.add(informixCheckBoxMenuItem);
         msqlCheckBoxMenuItem = new JCheckBoxMenuItem("MSQL");
         databaseMenu.add(msqlCheckBoxMenuItem);
         sybaseCheckBoxMenuItem = new JCheckBoxMenuItem("Sybase");
         databaseMenu.add(sybaseCheckBoxMenuItem);
         postgresqlCheckBoxMenuItem = new JCheckBoxMenuItem("PostgreSQL");
         databaseMenu.add(postgresqlCheckBoxMenuItem);
         dodsMenuBar.add(databaseMenu);
         helpMenu = new JMenu("Help");
         aboutMenuItem = new JMenuItem("About DODS");
         helpMenu.add(aboutMenuItem);
         dodsMenuBar.add(helpMenu);
         setJMenuBar(dodsMenuBar);
      
        //Setting up the DataBase MenuGroup.
      
         menuGroup = new ButtonGroup();
         menuGroup.add(standardJDBCCheckBoxMenuItem);
         menuGroup.add(oracleCheckBoxMenuItem);
         menuGroup.add(informixCheckBoxMenuItem);
         menuGroup.add(msqlCheckBoxMenuItem);
         menuGroup.add(sybaseCheckBoxMenuItem);
         menuGroup.add(postgresqlCheckBoxMenuItem);
      
        //Initialize Listeners and add them where appropriate.
      
         SymWindow aSymWindow = new SymWindow();
         this.addWindowListener(aSymWindow);
         SymAction lSymAction = new SymAction();
         exitMenuItem.addActionListener(lSymAction);
         insertAttributeMenuItem.addActionListener(lSymAction);
         insertDOMenuItem.addActionListener(lSymAction);
      
      	//****opensource contribution by Larry D'Wolcoto
         insertImportedObjectMenuItem.addActionListener(lSymAction);
         //****end
      
         insertPackageMenuItem.addActionListener(lSymAction);
         saveMenuItem.addActionListener(lSymAction);
         openMenuItem.addActionListener(lSymAction);
         editDOMenuItem.addActionListener(lSymAction);
         saveAsMenuItem.addActionListener(lSymAction);
         newMenuItem.addActionListener(lSymAction);
         closeMenuItem.addActionListener(lSymAction);
         editPackageMenuItem.addActionListener(lSymAction);
         deletePackageMenuItem.addActionListener(lSymAction);
         deleteDataObjectMenuItem.addActionListener(lSymAction);
         deleteAttributeMenuItem.addActionListener(lSymAction);
         undoMenuItem.addActionListener(lSymAction);
         SymItem lSymItem = new SymItem();
         toolbarCheckBoxMenuItem.addItemListener(lSymItem);
         standardJDBCCheckBoxMenuItem.addItemListener(lSymItem);
         oracleCheckBoxMenuItem.addItemListener(lSymItem);
         informixCheckBoxMenuItem.addItemListener(lSymItem);
         msqlCheckBoxMenuItem.addItemListener(lSymItem);
         sybaseCheckBoxMenuItem.addItemListener(lSymItem);
         postgresqlCheckBoxMenuItem.addItemListener(lSymItem);
         refreshMenuItem.addActionListener(lSymAction);
         editAttributeMenuItem.addActionListener(lSymAction);
         newButton.addActionListener(lSymAction);
         openButton.addActionListener(lSymAction);
         saveButton.addActionListener(lSymAction);
         editPackageButton.addActionListener(lSymAction);
         editDataObjectButton.addActionListener(lSymAction);
         editAttributeButton.addActionListener(lSymAction);
         insertPackageButton.addActionListener(lSymAction);
         insertDataObjectButton.addActionListener(lSymAction);
      
      	//****opensource contribution by Larry D'Wolcoto
         insertImportedObjectButton.addActionListener(lSymAction);
         //****end
      
         insertAttributeButton.addActionListener(lSymAction);
         buildAllButton.addActionListener(lSymAction);
         buildAllMenuItem.addActionListener(lSymAction);
         aboutMenuItem.addActionListener(lSymAction);
         findErrorsButton.addActionListener(lSymAction);
         findErrorsMenuItem.addActionListener(lSymAction);
      
        //initialize menu enablings.
      
         boolean errors = errorsExist();
         updateMenus();
      
        //Editor Dialog initializations.
      
         packageEditor = new PackageEditor(tree, this);
         dataObjectEditor = new DataObjectEditor(tree, this);
         attributeEditor = new AttributeEditor(tree, this);
      
      	//****opensource contribution by Larry D'Wolcoto
         tableBrowser = new TableBrowser(tree, this);
         //****end
      
      }
    /**
     * The main method creates an instance of DataObjectApplication and shows it.
     */
      public static void main(String args[]) {
         DataObjectApplication app = new DataObjectApplication();

	 boolean cool = true;
	 if ( args.length > 0 ) {
//System.err.println("args[0]="+args[0]);
	    try {
		File file = new File( args[0] );
		if ( null != file && file.isFile() ) {
		    FileInputStream finput = new FileInputStream(file);
		    ObjectInputStream oInput = new ObjectInputStream(finput);
		    app.dodsProject = (DODSProject)oInput.readObject();
		    app.dodsProject.setFile(file);
         	    app.updateTree();

		    app.treeRoot = new DOTreeNode(app.dodsProject, true);
		    app.graphicalView.setDODSProject(app.dodsProject);
		    app.updateTree();
		    app.tree.setRootVisible(true);
		    app.setDataObjectInTable(null);

		    app.updateTable();
		    app.updateMenus();
		    app.setCursor(app.normal);

		} else {
		    System.err.println(
			args[0] + " is not a project file.  Ignored.");
		    cool = false;
		}
	     } catch ( Exception e ) {
		System.err.println("First argument error:  "+e.getMessage());
		cool = false;
		// FIX remove next line
		e.printStackTrace();
	     }
	     if ( args.length > 1 ) {
//System.err.println("args[1]="+args[1]);
	         try {
		     File outputDir = new File( args[1] );
		     if ( null != outputDir && 
			  ( (!outputDir.exists()) || outputDir.isDirectory())) {
		         app.targetDir = outputDir;
		     } else {
		         System.err.println( args[1] + 
				" cannot be the output directory.  Ignored.");
			 cool = false;
		     }
	         } catch ( Exception e ) { 
		     System.err.println(
			"Second argument error:  "+e.getMessage());
		     cool = false;
		     // FIX remove next line
	             e.printStackTrace();
	         }
		 if ( args.length > 2 ) {
		     int exit_code = 0;
		     if ( args[2].equals( "regen" ) ) {
			// If 3rd arg is "regen", we just regenerate the code.
			// we do not show the gui.
			// 
			// Useful for testing changes to .template files.
			// 
			// Also useful for rebuilding an entire application
			// when a DODS project file is included in the 
			// source code that is put under version control.
			// The "regen" method of invoking DODS would be
			// used in the application's makefile.
			// 
			// NOTE:  use with caution!  Directory specified
			// in 2nd arg is deleted and rewritten!
			try {
			    if ( cool )
				Builder.run( args[1], app.dodsProject );
			} catch ( Exception e ) {
			    exit_code = 2;
			    System.err.println( "Code Generator failed:"
			       + e.getMessage() );
          e.printStackTrace();
			}
		     } else {
		        System.err.println( "Third argument was not 'regen'." );
			exit_code = 1;
		     }
		     System.exit( exit_code );
		 }
	     }
	 }

         app.setVisible(true);
      }
   
    //All of the attributes for the DataObjectApplication.
   
      private PackageEditor packageEditor;
      private DataObjectEditor dataObjectEditor;
      private AttributeEditor attributeEditor;
   
   	//****opensource contribution by Larry D'Wolcoto
      private TableBrowser tableBrowser;
      //****end

      private BuildMonitorDialog bmd = null;
   
      private DODSFileChooser fileChooser;
   
      private File targetDir = null;
      private JMenuBar dodsMenuBar;
      private JSplitPane verticalBar;
      private JSplitPane horizontalBar;
      private DODSProject dodsProject;
      private DOTableModel doTableModel;
      private DOTreeModel doTreeModel;
      private JTree tree;
      private JTable table;
      private JScrollPane top;
      private GraphicalPanel graphicalView;
      private DOTreeSelectionListener doTreeSelectionListener;
      private DOTreeNode treeRoot;
      private JToolBar toolBar;
   
    //The holding spots for deleted objects in case of undo.
   
      private Vector deletedNodeExtendedBy;
      private DataObject deletedNodeExtensionOf;
      private DOTreeNode deletedNode;
      private DOTreeNode deletedNodeParent;
      private Vector deletedAttributes = new Vector();
   
    //The cursors
   
      private Cursor wait = Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR);
      private Cursor normal = Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR);
   
    //All of the Buttons
   
      private JButton newButton;
      private JButton openButton;
      private JButton saveButton;
      private JButton editPackageButton;
      private JButton editDataObjectButton;
      private JButton editAttributeButton;
      private JButton insertPackageButton;
   
   	//****opensource contribution by Larry D'Wolcoto
      private JButton insertImportedObjectButton;
      //****end
   
      private JButton insertDataObjectButton;
      private JButton insertAttributeButton;
      private JButton buildAllButton;
      private JButton findErrorsButton;
   
    //All of the Menus.
   
      private JMenu fileMenu;
      private JMenuItem newMenuItem;
      private JMenuItem openMenuItem;
      private JMenuItem closeMenuItem;
      private JMenuItem saveMenuItem;
      private JMenuItem saveAsMenuItem;
      private JMenuItem exitMenuItem;
      private JMenu editMenu;
      private JMenuItem undoMenuItem;
      private JMenu deleteMenu;
      private JMenuItem deletePackageMenuItem;
      private JMenuItem deleteDataObjectMenuItem;
      private JMenuItem deleteAttributeMenuItem;
      private JMenuItem findErrorsMenuItem;
      private JMenuItem editPackageMenuItem;
      private JMenuItem editDOMenuItem;
      private JMenuItem editAttributeMenuItem;
      private JMenu viewMenu;
      private JCheckBoxMenuItem toolbarCheckBoxMenuItem;
      private JMenuItem refreshMenuItem;
      private JMenu insertMenu;
      private JMenuItem insertPackageMenuItem;
      private JMenuItem insertDOMenuItem;
   
   	//****opensource contribution by Larry D'Wolcoto
      private JMenuItem insertImportedObjectMenuItem;
      //****end
   
      private JMenuItem insertAttributeMenuItem;
      private JMenuItem buildAllMenuItem;
      private JMenu databaseMenu;
      private JCheckBoxMenuItem standardJDBCCheckBoxMenuItem;
      private JCheckBoxMenuItem oracleCheckBoxMenuItem;
      private JCheckBoxMenuItem informixCheckBoxMenuItem;
      private JCheckBoxMenuItem msqlCheckBoxMenuItem;
      private JCheckBoxMenuItem sybaseCheckBoxMenuItem;
      private JCheckBoxMenuItem postgresqlCheckBoxMenuItem;
      private JMenu helpMenu;
      private ButtonGroup menuGroup;
      private JMenuItem aboutMenuItem;

      // All accessible DODS Types
      DODSTypes dodsTypes = DODSTypes.getInstance();

    /**
     * This method changes the DataObject that has its Attributes displayed in the Table.
     * @param DO The new DataObject in the table.
     */
      public void setDataObjectInTable(DataObject DO) {
         if(doTableModel == null) {
            return;
         }
         else {
            doTableModel.setDataObject(DO);
         }
         boolean errors = errorsExist();
         table.revalidate();
         table.repaint();
         updateMenus();
      }
    /*
     * This method is called when the window receives a closing event.
     */
      private void DataObjectApplication_WindowClosing(WindowEvent event) {
         if(showSaveOptionDialog() == 2) {
            //do nothing because exiting was canceled.
         }
         else {
            setVisible(false);
            dispose();
            System.exit(0);
         }
      }
    /*
     * This method is called when the Menu receives a exit event.
     */
      private void exitMenuItem_ActionPerformed(ActionEvent event) {
         if(showSaveOptionDialog() == 2) {
            //do nothing because the exiting was canceled.
         }
         else {
            setVisible(false);
            dispose();
            System.exit(0);
         }
      }
    /*
     * This Method saves the Data Object tree.
     */
      private void saveTree() throws Exception {
         File file = dodsProject.getFile();
         if ( null == file ) {
            saveAsMenuItem_ActionPerformed( null );
         } 
         else {
            saveMenuItem_ActionPerformed( null );
         }
      }
    /*
     * This Method saves the Data Object tree when asked to.
     */
      private void saveMenuItem_ActionPerformed(ActionEvent event) {
        //When an exception is caught, the SaveAs.. Dialog is called so
        //the file can be renamed.
         try{
            File file = dodsProject.getFile();
            FileOutputStream foutput = new FileOutputStream(file);
            ObjectOutputStream ooutput = new ObjectOutputStream(foutput);
            ooutput.writeObject(dodsProject);
         }
            catch(Exception e) {
               saveAsMenuItem_ActionPerformed(new ActionEvent(this, 0, ""));
               return;
            }
         boolean errors = errorsExist();
         updateMenus();
      }
    /*
     * This method opens a saved DODSProject and throws away the old one,
     * but first it gives the option to save.
     */
      private void openMenuItem_ActionPerformed(ActionEvent event) {
         if(showSaveOptionDialog() == 2)
            return;
         int choice = fileChooser.showOpenDialog(this);
         if(choice == DODSFileChooser.CANCEL_OPTION) {
            return;
         }
         setCursor(wait);
         try {
            File file = fileChooser.getSelectedFile();
         
            FileInputStream finput = new FileInputStream(file);
            ObjectInputStream oInput = new ObjectInputStream(finput);
            DODSProject newdods = (DODSProject)oInput.readObject();
            dodsProject = newdods;
            dodsProject.setFile(file);
         }
            catch(Exception e) {
            //FIX:
               System.out.println(e.toString());
            }
         treeRoot = new DOTreeNode(dodsProject, true);
         graphicalView.setDODSProject(dodsProject);
         updateTree();
         tree.setRootVisible(true);
         boolean errors = errorsExist();
         setDataObjectInTable(null);
         updateTable();
         updateMenus();
         setCursor(normal);
      }
    /**
     * this Method updates the table's comboBox Editor to include the appropriate options.
     */
      public void updateTable() {
         boolean errors = errorsExist();
         updateMenus();
         table.revalidate();
         table.repaint();
      }
    /**
     * This method resets the tree when the set of DataObjects and packages changes,
     * while ensuring that the open and selected nodes remain open and selected.
     */
      public void updateTree() {
         TreePath selectionPath = tree.getSelectionPath();
         treeRoot.updateChildrenOrder();
         Vector openPaths = new Vector();
         for(int i = 0; i < tree.getRowCount(); i++) {
            if(tree.isExpanded(tree.getPathForRow(i))) {
               openPaths.addElement(tree.getPathForRow(i));
            }
         }
      
         doTreeModel = new DOTreeModel(treeRoot, this);
         tree.setModel(doTreeModel);
      
         for(int i = 0; i < openPaths.size(); i++) {
            tree.expandPath((TreePath)openPaths.elementAt(i));
         }
         setDataObjectInTable(null);
         tree.setSelectionPath(selectionPath);
         dodsProject.updatePackageName();
         updateGraphicalView();
      }
    /**
     * This method will update the graphical View when needed.
     */
      public void updateGraphicalView() {
         if(graphicalView == null) {
            return;
         }
         setCursor(wait);
         graphicalView.setDODSProject(dodsProject);
         graphicalView.updateBranches();
         graphicalView.getPreferredSize();
         graphicalView.repaint();
         setCursor(normal);
      }
    /*
     * This method is called when a new Data object is to be inserted.
     */
      private void insertDOMenuItem_ActionPerformed(ActionEvent event) {
         DataObject DO = new DataObject();
         editDataObject(DO);
      }
   
   
   	//****opensource contribution by Larry D'Wolcoto
    /*
     * This method is called when a new Imported Table object is to be inserted.
     */
      private void insertImportedObjectMenuItem_ActionPerformed(ActionEvent event) {
      
         DataObject DO = new DataObject();
         ImportObject(DO);
      }
      //****end
   
    /*
     * This method is called when a new Package is to be inserted.
     */
      private void insertPackageMenuItem_ActionPerformed(ActionEvent event) {
         DOPackage DOP = new DOPackage();
         editDOPackage(DOP);
      }
    /*
     * This Method shows the Attribute Editor with a new Attribute which.
     * is then created.
     */
      private void insertAttributeMenuItem_ActionPerformed(ActionEvent event) {
         Attribute att = new Attribute();
         editAttribute(att);
         table.setRowSelectionInterval(doTableModel.getRowCount()-1, doTableModel.getRowCount()-1);
         table.repaint();
      }
    /*
     * This method is called when a Data object is to be edited.
     */
      private void editDOMenuItem_ActionPerformed(ActionEvent event) {
         TreePath path = tree.getSelectionPath();
         DOTreeNode node = (DOTreeNode)path.getLastPathComponent();
         DataObject DO = (DataObject)node.getDataObject();
         Object[] newpath = new Object[path.getPath().length-1];
         for(int i = 0; i < path.getPath().length-1; i++) {
            newpath[i] = path.getPath()[i];
         }
        //This is done so the dataObjectEditor will set up the correct
        //path for the PackageName.
         tree.setSelectionPath(new TreePath(newpath));
      
         editDataObject(DO);
      }
    /*
     * This method is called when a Package is to be edited.
     */
      private void editPackageMenuItem_ActionPerformed(ActionEvent event) {
         TreePath path = tree.getSelectionPath();
         DOTreeNode node = (DOTreeNode)path.getLastPathComponent();
         DOPackage DOP = (DOPackage)node.getDataObject();
         if(DOP instanceof DODSProject) {
            editRoot(DOP);
         }
         else {
            Object[] newpath = new Object[path.getPath().length-1];
            for(int i = 0; i < path.getPath().length-1; i++) {
               newpath[i] = path.getPath()[i];
            }
            //This is done so the packageEditor will set up the correct
            //path for the PackageName.
            tree.setSelectionPath(new TreePath(newpath));
         
            editDOPackage(DOP);
         }
      }
    /*
     * This method is called when an Attribute is to be edited.
     */
      private void editAttributeMenuItem_ActionPerformed(ActionEvent event) {
         Attribute att = (Attribute)(doTableModel.getDataObject().getAttributes().elementAt(table.getSelectedRow()));
         editAttribute(att);
      }
    /**
     * This method takes care of the initialization of dataObjbectEditor and shows it.
     * @param DO The DataObject to be edited.
     */
      public void editDataObject(DataObject DO) {
         setCursor(wait);
         dataObjectEditor.setDataObject(DO);
         dataObjectEditor.setVisible(true);
         updateGraphicalView();
         boolean errors = errorsExist();
         updateMenus();
         setCursor(normal);
      }
   
     //****opensource contribution by Larry D'Wolcoto
    /**
     * This method takes care of the initialization of tableBrowser and shows it.
     * @param DO The DataObject to be edited.
     */
      public void ImportObject(DataObject DO) {
      
         setCursor(wait);
         tableBrowser.setDataObject(DO);
         tableBrowser.setVisible(true);
         updateGraphicalView();
         boolean errors = errorsExist();
         updateMenus();
         setCursor(normal);
      }
     //****end	
   
    /**
     * This method takes care of the initialization of packageEditor and shows it.
     * @param DOP The DOPackage to be edited.
     */
      public void editDOPackage(DOPackage DOP) {
         setCursor(wait);
         packageEditor.setDOPackage(DOP);
         packageEditor.setVisible(true);
         boolean errors = errorsExist();
         updateMenus();
         setCursor(normal);
      }
    /**
     * This method takes care of the initialization of attributeEditor and shows it.
     * @param att The Attribute to be edited.
     */
      public void editAttribute(Attribute att) {
         setCursor(wait);
         attributeEditor.setAttribute(att, doTableModel.getDataObject());

         // Highlight the Attribute name, if this is a new Attribute.
         if ( att.getName().equals("NewAttribute") ) {
           JTextField nameField = attributeEditor.getNameComponent();
           nameField.selectAll();
         }

         attributeEditor.setVisible(true);
         boolean errors = errorsExist();
         updateMenus();
         doTableModel.resetRows();
         table.repaint();
         updateGraphicalView();
         setCursor(normal);
      }
    /**
     * This method allows you to change the name of the root package.
     */
      public void editRoot(DOPackage DOP) {
         String newName = JOptionPane.showInputDialog(this,
         "Please enter the new" + "\n" +
         "name for the Root Package.",
         "Root Editor",
         JOptionPane.QUESTION_MESSAGE);
         try {
            dodsProject.setName(newName);
         }
            catch(DataObjectNameException e) {
               namingExceptionOccurred(e, this);
               editRoot(DOP);
            }
            catch(NullPointerException e) {
            }
         boolean errors = errorsExist();
         updateTree();
         updateMenus();
      }
    /*
     * This method opens a save File Dialog and saves the project.
     */
      private void saveAsMenuItem_ActionPerformed(ActionEvent event) {
         int choice = fileChooser.showSaveDialog(this);
         if(choice == DODSFileChooser.CANCEL_OPTION) {
            return;
         }
         try {
            File file = fileChooser.getSelectedFile();
            dodsProject.setFile(file);

            // If the file exists, confirm that we want to overwrite it.
            if ( file.exists() ) {
              if ( JOptionPane.YES_OPTION == JOptionPane.showConfirmDialog( this, "That file exists.  Overwrite it?", "Confirm File Save", JOptionPane.YES_NO_CANCEL_OPTION) ) {
                FileOutputStream foutput = new FileOutputStream(file);
                ObjectOutputStream ooutput = new ObjectOutputStream(foutput);
                ooutput.writeObject(dodsProject);
              }
            } else {
              // Otherwise, just save it.
              FileOutputStream foutput = new FileOutputStream(file);
              ObjectOutputStream ooutput = new ObjectOutputStream(foutput);
              ooutput.writeObject(dodsProject);
            }
         } catch(Exception e) {
            System.err.println(e.toString());
            JOptionPane.showMessageDialog( this, "Error saving file!", "Error!", JOptionPane.ERROR_MESSAGE);
         }
         boolean errors = errorsExist();
         updateMenus();
      }
    /*
     * This method removes the currentproject(with an option to save) and
     * creates a new one.
     */
      private void newMenuItem_ActionPerformed(ActionEvent event) {
         if(showSaveOptionDialog() == 2)
            return;
         dodsProject = new DODSProject("root");
         treeRoot = new DOTreeNode(dodsProject, true);
         updateTree();
         tree.setRootVisible(true);
         setDataObjectInTable(null);
         updateTable();
         boolean errors = errorsExist();
         updateMenus();
      }
    /*
     * This Method closes the current project but leaves the program open.
     */
      private void closeMenuItem_ActionPerformed(ActionEvent event) {
         if(showSaveOptionDialog() == 2)
            return;
         setDataObjectInTable(null);
         table.repaint();
         updateGraphicalView();
         dodsProject = new DODSProject("root");
         treeRoot = new DOTreeNode(dodsProject, true);
         updateTree();
         tree.setRootVisible(false);
         boolean errors = errorsExist();
         updateMenus();
      }
    /**
     * This method is called frequently to update which menus are enabled and disabled.
     * A menuItem will be disabled if It cannot currently be activated.
     */
      public void updateMenus() {
         updateDatabaseMenu();   //Sets up which database is selected.
         if(tree.isRootVisible()){
            saveAsMenuItem.setEnabled(true);
         }
         else {
            saveAsMenuItem.setEnabled(false);
         }
         if(checkSavedStatus()) {
            saveMenuItem.setEnabled(false);
            saveButton.setEnabled(false);
         }
         else {
            saveMenuItem.setEnabled(true);
            saveButton.setEnabled(true);
         }
         if(tree.getSelectionPath() != null) {
            TreePath path = tree.getSelectionPath();
            DOTreeNode node = (DOTreeNode)path.getLastPathComponent();
            if(node.getDataObject() instanceof DOPackage) {
               deletePackageMenuItem.setEnabled(true);
               deleteDataObjectMenuItem.setEnabled(false);
            }
            else {
               deleteDataObjectMenuItem.setEnabled(true);
               deletePackageMenuItem.setEnabled(false);
            }
            if(node.isLeaf()) {
               editPackageMenuItem.setEnabled(false);
               editPackageButton.setEnabled(false);
               editDOMenuItem.setEnabled(true);
               editDataObjectButton.setEnabled(true);
            }
            else {
               editPackageMenuItem.setEnabled(true);
               editPackageButton.setEnabled(true);
               editDOMenuItem.setEnabled(false);
               editDataObjectButton.setEnabled(false);
            }
            if(path.getPath().length == 1) {
               deletePackageMenuItem.setEnabled(false);
               deleteDataObjectMenuItem.setEnabled(false);
            }
         }
         else {
            editPackageMenuItem.setEnabled(false);
            editPackageButton.setEnabled(false);
            editDOMenuItem.setEnabled(false);
            editDataObjectButton.setEnabled(false);
            deletePackageMenuItem.setEnabled(false);
            deleteDataObjectMenuItem.setEnabled(false);
         }
         if(tree.isRootVisible()) {
            closeMenuItem.setEnabled(true);
            insertPackageMenuItem.setEnabled(true);
            insertDOMenuItem.setEnabled(true);
            insertPackageButton.setEnabled(true);
            insertDataObjectButton.setEnabled(true);
         }
         else {
            closeMenuItem.setEnabled(false);
            insertPackageMenuItem.setEnabled(false);
            insertDOMenuItem.setEnabled(false);
            insertDataObjectButton.setEnabled(false);
            insertPackageButton.setEnabled(false);
         }
         if(deletedNode != null){
            undoMenuItem.setEnabled(true);
         }
         else {
            undoMenuItem.setEnabled(false);
         }
         Vector allDOS = new Vector();
         DataObject DO = treeRoot.getDataObject();
         ((DODSProject)DO).getAllDataObjects(allDOS);
         if(allDOS.size() == 0) {
            insertAttributeMenuItem.setEnabled(false);
            insertAttributeButton.setEnabled(false);
         }
         else {
            insertAttributeMenuItem.setEnabled(true);
            insertAttributeButton.setEnabled(true);
         }
         if(table.getSelectedRow() < 0 || 
         table.getRowCount() == 0 ||
         table.getSelectedRow() > doTableModel.getDataObject().getAttributes().size()-1 ) {
            editAttributeMenuItem.setEnabled(false);
            editAttributeButton.setEnabled(false);
            deleteAttributeMenuItem.setEnabled(false);
         }
         else {
            editAttributeMenuItem.setEnabled(true);
            editAttributeButton.setEnabled(true);
            deleteAttributeMenuItem.setEnabled(true);
         }
         if(errorsExist()) {
            buildAllMenuItem.setEnabled(false);
            buildAllButton.setEnabled(false);
            if(allDOS.size() != 0) {
               findErrorsMenuItem.setEnabled(true);
               findErrorsButton.setEnabled(true);
            }
            else {
               findErrorsMenuItem.setEnabled(false);
               findErrorsButton.setEnabled(false);
            }
         }
         else {
            buildAllMenuItem.setEnabled(true);
            buildAllButton.setEnabled(true);
            findErrorsMenuItem.setEnabled(false);
            findErrorsButton.setEnabled(false);
         }
      }
    /**
     * This method compares the current project to it's saved version.
     * @return True when the two are equal, meaning it has been saved.
     */
      public boolean checkSavedStatus() {
         if(treeRoot.getChildren().size() == 0) 
            return true;
         try {
            File file = dodsProject.getFile();
            FileInputStream input = new FileInputStream(file);
            ObjectInputStream oInput = new ObjectInputStream(input);
            DODSProject dop2 = (DODSProject)oInput.readObject();
            if(dodsProject.equals(dop2)){
               return true;
            }
            else {
               return (!tree.isRootVisible());
            }
         }
            catch(Exception e) {
               return (!tree.isRootVisible());
            }
      }
    /**
     * This method is called when the current project is being closed.  It
     * presents a dialog asking if the user wants to save changes to the 
     * current project. If they do, then the save action takes place in this method.
     * @return 0 if the user wants to save. <br> 1 if the user doesn't want 
     *   to save. <br> 2 if the user wants to cancel their action.
     */
      public int showSaveOptionDialog() {
         if(!checkSavedStatus()) {
            String message;
            try {
               message = "Save Changes to " + dodsProject.getFile().getName() + "?";
            }
               catch(NullPointerException x) {
                  message = "Save Changes to Untitled?";
               }
            JOptionPane saveOptionPane = new JOptionPane(message,
            JOptionPane.QUESTION_MESSAGE,
            JOptionPane.YES_NO_CANCEL_OPTION);
            JDialog saveOptionDialog = saveOptionPane.createDialog(this, "DataObject Application");
            saveOptionDialog.setVisible(true);
            if(saveOptionPane.getValue().equals(new Integer(2)))
               return 2;
            if(saveOptionPane.getValue().equals(new Integer(0)))
               saveMenuItem_ActionPerformed(new ActionEvent(this, 0, ""));
            return 0;
         }
         return 1;
      }
    /**
     * When a naming exception occurs, this method is called, and an appropriate
     * warning dialog is displayed.
     * @param e The Name Exception which contains the message.
     * @param par The Component where the error occurred.
     */
      public void namingExceptionOccurred(DataObjectNameException e, Component par) {
         JOptionPane.showMessageDialog(par, e.getMessage(), "Naming Error", JOptionPane.WARNING_MESSAGE);
      }
    /*
     * This method deletes a package or DataObject.
     */
      private void deleteMenuItem_ActionPerformed(ActionEvent event) {
         setCursor(wait);
         deletedAttributes.removeAllElements();
         TreePath path = tree.getSelectionPath();
         Object[] pathComponents = path.getPath();
         DOTreeNode node = (DOTreeNode)pathComponents[pathComponents.length-1];
         DOTreeNode parentNode = (DOTreeNode)pathComponents[pathComponents.length-2];
         if(node.isLeaf()) {
            int result;
            if(node.getDataObject().getDescendents().size()!=0){
               setCursor(normal);
               result = JOptionPane.showConfirmDialog(this, "This Data Object is extended by other Objects. \nAre you sure you want to delete it?", "Delete Object", JOptionPane.YES_NO_OPTION);
            }
            else {
               result = JOptionPane.YES_OPTION;
            }
            switch(result) {
               case JOptionPane.YES_OPTION:
                  setCursor(wait);

		  // FIX?  This is not undoable.
		  deleteReferringAttributes( node );

                  deletedNodeExtensionOf = node.getDataObject().getExtensionOf();
                  deletedNodeExtendedBy = new Vector();
                  for(int i = 0; i < node.getDataObject().getExtendedBy().size(); i++) {
                     deletedNodeExtendedBy.addElement(node.getDataObject().getExtendedBy().elementAt(i));
                  }
                  node.getDataObject().setExtensionOf(null);
                  for(int i = 0; i < node.getDataObject().getExtendedBy().size(); i += 0) {
                     DataObject descend = (DataObject)node.getDataObject().getExtendedBy().elementAt(i);
                     descend.setExtensionOf(null);
                  }
                  ((DOPackage)parentNode.getDataObject()).getDataObjects().removeElement(node.getDataObject());
                  parentNode.getChildren().removeElement(node);
                  deletedNode = node;
                  deletedNodeParent = parentNode;
                  break;
               case JOptionPane.NO_OPTION:
                  setCursor(normal);
                  return;
               default:
                  setCursor(normal);
                  return;
            }
         }
         else {
            if(node.getChildren().size() != 0) {
               setCursor(normal);
               JOptionPane.showMessageDialog(this,
                  "This Package cannot be deleted because it contains other Objects.",
                  "Delete Package",
                  JOptionPane.WARNING_MESSAGE);
               return;
            }
            else {
               ((DOPackage)parentNode.getDataObject()).getDataObjects().removeElement(node.getDataObject());
               parentNode.getChildren().removeElement(node);
               deletedNode = node;
               deletedNodeParent = parentNode;
            }
         }
         boolean errors = errorsExist();
         updateTree();
         updateMenus();
         setCursor(normal);
      }

    /*
     * Search all DataObjects for Attributes that refer to this DataObject,
     * and remove those Attributes.
     * Not (currently) undoable.
     */
    private void deleteReferringAttributes( DOTreeNode deletedNode ) {
	if ( null == deletedNode )
	    return;
	DataObject deletedDO = deletedNode.getDataObject();
	DOTreeNode treeRoot = new DOTreeNode( dodsProject, true );
	deleteReferringAttributes( treeRoot, deletedDO );
        refreshDODS();
    }
    private void deleteReferringAttributes( DOTreeNode checkNode,
					    DataObject deletedDO ) {
	DataObject DO = checkNode.getDataObject();
	if ( DO instanceof DataObject ) {
	    Vector attrs = DO.getAttributes();
	    for ( int j = 0; j < attrs.size(); j++ ) {
		Attribute attr = (Attribute) attrs.elementAt( j );
		DataObject ref = attr.getObjectRefDO();
		if ( null != ref && deletedDO == ref ) {
		    attrs.removeElement( attr );
		    j--;  // otherwise we'd skip the next one
		}
	    }
	}

	Vector children = checkNode.getChildren();
	if ( null != children )
	    for ( int i = 0; i < children.size(); i++ ) {
		DOTreeNode node = (DOTreeNode) children.elementAt( i );
		deleteReferringAttributes( node, deletedDO );
	    }
    }

    /*
     * This reverses the most recent delete action.
     */
      private void undoMenuItem_ActionPerformed(ActionEvent event) {
         setCursor(wait);
         if(deletedAttributes.size() == 0) {
            if(deletedNode.getDataObject() instanceof DOPackage) {
               ((DOPackage)deletedNodeParent.getDataObject()).addDOPackage((DOPackage)deletedNode.getDataObject());
            }
            else {
               ((DOPackage)deletedNodeParent.getDataObject()).addDataObject(deletedNode.getDataObject());
               if(deletedNodeExtensionOf != null) {
                  deletedNode.getDataObject().setExtensionOf(deletedNodeExtensionOf);
               }
               if(deletedNodeExtendedBy != null) {
                  for(int i = 0; i < deletedNodeExtendedBy.size(); i++) {
                     ((DataObject)deletedNodeExtendedBy.elementAt(i)).setExtensionOf(deletedNode.getDataObject());
                  }
               }
            }
            deletedNodeParent.getChildren().addElement(deletedNode);
            deletedNodeExtensionOf = null;
            deletedNodeExtendedBy = null;
            deletedNode = null;
            deletedNodeParent = null;
            boolean errors = errorsExist();
            updateTree();
            updateMenus();
         }
         else {
            for(int i = 0; i < deletedAttributes.size(); i++) {
               deletedNode.getDataObject().addAttribute((Attribute)deletedAttributes.elementAt(i));
            }
            deletedAttributes.removeAllElements();
            deletedNode = null;
            boolean errors = errorsExist();
            doTableModel.resetRows();
            table.setRowSelectionInterval(-1, -1);
            errors = errorsExist();
            table.repaint();
            updateGraphicalView();
            updateMenus();
         }
         setCursor(normal);
      }
    /*
     * This method adds or removes the toolbar depending on the state.
     */
      private void toolbarCheckBoxMenuItem_ItemStateChanged(ItemEvent event) {
         if(!toolbarCheckBoxMenuItem.getState()) {
            getContentPane().remove(toolBar);
            SwingUtilities.updateComponentTreeUI(this);
         }
         else {
            getContentPane().add("North", toolBar);
            SwingUtilities.updateComponentTreeUI(this);
         }
      }
    /*
     * The refresh menu item is for when inexplicable painting errors occur
     * and the user wants to repaint everything to remove them.
     */
      private void refreshMenuItem_ActionPerformed(ActionEvent event) {
         refreshDODS();
      }
   
    /*
     * This function will refresh the entire DODS application.
     */
   
      public void refreshDODS() {
         errorsExist();
         updateTree();
         updateMenus();
         repaint();
      }
   
   
    /**
     * This method checks all of the Attributes and if they have errors, then
     * takes the appropriate action.
     */
      public void checkForErrors() {

         // TODO:  Most error checking is done based on the Java type.
         //        This needs to be changed to be based on the DODS type,
         //        as well.  Maybe a better approach is needed...

         DataObject dataobject = doTableModel.getDataObject();
         Vector atts = dataobject.getAttributes();
         for(int i = 0; i < atts.size(); i++) {
            Attribute att = (Attribute)atts.elementAt(i);
            boolean hasError = false;
            String errorText = new String("");

            // The currently selected DODS Type.
            String curDodsType = att.getDODSType();

            // The current Java type.
            String javatype = dodsTypes.getJavaType( curDodsType );
            String iJavaValue = att.getInitialJavaValue();

            // DBG
            //System.err.println("     Attrib: " + att.toString() + ", iJavaValue: *" + iJavaValue + "*");

            //Most of the error checking involves valid initial values.
            if ( null != javatype ) {
              if(javatype.equals("String")) {
                 hasError = false; //any string is valid.
              }
              else if( null != iJavaValue && javatype.equals("java.math.BigDecimal")) {
                 try {
                    java.math.BigDecimal bd = new java.math.BigDecimal(iJavaValue);
                    hasError = false;
                 }
                    catch(NumberFormatException e) {
                       hasError = true;
                       errorText = "Invalid Initial Value for Java Type.";
                    }
              }
              else if( null != iJavaValue && javatype.equals("boolean")) {
                 if ( !iJavaValue.equals("true") && !iJavaValue.equals("false") ) {
                   hasError = true;
                   errorText = "Initial Java value must be \"true\" or \"false.\"";
                 } else {
                    hasError = false;
                 }
              }
              else if( null != iJavaValue && javatype.equals("byte[]")) {
                 if(!iJavaValue.equals("")) {
                    hasError = true;
                    errorText = "No Initial Value allowed for this Java Type.";
                      //FIX: Should you allow "{byte1, byte2, byte3}"?
                 }
                 else {
                    hasError = false;
                 }
              }
              else if( null != iJavaValue && javatype.equals("byte")) {
                 try {
                    Byte bt = new Byte(iJavaValue);
                    hasError = false;
                 }
                    catch(NumberFormatException e) {
                       hasError = true;
                       errorText = "Invalid Initial Value for Java Type.";
                    }
              }
              else if( null != iJavaValue && javatype.equals("short")) {
                 try {
                    Short st = new Short(iJavaValue);
                    hasError = false;
                 }
                    catch(NumberFormatException e) {
                       hasError = true;
                       errorText = "Invalid Initial Value for Java Type.";
                    }
              }
              else if( null != iJavaValue && javatype.equals("int")) {
                 try {
                    Integer inte = new Integer(iJavaValue);
                    hasError = false;
                 }
                    catch(NumberFormatException e) {
                       hasError = true;
                       errorText = "Invalid Initial Value for Java Type.";
                    }
              }
              else if( null != iJavaValue && javatype.equals("long")) {
                 try {
                    Long lo = new Long(iJavaValue);
                    hasError = false;
                 }
                    catch(NumberFormatException e) {
                       hasError = true;
                       errorText = "Invalid Initial Value for Java Type.";
                    }
              }
              else if( null != iJavaValue && javatype.equals("float")) {
                 try {
                    Float lo = new Float(iJavaValue);
                    hasError = false;
                 }
                    catch(NumberFormatException e) {
                       hasError = true;
                       errorText = "Invalid Initial Value for Java Type.";
                    }
              }
              else if( null != iJavaValue && javatype.equals("double")) {
                 try {
                    Double lo = new Double(iJavaValue);
                    hasError = false;
                 }
                    catch(NumberFormatException e) {
                       hasError = true;
                       errorText = "Invalid Initial Value for Java Type.";
                    }
              }
              else if( null != iJavaValue && javatype.equals("java.sql.Date")) {
                 try {
                    String s = new String(iJavaValue);
                    boolean b = (java.sql.Date.valueOf(s)).toString().equals(iJavaValue);
                    hasError = false;
                 }
                    catch(Exception e) {
                       hasError = true;
                       errorText = "Initial Value must be in the form: \"yyyy-mm-dd\"";
                    }
              }
              else if( null != iJavaValue && javatype.equals("java.sql.Time")) {
                 try {
                    String s = new String(iJavaValue);
                    boolean b = (java.sql.Time.valueOf(s)).toString().equals(iJavaValue);
                    hasError = false;
                 }
                    catch(Exception e) {
                       hasError = true;
                       errorText = "Initial Value must be in the form: \"hh:mm:ss\"";
                    }
              }
              else if( null != iJavaValue && javatype.equals("java.sql.Timestamp")) {
                 try {
                    String s = new String(iJavaValue);
                    boolean b = (java.sql.Timestamp.valueOf(s)).toString().equals(iJavaValue);
                    hasError = false;
                 }
                    catch(Exception e) {
                       hasError = true;
                       errorText = "Initial Value must be in the form: \"yyyy-mm-dd hh:mm:ss.fffffffff\"";
                    }
              }
            } // end of null checking for javatype.
            else { //for all Object References.
               if( null != iJavaValue && !iJavaValue.equals("")) {
                  hasError = true;
                  errorText = "Invalid Initial Value for Java Type.";
               }
               else {
                  hasError = false;
               }
            }
            if ( null == iJavaValue || iJavaValue.equals("")) {
               hasError = false; //if there is no initial value then it can't be invalid.
            }
            if(sizeHasError(att)) { //If the attribute has an invalid size.
               hasError = true;
               if(!errorText.equals("")) {
                  errorText += " \n ";
               }
               errorText = "Invalid size for this DODS Type";
            }

            //Duplicated names in an Attribute cause an error.
            for(int j = 0; j < atts.size(); j++) {
               Attribute at = (Attribute)atts.elementAt(j);
               if(at != att){
                  if(at.getName().equals(att.getName())) {
                     hasError = true;
                     if(!errorText.equals("")) {
                        errorText += " \n ";
                     }
                     errorText += "Duplicate Attribute Names";
                  }
               }
            }
            if(att.isObjectRef()) {
               if(att.getObjectRefDO().isAbstract()) {
                  hasError = true;
                  if(!errorText.equals("")) {
                     errorText += " \n ";
                  }
                  errorText += "Referencing an abstract Data Object";
               }
            }
            att.setError(hasError);
            att.setErrorText(errorText);
         }
      }

    /*
     *  Reports an error if the Size field of an Attribute is incorrect.
     */
      private boolean sizeHasError(Attribute att) {

        // The currently selected DODS Type.
        String curDodsType = att.getDODSType();

        // Generate an error if this DODS Type allows a Size to be set
        // but this attribute has not set one.  (Sizes are either
        // not allowed or mandatory).
        if ( ( dodsTypes.isSizeAllowed( curDodsType ) ) && ( null == att.getSize() || att.getSize().equals("") ) ) {
          return true;
        }
        return false;

      }

    /*
     * called when Standard JDBC is chosen as the Database.
     */
      private void standardJDBCCheckBoxMenuItem_ItemStateChanged(ItemEvent event) {
         if(standardJDBCCheckBoxMenuItem.getState()) {
            dodsProject.setDatabase("Standard_JDBC");
         }
      }

    /*
     * called when Oracle is chosen as the Database.
     */
      private void oracleCheckBoxMenuItem_ItemStateChanged(ItemEvent event) {
         if(oracleCheckBoxMenuItem.getState()) {
            dodsProject.setDatabase("Oracle");
         }
      }

    /*
     * called when Informix is chosen as the Database.
     */
      private void informixCheckBoxMenuItem_ItemStateChanged(ItemEvent event) {
         if(informixCheckBoxMenuItem.getState()) {
            dodsProject.setDatabase("Informix");
         }
      }

    /*
     * called when MSQL is chosen as the Database.
     */
      private void msqlCheckBoxMenuItem_ItemStateChanged(ItemEvent event) {
         if(msqlCheckBoxMenuItem.getState()) {
            dodsProject.setDatabase("MSQL");
         }
      }

    /*
     * called when Sybase is chosen as the Database.
     */
      private void sybaseCheckBoxMenuItem_ItemStateChanged(ItemEvent event) {
         if(sybaseCheckBoxMenuItem.getState()) {
            dodsProject.setDatabase("Sybase");
         }
      }

    /*
     * called when PostgreSQL is chosen as the Database.
     */
      private void postgresqlCheckBoxMenuItem_ItemStateChanged(ItemEvent event) {
         if(postgresqlCheckBoxMenuItem.getState()) {
            dodsProject.setDatabase("PostgreSQL");
         }
      }

    /**
     * @return True if any of the attributes have errors in them.
     */
      public boolean errorsExist() {
         //System.err.println("errorsExist() was called!");
         checkForErrors();
         Vector allDOS = new Vector();
         dodsProject.getAllDataObjects(allDOS);
         for(int i = 0; i < allDOS.size(); i++) {
            DataObject DO = (DataObject)allDOS.elementAt(i);
            for(int j = 0; j < DO.getAttributes().size(); j++) {
               if(((Attribute)DO.getAttributes().elementAt(j)).hasError()) {
                  return true;
               }
            }
         }
         if(allDOS.size() == 0) {
            return true; //no DataObjects existing counts as an error.
         }
         return false;
      }

    /*
     * Launch Builder to generate and compile source code.
     */
      private void buildAllMenuItem_ActionPerformed(ActionEvent event) {
         File targetDirectory;
         saveMenuItem_ActionPerformed(new ActionEvent(this, 0, ""));
         DODSDirectoryChooser chooser = new DODSDirectoryChooser( targetDir );
         int choice = chooser.showDialog(this, "Re/Create Directory");
         if(choice == chooser.CANCEL_OPTION)
            return;
         targetDirectory = chooser.getSelectedFile();
      //System.err.println("targetDirectory ="+targetDirectory );
      //System.err.println("targetDirectory.getAbsolutePath()="+targetDirectory.getAbsolutePath());
      
         if(targetDirectory == null) {
            JOptionPane.showMessageDialog(this,
               "You mush choose a directory to put these data Objects into.",
               "Choose Directory",
               JOptionPane.WARNING_MESSAGE);
            return;
         }
         if ( targetDirectory.exists() ) {
            if ( ! targetDirectory.isDirectory() ) {
               JOptionPane.showMessageDialog(this,
                  "Your selection was not a directory.",
                  "Bad Directory",
                  JOptionPane.WARNING_MESSAGE);
               return;
            }
         
            int result = JOptionPane.showConfirmDialog( this,
            "Are you sure?\n" +
            "Everything in this directory will be destroyed:\n" +
            targetDirectory.getAbsolutePath(), 
            "Directory contents will be destroyed", 
            JOptionPane.YES_NO_OPTION,
            JOptionPane.QUESTION_MESSAGE );
            if(result == JOptionPane.NO_OPTION) {
               return;
            }
         
            String stuck = FileSystem.rmdir( targetDirectory );
            if ( null != stuck ) {
               JOptionPane.showMessageDialog(this,
                  "Unable to delete: \n" + stuck,
                  "Bad Directory",
                  JOptionPane.WARNING_MESSAGE);
               return;
            }
         }
         if ( ! targetDirectory.mkdirs() ) {
            JOptionPane.showMessageDialog(this,
               "Unable to mkdir: \n" +
               targetDirectory.getAbsolutePath(),
               "Bad Directory",
               JOptionPane.WARNING_MESSAGE);
            return;
         }
      
         targetDir = targetDirectory;
      
      /*
      if ( null == targetDir || 0 == targetDir.length() )
       targetDir = System.getProperty( "DODS_OUTPUT_DIR" );
      DirectoryChooser chooser = new DirectoryChooser( this, targetDir );
      //System.err.println("chooser.dir="+chooser.dir);
      if ( null == chooser.dir )
       return;
      targetDir = chooser.dir.getAbsolutePath();
      
      */
      
      /* We must remove the target directory (and everything under it)
      * because it was likely used for a previous Build run
      * and could contain source files for DataObjects that no longer 
      * exist in the project.
      * This means that the user should never add any files to
      * a directory tree generated by DODS.
      * No check for user files is currently performed.
      */
      
         try {
         //System.err.println("BEFORE saveTree");
            saveTree();
         //System.err.println("BEFORE Builder");

	    if ( null != bmd )
		bmd.dispose();
	    bmd = new BuildMonitorDialog();
	    bmd.setModal( false );
	    bmd.setResizable( false );
	    bmd.setVisible( true );

	    (new Thread( new BuildRunner( 
		targetDirectory.getAbsolutePath(), dodsProject ) ) ).start();
/*             Builder.run( targetDirectory.getAbsolutePath(), dodsProject ); */
         } 
            catch ( Exception e ) {  
               e.printStackTrace();
               JOptionPane.showMessageDialog(this, "ERROR: " + e.getMessage(),
                  "Building Data Objects",
                  JOptionPane.WARNING_MESSAGE);
            }
      }
    /*
     * This method deletes any selected attributes in the table.
     */
      private void deleteAttributeMenuItem_ActionPerformed(ActionEvent event) {
         int[] selrows = table.getSelectedRows();
         int[] rows = new int[selrows.length + 1];
         rows[0] = -1;
         for(int j = 1; j < rows.length; j++) {
            int row = rows[j-1];
            for(int i = 0; i < selrows.length; i++) {
               if(selrows[i] > row) {
                  row = selrows[i];
               }
            }
            if(row != rows[j-1]) {
               rows[j] = row;
            }
         }
         DataObject dataObject = doTableModel.getDataObject();
         deletedAttributes.removeAllElements();
         for(int i = 1; i < rows.length; i++) {
            Attribute att = (Attribute)((DOTableModel)table.getModel()).getDataObject().getAttributes().elementAt(rows[i]);
            deletedAttributes.addElement(att);
            dataObject.getAttributes().removeElement(att);
         }
         if(deletedAttributes.size() != 0){
            TreePath path = tree.getSelectionPath();
            deletedNode = (DOTreeNode)tree.getLastSelectedPathComponent();
         }
         table.setRowSelectionInterval(-1, -1);
         refreshDODS();
      }
   
    /*
     * This method makes sure that the Database for this project is correct.
     */
      private void updateDatabaseMenu() {
         if(dodsProject.getDatabase().equals("Standard_JDBC")) {
            standardJDBCCheckBoxMenuItem.doClick();
         }
         else if(dodsProject.getDatabase().equals("Oracle")) {
            oracleCheckBoxMenuItem.doClick();
         }
         else if(dodsProject.getDatabase().equals("Informix")) {
            informixCheckBoxMenuItem.doClick();
         }
         else if(dodsProject.getDatabase().equals("MSQL")) {
            msqlCheckBoxMenuItem.doClick();
         }
         else if(dodsProject.getDatabase().equals("Sybase")) {
            sybaseCheckBoxMenuItem.doClick();
         }
         else if(dodsProject.getDatabase().equals("PostgreSQL")) {
            postgresqlCheckBoxMenuItem.doClick();
         }
      }
    /*
     * Displays a Dialog which should display the appropriate information.
     */
      private void aboutMenuItem_ActionPerformed(ActionEvent event) {
         JLabel jl = new JLabel();
         jl.setText("");
         JOptionPane.showMessageDialog(this,
            jl,
            "About DODS",
            JOptionPane.PLAIN_MESSAGE,
            null);
      }
    /*
     * This method finds the first attribute with an error and takes you there.
     */
      private void findErrorsMenuItem_ActionPerformed(ActionEvent event) {
         for(int i = 0; i < tree.getRowCount(); i++) {
            TreePath path = tree.getPathForRow(i);
            if(((DOTreeNode)path.getLastPathComponent()).isLeaf()) {
               if(((DOTreeNode)path.getLastPathComponent()).getDataObject().hasAttributeErrors()) {
                  tree.setSelectionPath(path);
                  return;
               }
            }
         }
      }
    /*
     * For debugging purposes
     */
      public void checkDOS(){
         Vector allDOS = new Vector();
         dodsProject.getAllDataObjects(allDOS);
         for(int i = 0; i < allDOS.size(); i++) {
            DataObject DO = (DataObject)allDOS.elementAt(i);
            System.out.println();
            System.out.println(DO.getName() + ":");
            System.out.println("    " + "ExtensionOf: " + DO.getExtensionOf());
            System.out.println("    " + "ExtendedBy:  " + DO.getExtendedBy());
            System.out.println("    " + "Ancestors:   " + DO.getAncestors());
            System.out.println("    " + "Descendents: " + DO.getDescendents());
         }
      }
    /*
     *
     *
     * The rest are all internal listener classes which call the above
     * methods when appropriate events occur.
     *
     *
     */
      class SymWindow extends WindowAdapter {
         public void windowClosing(WindowEvent event) {
            Object object = event.getSource();
            if (object == DataObjectApplication.this)
               DataObjectApplication_WindowClosing(event);
         }
      }
      class SymAction implements ActionListener {
         public void actionPerformed(ActionEvent event) {
            Object object = event.getSource();
            if (object == exitMenuItem)
               exitMenuItem_ActionPerformed(event);
            else if (object == insertAttributeMenuItem)
               insertAttributeMenuItem_ActionPerformed(event);
            else if (object == insertDOMenuItem)
               insertDOMenuItem_ActionPerformed(event);
            
            //****opensource contribution by Larry D'Wolcoto
            else if (object == insertImportedObjectMenuItem)
               insertImportedObjectMenuItem_ActionPerformed(event);
            //****end	
            
            else if (object == insertPackageMenuItem)
               insertPackageMenuItem_ActionPerformed(event);
            else if (object == saveMenuItem)
               saveMenuItem_ActionPerformed(event);
            else if (object == openMenuItem)
               openMenuItem_ActionPerformed(event);
            else if (object == editDOMenuItem)
               editDOMenuItem_ActionPerformed(event);
            else if (object == saveAsMenuItem)
               saveAsMenuItem_ActionPerformed(event);
            else if (object == newMenuItem)
               newMenuItem_ActionPerformed(event);
            else if (object == closeMenuItem)
               closeMenuItem_ActionPerformed(event);
            else if (object == editPackageMenuItem)
               editPackageMenuItem_ActionPerformed(event);
            else if (object == deletePackageMenuItem)
               deleteMenuItem_ActionPerformed(event);
            else if (object == deleteDataObjectMenuItem)
               deleteMenuItem_ActionPerformed(event);
            else if (object == deleteAttributeMenuItem)
               deleteAttributeMenuItem_ActionPerformed(event);
            else if (object == undoMenuItem)
               undoMenuItem_ActionPerformed(event);
            else if (object == refreshMenuItem)
               refreshMenuItem_ActionPerformed(event);
            else if (object == editAttributeMenuItem)
               editAttributeMenuItem_ActionPerformed(event);
            else if (object == newButton)
               newMenuItem_ActionPerformed(event);
            else if (object == openButton)
               openMenuItem_ActionPerformed(event);
            else if (object == saveButton)
               saveMenuItem_ActionPerformed(event);
            else if (object == editDataObjectButton)
               editDOMenuItem_ActionPerformed(event);
            else if (object == editPackageButton)
               editPackageMenuItem_ActionPerformed(event);
            else if (object == editAttributeButton)
               editAttributeMenuItem_ActionPerformed(event);
            else if (object == insertDataObjectButton)
               insertDOMenuItem_ActionPerformed(event);
            
            //****opensource contribution by Larry D'Wolcoto
            else if (object == insertImportedObjectButton)
               insertImportedObjectMenuItem_ActionPerformed(event);
            //****end	
            
            else if (object == insertPackageButton)
               insertPackageMenuItem_ActionPerformed(event);
            else if (object == insertAttributeButton)
               insertAttributeMenuItem_ActionPerformed(event);
            else if (object == buildAllMenuItem)
               buildAllMenuItem_ActionPerformed(event);
            else if (object == buildAllButton)
               buildAllMenuItem_ActionPerformed(event);
            else if (object == aboutMenuItem)
               aboutMenuItem_ActionPerformed(event);
            else if (object == findErrorsButton)
               findErrorsMenuItem_ActionPerformed(event);
            else if (object == findErrorsMenuItem)
               findErrorsMenuItem_ActionPerformed(event);
         }
      }
      class SymItem implements ItemListener {
         public void itemStateChanged(ItemEvent event) {
            Object object = event.getSource();
            if (object == toolbarCheckBoxMenuItem)
               toolbarCheckBoxMenuItem_ItemStateChanged(event);
            if (object == standardJDBCCheckBoxMenuItem)
               standardJDBCCheckBoxMenuItem_ItemStateChanged(event);
            if (object == oracleCheckBoxMenuItem)
               oracleCheckBoxMenuItem_ItemStateChanged(event);
            if (object == informixCheckBoxMenuItem)
               informixCheckBoxMenuItem_ItemStateChanged(event);
            if (object == msqlCheckBoxMenuItem)
               msqlCheckBoxMenuItem_ItemStateChanged(event);
            if (object == sybaseCheckBoxMenuItem)
               sybaseCheckBoxMenuItem_ItemStateChanged(event);
            if (object == postgresqlCheckBoxMenuItem)
               postgresqlCheckBoxMenuItem_ItemStateChanged(event);
         }
      }
   }

class BuildMonitorDialog extends JDialog
implements PropertyChangeListener
{
    JOptionPane optPane;
    public BuildMonitorDialog() {

	JTextArea textArea = new JTextArea( 30, 80 );
	textArea.setEditable( false );
	textArea.setFont( new Font( "Courier", Font.PLAIN, 12 ) );
	Builder.setTextArea( textArea );
	textArea.append( "Generating .java and .sql files...\n\n" );

	JPanel textPanel = new JPanel(new BorderLayout());
	textPanel.add( new JScrollPane( textArea ), BorderLayout.CENTER );

	String[] options = { "Close" };
	Object[] message = new Object[] {
	    textPanel,
	    "Please check for warnings and errors"
	};
	optPane = new JOptionPane( message,
				    JOptionPane.PLAIN_MESSAGE,
				    JOptionPane.DEFAULT_OPTION,
				    null,
				    options, options[0] );

	optPane.addPropertyChangeListener( this );
	this.getContentPane().add( optPane, BorderLayout.CENTER );

	this.setTitle( "Generating source code and compiling..." );
	this.setResizable( true );
	this.pack();

    }
    public void propertyChange( PropertyChangeEvent e ) {
	if ( e.getPropertyName().equals( JOptionPane.VALUE_PROPERTY ) )
	    this.setVisible( false );
    }
}

class BuildRunner implements Runnable {
    String dir; 
    DODSProject project;
    public BuildRunner( String s, DODSProject p ) {
	    dir = s;
	    project = p;
    }
    public void run() {
	    try {
		Builder.run( dir, project );
	    } catch ( Exception e ) { 
		Builder.print( "ERROR: " + e.getMessage() );
		e.printStackTrace();
	    }
    }
}
