/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: DirectoryChooser.java,v 1.4 1999/02/12 12:18:07 andy Exp $
 *-----------------------------------------------------------------------------
 */



package dods.gui;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

public final class DirectoryChooser extends Dialog
implements ActionListener, TextListener {
    public  File	dir       = null;

    private TextField	tfDir     = new TextField( 50 );
    private JButton	setBtn    = new JButton( "OK" );
    private JButton	cancelBtn = new JButton( "Cancel" );

    public DirectoryChooser( Frame f, String defaultDir ) {
	super( f, "Choose Target Directory for Source Tree Generation", true );
	setLayout( new GridLayout( 3, 1 ) );
	setSize( 200, 500 );
	setLocation( 50, 100 );
	if ( null != defaultDir )
	    tfDir.setText( defaultDir );
	setBtn.addActionListener( this );
	cancelBtn.addActionListener( this );
	Panel p1 = new Panel();
	p1.add( new Label( "Target Directory:", Label.LEFT ) );
	p1.add( tfDir );
	Panel p3 = new Panel();
	p3.add( setBtn );
	p3.add( cancelBtn );
	add( p1 );
	add( p3 );
	pack();
	show();
    }

    public void actionPerformed( ActionEvent event ) {
	dir = null;
	String what = event.getActionCommand();
	if ( "Cancel".equals( what ) ) {
	    dispose();
	} else {
	    String path = tfDir.getText();
	    if ( null == path || 0 == path.length() ) {
/* 		path = ""; */
		int result = JOptionPane.showConfirmDialog( this,
				"Do you wish to Build your project?", 
				"No directory specified", 
				JOptionPane.YES_NO_OPTION,
				JOptionPane.QUESTION_MESSAGE );
		if ( JOptionPane.NO_OPTION == result )
		    dispose();
		return;
	    } else {
		dir = new File( path );
		if ( dir.isFile() ) {
		    int result = JOptionPane.showConfirmDialog( this,
				"That is an existing file, not a directory.\n" +
				"Do you wish to enter a different path?", 
				"Cannot create directory", 
				JOptionPane.YES_NO_OPTION,
				JOptionPane.ERROR_MESSAGE );
		    if ( JOptionPane.NO_OPTION == result ) {
			dir = null;
			dispose();
		    }
		    return;
		} else if ( ! dir.isDirectory() ) {
		    int result = JOptionPane.showConfirmDialog( this,
				"Create that directory?", 
				"Directory does not exist", 
				JOptionPane.YES_NO_CANCEL_OPTION,
				JOptionPane.QUESTION_MESSAGE );
		    if ( JOptionPane.CANCEL_OPTION == result ) {
			dir = null;
			dispose();
			return;
		    }
		    if ( JOptionPane.NO_OPTION == result ) {
			dir = null;
			return;
		    }
		    dir.mkdir();
		    dispose();
		} else {
		    int result = JOptionPane.showConfirmDialog( this,
				"Destroy current contents of that directory?", 
				"Directory contents will be destroyed", 
				JOptionPane.YES_NO_CANCEL_OPTION,
				JOptionPane.QUESTION_MESSAGE );
		    if ( JOptionPane.CANCEL_OPTION == result ) {
			dir = null;
			dispose();
			return;
		    }
		    if ( JOptionPane.NO_OPTION == result ) {
			dir = null;
			return;
		    }
		    dispose();
		}
	    }
	}
    }

    public void textValueChanged( TextEvent e ) {
	int cols = tfDir.getColumns();
	if ( tfDir.getText().length() == cols ) {
	    tfDir.setColumns( cols + cols );
	    pack();
	}
    }
}
