/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: GraphicalBranch.java,v 1.4 1999/02/12 12:18:07 andy Exp $
 *-----------------------------------------------------------------------------
 */



package dods.gui;

import dods.data.*;
import javax.swing.JLabel;
import java.awt.Graphics;
import java.awt.Color;
import java.awt.Font;
import java.awt.Rectangle;
import java.util.Vector;
/**
 * This represents one node of the Graphical Tree in the GraphicalPanel and
 * the connections to other nodes.
 *
 * @author  Philip Martin
 * @version $Revision: 1.4 $
 */
public class GraphicalBranch {
    //FIX: comments
    private int Y;
    private int X;
    private int width;
    private int row;
    private boolean extended;
    private Vector branches = new Vector();
    private int bottomRow;
    private boolean mouseOver = false;
    private boolean selected = false;
    private Rectangle rectangle;
    private int labelWidth;
    private int labelHeight;
    private GraphicalPanel gPanel;
    /*
     * This is the DataObject that this GraphicalDO represents.
     */
    private DataObject DO;
    /**
     * The constuctor initializes variables using the DataObject given.
     */
    public GraphicalBranch(DataObject Do, int ro, GraphicalPanel gp) {
        DO = Do;
        row = ro;
        gPanel = gp;
        bottomRow = row;
        JLabel label = new JLabel(DO.getName());
        labelWidth = Math.max(label.getPreferredSize().width + 20, 100);
        labelHeight = Math.max(label.getPreferredSize().height, 20);
        X = 0;
        Y = 0;
        width = labelWidth + 20;
        if(DO.getExtendedBy().size() > 0) {
            extended = true;
        }
        else {
            extended = false;
        }
    }
    /**
     * This method gathers all of the DataObjects into Branches which can
     * be displayed.
     */
    public void updateBranches() {
        branches = new Vector();
        Vector extendedBy = DO.getExtendedBy();
        extendedBy = DataObject.alphabetizeVector(extendedBy);
        int branchesWidth = 0;
        for(int i = 0; i < extendedBy.size(); i++) {
            DataObject Data = (DataObject)extendedBy.elementAt(i);
            GraphicalBranch branch = new GraphicalBranch(Data, row + 1, gPanel);
            branches.addElement(branch);
            branch.updateBranches();
            if(branch.getBottomRow() > bottomRow) {
                bottomRow = branch.getBottomRow();
            }
            branchesWidth += branch.getWidth();
        }
        width = Math.max(branchesWidth, labelWidth + 20);
    }
    /**
     * This method updates where all of the branches are located.
     */
    public void updateBranchPositions() {
        int newX = X;
        int branchesWidth = 0;
        for(int i = 0; i < branches.size(); i++) {
            GraphicalBranch branch = (GraphicalBranch)branches.elementAt(i);
            branchesWidth += branch.getWidth();
        }
        if(branchesWidth >= width) {
            for(int i = 0; i < branches.size(); i++) {
                GraphicalBranch branch = (GraphicalBranch)branches.elementAt(i);
                if(branch.isExtended()) {
                    branch.setPosition(newX, branch.getRow()*50-20);
                    newX += branch.getWidth();
                }
                else {
                    //FIX:
                    branch.setPosition(newX, branch.getRow()*50-20);
                    newX += branch.getWidth();
                }
                branch.updateBranchPositions();
            }
        }
        else {
            int newXtwo = X + width/2 - branchesWidth/2;
            for(int i = 0; i < branches.size(); i++) {
                GraphicalBranch branch = (GraphicalBranch)branches.elementAt(i);
                branch.setPosition(newXtwo, branch.getRow()*50);
                newXtwo += branch.getWidth();
                branch.updateBranchPositions();
            }
        }
        //FIX: Should this be here?
        rectangle = new Rectangle(X + width/2- labelWidth/2 + 2, Y-labelHeight + 2, labelWidth + 2, labelHeight + 2);
    }
    /**
     * Sets the position of the entire branches upper left corner.
     */
    public void setPosition(int x, int y){
        X = x;
        Y = y;
    }
    /**
     * This method draws The GraphicalBranch in the given Graphics setting.
     */
    public void draw(Graphics g) {
        int xoff = 0;
        int yoff = 0;
        if(mouseOver) {
            xoff++;
            yoff++;
        }
        for(int i = 0 ; i < branches.size(); i++) {
            GraphicalBranch branch = (GraphicalBranch)branches.elementAt(i);
            g.setColor(Color.green.darker());
            g.drawLine(X + width/2, Y+2,
                       branch.getX() + branch.getWidth()/2,
		       branch.getY()-15);
            //g.setColor(Color.white);
            //g.fillRect(branch.getX() + branch.getWidth()/2 - labelWidth/2, branch.getY()-15, labelWidth, labelHeight);
            g.setColor(Color.black);
            branch.draw(g);
        }
        g.fillRect(X + width/2- labelWidth/2 + 2-xoff, Y-labelHeight + 2-yoff, labelWidth + 2, labelHeight + 2);
        g.setColor(Color.blue);
        if(DO.isAbstract()) {
            g.setColor(Color.yellow);
        }
        if(DO.isFinal()) {
            g.setColor(Color.red);
        }
        g.fillRect(X + width/2- labelWidth/2 - 1+xoff, Y-labelHeight - 1+xoff, labelWidth + 2, labelHeight + 2);
        if(selected) {
            g.setColor(Color.green);
            g.fillRect(X + width/2- labelWidth/2 + 2+xoff, Y-labelHeight + 2+yoff, labelWidth - 5, labelHeight - 4);
        }
        else {
            g.setColor(Color.white);
            g.fillRect(X + width/2- labelWidth/2 + 2+xoff, Y-labelHeight + 2+yoff, labelWidth - 4, labelHeight - 4);
        }
        g.setColor(Color.black);
            Font oldfont = g.getFont(); 
        if(DO.isAbstract()) {
            Font newfont = new Font(oldfont.getName(), Font.ITALIC, oldfont.getSize());
	    g.setFont(newfont);
	}
        g.drawString(DO.getName(), X + width/2 - (labelWidth/2 -10)+xoff, Y-5+yoff);
	g.setFont(oldfont);
    }
    /**
     * @return the current Width of this branch.
     */
    public int getWidth() {
        return width;
    }
    /**
     * @return the current Row of this branch.
     */
    public int getRow() {
        return row;
    }
    public int getX() {
        return X;
    }
    public int getY() {
        return Y;
    }
    public String getName() {
        return DO.getName();
    }
    /**
     * @return if this branch is extended.
     */
    public boolean isExtended() {
        return extended;
    }
    public int getBottomRow() {
        return bottomRow;
    }
    public void addBranches(Vector v) {
        for(int i = 0; i < branches.size(); i++) {
            GraphicalBranch b = (GraphicalBranch)branches.elementAt(i);
            v.addElement(b);
            b.addBranches(v);
        }
    }
    public Rectangle getRectangle() {
        return rectangle;
    }
    public void setMouseOver(boolean bool) {
        mouseOver = bool;
    }
    public boolean getMouseOver() {
        return mouseOver;
    }
    public void setSelected(boolean bool) {
        selected = bool;
    }
    public boolean getSelected() {
        return selected;
    }
    public DataObject getDataObject() {
        return DO;
    }
}





