/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: AttributeEditor.java,v 1.7 1999/06/08 01:06:49 jay Exp $
 *-----------------------------------------------------------------------------
 *
 * Changes in layout and actions to support the 'DODS Type' for
 * attribute properties by David Wood (dwood@pisoftware.com).
 * (Changes to allow a unique column added by Leigh Dodds (ldodds@ingenta.com).
 */


   package dods.gui.editors;

   import dods.gui.*;
   import dods.gui.tree.*;
   import dods.data.*;
   import javax.swing.JDialog;
   import javax.swing.JFrame;
   import javax.swing.JOptionPane;
   import javax.swing.JTree;
   import javax.swing.JButton;
   import javax.swing.JPanel;
   import javax.swing.JTabbedPane;
   import javax.swing.JTextField;
   import javax.swing.JLabel;
   import javax.swing.JScrollPane;
   import javax.swing.JComboBox;
   import javax.swing.JCheckBox;
   import javax.swing.JTextArea;
   import javax.swing.border.TitledBorder;
   import javax.swing.event.TreeSelectionEvent;
   import javax.swing.event.ChangeListener;
   import javax.swing.event.ChangeEvent;
   import javax.swing.tree.TreePath;
   import java.awt.*;
   import java.awt.Dimension;
   import java.awt.GridBagLayout;
   import java.awt.Color;
   import java.awt.GridBagConstraints;
   import java.awt.Component;
   import java.awt.FlowLayout;
   import java.awt.Insets;
   import java.awt.event.ActionListener;
   import java.awt.event.ActionEvent;
   import java.util.Vector;

/**
 * This class allows you to edit an Attribute and all of its attributes.
 *
 * @author  Philip Martin
 * @version $Revision: 1.7 $
 */
   public class AttributeEditor extends JDialog {

      // Instance variables.
      private JPanel jp;
      private JPanel buttonPanel;
      private JButton okButton;
      private JButton cancelButton;
      private JButton helpButton;
      private JTabbedPane jtp;
      private JPanel idPanel;
      private JLabel nameLabel;
      private JTextField name;
      private JLabel viewLabel;
      private JScrollPane treeView;
      private JTextField className;
      private JPanel typePanel;
      private JLabel dodsTypeLabel;
      private JComboBox dodsType;
      private JPanel typesPanel;
      private JLabel javaTypeLabel;
      private JLabel javaTypeValue;
      private JLabel jdbcTypeLabel;
      private JLabel jdbcTypeValue;
      private JLabel dbTypeLabel;
      private JLabel dbTypeValue;
      private JLabel javadocLabel;
      private JScrollPane javadocView;
      private JPanel propertiesPanel;
      private JPanel javaValuePanel;
      private JLabel javaTypeLabel2;
      private JLabel javaTypeValue2;
      private JLabel initJavaLabel;
      private JTextField initJavaValue;
      private JPanel dbValuePanel;
      private JLabel dbTypeLabel2;
      private JLabel dbTypeValue2;
      private JLabel initDBLabel;
      private JTextField initDBValue;
      private JPanel sizeValuePanel;
      private JLabel sizeLabel;
      private JTextField size;
      private JPanel optionPanel;
      private JCheckBox isIndex;
      private JCheckBox canBeNull;
      private JCheckBox usedForQuery;
      private JCheckBox refConstraint;
      private JCheckBox uniqueConstraint;
      private JCheckBox constant;
      /*
      private JPanel arrayPanel;
      private JCheckBox isArray;
      private JPanel arrayAttributesPanel;
      private JPanel arraySizePanel;
      private JLabel arraySizeLabel;
      private JTextField arraySize;
      private TitledBorder arrayAttributesBorder;
      */
      private JTextArea jta;
      private AttributeTreeSelectionListener listener;
      private JTree tree;
      private JTree oldTree;
      private DataObjectApplication doa;
      private DataObject DO;
      private Attribute att;

      // Object which holds all possible DODS Types
      DODSTypes dodsTypes = DODSTypes.getInstance();

      // The list of currently appropriate DODS Types for the
      // selected database.
      String[] curDodsTypes;

    /**
     * This Constructor lays out all of the panels neatly.
     * @param oldtre The original package tree from the parent frame.
     * @param da The parent DataObjectApplication.
     */
      public AttributeEditor(JTree oldtre, DataObjectApplication da) {
         super(da);
        //Initializing the tree.
         oldTree = oldtre;
         doa = da;
         tree = new JTree();
         tree.setMinimumSize(new Dimension(0,0));
        //Initializing the Frame.
         setTitle("AttributeEditor");
         setModal(true);
         setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
         getContentPane().setLayout(new GridBagLayout());
         GridBagConstraints gbc;
         setBackground(Color.lightGray);
        //Initializing the panel container.
         jp = new JPanel();
         jp.setLayout(new GridBagLayout());
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 0;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.fill = GridBagConstraints.BOTH;
         gbc.insets = new Insets(0,0,0,0);
         ((GridBagLayout)getContentPane().getLayout()).setConstraints(jp, gbc);
         getContentPane().add(jp);
        //Initializing Button Panel.
         buttonPanel = new JPanel();
         buttonPanel.setLayout(new FlowLayout(FlowLayout.CENTER,5,5));
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 1;
         gbc.weightx = 1.0;
         gbc.weighty = 0.3;
         gbc.fill = GridBagConstraints.HORIZONTAL;
         gbc.insets = new Insets(0,0,0,5);
         ((GridBagLayout)jp.getLayout()).setConstraints(buttonPanel, gbc);
         jp.add(buttonPanel);
         okButton = new JButton();
         okButton.setEnabled(false);
         okButton.setText("OK");
         buttonPanel.add(okButton);
         cancelButton = new JButton();
         cancelButton.setText("Cancel");
         buttonPanel.add(cancelButton);
         helpButton = new JButton();
         helpButton.setText("Help");
         buttonPanel.add(helpButton);
         helpButton.setEnabled(false);
        //Initializing the TabbedPane.
         jtp = new JTabbedPane();
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 0;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.fill = GridBagConstraints.BOTH;
         gbc.insets = new Insets(10,10,0,10);
         ((GridBagLayout)jp.getLayout()).setConstraints(jtp, gbc);
         jp.add(jtp);

        //Initializing the idPanel.
         idPanel = new JPanel();
         idPanel.setLayout(new GridBagLayout());
         nameLabel = new JLabel();
         nameLabel.setText("Name:");
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 0;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.SOUTHWEST;
         gbc.fill = GridBagConstraints.NONE;
         gbc.insets = new Insets(20,20,2,0);
         ((GridBagLayout)idPanel.getLayout()).setConstraints(nameLabel, gbc);
         idPanel.add(nameLabel);
         name = new JTextField();
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 1;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.NORTHWEST;
         gbc.fill = GridBagConstraints.HORIZONTAL;
         gbc.insets = new Insets(0,20,0,20);
         ((GridBagLayout)idPanel.getLayout()).setConstraints(name, gbc);
         idPanel.add(name);
         viewLabel = new JLabel();
         viewLabel.setText("Place In:");
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 2;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.SOUTHWEST;
         gbc.fill = GridBagConstraints.NONE;
         gbc.insets = new Insets(0,20,2,0);
         ((GridBagLayout)idPanel.getLayout()).setConstraints(viewLabel, gbc);
         idPanel.add(viewLabel);
         treeView = new JScrollPane();
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 3;
         gbc.weightx = 1.0;
         gbc.weighty = 50.0;
         gbc.anchor = GridBagConstraints.NORTHWEST;
         gbc.fill = GridBagConstraints.BOTH;
         gbc.insets = new Insets(0,20,0,20);
         ((GridBagLayout)idPanel.getLayout()).setConstraints(treeView, gbc);
         idPanel.add(treeView);
         className = new JTextField();
         className.setEditable(false);
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 4;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.fill = GridBagConstraints.HORIZONTAL;
         gbc.insets = new Insets(5,20,20,20);
         ((GridBagLayout)idPanel.getLayout()).setConstraints(className, gbc);
         idPanel.add(className);
         className.setEnabled(false);

        //Initializing typePanel.
         typePanel = new JPanel();
         typePanel.setLayout(new GridBagLayout());

         // DODS Type label and pulldown.
         dodsTypeLabel = new JLabel();
         dodsTypeLabel.setText("DODS Type:");
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 0;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.SOUTHWEST;
         gbc.fill = GridBagConstraints.NONE;
         gbc.insets = new Insets(20,20,2,0);
         ((GridBagLayout)typePanel.getLayout()).setConstraints(dodsTypeLabel, gbc);
         typePanel.add(dodsTypeLabel);
         dodsType = new JComboBox();
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 1;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.NORTHWEST;
         gbc.fill = GridBagConstraints.HORIZONTAL;
         gbc.insets = new Insets(0,20,0,20);
         ((GridBagLayout)typePanel.getLayout()).setConstraints(dodsType, gbc);
         typePanel.add(dodsType);

         // Java, JDBC and Database Type labels.
         typesPanel = new JPanel();
         typesPanel.setLayout( new GridLayout(3,2,10,1) );

         javaTypeLabel = new JLabel();
         javaTypeLabel.setText("Java Type: ");
         typesPanel.add( javaTypeLabel );

         javaTypeValue = new JLabel();
         javaTypeValue.setForeground( Color.red );
         javaTypeValue.setText("Not Set");
         typesPanel.add( javaTypeValue );

         jdbcTypeLabel = new JLabel();
         jdbcTypeLabel.setText("JDBC Type: ");
         typesPanel.add( jdbcTypeLabel );

         jdbcTypeValue = new JLabel();
         jdbcTypeValue.setForeground( Color.red );
         jdbcTypeValue.setText("Not Set");
         typesPanel.add( jdbcTypeValue );

         dbTypeLabel = new JLabel();
         dbTypeLabel.setText("Database Type: ");
         typesPanel.add( dbTypeLabel );

         dbTypeValue = new JLabel();
         dbTypeValue.setForeground( Color.red );
         dbTypeValue.setText("Not Set");
         typesPanel.add( dbTypeValue );

         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 2;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.WEST;
         gbc.fill = GridBagConstraints.NONE;
         gbc.insets = new Insets(20,20,20,0);
         ((GridBagLayout)typePanel.getLayout()).setConstraints(typesPanel, gbc);
         typePanel.add(typesPanel);

         // Javadoc label and textarea.
         javadocLabel = new JLabel();
         javadocLabel.setText("Javadoc Text:");
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 5;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.SOUTHWEST;
         gbc.fill = GridBagConstraints.NONE;
         gbc.insets = new Insets(0,20,2,0);
         ((GridBagLayout)typePanel.getLayout()).setConstraints(javadocLabel, gbc);
         typePanel.add(javadocLabel);
         javadocView = new JScrollPane();
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 6;
         gbc.weightx = 1.0;
         gbc.weighty = 50.0;
         gbc.fill = GridBagConstraints.BOTH;
         gbc.insets = new Insets(0,20,20,20);
         ((GridBagLayout)typePanel.getLayout()).setConstraints(javadocView, gbc);
         typePanel.add(javadocView);

        //Initializing propertiesPanel.
         propertiesPanel = new JPanel();
         propertiesPanel.setLayout(new GridBagLayout());

         // Add a panel to hold the Java value info. 
         javaValuePanel = new JPanel();
         javaValuePanel.setLayout(new GridLayout(2,2));
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 0;
         gbc.weightx = 0.0;
         gbc.weighty = 0.0;
         gbc.fill = GridBagConstraints.HORIZONTAL;
         gbc.insets = new Insets(20,20,5,20);
         ((GridBagLayout)propertiesPanel.getLayout()).setConstraints(javaValuePanel, gbc);
         propertiesPanel.add(javaValuePanel);

         // Insert Java Type label onto the java value panel.
         javaTypeLabel2 = new JLabel();
         javaTypeLabel2.setText("Java Type:");
         javaValuePanel.add( javaTypeLabel2 );

         // Insert Java Type value onto the java value panel.
         javaTypeValue2 = new JLabel();
         javaTypeValue2.setForeground( Color.red );
         javaTypeValue2.setText("Not Set");
         javaValuePanel.add( javaTypeValue2 );

         initJavaLabel = new JLabel();
         initJavaLabel.setText("Initial Java Value:");
         javaValuePanel.add( initJavaLabel );

         initJavaValue = new JTextField();
         initJavaValue.setColumns( 10 );
         javaValuePanel.add( initJavaValue );

         // Add a panel to hold the DB value info. 
         dbValuePanel = new JPanel();
         dbValuePanel.setLayout(new GridLayout(2,2));
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 1;
         gbc.weightx = 0.0;
         gbc.weighty = 0.0;
         gbc.fill = GridBagConstraints.HORIZONTAL;
         gbc.insets = new Insets(5,20,5,20);
         ((GridBagLayout)propertiesPanel.getLayout()).setConstraints(dbValuePanel, gbc);
         propertiesPanel.add(dbValuePanel);

         // Insert DB Type label onto the db value panel.
         dbTypeLabel2 = new JLabel();
         dbTypeLabel2.setText("DB Type:");
         dbValuePanel.add( dbTypeLabel2 );

         // Insert DB Type value onto the db value panel.
         dbTypeValue2 = new JLabel();
         dbTypeValue2.setForeground( Color.red );
         dbTypeValue2.setText("Not Set");
         dbValuePanel.add( dbTypeValue2 );

         initDBLabel = new JLabel();
         initDBLabel.setText("Initial DB Value:");
         dbValuePanel.add( initDBLabel );

         initDBValue = new JTextField();
         initDBValue.setColumns( 10 );
         dbValuePanel.add( initDBValue );

         // Add a panel to hold the size value info. 
         sizeValuePanel = new JPanel();
         sizeValuePanel.setLayout(new GridLayout(1,2));
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 2;
         gbc.weightx = 0.0;
         gbc.weighty = 0.0;
         gbc.fill = GridBagConstraints.HORIZONTAL;
         gbc.insets = new Insets(5,20,10,20);
         ((GridBagLayout)propertiesPanel.getLayout()).setConstraints(sizeValuePanel, gbc);
         propertiesPanel.add(sizeValuePanel);

         // Size label and textfield.
         sizeLabel = new JLabel();
         sizeLabel.setText("Size:");
         sizeValuePanel.add(sizeLabel);

         size = new JTextField();
         size.setColumns(10);
         sizeValuePanel.add(size);


         // Option panel.
         optionPanel = new JPanel();
      /*         optionPanel.setLayout(new FlowLayout(FlowLayout.CENTER,5,5)); */
         optionPanel.setLayout(new GridBagLayout());
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 3;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.fill = GridBagConstraints.BOTH;
         gbc.insets = new Insets(0,20,20,20);
         ((GridBagLayout)propertiesPanel.getLayout()).setConstraints(optionPanel, gbc);
         propertiesPanel.add(optionPanel);
      
         gbc.gridy = 0;
         gbc.insets = new Insets(0,10,0,10);
      
         canBeNull = new JCheckBox();
         canBeNull.setText("Can be Null");
         ((GridBagLayout)optionPanel.getLayout()).setConstraints(canBeNull, gbc);
         optionPanel.add(canBeNull);
      
         gbc.gridy++;
      
         isIndex = new JCheckBox();
         isIndex.setText("Is an Index");
         ((GridBagLayout)optionPanel.getLayout()).setConstraints(isIndex, gbc);
         optionPanel.add(isIndex);
      
         gbc.gridy++;
      
         usedForQuery = new JCheckBox();
         usedForQuery.setText("Can be queried");
         ((GridBagLayout)optionPanel.getLayout()).setConstraints(usedForQuery, gbc);
         optionPanel.add(usedForQuery);
      
         gbc.gridy++;
      
         refConstraint = new JCheckBox();
         refConstraint.setText("Referenced DO must exist");
         ((GridBagLayout)optionPanel.getLayout()).setConstraints(refConstraint, gbc);
         optionPanel.add(refConstraint);
      
         gbc.gridy++;
      
         uniqueConstraint = new JCheckBox();
         uniqueConstraint.setText("Unique");
         ((GridBagLayout)optionPanel.getLayout()).setConstraints(uniqueConstraint, gbc
);
         optionPanel.add(uniqueConstraint);

         gbc.gridy++;

         constant = new JCheckBox();
         constant.setText("Value is constant");
         ((GridBagLayout)optionPanel.getLayout()).setConstraints(constant, gbc);
         optionPanel.add(constant);
      
	/* FIX
		disabled until code generation is more robust.

        //Initializing ArrayPanel.
         arrayPanel = new JPanel();
         arrayPanel.setLayout(new GridBagLayout());
         isArray = new JCheckBox();
         isArray.setText("Is an Array");
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 0;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.WEST;
         gbc.fill = GridBagConstraints.NONE;
         gbc.insets = new Insets(20,20,10,0);
         ((GridBagLayout)arrayPanel.getLayout()).setConstraints(isArray, gbc);
         arrayPanel.add(isArray);
         arrayAttributesPanel = new JPanel();
         arrayAttributesPanel.setLayout(new GridBagLayout());
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 1;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.fill = GridBagConstraints.BOTH;
         gbc.insets = new Insets(0,20,20,20);
         ((GridBagLayout)arrayPanel.getLayout()).setConstraints(arrayAttributesPanel, gbc);
         arrayPanel.add(arrayAttributesPanel);
         arraySizePanel = new JPanel();
         arraySizePanel.setLayout(new FlowLayout(FlowLayout.CENTER,5,5));
         gbc = new GridBagConstraints();
         gbc.gridx = 0;
         gbc.gridy = 0;
         gbc.weightx = 1.0;
         gbc.weighty = 1.0;
         gbc.anchor = GridBagConstraints.WEST;
         gbc.fill = GridBagConstraints.NONE;
         gbc.insets = new Insets(0,0,0,0);
         ((GridBagLayout)arrayAttributesPanel.getLayout()).setConstraints(arraySizePanel, gbc);
         arrayAttributesPanel.add(arraySizePanel);
         arraySizeLabel = new JLabel();
         arraySizeLabel.setText("Size:");
         arraySizePanel.add(arraySizeLabel);
         arraySize = new JTextField();
         arraySize.setColumns(5);
         arraySizePanel.add(arraySize);
	*/

        //Initializing Tree view.
         listener = new AttributeTreeSelectionListener(tree, className);
         tree.addTreeSelectionListener(listener);
         tree.setMinimumSize(new Dimension(0,0));
         treeView.setViewportView(tree);
         treeView.setMinimumSize(new Dimension(0,0));

        //Initializing borders.
         javaValuePanel.setBorder(new TitledBorder(""));
         dbValuePanel.setBorder(new TitledBorder(""));
         sizeValuePanel.setBorder(new TitledBorder(""));
         optionPanel.setBorder(new TitledBorder("Options"));
	 /*
         arrayAttributesBorder = new TitledBorder("Array Attributes");
         arrayAttributesPanel.setBorder(arrayAttributesBorder);
	 */

        //Initializing javadoc TextArea.
         jta = new JTextArea();
         jta.setMinimumSize(new Dimension(0,0));
         javadocView.setViewportView(jta);
         javadocView.setPreferredSize(new Dimension(0,0));

        //Other Frame Initializations.
         setSize(350,500);
         setResizable(false);
         jtp.addTab("Identification", idPanel);
         jtp.addTab("Type", typePanel);
         jtp.addTab("Properties", propertiesPanel);
	 /*
         jtp.addTab("Array", arrayPanel);
	 */

        //Initializing Listeners.
         SymAction lSymAction = new SymAction();
         okButton.addActionListener(lSymAction);
         cancelButton.addActionListener(lSymAction);
         name.addActionListener(lSymAction);
         dodsType.addActionListener(lSymAction);
         initJavaValue.addActionListener(lSymAction);
         SymChange lSymChange = new SymChange();
	 /*
         isArray.addChangeListener(lSymChange);
	 */

      }

    /*
     * When this method is called, All of the Attribute's attributes are set
     * to the appropriate values and then the dialog is hidden.
     */
      private void okButton_actionPerformed(ActionEvent event) {
      //System.err.println("AttributeEditor.okButton_actionPerformed");
         if(className.getText().equals("")) {
            jtp.setSelectedIndex(0);
            JOptionPane.showMessageDialog(this, 
               "Please select a class for this Attribute.",
               "Error creating Attribute.",
               JOptionPane.WARNING_MESSAGE);
            return;
         }
         DataObject DOA = ((DOTreeNode)tree.getLastSelectedPathComponent()).getDataObject();
         if(DO.getAttributes().contains(att)) {
            DO.getAttributes().removeElement(att);
         }
         if(!DOA.getAttributes().contains(att)) {
            DOA.addAttribute(att);
         }
         att.setName(name.getText());

         att.setDODSType(dodsType.getSelectedItem().toString());

         att.setInitialJavaValue(initJavaValue.getText());
         att.setInitialDBValue(initDBValue.getText());
         att.setJavadoc(jta.getText());
         att.setUnique(uniqueConstraint.isSelected()) ;
         att.setIndex(isIndex.isSelected());

         // Make sure that we don't allow a field to be null for a Java primitive.
         String curDodsType = att.getDODSType();
         String javatype = dodsTypes.getJavaType( curDodsType );
         if ( ! dodsTypes.isJavaPrimitive( javatype ) ) {
           att.setCanBeNull(canBeNull.isSelected());
         } else {
           att.setCanBeNull( false );
         }

         att.setUsedForQuery(usedForQuery.isSelected());
         att.setRefConstraint(refConstraint.isSelected());
         att.setConstant(constant.isSelected());
         att.setSize(size.getText());

// TODO - Do something about these commentted areas!
	 /*
         att.setArray(isArray.isSelected());
	 */
          if(dodsType.getSelectedItem() instanceof DataObject) {
             att.setObjectRefDO((DataObject)dodsType.getSelectedItem());
          }
          else {
            att.setObjectRefDO(null);
          }
	 /*
         try{
            att.setArraySize((new Integer(arraySize.getText())).intValue());
         }
         catch(NumberFormatException e) {
               att.setArraySize(0);
         }
	 */
	 /*
         if(dbType.getSelectedItem().toString().equals("none")){
            att.setObjectRef(true);
         }
         else{
            att.setObjectRef(false);
         }
	 */
      
      	//this ensures that the new attribute will show in the table of attributes
         doa.refreshDODS();
      	//hide this window
         setVisible(false);
      }
    /*
     * This Method just hides the dialog when cancel is pressed.
     */
      private void cancelButton_actionPerformed(ActionEvent event) {
         setVisible(false);
      }
    /*
     * When enter is pressed in the name field, it is as if the ok button was pressed.
     */
      private void name_actionPerformed(ActionEvent event) {
         okButton.doClick();
      }
    /*
     * This method initializes the choices for DODS Types.
     */
      private void initializeDodsChoices() {

         // Get the appropriate DODS Types for the
         // selected DBMS.
         curDodsTypes = dodsTypes.getDODSTypes();

         try {
            dodsType.removeAllItems();
         }
            catch(IndexOutOfBoundsException e){
            }
         Vector DOS = new Vector();
         ((DOPackage)((DOTreeNode)tree.getModel().getRoot()).getDataObject()).getAllDataObjects(DOS);

         // Fill the Vector with appropriate DODS Types.
         for ( int i = 0 ; i < curDodsTypes.length ; i++ ) {
           DOS.addElement( curDodsTypes[ i ] );
         }

         // Add DODS Types to the pulldown.
         for(int i = 0; i < DOS.size(); i++) {
           dodsType.addItem( DOS.elementAt(i) );
         }

      }

    /*
     *  When the dodsType is chosen, the subtypes need to be updated
     *  and widgets changed.
     */
     private void dodsType_actionPerformed(ActionEvent event) {

       // Take actions when the DODS type is selected or unselected.

       String curDodsType;  // The current DODS type selected.
       String javatype;     // The Java type for the current DODS type.

       if ( ( null == dodsType.getSelectedItem() ) || ( 0 > dodsType.getSelectedItem().toString().length() ) ) {

         javaTypeValue.setForeground( Color.red );
         jdbcTypeValue.setForeground( Color.red );
         dbTypeValue.setForeground( Color.red );
         javaTypeValue2.setForeground( Color.red );
         dbTypeValue2.setForeground( Color.red );

         // Clear the labels showing the Java, JDBC and Database
         // types.
         javaTypeValue.setText( "Not Set" );
         jdbcTypeValue.setText( "Not Set" );
         dbTypeValue.setText( "Not Set" );
         javaTypeValue2.setText( "Not Set" );
         dbTypeValue2.setText( "Not Set" );

         // Disable the initial value entry textfields.
         initJavaValue.setBackground(Color.lightGray);
         initJavaValue.setEditable( false );
         initDBValue.setBackground(Color.lightGray);
         initDBValue.setEditable( false );

         // Ghost the size textfield.
         size.setBackground(Color.lightGray);
         size.setEditable( false );


         // Make sure we ensure that the component is valid here or it will
         // clear the values upon initialization!
         if ( ((JComboBox)event.getSource()).isValid() ) {
           // Clear the initial values and size.
           initJavaValue.setText( "" );
           initDBValue.setText( "" );
           size.setText( "" );
         }

         // Disable the option checkboxes.
         isIndex.setEnabled( false );
         canBeNull.setEnabled( false );
         usedForQuery.setEnabled( false );
         refConstraint.setEnabled( false );
         uniqueConstraint.setEnabled( false );
         constant.setEnabled( false );

         // Disable the OK button.
         okButton.setEnabled(false);
       
       } else {

         // We should have a valid DODS type.

         curDodsType = dodsType.getSelectedItem().toString();
         javatype = dodsTypes.getJavaType( curDodsType );

         javaTypeValue.setForeground( Color.black );
         jdbcTypeValue.setForeground( Color.black );
         dbTypeValue.setForeground( Color.black );
         javaTypeValue2.setForeground( Color.black );
         dbTypeValue2.setForeground( Color.black );

         // Set the labels showing the Java, JDBC and Database
         // types associated with this DODS type.
         javaTypeValue.setText( dodsTypes.getJavaType( curDodsType ) );
         jdbcTypeValue.setText( dodsTypes.getJDBCType( curDodsType ) );
         dbTypeValue.setText( dodsTypes.getSQLType( curDodsType ) );
         javaTypeValue2.setText( dodsTypes.getJavaType( curDodsType ) );
         dbTypeValue2.setText( dodsTypes.getSQLType( curDodsType ) );

         if ( ! ( att.isObjectRef() ) ) {

           // Enable the initial value entry textfields if the DODS type is
           // not a DataObject.
           if ( dodsType.getSelectedItem() instanceof DataObject ) {
             initJavaValue.setBackground(Color.lightGray);
             initJavaValue.setEditable( false );
             //initJavaValue.setText( "" );
             initDBValue.setBackground(Color.lightGray);
             initDBValue.setEditable( false );
             //initDBValue.setText( "" );
           } else {
             initJavaValue.setBackground(Color.white);
             initJavaValue.setEditable( true );
             initDBValue.setBackground(Color.white);
             initDBValue.setEditable( true );
           }

           // (un)Ghost the size textfield depending on the DODS type
           // selected.
           if ( dodsTypes.isSizeAllowed( curDodsType ) ) {
             size.setBackground(Color.white);
             size.setEditable( true );
           } else {
             size.setBackground(Color.lightGray);
             size.setEditable( false );
             //size.setText( "" );
           }

           // Check to see if the DODS Type for this Attribute has
           // changed.  If it has, clear the initial values and the size.
           if ( ! curDodsType.equals(att.getDODSType() ) ) {

             // This is a bit of a hack :)
             // We need to ignore changes to curDodsType when the JComboBox component
             // is initialized.  So, we check to see if it is invalid.  If so, we
             // know that the "change" that we are seeing is really just the initial
             // setting of the component's value so we can ignore it.
             if ( ((JComboBox)event.getSource()).isValid() ) {
               initJavaValue.setText( "" );
               initDBValue.setText( "" );
               size.setText( "" );
             }

           }

         } else {

           // The Attribute is an Object reference.
           // Ghost and disable all fields.
           initJavaValue.setBackground(Color.lightGray);
           initJavaValue.setEditable( false );
           initJavaValue.setText( "" );
           initDBValue.setBackground(Color.lightGray);
           initDBValue.setEditable( false );
           initDBValue.setText( "" );
           size.setBackground(Color.lightGray);
           size.setEditable( false ); 
           size.setText( "" );

         } // end if

         // Selectively enable the option checkboxes.
         isIndex.setEnabled( true );
         if ( ! dodsTypes.isJavaPrimitive( javatype ) ) {
           canBeNull.setEnabled( true );
         } else {
           canBeNull.setEnabled( false );
           canBeNull.setSelected( false );
         }
         usedForQuery.setEnabled( true );
         refConstraint.setEnabled( true );
         uniqueConstraint.setEnabled( true );
         constant.setEnabled( true );

         // Enable the OK button.
         okButton.setEnabled(true);
       
       } // end if/else

     }

    /*
     *  When the initial Java value is completed, the initial DODS value may need to be updated.
     */
     private void initJavaValue_actionPerformed(ActionEvent event) {

       // If the initial DODS value is not empty, copy the initial Java value to it.
       if ( ( null == initDBValue.getText() ) || ( initDBValue.getText().equals("") ) ) {
         initDBValue.setText( initJavaValue.getText() );
       }

     }

    /*
     * When the isArray choice changes, the array options are either
     * enabled or disabled.
     */
      /*
      private void isArray_stateChanged(ChangeEvent event) {
         if(isArray.isSelected()) {
            arraySize.setEnabled(true);
            arraySize.setBackground(Color.white);
            arraySizeLabel.setForeground(Color.black);
            arrayAttributesBorder.setTitleColor(Color.black);
         }
         else {
            arraySize.setEnabled(false);
            arraySize.setBackground(Color.lightGray);
            arraySizeLabel.setForeground(Color.gray);
            arrayAttributesBorder.setTitleColor(Color.gray);
         }
         repaint();
      }
      */
    /**
     * This method initializes all of the fields to their appropriate values given
     * the attribute being edited and the DataObject it is in.
     * @param at The attribute to be edited.
     * @param Do The DataObject it is in.
     */
      public void setAttribute(Attribute at, DataObject Do) {
         jtp.setSelectedIndex(0);
         att = at;
         DO = Do;
         jta.setText(att.getJavadoc());
         name.setText(att.getName());
         initJavaValue.setText(att.getInitialJavaValue());
         initDBValue.setText(att.getInitialDBValue());
         isIndex.setSelected(att.isIndex());
         canBeNull.setSelected(att.canBeNull());
         usedForQuery.setSelected(att.isUsedForQuery());
         refConstraint.setSelected(att.isRefConstraint());
         uniqueConstraint.setSelected(att.isUnique()) ;
         constant.setSelected(att.isConstant());
         size.setText(att.getSize());
	 /*
         isArray.setSelected(att.isArray());
         arraySize.setText(att.getArraySize()+"");
         if(!att.isArray()) {
            arraySize.setEnabled(false);
            arraySize.setBackground(Color.lightGray);
            arraySizeLabel.setForeground(Color.gray);
            arrayAttributesBorder.setTitleColor(Color.gray);
         }
	 */
         DOTreeModel treeModel = new DOTreeModel((DOTreeNode)oldTree.getModel().getRoot(), doa);
         tree.setModel(treeModel);
         tree.setCellRenderer(new DOTreeRenderer());
         tree.setSelectionPath(oldTree.getSelectionPath());
         tree.expandPath(oldTree.getSelectionPath());
         initializeDodsChoices();
         if(att.getObjectRefDO()!=null) {
            dodsType.setSelectedItem(att.getObjectRefDO());
         }
         else {
            dodsType.setSelectedItem(att.getDODSType());
         }
         if(tree.getSelectionPath()==null){
            Object[] rootpath = {tree.getModel().getRoot()};
            tree.setSelectionPath(new TreePath(rootpath));
         }
         listener.valueChanged(new TreeSelectionEvent(this, null, null, null, null));
      }

    /*
     *
     * Exposes the JTextField containing the Attribute's name.
     *
     */
      public JTextField getNameComponent() {
        return name;
      }

    /*
     *
     * the inner classes are just listeners for events.
     *
     */
      class SymAction implements ActionListener {
         public void actionPerformed(ActionEvent event) {
            Object object = event.getSource();
            if (object == okButton)
               okButton_actionPerformed(event);
            else if (object == cancelButton)
               cancelButton_actionPerformed(event);
            else if (object == name)
               name_actionPerformed(event);
            else if (object == dodsType)
               dodsType_actionPerformed(event);
            else if (object == initJavaValue)
               initJavaValue_actionPerformed(event);
         }
      }
      class SymChange implements ChangeListener {
         public void stateChanged(ChangeEvent event) {
            Object object = event.getSource();
	    /*
            if (object == isArray)
               isArray_stateChanged(event);
	    */
         }
      }
   }
