/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: DataObjectEditor.java,v 1.7 1999/06/17 20:55:17 jay Exp $
 *-----------------------------------------------------------------------------
 */



package dods.gui.editors;

import dods.gui.*;
import dods.data.*;
import dods.gui.tree.*;
import dods.exceptions.*;
import javax.swing.JFrame;
import javax.swing.ButtonGroup;
import javax.swing.JDialog;
import javax.swing.JTabbedPane;
import javax.swing.JOptionPane;
import javax.swing.JTree;
import javax.swing.JScrollPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JComboBox;
import javax.swing.JCheckBox;
import javax.swing.JRadioButton;
import javax.swing.border.TitledBorder;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import javax.swing.tree.TreePath;
import java.awt.Color;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Dimension;
import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.Vector;
/**
 * This class is a pop-up window which lets you edit all of a DataObject's
 * attributes (not Attributes) in a convenient window.
 *
 * @author  Philip Martin
 * @version $Revision: 1.7 $
 */
public final class DataObjectEditor extends JDialog {
    /**
     * This Constructor lays out all of the input devices.
     * @param tre The JTree from the DataObjectApplication.
     * @param da The Parent DataObjectApplication.
     */
    public DataObjectEditor(JTree tre, DataObjectApplication da) {
        super(da);
        //Initializing the tree for the package selection.
        oldTree = tre;
        doa = da;
        tree = new JTree();
        DOEditorTreeModel treeModel = new DOEditorTreeModel((DOTreeNode)oldTree.getModel().getRoot(), doa, new DOPackage("adsfasdfadsfqp"));
        tree.setModel(treeModel);
        tree.setMinimumSize(new Dimension(0,0));
        //Initializing the Frame.
        setTitle("Data Object Editor");
        setResizable(false);
        setModal(true);
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        setBackground(Color.lightGray);
        getContentPane().setLayout(new GridBagLayout());
        GridBagConstraints gbc = new GridBagConstraints();
        setSize(300,500);
        JPanel jp = new JPanel();
        jp.setLayout(new GridBagLayout());
        //ScrollPanePanel initialization
        scrollPanePanel = new JPanel();
        scrollPanePanel.setLayout(new GridBagLayout());
        treeView = new JScrollPane();
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 2;
        gbc.weightx = 1.0;
        gbc.weighty = 50.0;
        gbc.anchor = GridBagConstraints.NORTH;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.insets = new Insets(2,20,0,20);
        ((GridBagLayout)scrollPanePanel.getLayout()).setConstraints(treeView, gbc);
        scrollPanePanel.add(treeView);
        packageName = new JTextField();
        packageName.setMargin(new java.awt.Insets(0,0,0,0));
        packageName.setEditable(false);
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 3;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.anchor = GridBagConstraints.NORTH;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(10,20,20,20);
        ((GridBagLayout)scrollPanePanel.getLayout()).setConstraints(packageName, gbc);
        scrollPanePanel.add(packageName);
        packageName.setEnabled(false);
        packageLabel = new JLabel();
        packageLabel.setText("Place In:");
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.anchor = GridBagConstraints.SOUTHWEST;
        gbc.fill = GridBagConstraints.NONE;
        gbc.insets = new Insets(10,20,0,0);
        ((GridBagLayout)scrollPanePanel.getLayout()).setConstraints(packageLabel, gbc);
        scrollPanePanel.add(packageLabel);
        //DBPanel Initialization.
        dbPanel = new JPanel();
        dbPanel.setLayout(new GridBagLayout());
        tableNameLabel = new JLabel();
        tableNameLabel.setText("db Table Name:");
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.weightx = 0.5;
        gbc.weighty = 1.0;
        gbc.anchor = GridBagConstraints.SOUTHWEST;
        gbc.fill = GridBagConstraints.NONE;
        gbc.insets = new Insets(0,20,0,0);
        ((GridBagLayout)dbPanel.getLayout()).setConstraints(tableNameLabel, gbc);
        dbPanel.add(tableNameLabel);
        tableName = new JTextField();
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.weightx = 20.0;
        gbc.weighty = 1.0;
        gbc.anchor = GridBagConstraints.NORTHWEST;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(2,20,0,20);
        ((GridBagLayout)dbPanel.getLayout()).setConstraints(tableName, gbc);
        dbPanel.add(tableName);
        dbTypePanel = new JPanel();
        dbTypePanel.setLayout(new GridBagLayout());
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 2;
        gbc.gridwidth = 2;
        gbc.weightx = 1.0;
        gbc.weighty = 2.0;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.insets = new Insets(0,20,20,20);
        ((GridBagLayout)dbPanel.getLayout()).setConstraints(dbTypePanel, gbc);
        dbPanel.add(dbTypePanel);
        isEveryClass = new JRadioButton();
        isEveryClass.setText("Every Class");
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(0,10,0,10);
        ((GridBagLayout)dbTypePanel.getLayout()).setConstraints(isEveryClass, gbc);
        dbTypePanel.add(isEveryClass);
        isEveryClass.setEnabled(false);
        isConcreteClass = new JRadioButton();
        isConcreteClass.setText("Every Concrete Class");
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(0,10,0,10);
        ((GridBagLayout)dbTypePanel.getLayout()).setConstraints(isConcreteClass, gbc);
        dbTypePanel.add(isConcreteClass);
        isHierarchy = new JRadioButton();
        isHierarchy.setText("Entire Hierarchy");
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 2;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(0,10,0,10);
        ((GridBagLayout)dbTypePanel.getLayout()).setConstraints(isHierarchy, gbc);
        dbTypePanel.add(isHierarchy);

        isLazyLoading = new JRadioButton();
        isLazyLoading.setText("Lazy Loading");
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 3;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(0,10,0,10);
        ((GridBagLayout)dbTypePanel.getLayout()).setConstraints(isLazyLoading, gbc);
        dbTypePanel.add(isLazyLoading);

        isCaching = new JRadioButton();
        isCaching.setText("Cached");
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 4;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(0,10,0,10);
        ((GridBagLayout)dbTypePanel.getLayout()).setConstraints(isCaching, gbc);
        dbTypePanel.add(isCaching);

        isFullCaching = new JRadioButton();
        isFullCaching.setText("Fully Cached");
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 5;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(0,10,0,10);
        ((GridBagLayout)dbTypePanel.getLayout()).setConstraints(isFullCaching, gbc);
        dbTypePanel.add(isFullCaching);

        isLazyLoading.setEnabled(true);
        isCaching.setEnabled(true);
        isFullCaching.setEnabled(true);

        isEveryClass.setEnabled(false);
        isHierarchy.setEnabled(false);
        //ButtonPanel initialization.
        buttonPanel = new JPanel();
        buttonPanel.setLayout(new FlowLayout(FlowLayout.CENTER,5,5));
        okButton = new JButton();
        okButton.setText("OK");
        buttonPanel.add(okButton);
        cancelButton = new JButton();
        cancelButton.setText("Cancel");
        buttonPanel.add(cancelButton);
        helpButton = new JButton();
        helpButton.setText("Help");
        buttonPanel.add(helpButton);
        helpButton.setEnabled(false);
        //ClassPanel initialization.
        classPanel = new JPanel();
        classPanel.setLayout(new GridBagLayout());
        nameLabel = new JLabel();
        nameLabel.setText("Name:");
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.anchor = GridBagConstraints.SOUTHWEST;
        gbc.fill = GridBagConstraints.NONE;
        gbc.insets = new Insets(0,20,0,0);
        ((GridBagLayout)classPanel.getLayout()).setConstraints(nameLabel, gbc);
        classPanel.add(nameLabel);
        extendsLabel = new JLabel();
        extendsLabel.setText("Extends:");
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 2;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.anchor = GridBagConstraints.SOUTHWEST;
        gbc.fill = GridBagConstraints.NONE;
        gbc.insets = new Insets(0,20,0,0);
        ((GridBagLayout)classPanel.getLayout()).setConstraints(extendsLabel, gbc);
        classPanel.add(extendsLabel);
        name = new JTextField();
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.anchor = GridBagConstraints.NORTHWEST;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(2,20,0,20);
        ((GridBagLayout)classPanel.getLayout()).setConstraints(name, gbc);
        classPanel.add(name);
        extensionOf = new JComboBox();
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 3;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.anchor = GridBagConstraints.NORTHWEST;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(2,20,0,20);
        ((GridBagLayout)classPanel.getLayout()).setConstraints(extensionOf, gbc);
        classPanel.add(extensionOf);
        typePanel = new JPanel();
        typePanel.setLayout(new FlowLayout(FlowLayout.CENTER,5,5));
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 4;
        gbc.gridwidth = 2;
        gbc.weightx = 1.0;
        gbc.weighty = 0.5;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(0,20,20,20);
        ((GridBagLayout)classPanel.getLayout()).setConstraints(typePanel, gbc);
        classPanel.add(typePanel);
        isAbstract = new JCheckBox();
        isAbstract.setText("Abstract");
        typePanel.add(isAbstract);
        isFinal = new JCheckBox();
        isFinal.setText("Final");
        typePanel.add(isFinal);
        //Initializing borders.
        TitledBorder border1 = new TitledBorder("Modifiers");
        border1.setTitlePosition(TitledBorder.TOP);
        border1.setTitleJustification(TitledBorder.LEFT);
        typePanel.setBorder(border1);
        TitledBorder border2 = new TitledBorder("DataBase Mapping Type");
        border2.setTitlePosition(TitledBorder.TOP);
        border2.setTitleJustification(TitledBorder.LEFT);
        dbTypePanel.setBorder(border2);
        //setting up the treeView.
        listener = new DOEditorTreeSelectionListener(tree, packageName);
        tree.addTreeSelectionListener(listener);
        tree.setMinimumSize(new Dimension(0,0));
        treeView.setViewportView(tree);
        treeView.setMinimumSize(new Dimension(0,0));
        //Initializing button group.
        ButtonGroup bg = new ButtonGroup();
        bg.add(isEveryClass);
        bg.add(isConcreteClass);
        bg.add(isHierarchy);

        //Initializing listeners.
        SymAction lSymAction = new SymAction();
        cancelButton.addActionListener(lSymAction);
        okButton.addActionListener(lSymAction);
        name.addActionListener(lSymAction);
        tableName.addActionListener(lSymAction);
        SymChange lSymChange = new SymChange();
        isEveryClass.addChangeListener(lSymChange);
        isConcreteClass.addChangeListener(lSymChange);
        isHierarchy.addChangeListener(lSymChange);

        isLazyLoading.addChangeListener(lSymChange);
        isCaching.addChangeListener(lSymChange);
        isFullCaching.addChangeListener(lSymChange);

        isAbstract.addChangeListener(lSymChange);
        extensionOf.addActionListener(lSymAction);
        //adding buttonPanel
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.weightx = 1.0;
        gbc.weighty = 0.3;
        gbc.anchor = GridBagConstraints.CENTER;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(0,0,0,5);
        ((GridBagLayout)jp.getLayout()).setConstraints(buttonPanel, gbc);
        jp.add(buttonPanel);
        //initializing the TabbedPane
        jtp = new JTabbedPane();
        jtp.addTab("Class", classPanel);
        jtp.addTab("Package", scrollPanePanel);
        jtp.addTab("DataBase", dbPanel);
        //Adding the TabbedPane
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.anchor = GridBagConstraints.CENTER;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.insets = new Insets(10,10,0,10);
        ((GridBagLayout)jp.getLayout()).setConstraints(jtp, gbc);
        jp.add(jtp);
        //Adding the Panel.
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.anchor = GridBagConstraints.CENTER;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.insets = new Insets(0,0,0,0);
        ((GridBagLayout)getContentPane().getLayout()).setConstraints(jp, gbc);
        getContentPane().add(jp);
    }
    
    private DataObject DO;
    private JTree tree;
    private JTree oldTree;
    private DataObjectApplication doa;
    private JTabbedPane jtp;
    private DOEditorTreeSelectionListener listener;
    private JPanel scrollPanePanel;
    private JScrollPane treeView;
    private JTextField packageName;
    private JLabel packageLabel;
    private JPanel dbPanel;
    private JLabel tableNameLabel;
    private JTextField tableName;
    private JPanel dbTypePanel;
    private JRadioButton isEveryClass;
    private JRadioButton isConcreteClass;
    private JRadioButton isHierarchy;
    private JRadioButton isLazyLoading;
    private JRadioButton isCaching;
    private JRadioButton isFullCaching;
    private JPanel buttonPanel;
    private JButton okButton;
    private JButton cancelButton;
    private JButton helpButton;
    private JPanel classPanel;
    private JLabel nameLabel;
    private JLabel extendsLabel;
    private JTextField name;
    private JComboBox extensionOf;
    private JPanel typePanel;
    private JCheckBox isAbstract;
    private JCheckBox isFinal;
    
    /*
     * This method hides the Dialog when cancel is pressed.
     */
    private void cancelButton_actionPerformed(ActionEvent event) {
        isAbstract.setEnabled(true);
        setVisible(false);
    }
    /*
     * This method changes all of the DataObject's attributes and hides the dialog.
     */
    private void okButton_actionPerformed(ActionEvent event) {
        //sets the DBTableName if the dataObject has a DBTable.
        if(tableName.isEnabled()) {
            try {
                DO.setDBTableName(tableName.getText());
            }
            catch(DataObjectNameException e) {
                doa.namingExceptionOccurred(e, this);
                return;
            }
        }
        //FIX:  If multiple types are allowed, this will have to be changed
        try {
            DO.setDBTableType(DataObject.CONCRETE);
        }
        catch(DataObjectTypeException e) {
        }
        //set the name
        try {
            DO.setName(name.getText());
        }
        catch(DataObjectNameException e) {
            doa.namingExceptionOccurred(e, this);
            return;
        }
        //sets what DataObject this extends.
        if(extensionOf.getSelectedItem() instanceof String) {
            DO.setExtensionOf(null);
        }
        else {
            DO.setExtensionOf((DataObject)extensionOf.getSelectedItem());
	    ((DataObject)extensionOf.getSelectedItem()).setAbstract(true);
        }

        DO.setLazyLoading(isLazyLoading.isSelected());
        DO.setCaching(isCaching.isSelected());
        DO.setFullCaching(isFullCaching.isSelected());

        DO.setAbstract(isAbstract.isSelected());
        DO.setFinal(isFinal.isSelected());
        //Sets the Package this dataObject goes into.
        DOTreeNode newNode = new DOTreeNode(DO, false);
        if(!packageName.getText().equals(DO.getPackageName()) || DO.getPackageName().equals("")) {
            DOTreeNode newParent = (DOTreeNode)tree.getLastSelectedPathComponent();
            for(int i = 0; i < newParent.getChildren().size(); i++) {
                DOTreeNode donode = (DOTreeNode)newParent.getChildren().elementAt(i);
                if(donode.getDataObject().getName().equals(name.getText()) && donode.getDataObject() != DO) {
		    jtp.setSelectedIndex(0);
                    JOptionPane.showMessageDialog(this,
                                      "An object with that Name already exists in this package.",
                                      "Error editing Data Object.",
                                      JOptionPane.WARNING_MESSAGE);
                    return;
                }
            }
            Vector allDOS = new Vector();
            ((DOPackage)((DOTreeNode)oldTree.getModel().getRoot()).getDataObject()).getAllDataObjects(allDOS);
            for(int i = 0; i < allDOS.size(); i++) {
                DataObject dobject = (DataObject)allDOS.elementAt(i);
                if(dobject.getDBTableName().equals(tableName.getText()) && dobject != DO && !(DO.isAbstract() || dobject.isAbstract())) {
                    jtp.setSelectedIndex(2);
		    JOptionPane.showMessageDialog(this,
                                      "An object with that DB Table Name already exists.",
                                      "Error editing Data Object.",
                                      JOptionPane.WARNING_MESSAGE);
                    return;
                }
            }
            DOTreeNode findnode = ((DOTreeNode)((DOTreeModel)tree.getModel()).getRoot()).findNodeWith(DO);
            DOTreeNode dotn;
            Vector children;
            if(findnode != null) {
                children = findnode.getChildren();
            }
            else {
                children = new Vector();
            }
            for(int i = 0; i < children.size(); i++) {
                dotn = (DOTreeNode)children.elementAt(i);
                if(DO.equals(dotn.getDataObject())){
                    children.removeElement(dotn);
                    ((DOPackage)findnode.getDataObject()).getDataObjects().removeElement(dotn.getDataObject());
                    break;
                }
            }
            ((DOPackage)newParent.getDataObject()).addDataObject(DO);
            newParent.getChildren().addElement(newNode);
        }
        else {
            DOTreeNode newParent = (DOTreeNode)tree.getLastSelectedPathComponent();
            for(int i = 0; i < newParent.getChildren().size(); i++) {
                DOTreeNode donode = (DOTreeNode)newParent.getChildren().elementAt(i);
                if(donode.getDataObject().getName().equals(name.getText()) && donode.getDataObject() != DO) {
                    jtp.setSelectedIndex(0);
		    JOptionPane.showMessageDialog(this,
                                      "An object with that Name already exists in this package.",
                                      "Error editing Data Object.",
                                      JOptionPane.WARNING_MESSAGE);
                    return;
                }
            }
            Vector allDOS = new Vector();
            ((DOPackage)((DOTreeNode)oldTree.getModel().getRoot()).getDataObject()).getAllDataObjects(allDOS);
            for(int i = 0; i < allDOS.size(); i++) {
                DataObject dobject = (DataObject)allDOS.elementAt(i);
                if(dobject.getDBTableName().equals(tableName.getText()) && dobject != DO && !(DO.isAbstract() || dobject.isAbstract())) {
                    jtp.setSelectedIndex(2);
		    JOptionPane.showMessageDialog(this,
                                      "An Object with that DB Table Name already exists.",
                                      "Error editing Data Object.",
                                      JOptionPane.WARNING_MESSAGE);
                    return;
                }
            }
        }
        //The following makes sure that the DataObject is selected after
        //it is created.
        TreePath path = tree.getSelectionPath();
        Object[] newPathComponents = new Object[path.getPath().length +1];
        for(int i = 0; i < path.getPath().length; i++) {
            newPathComponents[i] = path.getPath()[i];
        }
        newPathComponents[path.getPath().length] = newNode;
        TreePath newPath = new TreePath(newPathComponents);
        //Final wrap up.
        doa.updateTree();
        oldTree.setSelectionPath(newPath);
        doa.updateTable();
	isAbstract.setEnabled(true);
        setVisible(false);
    }
    /*
     * When you hit enter in the name Field, it's as if the okButton was pressed.
     */
    private void name_actionPerformed(ActionEvent event) {
        okButton.doClick();
    }
    /*
     * When you hit enter in the table name Field, it's as if the okButton was pressed.
     */
    private void tableName_actionPerformed(ActionEvent event) {
        okButton.doClick();
    }
    /*
     * This Method checks the mapping type, and depending on what type of
     * mapping is chosen, will enable or disable the option to set the table name.
     */
    private void checkEnabledMapping() {
        if(isEveryClass.isSelected()){
            tableName.setEnabled(true);
            tableName.setBackground(Color.white);
            tableNameLabel.setForeground(Color.black);
        }
        else if(isConcreteClass.isSelected()){
            if(isAbstract.isSelected()){
                tableName.setEnabled(false);
                tableName.setBackground(Color.lightGray);
                tableNameLabel.setForeground(Color.gray);
            }
            else {
                tableName.setEnabled(true);
                tableName.setBackground(Color.white);
                tableNameLabel.setForeground(Color.black);
            }
        }
        else if(isHierarchy.isSelected()){
            if(extensionOf.getSelectedItem() instanceof String) {
                tableName.setEnabled(true);
                tableName.setBackground(Color.white);
                tableNameLabel.setForeground(Color.black);
            }
            else {
                tableName.setEnabled(false);
                tableName.setBackground(Color.lightGray);
                tableNameLabel.setForeground(Color.gray);
            }
        }

        if(isLazyLoading.isSelected()){
        }
        if(isCaching.isSelected()){
            isFullCaching.setEnabled(true);
        } else {
            isFullCaching.setEnabled(false);
/*             isFullCaching.setSelected(false); */
	}
        if(isFullCaching.isSelected()){
		// FIX recursion if either of these is uncommented
/*             isCaching.setSelected(true); */
        } else {
/*             isCaching.setSelected(false); */
	}
    }
    /**
     * This method sets up the DataObject which will be edited by the DOE.
     * It also initializes all of the fields to what the value is in the DO.
     * @param Do The new DataObject.
     */
    public void setDataObject(DataObject Do) {
        jtp.setSelectedIndex(0);
        DO = Do;
        tableName.setText(DO.getDBTableName());
        tableName.setEnabled(!DO.isAbstract());
        isConcreteClass.setSelected(true);//FIX:if multiple mapping types
                                          //    are allowed, this will need
                                          //    to be changed.
        name.setText(DO.getName());
        isAbstract.setSelected(DO.isAbstract());

	if ( DO.isLazyLoading() ) {
	    isLazyLoading.setEnabled( true );
	    isLazyLoading.setSelected( true );
	} else {
	    isLazyLoading.setSelected( false );
	}
	if ( DO.isCaching() ) {
	    isCaching.setEnabled( true );
	    isCaching.setSelected( true );
	    isFullCaching.setEnabled( true );
	    if ( DO.isFullCaching() ) {
		isFullCaching.setSelected( true );
	    }
	} else {
	    isCaching.setSelected( false );
	    isFullCaching.setSelected( false );
	}

	if(DO.getExtendedBy().size() != 0) {
	    isAbstract.setEnabled(false);
	}
        isFinal.setSelected(DO.isFinal());
	try {
            extensionOf.removeAllItems();
	}
	catch(IndexOutOfBoundsException e){}
        extensionOf.addItem("Nothing");
        Vector DOS = new Vector();
        DOEditorTreeModel treeModel = new DOEditorTreeModel((DOTreeNode)oldTree.getModel().getRoot(), doa, new DOPackage("adsfasdfadsfqp"));
        tree.setModel(treeModel);
        ((DOPackage)((DOTreeNode)((DOEditorTreeModel)tree.getModel()).getRoot()).getDataObject()).getAllDataObjects(DOS);
        for(int i = 0; i < DOS.size(); i++) {
            DataObject data = (DataObject)DOS.elementAt(i);
            if(!DO.getDescendents().contains(data) && !DO.equals(data) && !data.isFinal()) {
                extensionOf.addItem(data);
            }
        }
        Vector extOf = new Vector();
        for(int i = 0; i < extensionOf.getItemCount(); i++){
            extOf.addElement(extensionOf.getItemAt(i));
        }
        extOf = DataObject.alphabetizeVector(extOf);
	try {
            extensionOf.removeAllItems();
        }
	catch(IndexOutOfBoundsException e){}
        for(int i = 0; i < extOf.size(); i++) {
            extensionOf.addItem(extOf.elementAt(i));
        }
        if(DO.getExtensionOf()!=null) {
            extensionOf.setSelectedItem(DO.getExtensionOf());
        }
        tree.setSelectionPath(oldTree.getSelectionPath());
        tree.expandPath(oldTree.getSelectionPath());
        if(tree.getSelectionPath()==null || ((DOTreeNode)tree.getLastSelectedPathComponent()).isLeaf()){
            Object[] rootpath = {tree.getModel().getRoot()};
            tree.setSelectionPath(new TreePath(rootpath));
        }
        listener.valueChanged(new TreeSelectionEvent(this, null, null, null, null));
    }
    /*
     *
     * All of the following private methods simply call checkEnabledMapping
     * it might have changed.
     *
     */
    private void isEveryClass_stateChanged(ChangeEvent event) {
        checkEnabledMapping();
    }
    private void isConcreteClass_stateChanged(ChangeEvent event) {
        checkEnabledMapping();
    }
    private void isHierarchy_stateChanged(ChangeEvent event) {
        checkEnabledMapping();
    }
    private void isLazyLoading_stateChanged(ChangeEvent event) {
        checkEnabledMapping();
    }
    private void isCaching_stateChanged(ChangeEvent event) {
        checkEnabledMapping();
    }
    private void isFullCaching_stateChanged(ChangeEvent event) {
        checkEnabledMapping();
    }
    private void isAbstract_stateChanged(ChangeEvent event) {
        checkEnabledMapping();
    }
    private void extensionOf_actionPerformed(ActionEvent event) {
        checkEnabledMapping();
    }
    /*
     *
     * The inner classes are just listeners for events.
     *
     */
    class SymAction implements ActionListener {
        public void actionPerformed(ActionEvent event) {
            Object object = event.getSource();
            if (object == cancelButton)
                cancelButton_actionPerformed(event);
            else if (object == okButton)
                okButton_actionPerformed(event);
            else if (object == name)
                name_actionPerformed(event);
            else if (object == tableName)
                tableName_actionPerformed(event);
            else if (object == extensionOf)
                extensionOf_actionPerformed(event);
        }
    }
    class SymChange implements ChangeListener {
        public void stateChanged(ChangeEvent event) {
            Object object = event.getSource();
            if (object == isEveryClass)
                isEveryClass_stateChanged(event);
            else if (object == isConcreteClass)
                isConcreteClass_stateChanged(event);
            else if (object == isHierarchy)
                isHierarchy_stateChanged(event);
            else if (object == isAbstract)
                isAbstract_stateChanged(event);
            else if (object == isLazyLoading)
                isLazyLoading_stateChanged(event);
            else if (object == isCaching)
                isCaching_stateChanged(event);
            else if (object == isFullCaching)
                isFullCaching_stateChanged(event);
        }
    }
}



