/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: PackageEditor.java,v 1.5 1999/02/12 12:18:12 andy Exp $
 *-----------------------------------------------------------------------------
 */


package dods.gui.editors;

import dods.data.*;
import dods.gui.*;
import dods.gui.tree.*;
import dods.exceptions.*;
import javax.swing.JFrame;
import javax.swing.JDialog;
import javax.swing.JTree;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.JScrollPane;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.tree.TreePath;
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.Font;
import java.awt.FlowLayout;
import java.awt.Dimension;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.Vector;
/**
 * This class creates a pop-up window which allows you to edit a package by
 * changing its name and location.
 *
 * @author  Philip Martin
 * @version $Revision: 1.5 $
 */
public final class PackageEditor extends JDialog {
    /**
     * This Constructor initializes all of the fields and lays everything out.
     * @param tre The JTree from the parent frame.
     * @param da The parent DataObjectApplication.
     */
    public PackageEditor(JTree tre, DataObjectApplication da) {
        super(da);
        //initializing the Tree.
        oldTree = tre;
        doa = da;
        tree = new JTree();
        DOEditorTreeModel treeModel = new DOEditorTreeModel((DOTreeNode)oldTree.getModel().getRoot(), doa, DOP);
        tree.setModel(treeModel);
        tree.setMinimumSize(new Dimension(0,0));
        //Initializing the frame.
        setTitle("Package Editor");
        setResizable(false);
        setModal(true);
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        setSize(300,250);
        getContentPane().setLayout(new GridBagLayout());
        GridBagConstraints gbc;
        //Initializing Fields;
        name = new JTextField();
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 1;
        gbc.gridwidth = 2;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.anchor = GridBagConstraints.NORTHWEST;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(0,20,0,20);
        ((GridBagLayout)getContentPane().getLayout()).setConstraints(name, gbc);
        getContentPane().add(name);
        packageName = new JTextField();
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 4;
        gbc.gridwidth = 2;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.insets = new Insets(0,20,0,20);
        ((GridBagLayout)getContentPane().getLayout()).setConstraints(packageName, gbc);
        getContentPane().add(packageName);
        packageName.setEnabled(false);
        treeView = new JScrollPane();
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 3;
        gbc.gridwidth = 3;
        gbc.weightx = 1.0;
        gbc.weighty = 10.0;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.insets = new Insets(0,20,0,20);
        ((GridBagLayout)getContentPane().getLayout()).setConstraints(treeView, gbc);
        getContentPane().add(treeView);
        packageLabel = new JLabel();
        packageLabel.setText("Place in:");
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 2;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.anchor = GridBagConstraints.SOUTHWEST;
        gbc.fill = GridBagConstraints.NONE;
        gbc.insets = new Insets(0,20,2,0);
        ((GridBagLayout)getContentPane().getLayout()).setConstraints(packageLabel, gbc);
        getContentPane().add(packageLabel);
        nameLabel = new JLabel();
        nameLabel.setText("Name:");
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.weightx = 1.0;
        gbc.weighty = 1.0;
        gbc.anchor = GridBagConstraints.SOUTHWEST;
        gbc.fill = GridBagConstraints.NONE;
        gbc.insets = new Insets(0,20,2,0);
        ((GridBagLayout)getContentPane().getLayout()).setConstraints(nameLabel, gbc);
        getContentPane().add(nameLabel);
        buttonPanel = new JPanel();
        buttonPanel.setLayout(new FlowLayout(FlowLayout.CENTER,5,5));
        gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 5;
        gbc.gridwidth = 3;
        gbc.fill = GridBagConstraints.BOTH;
        gbc.insets = new Insets(5,15,0,15);
        ((GridBagLayout)getContentPane().getLayout()).setConstraints(buttonPanel, gbc);
        getContentPane().add(buttonPanel);
        okButton = new JButton();
        okButton.setText("OK");
        buttonPanel.add(okButton);
        cancelButton = new JButton();
        cancelButton.setText("Cancel");
        buttonPanel.add(cancelButton);
        helpButton = new JButton();
        helpButton.setText("Help");
        buttonPanel.add(helpButton);
        helpButton.setEnabled(false);
        //Initializing TreeView.
        listener = new DOEditorTreeSelectionListener(tree, packageName);
        tree.addTreeSelectionListener(listener);
        tree.setMinimumSize(new Dimension(0,0));
        treeView.setViewportView(tree);
        treeView.setMinimumSize(new Dimension(0,0));
        //Initializing Listeners.
        SymAction lSymAction = new SymAction();
        okButton.addActionListener(lSymAction);
        cancelButton.addActionListener(lSymAction);
        name.addActionListener(lSymAction);
    }
    private DOPackage DOP;
    private JTree tree;
    private JTree oldTree;
    private DataObjectApplication doa;
    private DOEditorTreeSelectionListener listener;
    private JTextField name;
    private JTextField packageName;
    private JScrollPane treeView;
    private JLabel packageLabel;
    private JLabel nameLabel;
    private JPanel buttonPanel;
    private JButton okButton;
    private JButton cancelButton;
    private JButton helpButton;
    /*
     * This method sets the packages name and location and then hides the dialog.
     */
    private void okButton_actionPerformed(ActionEvent event) {
        try {
            DOP.setName(name.getText());
        }
        catch(DataObjectNameException e) {
            doa.namingExceptionOccurred(e, this);
            return;
        }
        DOTreeNode newNode = new DOTreeNode(DOP, true);
        if(!packageName.getText().equals(DOP.getPackageName())) {
            DOTreeNode newParent = (DOTreeNode)tree.getLastSelectedPathComponent();
            for(int i = 0; i < newParent.getChildren().size(); i++) {
                DOTreeNode donode = (DOTreeNode)newParent.getChildren().elementAt(i);
                if(donode.getDataObject().getName().equals(name.getText()) && donode.getDataObject() != DOP) {
                    JOptionPane.showMessageDialog(this, 
                                      "An object with that name already exists in this package.",
                                      "Error creating package.",
                                      JOptionPane.WARNING_MESSAGE);
                    return;
                }
            }
            DOTreeNode findnode = ((DOTreeNode)((DOTreeModel)tree.getModel()).getRoot()).findNodeWith(DOP);
            DOTreeNode dotn;
            Vector children;
            if(findnode != null) {
                children = findnode.getChildren();
            }
            else {
                children = new Vector();
            }
            for(int i = 0; i < children.size(); i++) {
                dotn = (DOTreeNode)children.elementAt(i);
                if(dotn.getDataObject() instanceof DOPackage) {
                    if(DOP.equals((DOPackage)dotn.getDataObject())){
                        children.removeElement(dotn);
                        ((DOPackage)findnode.getDataObject()).getDataObjects().removeElement(dotn.getDataObject());
                        break;
                    }
                }
            }
            ((DOPackage)newParent.getDataObject()).addDOPackage(DOP);
            newParent.getChildren().addElement(newNode);
        }
        else {
            DOTreeNode newParent = (DOTreeNode)tree.getLastSelectedPathComponent();
            for(int i = 0; i < newParent.getChildren().size(); i++) {
                DOTreeNode donode = (DOTreeNode)newParent.getChildren().elementAt(i);
                if(donode.getDataObject().getName().equals(name.getText()) && donode.getDataObject() != DOP) {
                    JOptionPane.showMessageDialog(this, 
                                      "An object with that name already exists in this package.",
                                      "Error creating package.",
                                      JOptionPane.WARNING_MESSAGE);
                    return;
                }
            }
        }
        TreePath path = tree.getSelectionPath();
        Object[] newPathComponents = new Object[path.getPath().length +1];
        for(int i = 0; i < path.getPath().length; i++) {
            newPathComponents[i] = path.getPath()[i];
        }
        newPathComponents[path.getPath().length] = newNode;
        TreePath newPath = new TreePath(newPathComponents);
        DOP.updatePackageName();
        doa.updateTree();
        oldTree.setSelectionPath(newPath);
        setVisible(false);
    }
    /*
     * This method just hides the dialog.
     */
    private void cancelButton_actionPerformed(ActionEvent event) {
        setVisible(false);
    }
    /*
     * When enter is pressed in the name field it is just as if the ok button was pressed.
     */
    private void name_actionPerformed(ActionEvent event) {
        okButton.doClick();
    }
    /**
     * This method initializes all of the Fields for the editor.
     * @param DoP The DOPackage being edited.
     */
    public void setDOPackage(DOPackage DoP) {
        DOP = DoP;
        name.setText(DOP.getName());
        DOEditorTreeModel treeModel = new DOEditorTreeModel((DOTreeNode)oldTree.getModel().getRoot(), doa, DOP);
        tree.setModel(treeModel);
        tree.setMinimumSize(new Dimension(0,0));
        tree.setSelectionPath(oldTree.getSelectionPath());
        tree.expandPath(oldTree.getSelectionPath());
        if(tree.getSelectionPath()==null || ((DOTreeNode)tree.getLastSelectedPathComponent()).isLeaf()){
            Object[] rootpath = {tree.getModel().getRoot()};
            tree.setSelectionPath(new TreePath(rootpath));
        }
        listener.valueChanged(new TreeSelectionEvent(this, null, null, null, null));
    }
    /*
     * This class is just an action listener for this dialog.
     */
    class SymAction implements ActionListener {
        public void actionPerformed(ActionEvent event) {
            Object object = event.getSource();
            if (object == okButton)
                okButton_actionPerformed(event);
            else if (object == cancelButton)
                cancelButton_actionPerformed(event);
            else if (object == name)
                name_actionPerformed(event);
        }
    }
}
