/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: DOTableColumnRenderer.java,v 1.6 1999/03/09 09:24:00 andy Exp $
 *-----------------------------------------------------------------------------
 */



package dods.gui.table;

import dods.gui.*;
import dods.data.*;
import javax.swing.JLabel;
import javax.swing.ImageIcon;
import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;
import java.awt.Component;
import java.awt.Color;
import java.awt.Dimension;
/**
 * This class is set up to display Columns in a JTable with an Icon instead of
 * Text.  It should be extended rather than used itself.
 *
 * @author  Philip Martin
 * @version $Revision: 1.6 $
 */
public class DOTableColumnRenderer extends JLabel implements TableCellRenderer {

    private boolean toShow = false;

    /**
     * The icon used in the column that this renderer is used for.
     */
    protected ImageIcon icon;
    /**
     * This general Constructor sets the icon to an Error Icon, but that should be
     * changed for any extension of this class.
     */
    DOTableColumnRenderer(boolean sh) {
        super();
        setOpaque(true);
        toShow = sh;
        setPreferredSize(new Dimension(18, 18));
        icon = Icons.getIcon("error.gif");
    }
    /**
     * This method sets when the icon is visible depending on the value of the cell.
     * @return this with the correct icon because this extends JLabel.
     */
    public Component getTableCellRendererComponent(JTable table,
                                                   Object value,
                                                   boolean isSelected,
                                                   boolean hasFocus,
                                                   int row,
                                                   int column) {
        if (isSelected) {
            super.setForeground(table.getForeground());
            super.setBackground(Color.lightGray);
        }
        else {
            super.setForeground(table.getForeground());
            super.setBackground(Color.lightGray);
        }
        if(toShow) {
            setIcon(icon);
            return this;
        }
        setIcon(null);
        if(value!=null) {
            if(((Boolean)value).booleanValue()) {
                setIcon(icon);
            }
        }
        setFont(table.getFont());
        
        return this;
    }
}
