/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: DOTableModel.java,v 1.7 1999/07/23 22:02:09 jay Exp $
 *-----------------------------------------------------------------------------
 * Changes to support DODS Types made by David Wood (dwood@pisoftware.com)
 */



package dods.gui.table;

import dods.data.*;
import dods.gui.*;
import javax.swing.JTable;
import javax.swing.JComboBox;
import javax.swing.table.TableModel;
import javax.swing.event.TableModelListener;
import java.util.Vector;
/**
 * This Table Model organizes everything for the table in how it is layed out
 * and how you edit it.
 *
 * @author  Philip Martin
 * @version $Revision: 1.7 $
 */
public class DOTableModel implements TableModel {
    /**
     * Any Listeners for this TableModel.
     */
    protected Vector listeners;
    /**
     * The DataObject whose attributes are being displayed.
     */
    protected DataObject DO;
    /**
     * The number of rows in the table.
     */
    protected int rows;
    /**
     * The parent DataObjectApplication.
     */
    protected DataObjectApplication doa;
    /**
     * True when a value has changed and the table needs to be updated.
     */
    protected boolean hasBeenEdited;
    /**
     * The table for this model.
     */
    protected JTable table;

    /**
     * Constructor which initializes all of the relevant variables.
     */
    public DOTableModel(DataObject Do, DataObjectApplication da, JTable tab) {
        listeners = new Vector();
        doa = da;
        setDataObject(Do);
        hasBeenEdited = false;
        table = tab;
    }

    /**
     * This takes the new value for a cell, and updates the appropriate Attribute.
     */
    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        Attribute att = (Attribute)DO.getAttributes().elementAt(rowIndex);
        switch(columnIndex) {
            //The editing for the first five columns is handled elsewhere.
            case 0:
                break;
            case 1:
                break;
            case 2:
                break;
	    /*
            case 3:
                break;
		*/
            case 3:
                break;
            case 4:
                break;
            case 5:
                break;
            case 6:
                break;
            case 7:
                break;
            case 8:
                att.setName(aValue.toString());
                break;
            case 9:
                // Changed from Java to DODS.  TODO - correct if needed.
                att.setDODSType(aValue.toString());
                break;
            case 10:
                att.setSize(aValue.toString());
                break;
            case 11:
                att.setInitialJavaValue(aValue.toString());
                break;
            case 12:
                // aValue.toString() seems bogus :(
                // e.g. '1' seems to display as 'true'  - TODO FIX THIS!!
                att.setInitialDBValue(aValue.toString());
                break;
            default:
                break;
        }
        doa.checkForErrors();
    }

    /**
     * Removes the given listener.
     */
    public void removeTableModelListener(TableModelListener l) {
        listeners.removeElement(l);
    }

    /**
     * Returns true if the given cell is editable.
     */
    public boolean isCellEditable(int rowIndex, int columnIndex) {
        // TODO:  Change if columns are changed!
        //if(columnIndex > 4 && columnIndex < 10 && columnIndex != 7) {
        if(columnIndex == 8) {
            return true;
        }
        return false;
    }

    /**
     * Returns the Object contained in the given cell.
     */
    public Object getValueAt(int rowIndex, int columnIndex) {
        Attribute att = (Attribute)DO.getAttributes().elementAt(rowIndex);

        // TODO - when we change the column numbers, change this!!
        switch(columnIndex) {
            case 0:
                return att;
            case 1:
                return new Boolean(att.isObjectRef());
	    /*
            case 2:
                return att;
		*/
            case 2:
                return new Boolean(att.canBeNull());
            case 3:
                return new Boolean(att.isIndex());
            case 4:
                return new Boolean(att.isUsedForQuery());
            case 5:
                return new Boolean(att.isRefConstraint());
            case 6:
                return new Boolean(att.isUnique());
            case 7:
                return new Boolean(att.isConstant());
            case 8:
                return att.getName();
            case 9:
		// FIX 
		// I really want to change the Attribute.setObjectRef() method
		// to call Attribute.setJavaType().  But that would cause
		// existing saved project (.doa) files to become invalid.
		// So instead we have this patch code.
		if ( att.isObjectRef() ) {
			DataObject o = att.getObjectRefDO();
			return o.getPackageName() + "." + o.getName() + "DO";
		}
		// END PATCH
                // TODO - changed this to DODS type - OK?
                return att.getDODSType();
            case 10:
                return att.getSize();
            case 11:
                return att.getInitialJavaValue();
            case 12:
                return att.getInitialDBValue();
            default:
                return "";
        }
    }

    /**
     * Returns the number of rows in the table.
     */
    public int getRowCount() {
        return rows;
    }

    /**
     * Returns the name that is meant for each column header.
     */
    public String getColumnName(int columnIndex) {
        switch(columnIndex) {
            case 8:
                return "Name";
            case 9:
                return "DODS Type";
            case 10:
                return "Size";
            case 11:
                return "Initial Java Value";
            case 12:
                return "Initial DB Value";
            default:
                return "";
        }
    }
    /**
     * Returns the number of Columns in the table.
     */
    public int getColumnCount() {
        return 13;
    }
    /**
     * Returns the class of Objects in a given column.
     */
    public Class getColumnClass(int columnIndex) {
        if(columnIndex < 8) {
            return Boolean.class;
        }
	/*
        if(columnIndex == 8) {
            return JComboBox.class;
        }
	*/
        return String.class;
    }
    /**
     * Adds a new Listener.
     */
    public void addTableModelListener(TableModelListener l) {
        if(!listeners.contains(l)) {
            listeners.addElement(l);
        }
    }
    /**
     * Sets the DataObject being displayed in a given table.
     */
    public void setDataObject(DataObject Do) {
        if(Do != null) {
            DO = Do;
            rows = DO.getAttributes().size();
        }
        else {
            rows = 0;
        }
    }
    /**
     * resets the row count when it has changed.
     */
    public void resetRows() {
        rows = DO.getAttributes().size();
    }
    /**
     * Returns the DataObject Being displayed in the table.
     */
    public DataObject getDataObject() {
        return DO;
    }
}
