/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: TableHeaderListener.java,v 1.6 1999/07/23 22:02:09 jay Exp $
 *-----------------------------------------------------------------------------
 */



package dods.gui.table;

import dods.gui.*;
import dods.data.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseAdapter;
import java.awt.Point;
import javax.swing.JTable;
import javax.swing.table.JTableHeader;
import java.util.Vector;
/**
 * This is a standard Mouse Listener for the TableHeader.  When a Column
 * Header is clicked, the table is sorted by that Column.
 *
 * @author  Philip Martin
 * @version $Revision: 1.6 $
 */
public class TableHeaderListener extends MouseAdapter {
    private boolean ascending = true;
    private Vector newOrder;
    private DataObjectApplication doa;
    /**
     * Tells this listener the DataObjectApplication it is using.
     */
    public TableHeaderListener(DataObjectApplication da) {
        doa = da;
    }
    /**
     * When the mouse is clicked in a column header, this method sorts according
     * to that column.
     */
    public void mouseClicked(MouseEvent e) {
        JTableHeader head = (JTableHeader)e.getSource();
        JTable table = head.getTable();
        table.setRowSelectionInterval(-1,-1);
        doa.checkForErrors();
        doa.updateMenus();
        int column = head.columnAtPoint(new Point(e.getX(), e.getY()));
        DataObject DO = ((DOTableModel)table.getModel()).getDataObject();
        Vector atts = DO.getAttributes();
        int size = atts.size();
        
        if(table.getRowCount() < 2) {
            return;
        }
        newOrder = new Vector();

        // TODO: Chg this number of the number of columns with icons changes.
        if(column < 8) {
            int trues;
            int untrues;
            if(ascending) {
                trues = 0;
                untrues = 0;
                for(int i = 0; i < size; i++) {
                    Attribute insert = (Attribute)atts.elementAt(i);
                    if(getBooleanValue(insert, column)) {
                        newOrder.insertElementAt(insert, 0);
                        trues++;
                    }
                    else {
                        newOrder.addElement(insert);
                        untrues++;
                    }
                }
                ascending = false;
            }
            else {
                trues = 0;
                untrues = 0;
                for(int i = 0; i < size; i++) {
                    Attribute insert = (Attribute)atts.elementAt(i);
                    if(getBooleanValue(insert, column)) {
                        newOrder.addElement(insert);
                        untrues++;
                    }
                    else {
                        newOrder.insertElementAt(insert, 0);
                        trues++;
                    }
                }
                ascending = true;
            }
            sortRowsByName(0, trues-1);
            sortRowsByName(size-untrues, size-1);
        }
        else {
            if(column != -1){
                for(int i = 0; i < size; i++) {
                    newOrder.addElement(atts.elementAt(i));
                }
                sortRowsByColumn(0, size-1, column);
            }
            if(ascending) {
                ascending = false;
            }
            else {
                ascending = true;
                Vector newOrder2 = new Vector();
                for(int i = size-1; i != -1 ; i--) {
                    newOrder2.addElement(newOrder.elementAt(i));
                }
                newOrder = newOrder2;
            }
            thenSortByName(column);
        }
        DO.setAttributes(newOrder);
        table.repaint();
    }
    /*
     * This returns the appropriate value for a column in an Attribute.
     */
    private boolean getBooleanValue(Attribute att, int column) {
        switch(column) {
            case 0:
                return att.hasError();
            case 1:
                return att.isObjectRef();
	    /*
            case 2:
                return att.isArray();
		*/
            case 2:
                return att.canBeNull();
            case 3:
                return att.isIndex();
            case 4:
                return att.isUsedForQuery();
            case 5:
                return att.isRefConstraint();
            case 6:
                return att.isUnique();
            case 7:
                return att.isConstant();
            default:
                return true;
        }
    }
    /*
     * This method sorts the rows between firstRow and lastRow alphabetically
     * by name.
     */
    private void sortRowsByName(int firstRow, int lastRow) {
        if(lastRow <= firstRow) {
            return;
        }
        Vector newOrder2 = new Vector();
        for(int i = 0; i < firstRow; i++) {
            newOrder2.addElement(newOrder.elementAt(i));
        }
        newOrder2.addElement(newOrder.elementAt(firstRow));
        Attribute start = (Attribute)newOrder.elementAt(firstRow);
        Attribute startplus = (Attribute)newOrder.elementAt(firstRow+1);
        if(startplus.getName().compareTo(start.getName())< 0) {
            newOrder2.insertElementAt(startplus, firstRow);
        }
        else {
            newOrder2.addElement(startplus);
        }
        for(int i = firstRow + 2; i < lastRow + 1; i++) {
            Attribute moving = (Attribute)newOrder.elementAt(i);
            int size2 = newOrder2.size();
            for(int j = firstRow; j < size2; j++) {
                Attribute low = (Attribute)newOrder2.elementAt(j);
                if(moving.getName().compareTo(low.getName()) < 0) {
                    newOrder2.insertElementAt(moving, j);
                    break;
                }
            }
            if(!newOrder2.contains(moving)){
                newOrder2.addElement(moving);
            }
        }
        for(int i = lastRow+1; i < newOrder.size(); i++) {
            newOrder2.addElement(newOrder.elementAt(i));
        }
        newOrder = newOrder2;
    }
    /*
     * This sorts rows by the given column between the two rows.
     */
    private void sortRowsByColumn(int firstRow, int lastRow, int column) {
        if(lastRow <= firstRow) {
            return;
        }
        Vector newOrder2 = new Vector();
        for(int i = 0; i < firstRow; i++) {
            newOrder2.addElement(newOrder.elementAt(i));
        }
        newOrder2.addElement(newOrder.elementAt(firstRow));
        Attribute start = (Attribute)newOrder.elementAt(firstRow);
        Attribute startplus = (Attribute)newOrder.elementAt(firstRow+1);
        if(getString(startplus, column).compareTo(getString(start, column))< 0) {
            newOrder2.insertElementAt(startplus, firstRow);
        }
        else {
            newOrder2.addElement(startplus);
        }
        for(int i = firstRow + 2; i < lastRow + 1; i++) {
            Attribute moving = (Attribute)newOrder.elementAt(i);
            int size2 = newOrder2.size();
            for(int j = firstRow; j < size2; j++) {
                Attribute low = (Attribute)newOrder2.elementAt(j);
                if(getString(moving, column).compareTo(getString(low, column)) < 0) {
                    newOrder2.insertElementAt(moving, j);
                    break;
                }
            }
            if(!newOrder2.contains(moving)){
                newOrder2.addElement(moving);
            }
        }
        for(int i = lastRow+1; i < newOrder.size(); i++) {
            newOrder2.addElement(newOrder.elementAt(i));
        }
        newOrder = newOrder2;
    }
    /*
     * Returns the string for the given column in the attribute.
     */
    private String getString(Attribute att, int column) {
        switch(column) {
            case 8:
                return att.getName();
            case 9:
                // FIX
                // I really want to change the Attribute.setObjectRef() method
                // to call Attribute.setJavaType().  But that would cause
                // existing saved project (.doa) files to become invalid.
                // So instead we have this patch code.
                if ( att.isObjectRef() ) {
                        DataObject o = att.getObjectRefDO();
                        return o.getPackageName() + "." + o.getName() + "DO";
                }
		// END PATCH
                //return att.getJavaType();
                return att.getDODSType();
            case 10:
                //return att.getDBType();
                return att.getDODSType();
            case 11:
                return att.getSize();
            case 12:
                //return att.getInitialValue();
                return att.getInitialJavaValue();
            default:
                return "";
        }
    }
    /*
     * After the rows are sorted by column, they will be sorted by name.
     */
    private void thenSortByName(int column) {
        Attribute same = (Attribute)newOrder.elementAt(0);
        int firstRow = 0;
        int lastRow = 0;
        while(lastRow < newOrder.size()-1) {
            String sameString = getString(same, column);
            int i = firstRow;
            while(getString(same, column).equals(sameString) && i < newOrder.size()) {
                i++;
                if(i==newOrder.size()) {
                }
                else {
                    same = (Attribute)newOrder.elementAt(i);
                }
            }
            lastRow = i-1;
            sortRowsByName(firstRow, lastRow);
            firstRow = i;
        }
    }
}
