/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: TableMouseListener.java,v 1.5 1999/07/23 22:02:09 jay Exp $
 *-----------------------------------------------------------------------------
 * Changes in layout and actions to support the 'DODS Type' for
 * attribute properties by David Wood (dwood@pisoftware.com).
 */



package dods.gui.table;

import dods.gui.*;
import dods.data.*;
import javax.swing.JTable;
import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
/**
 * This class tells the table which row is selected, and it handles editing
 * the boolean values in the table.
 */
public class TableMouseListener extends MouseAdapter {
    private JTable table;
    private DataObjectApplication doa;

    // All accessible DODS Types
    DODSTypes dodsTypes = DODSTypes.getInstance(); 

    /**
     * Tells the listener which table and DataObjectApplication are using it.
     */
    public TableMouseListener(JTable tabl, DataObjectApplication da) {
        table = tabl;
        doa=da;
    }
    /**
     * When the mouse is released, the table is checked for errors and
     * the menus are updated.
     */
    public void mouseReleased(MouseEvent e) {
        doa.checkForErrors();
        doa.updateMenus();
    }
    /**
     * The row where the mouse is pressed is selected, and if it is in one
     * of the boolean columns that value is toggled.
     */
    public void mousePressed(MouseEvent e) {

        Point point = new Point(e.getX(), e.getY());
        int row = table.rowAtPoint(point);
        if(row == -1) {
            table.repaint();
            table.setRowSelectionInterval(row, row);
            doa.updateMenus();
            return;
        }
        int column = table.columnAtPoint(point);
        DataObject DO = ((DOTableModel)table.getModel()).getDataObject();
        Attribute att = (Attribute)DO.getAttributes().elementAt(row);

        // The currently selected DODS Type.
        String curDodsType = att.getDODSType();

        // The current Java type.
        String javatype = dodsTypes.getJavaType( curDodsType );

        // If the right mouse button is clicked (or use of meta-click),
        // pop up the Attribute editor.
        if ( e.isMetaDown() ) {
          table.setRowSelectionInterval(row, row);
          doa.editAttribute(att);
        } else {
          // Otherwise, toggle the state on the icon columns.
          switch(column) {
          /*
            case 2:
                att.setArray(!att.isArray());
                break;
          */
            case 2:
                // Don't allow this field to be null if the Java type
                // is a primitive.  That could ruin your day :)
                if ( ! dodsTypes.isJavaPrimitive( javatype ) ) {
                  att.setCanBeNull(!att.canBeNull());
                } else {
                  att.setCanBeNull( false );
                }
                break;
            case 3:
                att.setIndex(!att.isIndex());
                break;
            case 4:
                att.setUsedForQuery(!att.isUsedForQuery());
                break;
            case 5:
                att.setRefConstraint(!att.isRefConstraint());
                break;
            case 6:
                att.setUnique(!att.isUnique());
                break;
            case 7:
                att.setConstant(!att.isConstant());
                break;
            default:
                break;
          }
        }
        table.repaint();
        table.setRowSelectionInterval(row, row);
        doa.checkForErrors();
        doa.updateMenus();
    }
}
