/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * $Id: DOTreeSelectionListener.java,v 1.5 1999/02/12 12:18:17 andy Exp $
 *-----------------------------------------------------------------------------
 */


package dods.gui.tree;

import dods.gui.*;
import dods.data.*;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.JTree;
/**
 * This class listens to what is selected in the tree and updates the Table
 * in the DataObjectApplication.
 */
public final class DOTreeSelectionListener implements TreeSelectionListener {
    private DataObjectApplication doa;
    private JTree tree;
    private GraphicalPanel gPanel;
    /**
     * Initializes the appropriate variables.
     */
    public DOTreeSelectionListener(DataObjectApplication da, JTree tre, GraphicalPanel gpanel) {
        doa = da;
        tree = tre;
        gPanel = gpanel;
    }
    /**
     * When a new Node is selected, if it is a leaf, then the DataObject in 
     * that node has its Attributes displayed.
     */
    public void valueChanged(TreeSelectionEvent e) {
        if(tree.getSelectionPath()==null) {
            doa.setDataObjectInTable(null);
            gPanel.setSelectedDO(null);
        }
        else {
            DOTreeNode DOtn = (DOTreeNode)tree.getSelectionPath().getLastPathComponent();
            DataObject DO = DOtn.getDataObject();
            gPanel.setSelectedDO(DO);
            doa.setDataObjectInTable(DO);
        }
    }
}


