package jstest.presentation;

import java.io.*;

import com.lutris.xml.xmlc.*;
import com.lutris.appserver.server.httpPresentation.*;
import org.mozilla.javascript.*;

public class JSWelcome implements HttpPresentation {

    public void run(HttpPresentationComms comms) throws HttpPresentationException {
	try {
	    /* 
	     * set up the Thread context class loader so the JavaScript has access
	     * to the Enhydra application's class loaded by MultiClassLoader.
	     * Only works for Java2, thu.
	     */
	    ClassLoader old = Thread.currentThread().getContextClassLoader();
	    Thread.currentThread().setContextClassLoader(this.getClass().getClassLoader());

	    /*
	     * initialize a JavaScript context for execution 
	     */
	    Context cx = Context.enter();
	    Scriptable scope = cx.initStandardObjects(new ImporterTopLevel());

	    /* 
	     * comms is binded to the JavaScript scope so the script has access to the 
	     * request and response 
	     */
	    Scriptable jsArgs = Context.toObject(comms, scope);
	    scope.put("comms", scope, jsArgs);

	    /* load the script for execution */
	    Object result = cx.evaluateReader(scope, 
					      new InputStreamReader (this.getClass().getResourceAsStream("JSWelcome.js")),
					      "JSWelcome.js", 1, null);

	    /* 
	     * clean up the JavaScript Context 
	     */
	    Context.exit();

	    /*
	     * reset the thread context class loader
	     */
	    Thread.currentThread().setContextClassLoader(old);
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }
}
