/*-----------------------------------------------------------------------------
 * PersonDO.java
 *-----------------------------------------------------------------------------
 */

package phoneList.data;

import com.lutris.appserver.server.*;
import com.lutris.appserver.server.sql.*;
import java.sql.*;
import java.math.BigDecimal;

/**
 * Example data object for the person database table.
 *
 * @author Kristen Pol
 * @since Enhydra2.0
 */
public class PersonDO extends BasicDO {

    // First name column name
    public static final String FIRST_NAME = "firstName";

    // Last name column name
    public static final String LAST_NAME = "lastName";

    // Phone number column name
    public static final String PHONE_NUMBER = "phoneNumber";

    // private data members

    // Parameter names
    // ORDER IS IMPORTANT! (use same order as in setData)
    private static String[] PARAMETER_NAMES = 
	{FIRST_NAME, LAST_NAME, PHONE_NUMBER};

    // First name for this person
    private String firstName = "";

    // Last name for this person
    private String lastName = "";

    // Phone number for this person
    private String phoneNumber = "";

    // Construct new person data object
    public PersonDO()
            throws ObjectIdException, DatabaseManagerException {
	super();
    }

    // Construct person data object from the given database info
    public PersonDO(ResultSet rs)
            throws SQLException, ObjectIdException {
        super(rs);
        setNewVersion(getVersion());
	setFirstName(rs.getString("firstName"));
	setLastName(rs.getString("lastName"));
	setPhoneNumber(rs.getString("phoneNumber"));
        setDirty(false);
    }

    // Set first name for this person
    public void setFirstName(String firstName) {
	if (! firstName.equals(this.firstName)) {
	    this.firstName = firstName;
	    setDirty(true);
	}
    }

    // Get first name for this person
    public String getFirstName() {
        return this.firstName;
    }

    // Set last name for this person
    public void setLastName(String lastName) {
	if (! lastName.equals(this.lastName)) {
	    this.lastName = lastName;
	    setDirty(true);
	}
    }

    // Get last name for this person
    public String getLastName() {
        return this.lastName;
    }

    // Set phone number for this person
    public void setPhoneNumber(String phoneNumber) {
	if (! phoneNumber.equals(this.phoneNumber)) {
	    this.phoneNumber = phoneNumber;
	    setDirty(true);
	}
    }

    // Get phone number for this person
    public String getPhoneNumber() {
        return this.phoneNumber;
    }

    // Get database table name
    protected String getTableName() {
	return getDBTableName();
    }

    // Get database table name
    public static String getDBTableName() {
	return "Person";
    }

    // Set database parameters for this person data object
    // ORDER IS IMPORTANT! (use same order as in getParameters)
    protected int setParameters(PreparedStatement stmt, int param)
            throws SQLException {
	param = setString(stmt, param, getFirstName());
	param = setString(stmt, param, getLastName());
	param = setString(stmt, param, getPhoneNumber());
	return param;
    }

    // Get database parameters names (columns in table)
    // ORDER IS IMPORTANT! (use same order as in setParameters)
    protected String[] getParameters() {
	return PARAMETER_NAMES;
    }
}
