/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-1999 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *-----------------------------------------------------------------------------
 * /home/proj/phoneList1/phoneList/tmp3/phoneList/data/PersonDO.java
 *-----------------------------------------------------------------------------
 */

package phoneList.data;

import com.lutris.appserver.server.sql.*;
import com.lutris.appserver.server.*;
import com.lutris.util.Config;
import com.lutris.util.KeywordValueTable;
import dods.builder.sourceGenerators.Query.NonUniqueQueryException;
import dods.builder.sourceGenerators.Query.DataObjectException;
import java.sql.*;
import java.math.*;
import java.util.Hashtable;
import java.util.Enumeration;


/**
 * Data core class, used to set, retrieve the PersonDO information.
 *
 * @version $Revision: 1.11 $
 * @author  sergiob
 * @since   phoneList
 */
 public class PersonDO extends dods.builder.sourceGenerators.DO.GenericDO {

    /**
     * The ObjectId is the unique key for the DO in the database.
     */

    /**
     * A DO class contains a reference to a DataStruct class.
     * This reference can be null (when the data for the DO 
     * has not yet been retrieved from the database),
     * allowing a DO object to be a lightweight placeholder
     * until its data is needed.
     */
    private PersonDataStruct data = null;

    /**
     * Protected constructor.  Only derived classes should call it.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     */
    protected PersonDO ()
    throws ObjectIdException, DatabaseManagerException {
        super();
    }

    /**
     * loadData() 
     * Load the fields for the DO from the database.
     *
     * @exception DataObjectException
     *   If an object id can't be allocated for this object.
     * @exception SQLException
     *   If an object id can't be allocated for this object.
     */
    public void loadData() 
    throws SQLException, ObjectIdException, DataObjectException 
    {
	if ( null == data ) {
	    
	    data = new PersonDataStruct ();
	}

	

	ObjectId id = getOId();
	if ( null == id )
	    return;

	// an id means we complain if no record in database
	PersonQuery query = new PersonQuery ();
	query.setQueryOId( id );
	query.requireUniqueInstance();
	PersonDO obj;
	try {
	    while ( null != ( obj = query.getNextDO() ) ) {
	        data = obj.data;
	        // obj.data.readOnly = true;
	    }
	} catch ( NonUniqueQueryException e ) { 
	    throw new ObjectIdException( "Duplicate ObjectId" );
	}

	
    }

    


    /**
     * createVirgin() 
     * Creates a DO that has no ObjectId or data.
     * Such a DO is used to insert a new database entry
     * after its data has been set.
     */
    
    public static PersonDO createVirgin()
    throws DatabaseManagerException, ObjectIdException {
	return new PersonDO ();
    }
    

    /**
     * createExisting() 
     * Creates a DO that represents an existing entry in the database.
     * Such a DO is used to examine and possibly update such an entry.
     * createExisting() is called only from the code that retrieves
     * an ObjectId from a ResultSet (database query result).
     * createExisting() is protected because no other DO or BO should ever
     * need to call it.
     * FIX unfortunately the createExisting(BigDecimal) form *does* need 
     * to be public because it is called by the public ctors of other DOs.
     * For example,
     *    AaaDO contains a ref to a BbbDO,
     *    so there is a method AaaDO.setBbb(BbbDO).
     *    In the ctor AaaDO(ResultSet), we have the call
     *       setBbb( BbbDO.createExisting( rs.getBigDecimal( "bbb", 0 )));
     * Since AaaDO is not in the same package as BbbDO,
     * BbbDO.createExisting(BigDecimal) must be public, not protected.
     * Java needs the C++ 'friend' idea.
     */
    
    public static PersonDO createExisting( BigDecimal bd ) 
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException
    {
	if ( null == bd )
	    return null;
	return createExisting( new ObjectId( bd ) );
    }
    protected static PersonDO createExisting( ObjectId id ) 
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException
    {
	PersonDO ret = new PersonDO ();
	  ret.setOId( id );
	if ( ! false )
	    ret.loadData();
	return ret;
    }

    protected static PersonDO createExisting( ResultSet rs )
    throws SQLException, ObjectIdException, DataObjectException
		, DatabaseManagerException
    {
	PersonDO ret = new PersonDO ( rs );
	return ret;
    }
    

    /**
     * createCopy() 
     * Creates a DO that has no ObjectId
     * but has a copy of an existing DO's data.
     * Such a DO is used to insert a new database entry
     * that is largely similar to an existing entry.
     */
    
    public static PersonDO createCopy( PersonDataStruct data )
    throws DatabaseManagerException, ObjectIdException {
	PersonDO ret = new PersonDO ();
	ret.data = ( PersonDataStruct ) data.duplicate();
	return ret;
    }
    

    /**
     * createCopy() 
     * Creates a DO that has no ObjectId
     * but has a copy of an existing DO's data.
     * Such a DO is used to insert a new database entry
     * that is largely similar to an existing entry.
     */
    
    public static PersonDO createCopy( PersonDO orig )
    throws DatabaseManagerException, ObjectIdException {
	PersonDO ret = new PersonDO ();
	if ( null != orig.data ) {
	    ret.data = ( PersonDataStruct ) orig.data.duplicate();
	}
	return ret;
    }
    

    /**
     * reload() 
     * Causes the DO to refresh itself from the database
     * the next time a set or get method is called.
     */
    
    public void reload() {
	data = null;
    }
    

    /** 
     * The methods <CODE>
     *     getHandle
     *     hasMatchingHandle
     * </CODE> are used by Presentation Objects that need to populate
     * HTML select lists with Data Objects as options.
     * The <CODE>getHandle()</CODE> method is used
     * to set the value for each option,
     * and the <CODE>hasMatchingHandle()<CODE>
     * methods are used to lookup the Data Object when the selection has
     * been made.
     *
     * @return id of this DO as a string
     *   If an object id can't be allocated for this object.
     */
    
    public        String  getHandle() 
    throws DatabaseManagerException {
	String ret = null;
	    if ( null == getOId() )
		   throw new DatabaseManagerException( "ID not set" );
	    ret = getOId().toString();
        return ret;
    }
    

   /**
     * hasMatchingHandle
     *
     * @param handle
     *   <CODE>String</CODE> version of DO id
     * @return boolean
     *   True if the string version of the id of this DO matches passed handle
     * @see getHandle
     */
    
    public        boolean hasMatchingHandle( String handle ) {
	boolean ret = false;
	    if ( null == getOId() )
		   ret = false;
	    else
		   ret = getOId().toString().equals( handle );
        return ret;
    }
    


    


   /**
    * Get firstName of the Person
    *
    * @return firstName of the Person
    */
   public String getFirstName () 
   throws DataObjectException {
      if ( null == data )
 	try {
 	  loadData();
 	} catch ( Exception e ) {
 	  throw new DataObjectException( 
 	    "Unable to load data for PersonDO id=" + getOId() +
 	    ", error = " + e.getMessage() );
        }
      return data.firstName;
   }

   /**
    * Set firstName of the Person
    *
    * @param firstName of the Person
    */
   
   public void setFirstName ( String firstName )
   throws DataObjectException {
      if ( null == data )
	try {
	  loadData();
	} catch ( Exception e ) {
	  throw new DataObjectException( 
	    "Unable to load data for PersonDO id=" + getOId() +
	    ", error = " + e.getMessage() );
	}
      data.firstName =  markNewValue(
	data.firstName, firstName , 0, 20, true );
   }
   

   /**
    * Get lastName of the Person
    *
    * @return lastName of the Person
    */
   public String getLastName () 
   throws DataObjectException {
      if ( null == data )
 	try {
 	  loadData();
 	} catch ( Exception e ) {
 	  throw new DataObjectException( 
 	    "Unable to load data for PersonDO id=" + getOId() +
 	    ", error = " + e.getMessage() );
        }
      return data.lastName;
   }

   /**
    * Set lastName of the Person
    *
    * @param lastName of the Person
    */
   
   public void setLastName ( String lastName )
   throws DataObjectException {
      if ( null == data )
	try {
	  loadData();
	} catch ( Exception e ) {
	  throw new DataObjectException( 
	    "Unable to load data for PersonDO id=" + getOId() +
	    ", error = " + e.getMessage() );
	}
      data.lastName =  markNewValue(
	data.lastName, lastName , 0, 20, false );
   }
   

   /**
    * Get phoneNumber of the Person
    *
    * @return phoneNumber of the Person
    */
   public String getPhoneNumber () 
   throws DataObjectException {
      if ( null == data )
 	try {
 	  loadData();
 	} catch ( Exception e ) {
 	  throw new DataObjectException( 
 	    "Unable to load data for PersonDO id=" + getOId() +
 	    ", error = " + e.getMessage() );
        }
      return data.phoneNumber;
   }

   /**
    * Set phoneNumber of the Person
    *
    * @param phoneNumber of the Person
    */
   
   public void setPhoneNumber ( String phoneNumber )
   throws DataObjectException {
      if ( null == data )
	try {
	  loadData();
	} catch ( Exception e ) {
	  throw new DataObjectException( 
	    "Unable to load data for PersonDO id=" + getOId() +
	    ", error = " + e.getMessage() );
	}
      data.phoneNumber =  markNewValue(
	data.phoneNumber, phoneNumber , 0, 20, false );
   }
   
    /**
     * Protected constructor.
     *
     * @param rs
     *   Result set from which to obtain product data.
     * @exception java.sql.SQLException
     *   If a database connection error occurs.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If the database contains an invalid object id.
     */
    protected PersonDO(ResultSet rs)
            throws SQLException, ObjectIdException, DataObjectException
 	    , DatabaseManagerException
/*  	      */
    {
        super(rs);

	// Constructing a DO from a ResultSet means we definitely need the 
	// DataStruct ready for the setXxx methods invoked below.
	data = new PersonDataStruct ();
 
	// writeMemberStuff uses the ResultSetExtraction.template
	// to build up the value for this tag:
	// the value is a series of calls to the DO set methods.
		
	setFirstName( 
	    
		rs.getString( 
			"firstName"  )
	    
	);
	
	
	setLastName( 
	    
		rs.getString( 
			"lastName"  )
	    
	);
	
	
	setPhoneNumber( 
	    
		rs.getString( 
			"phoneNumber"  )
	    
	);
	

 
        markClean();
    }

    

    

    private int[] param = null;

    /**
     * Prepares the statement used to insert this object
     * into the database.
     * @param conn the database connection.
     * @return the insert statement.
     * @exception java.sql.SQLException if an error occurs.
     */
    public PreparedStatement getInsertStatement(DBConnection conn)
            throws SQLException {
	/* FIX:  here I've hacked in some cache maintenance code.
	 * It would probably be better to have CoreDO implement
	 * 	void addToCache(CoreDO DO) {}
	 * and have each DO that has caching enabled override it as
	 *      void addToCache(CoreDO DO) { cache.put( DO.getOId(), DO ); }
	 * and change CoreDO to invoke addToCache()
	 * when it invokes getInsertStatement().
	 */

        ObjectId oid;

        PreparedStatement stmt = conn.prepareStatement( 
	    "insert into Person values ( ?, ?, ?, ?, ? )" );

	param = new int[1]; param[0] = 1;
	// writeMemberStuff uses the JDBCsetCalls.template
	// to build up the value for this tag:
	// the value is a series of calls to setPrepStmtParam_TYPE methods.
	// Those methods are defined in GenericDO.
	try {
	    	setPrepStmtParam_String( stmt, param,
		getFirstName() );
	setPrepStmtParam_String( stmt, param,
		getLastName() );
	setPrepStmtParam_String( stmt, param,
		getPhoneNumber() );


	    /* The order of the values being inserted must match
	     * the order of the columns listed in the CREATE TABLE command
	     * that appears in the .sql file for this DO.
	     * Since the id and version number are always the last columns
	     * listed in the CREATE TABLE command, their values are added
	     * to the PreparedStatement after all other values.
	     */
	    setPrepStmtParam_BigDecimal( stmt, param, getOId().toBigDecimal() );
	    setPrepStmtParam_int(        stmt, param, getNewVersion() );

	} catch ( Exception e ) {
	    throw new SQLException( "Data Object error: " + e.getMessage() );
	}

        return stmt;
    }

    /**
     * Prepares the statement used to update this object
     * in the database.
     * @param conn the database connection
     * @return the update statement.
     * @exception java.sql.SQLException if an error occurs.
     */
    public PreparedStatement getUpdateStatement(DBConnection conn)
            throws SQLException {

        ObjectId oid;

        PreparedStatement stmt = conn.prepareStatement(
	    "update Person set version = ?, firstName = ?, lastName = ?, phoneNumber = ? " +
	    "where oId = ? and version = ?" );

	param = new int[1]; param[0] = 1;
	//int[] param = {1};
	// writeMemberStuff builds up the value for this tag:
	// the value is a series of calls to setPrepStmtParam_TYPE methods.
	// Those methods are defined below.
	try {
	    setPrepStmtParam_int( stmt, param, getNewVersion() );
	    	setPrepStmtParam_String( stmt, param,
		getFirstName() );
	setPrepStmtParam_String( stmt, param,
		getLastName() );
	setPrepStmtParam_String( stmt, param,
		getPhoneNumber() );


	    /* When updating a persistent object, the UPDATE_WHERE_CLAUSE tag
	     * used to build the SQL WHERE clause (above) uses the 
	     * DO's id and version number to locate the correct row in
	     * the database table.
	     */
	    setPrepStmtParam_BigDecimal( stmt, param, getOId().toBigDecimal() );
	    setPrepStmtParam_int(        stmt, param, getVersion() );

	} catch ( Exception e ) {
	    throw new SQLException( "Data Object error: " + e.getMessage() );
	}

        return stmt;
    }

    /**
     * Prepares the statement used to delete this object
     * from the database.
     * @param conn the database connection
     * @return the delete statement.
     * @exception java.sql.SQLException if an error occurs.
     */
    public PreparedStatement getDeleteStatement(DBConnection conn)
            throws SQLException {

        String sql =
            "delete from Person \n" +
            "where oId = ?";
        PreparedStatement stmt = conn.prepareStatement(sql);
        stmt.setBigDecimal(1, getOId().toBigDecimal());
        return stmt;
    }

    

    /*
     * toString - for debugging
     */
    public String toString(){
	String str = "PersonDO:";
	ObjectId oid = getOId();
	String id = "virgin";
	if ( null != oid ) 
	    id = oid.toString();
	str += " OID=" + id;
	if ( null != data ) 
	    str = str + " firstName=" + data.firstName
+ " lastName=" + data.lastName
+ " phoneNumber=" + data.phoneNumber
;
        return str + "; " + super.toString();
    }

}
