#!/usr/bin/perl -w

# Takes a list of HTML filenames on the command line (optionally with
# a "-a" argument) and searches for the attribute name in starting
# tags.

# (c) Jason Henry Parker, jparker@pisoftware.com, and licensed under
# the same terms as Platypus (the GNU LGPL).

# $Log: list-attrs.pl,v $
# Revision 1.2  1999/10/22 07:31:00  dwood
# Added LGPL license info.
#
# Revision 1.1  1999/10/15 03:21:14  jparker
# Added list-attrs.pl perl script.
#
# Revision 1.1  1999/10/15 03:16:31  jparker
# Initial revision
#
# Revision 1.2  1999/10/15 02:07:10  jparker
# Works, but only gets ID tags.
#
# Revision 1.1  1999/10/14 01:28:22  jparker
# Initial revision
#

#
#  Copyright (C) 1999 Plugged In Software Pty Ltd.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  Plugged In Software
#  PO Box 4130
#  St. Lucia South Queensland
#  Australia  4067
#  info@PIsoftware.com
#

use strict;
use HTML::TokeParser;
use Getopt::Std;

my $RCSId = q( $Id: list-attrs.pl,v 1.2 1999/10/22 07:31:00 dwood Exp $ );

my %results;
my %opts;

my $tagname;

sub usage {
  die "Usage:  $0 [ -h | -a attribute ] <html files>\n";
}

if(!getopts('a:h',\%opts)) {
  usage;
}

if(defined($opts{h}) or $#ARGV<0) {
  usage;
}

if(!defined($opts{a})) {
  $tagname="id";
} else {
  $tagname=$opts{a};
}

while(my $fn = pop @ARGV) { # or die "no argument!\n";

  my $p = HTML::TokeParser->new($fn);

  $" = ", "; #"

  while(defined(my $tokeref = $p->get_tag)) {
    if($#{$tokeref}>1) { # if it's not an end tag...
      if(exists( $tokeref->[1]->{$tagname} )) {
        push @{$results{$tokeref->[1]->{$tagname}}}, "$fn:$tokeref->[-1]";
      }
    }
  }
}

for my $attrname (keys %results) {
  print $attrname;
  map { print "\t",$_,"\n" } @{$results{$attrname}};
}
