#!/usr/bin/perl -w
# converts sql table decls into dev.conf files (sent to standard
# output).
# Author:  jason parker, jparker@pisoftware.com, 19991004
# Licensed under the same terms as the product containing this
# script, or the GPL, whichever is more liberal.

#
#  Copyright (C) 1999 Plugged In Software Pty Ltd.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  Plugged In Software
#  PO Box 4130
#  St. Lucia South Queensland
#  Australia  4067
#  info@PIsoftware.com
#

use strict;
use English;

$" = ', ';

sub Next {
	if(defined($_=<FIL>)) {
		return chomp $_
	} else {
		if(eof) {
			return undef
		} else {
			die "unexpected error at line $.: $!\n"
		}
	}
}

### First pass, build a mapping from table names to class names.

my %table_to_class = ();

open FIL, "$ARGV[0]" or die "Could not open $ARGV[0]: $!\n";

while(Next) {
    my ($tblnam,$classname);
                                        
    if(($tblnam) = m/create table (\w+)/) {
        if(m/\(/) {          
            goto postopenparen
        }   
            
        if(Next and !m/^\(/) {
            die "expecting `(' at line $., got `$_'\n";
        }
	} else { next }

    postopenparen:

	while(Next and !m/^\)\;/) { 
		if(m{/\*\s+class\s+(\w+)\s+\*/}) {
			$classname = $1;
			if(exists($table_to_class{$tblnam}) and $table_to_class{$tblnam} ne $classname) {
				die "Name clash on $tblnam => $classname vs $table_to_class{$tblnam} at line $.\n";
			} else {
				print STDERR "<<<[$.] $tblnam\n";
				$table_to_class{$tblnam}=$classname;
			}
		}
	}
}

close FIL or die "Could not close $ARGV[0]: $!\n";

print STDERR "Finished first pass\n";

### Second pass, create output this time.

open FIL, "$ARGV[0]" or die "Could not open $ARGV[0]: $!\n";

while(Next) {
	my ($tblnam,$classname);

	if(($tblnam) = m/create table (\w+)/) { 
		if(m/\(/) {
			goto postopenparen
		}

		if(Next and !m/^\(/) {
			die "expecting `(' at line $., got `$_'\n";
		}

	postopenparen:

		my @decls;

		while(!m/^\)\;/) { 
			if(!Next and !m/^\)\;/) {
				die "failed to get `)' by line $.: $!\n";
			}

			if(m/class (\w+)/) {
				$classname = $1;
			} else { next }

			print "DO.TABLE.$classname.SQL_NAME=$tblnam\n";

			while(Next and !m/^\)\;/) {
				my ($colnam,$coltyp,$rest);
				next if m{/\*}; # need better comment regex.
				if(($colnam,$coltyp,$rest) = m/(\w+)\W+(\S+)\W*(.*),/) {
					if($coltyp =~ s/\((.+)\)//) { # look for something in brackets
						my $size = $1;
						print "DO.TABLE.$classname.COLUMN.$colnam.SIZE=$size\n";
					}

					print "DO.TABLE.$classname.COLUMN.$colnam.DODS_TYPE=$coltyp\n";

					if($rest =~ m/NOT\s+NULL/) {
						print "DO.TABLE.$classname.COLUMN.$colnam.NULL_ALLOWED=\"false\"\n"
					} else {
						print "DO.TABLE.$classname.COLUMN.$colnam.NULL_ALLOWED=\"true\"\n"
					}

					if($rest =~ m/DEFAULT\s+(\S+)/) {
						print "DO.TABLE.$classname.COLUMN.$colnam.DEFAULT_VALUE=\"$1\"\n"
					}

					if($rest =~ m/PRIMARY KEY/) {
						print "DO.TABLE.$classname.COLUMN.$colnam.PRIMARY_KEY=\"true\"\n";
					}

					# REFERENCES t_village(objectId)
					if((my $reffed) = ($rest =~ m/REFERENCES\s+(\w+)/)) {
						print STDERR ">>> $reffed\n";
						if(exists($table_to_class{$reffed})) {
							print "DO.TABLE.$classname.COLUMN.$colnam.REFERENCES=$table_to_class{$reffed}\n";
						}
					}
					print "\n";
				}
			}
		}

	postcloseparen:
		print "\n";
	}
}
