package com.pisoftware.platypus.billabong;

import  com.lutris.appserver.server.session.*;
import  com.lutris.util.*;
import  com.lutris.dods.builder.engine.*;
import  dods.data.*;
import  java.io.*;
import  java.util.*;
import  java.math.*;
import  java.net.*;
import  java.text.*;
import  com.pisoftware.platypus.util.*;
import  com.pisoftware.platypus.pond.*;
import  com.pisoftware.platypus.production.*;


/**
 *  Author: <A HREF="MAILTO:cryan@plugged.net.au">Chris Ryan</A>
 *<P>
 * Company: Plugged In Software<BR>
 *          P.O. Box 4130<BR>
 *          St Lucia South<BR>
 *          Brisbane Queensland 4067<BR>
 *          Australia<BR>
 * <A HREF="http://www.plugged.net.au/">http://www.plugged.net.au</A><BR>
 *<P>
 *    Date: 19 July 1999
 *<P>
 * Purpose: This application reads the application's global
 *          config file to determine the list of PO files in
 *          the system.  From this list, BO stub files are
 *          generated for each BO file that doesn't currently
 *          exist.  BO stub files are also generated for each
 *          sub-directory and for the application base BO.  Again
 *          these are only generated if they do not already exist.
 *
 *          Each sub-directory BO class extends the application
 *          base BO class.  All other BO stub classes extend
 *          the sub-directory BO class for which they are in.
 *          Those in the application base directory extend the
 *          application base BO class directly.
 *
 *          A special file is created in each directory after
 *          each run that holds the list of each new BO created.
 *          The contents of this file can be cut'n'paste into
 *          the Makefile for the directory.  (TODO)
 *
 *    TODO: 1. Extend this code to allow overwriting existing
 *             files, but to keep third party added code/comments.  
 *             This will probably require the third party additions 
 *             to be in specially marked blocks.
 *
 */

/*
 *  Copyright (C) 1999 Plugged In Software Pty Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Plugged In Software
 *  PO Box 4130
 *  St. Lucia South Queensland
 *  Australia  4067
 *  info@PIsoftware.com
 */

public class Billabong {


/**
 * The application name
 */
  private String application_name = null;


/**
 * The application global config filename - with path
 */
  private String application_config = null;


/**
 * The base package name
 */
  private String base_package_name = null;


/**
 * The starting output directory for the stubs, which
 * will probably be <appname>/<appname>/business.
 */
  private String output_dir = null;


/**
 * CORE is a required key in the global conf that contains
 * the PO subsection.
 */
  private static final String CORE = "CORE";


 /**
  * PO is a required key in the global conf that contains
  * the PO files within the application.  If there are
  * subsections to this key, then the listed POs
  * are in subdirectories under the base PO level.
  *
  * Some examples:
  *
  * CORE.PO.Welcome = true  <- Welcome PO in base dir.
  * CORE.PO.reports.Report1 = true   <- reports sub dir.
  */
  private static final String CORE_PO = "CORE.PO";
  private static final String PO = "PO";
  private static final String DO_TABLE = "DO.TABLE";


/**
 * CORE_APPLICATION refers to the application name in the
 * config file
 */
  private static final String CORE_APPLICATION = "CORE.APPLICATION_NAME";


/**
 * BASE_PACKAGE_NAME refers to the base java package name in the
 * config file.  If this doesn't exist, it is set to the
 * application name.
 */
  private static final String BASE_PACKAGE_NAME = "CORE.BASE_PACKAGE_NAME";


/**
 * The base application template filename & internal holder
 */
  private String base_application_template_file = null;
  private String base_app_template = null;


/**
 * The stubs application template filename & internal holder
 */
  private String stubs_template_file = null;
  private String stubs_template = null;


/**
 * The access method template filename & internal holder
 */
  private String access_template_file = null;
  private String access_template = null;


/**
 * The commit method template filename & internal holder
 */
  private String commit_template_file = null;
  private String commit_template = null;


/**
 * The getIdValue access method template filename & internal holder
 */
  private String getaccess_template_file = null;
  private String getaccess_template = null;


/**
 * The getBusinessIdValue method template filename & internal holder
 */
  private String business_template_file = null;
  private String business_template = null;


/**
 * The id template filename & internal holder
 */
  private String id_template_file = null;
  private String id_template = null;


/**
 * The id (int index) template filename & internal holder
 */
  private String int_index_template_file = null;
  private String int_index_template = null;


/**
 * The BDO related templates - filename & internal holder
 */
  private String bdoid_template_file = null;
  private String bdoid_template = null;
  private String bdogetaccess_template_file = null;
  private String bdogetaccess_template = null;
  private String bdostubs_template_file = null;
  private String bdostubs_template = null;


/**
 * The hashtable containing the strings for substitution.
 */
  private Hashtable substitutions = new Hashtable(100);


/**
 * The following strings are used as the keys to the substitution
 * hashtable, config files, etc.
 *
 * TODO: separate & doco...
 */
  private final static String SUB_DATE = "DATE";   // current date
  private final static String SUB_APPLICATION = "APP_NAME";  // application name
  private final static String SUB_JNAME = "APP_JNAME";  // application name, Java-ised
  private final static String SUB_BASE_PACKAGE = "BASE_PACKAGE_NAME";  // application name, Java-ised
  private final static String BDO_NAME_DEFS = "BDO_NAME_DEFS";
  private final static String BO_KEYWORD_VALUES = "BO_KEYWORD_VALUES";
  private final static String CREATE_SAVE_BO_OBJECTS = "CREATE_SAVE_BO_OBJECTS";
  private final static String RESTORE_BO_OBJECTS = "RESTORE_BO_OBJECTS";
  private final static String PACKAGE = "PACKAGE";
  private final static String CLASS_JNAME = "CLASS_JNAME";
  private final static String EXTEND_JNAME = "EXTEND_JNAME";
  private final static String BEGIN_ACCESS_METHODS = "BEGIN_ACCESS_METHODS";
  private final static String END_ACCESS_METHODS = "END_ACCESS_METHODS";
  private final static String ACCESS_METHODS = "ACCESS_METHODS";
  private final static String BEGIN_COMMIT_METHODS = "BEGIN_COMMIT_METHODS";
  private final static String END_COMMIT_METHODS = "END_COMMIT_METHODS";
  private final static String COMMIT_METHODS = "COMMIT_METHODS";
  private final static String ACCESSES = "ACCESSES";
  private final static String COMMITS = "COMMITS";
  private final static String ACCESS_NAME = "ACCESS_NAME";
  private final static String ACCESS_PARAMS = "ACCESS_PARAMS";
  private final static String ACCESS_JAVADOC_PARAMS = "ACCESS_JAVADOC_PARAMS";
  private final static String COMMIT_NAME = "COMMIT_NAME";
  private final static String COMMIT_PARAMS = "COMMIT_PARAMS";
  private final static String COMMIT_JAVADOC_PARAMS = "COMMIT_JAVADOC_PARAMS";
  private final static String QUERYARGCOUNT = "QUERYARGCOUNT";
  private final static String BEGIN_GET_ACCESS_METHODS = "BEGIN_GET_ACCESS_METHODS";
  private final static String END_GET_ACCESS_METHODS = "END_GET_ACCESS_METHODS";
  private final static String GET_ACCESS_METHODS = "GET_ACCESS_METHODS";
  private final static String BEGIN_BUSINESS_METHODS = "BEGIN_BUSINESS_METHODS";
  private final static String END_BUSINESS_METHODS = "END_BUSINESS_METHODS";
  private final static String BUSINESS_METHODS = "BUSINESS_METHODS";
  private final static String IDS = "IDS";
  private final static String DATAFROM = "DATAFROM";
  private final static String DATAFROM_DATABASE = "DATABASE";
  private final static String DATAFROM_BUSINESS = "BUSINESS";
  private final static String ACCESS_DEFS = "ACCESS_DEFS";
  private final static String ELSE = "ELSE";
  private final static String INT_INDEX_SECTION = "INT_INDEX_SECTION";
  private final static String ID_LIST = "ID_LIST";
  private final static String INT_ID_LIST = "INT_ID_LIST";
  private final static String OID_ID_LIST = "OID_ID_LIST";
  private final static String GET_CALL = "GET_CALL";
  private final static String RESULT_DEF = "RESULT_DEF";
  private final static String ACCESS_TABLE = "TABLE";
  private final static String ID_NAME = "ID_NAME";
  private final static String FIELDPATH = "FIELDPATH";
  private final static String ACCESS = "ACCESS";
  private final static String ID_TABLE_NAME = "ID_TABLE_NAME";
  private final static String SET_QUERY_RESULTS = "SET_QUERY_RESULTS";
  private final static String DB_HANDLE = "ObjectId";
  private final static String EXCEPTION_TYPE = "EXCEPTION_TYPE";
  private final static String EXCEPTION = "Exception";
  private final static String DATAOBJECT = "DataObjectException";
  private final static String APP_GET_BUSINESS_ID_VALUE = "APP_GET_BUSINESS_ID_VALUE";
  private final static String BUSINESS_ID_RETURN = "BUSINESS_ID_RETURN";


/**
 * Holds the table names within the DO layer
 */
  private String[] table_names = null;

/**
 * A hashtable containing the columns within a table.
 * The key to the hashtable is the table name and the
 * object inside is a String[].
 */
  private Hashtable column_names = new Hashtable(100);

/**
 * A hashtable containing the code generated for each
 * BDO - BO class.
 * The key to the hashtable is the table name and the
 * object inside is a String[] with 2 entries - one
 * for the getIdValue(int...) and the other is
 * getIdValue(oidstr...).
 */
  private Hashtable bdo_code = new Hashtable(100);


/**
 * Indicates if a a file should be overwritten.
 */
  private boolean overwrite = false;


/**
 * Data about the DODS types available
 */
  private DODSTypes dodsTypes = null;


/**
 * Constructor - keeps values passed.
 * 
 * @param application_config   The application global config file name
 * @param output_dir  The destination base directory for the stubs
 */
  public Billabong(String application_config, String output_dir) {

    this.application_config = application_config;
    this.output_dir = output_dir;
    this.dodsTypes = DODSTypes.getInstance();

  }


/**
 * The arguments expected to the program are:
 *
 * 1. The application global config file name
 * 2. Output directory name for the generated code
 */
  public static void main( String argv[] ) throws Exception {

    if (argv.length != 2 && argv.length != 3) {

      System.err.println("Not enough parameters!\n");
      System.err.println("Usage: Billabong <app_conf> <output_dir>\n");
      System.err.println("<app_conf> is the application global config file");
      System.err.println("<output_dir> is the output directory name for the generated code");
      System.err.println("[OVERWRITE] optional, must be in caps.  Overwrites any existing files.  USE WITH CAUTION!");
      System.exit(1);

    }

    Billabong billabong = new Billabong(argv[0], argv[1]);

    // set overwrite flag, if necessary
    if (argv.length == 3) {

      if (argv[2].equals("OVERWRITE")) {

        System.out.println("OVERWRITING files...");
        billabong.overwrite = true;

      } else {

        System.err.println("Unknown third parameter: "+argv[2]);
        System.exit(1);

      }

    }

    // get the Billabong base directory from the system properties.  
    // If not found, then current directory is assumed
    String bos_base_dir = System.getProperty("billabong.location", ".");

    // create the template filenames
    billabong.base_application_template_file = bos_base_dir + File.separator + "templates" + File.separator + "appbase.template";
    billabong.stubs_template_file = bos_base_dir + File.separator + "templates" + File.separator + "stubs.template";
    billabong.access_template_file = bos_base_dir + File.separator + "templates" + File.separator + "access.template";
    billabong.commit_template_file = bos_base_dir + File.separator + "templates" + File.separator + "commit.template";
    billabong.getaccess_template_file = bos_base_dir + File.separator + "templates" + File.separator + "getaccess.template";
    billabong.business_template_file = bos_base_dir + File.separator + "templates" + File.separator + "business.template";
    billabong.id_template_file = bos_base_dir + File.separator + "templates" + File.separator + "idaccess.template";
    billabong.int_index_template_file = bos_base_dir + File.separator + "templates" + File.separator + "intidaccess.template";
    billabong.bdoid_template_file = bos_base_dir + File.separator + "templates" + File.separator + "bdoidaccess.template";
    billabong.bdogetaccess_template_file = bos_base_dir + File.separator + "templates" + File.separator + "bdogetaccess.template";
    billabong.bdostubs_template_file = bos_base_dir + File.separator + "templates" + File.separator + "bdostubs.template";

    // read the templates
    billabong.base_app_template = billabong.readTemplate(billabong.base_application_template_file);
    billabong.stubs_template = billabong.readTemplate(billabong.stubs_template_file);
    billabong.access_template = billabong.readTemplate(billabong.access_template_file);
    billabong.commit_template = billabong.readTemplate(billabong.commit_template_file);
    billabong.getaccess_template = billabong.readTemplate(billabong.getaccess_template_file);
    billabong.business_template = billabong.readTemplate(billabong.business_template_file);
    billabong.id_template = billabong.readTemplate(billabong.id_template_file);
    billabong.int_index_template = billabong.readTemplate(billabong.int_index_template_file);
    billabong.bdoid_template = billabong.readTemplate(billabong.bdoid_template_file);
    billabong.bdogetaccess_template = billabong.readTemplate(billabong.bdogetaccess_template_file);
    billabong.bdostubs_template = billabong.readTemplate(billabong.bdostubs_template_file);

    // kick the process off...
    billabong.process();

  }


/**
 * Reads a template file and holds onto the contents in a String.
 *
 * @param template_file  The template file to read. 
 */
  public String readTemplate(String template_file) throws Exception {

    return Util.readFile(template_file);

  }


/**
 * Handle to the application wide config file
 */
  private KeywordValueTable app_cfg = null;


/**
 * Processes the configuration keyword value table to
 * generate the necessary stub files.
 */
  public void process() throws Exception {

    // Get the data from the config file
    KeywordValueFiles cfg = new KeywordValueFiles(application_config,false);

    // ensure all sections are read in now
    cfg.readAllSections(true);

    // keep the application config
    app_cfg = cfg;

    // DBG
    // System.err.println("Config toString: "+cfg.toString());

/*
    KeywordValueTable cfg = new KeywordValueTable();

    cfg.set(CORE_APPLICATION,"test");
    cfg.set("CORE.PO.Welcome","true");
    cfg.set("CORE.PO.reports.Sales","true");
    cfg.set("CORE.PO.maintenance.AddNewWidget","true");
    cfg.set("CORE.PO.entry.managers.NewItem","true");

    // DB stuff...
    cfg.set("DO.TABLE.Person.COLUMN.FirstName.JAVA_TYPE", "String");
    cfg.set("DO.TABLE.Person.COLUMN.Surname.JAVA_TYPE", "String");
    cfg.set("DO.TABLE.Person.COLUMN.Age.JAVA_TYPE", "int");
    cfg.set("DO.TABLE.Person.COLUMN.Birthday.JAVA_TYPE", "java.sql.Date");

    cfg.set("DO.TABLE.Company.COLUMN.Name.JAVA_TYPE", "String");
    cfg.set("DO.TABLE.Company.COLUMN.Address.JAVA_TYPE", "String");

    cfg.set("DO.TABLE.Product.COLUMN.Name.JAVA_TYPE", "String");
    cfg.set("DO.TABLE.Product.COLUMN.Amount.JAVA_TYPE", "double");

*/
// End DBG


    // keep the application name handy...
    application_name = cfg.getString(CORE_APPLICATION);

    if (application_name == null) {

      System.err.println(CORE_APPLICATION+" is not set in the configuration file!");
      System.exit(1);

    }

    // keep the base package name handy...
    base_package_name = cfg.getString(BASE_PACKAGE_NAME);

    if (base_package_name == null) {

      // not set, use application name.
      base_package_name = application_name;

    }

    // get the section containing the DO layer info
    KeywordValueTable do_kvt = cfg.getSection(DO_TABLE);

    // any DO layer data?
    if (do_kvt != null) {

      // set the DBMS for the DODS Types
      dodsTypes.setDBMS(app_cfg.getString("DO.DBMS"));

      // get the database tables
      table_names = do_kvt.keys();

      // get the columns of the tables
      for (int i=0; i<table_names.length; i++) {

        // get the column section within this particular table
        KeywordValueTable col_kvt = do_kvt.getSection(table_names[i]+".COLUMN");

        // get the column names
        String[] columns = col_kvt.keys();
        
        // save these in the hastable
        column_names.put(table_names[i], columns);

      }

    }

    // setup the substitution hashtable

    // current date
    substitutions.put(SUB_DATE, (DateFormat.getDateInstance()).format(new Date()).toString());

    // application name
    substitutions.put(SUB_APPLICATION, application_name);

    // application name, Java-ised
    substitutions.put(SUB_JNAME, Util.toWIC(application_name));

    // base java package name
    substitutions.put(SUB_BASE_PACKAGE, base_package_name);

    // end setup of substitution hashtable


    // Generate the base application BO stub
    generateBaseStub(application_name, output_dir);

    KeywordValueTable pocfg = cfg.getSection(PO);

    if (pocfg == null) {

      System.err.println("Can't find the PO section!");
      System.exit(1);

    }

    // Generate the stubs
    generateStubs(cfg.getSection(CORE_PO), pocfg, output_dir, "");

    // Generate the BDO stubs - if we've got something to generate...
    if (bdo_code.size() > 0)
      generateBDOStubs(output_dir);

    // The End

  }


/**
 * Generates the Base application BO stub.
 *
 * @param app_name  The name of the application
 * @param output_dir  The destination base directory for the stub
 * @exception Exception  If an unrecoverable error occurs.
 */
  private void generateBaseStub(String app_name, String output_dir) throws Exception {

    // make the output directory
    Util.makeDirectory(output_dir);

    // create a Java-ised version of the application name
    String jname = Util.toWIC(app_name);

    // make the filename for this base stub
    String app_filename = output_dir + File.separator + jname + "BO.java";

    // do we have DO data to use?
    if (table_names != null) {

      // yes, setup the related information

      /*
       * The following block creates the code for the following tags:
       * BDO_NAME_DEFS - the BDO-BO variable definitions
       * BO_KEYWORD_VALUES - the static strings for session data lookups
       * CREATE_SAVE_BO_OBJECTS - the instantiation of the BDO-BOs and saving them in the session data
       * RESTORE_BO_OBJECTS - the restoration of the BDO-BOs from the session data
       */
      StringBuffer bdo_name_defs = new StringBuffer();
      StringBuffer bo_keyword_values = new StringBuffer();
      StringBuffer create_save_bo_objects = new StringBuffer();
      StringBuffer restore_bo_objects = new StringBuffer();
      for (int i=0; i<table_names.length; i++) {

        // ensure this table has an entry in the hash for BDO code generation later
        String[] tmp_array = { "", "" };
        bdo_code.put(table_names[i], tmp_array);

        // create a variable name for this table
        char first_char = Character.toLowerCase(table_names[i].charAt(0));
        String varname = first_char + table_names[i].substring(1); 

        bdo_name_defs.append("  public ");
        bdo_name_defs.append(table_names[i]);
        bdo_name_defs.append("BO ");
        bdo_name_defs.append(varname);
        bdo_name_defs.append("BO = null;\n");

        bo_keyword_values.append("  protected final static String BO_");
        bo_keyword_values.append(table_names[i].toUpperCase());
        bo_keyword_values.append(" = \"");
        bo_keyword_values.append(table_names[i].toLowerCase());
        bo_keyword_values.append("\";\n");

        create_save_bo_objects.append("        ");
        create_save_bo_objects.append(varname);
        create_save_bo_objects.append("BO = new ");
        create_save_bo_objects.append(table_names[i]);
        create_save_bo_objects.append("BO();\n");
        create_save_bo_objects.append("        ");
        create_save_bo_objects.append("comms.sessionData.set(BO_KEYWORD+\".\"+BO_");
        create_save_bo_objects.append(table_names[i].toUpperCase());
        create_save_bo_objects.append(",");
        create_save_bo_objects.append(varname);
        create_save_bo_objects.append("BO);\n\n");

        restore_bo_objects.append("        ");
        restore_bo_objects.append(varname);
        restore_bo_objects.append("BO = (");
        restore_bo_objects.append(table_names[i]);
        restore_bo_objects.append("BO) BO_KeywordValueTable.get(BO_");
        restore_bo_objects.append(table_names[i].toUpperCase());
        restore_bo_objects.append(");\n");

      }

      // put the results into the substitutions hashtable
      substitutions.put(BDO_NAME_DEFS, bdo_name_defs.toString());
      substitutions.put(BO_KEYWORD_VALUES, bo_keyword_values.toString());
      substitutions.put(CREATE_SAVE_BO_OBJECTS, create_save_bo_objects.toString());
      substitutions.put(RESTORE_BO_OBJECTS, restore_bo_objects.toString());

    } else {

      // No DO stuff, ensure it's commented out
      substitutions.put(BDO_NAME_DEFS, "/* No BDO Definitions */");
      substitutions.put(BO_KEYWORD_VALUES, "/* No BDO Definitions */");
      substitutions.put(CREATE_SAVE_BO_OBJECTS, "/* No BDO Definitions to create/save */");
      substitutions.put(RESTORE_BO_OBJECTS, "/* No BDO Definitions to restore */");

    }

    // create the application business id values
    substitutions.put(BUSINESS_ID_RETURN,"return \"ID value is: \"+id+\", format method is: <\"+format+\">\";");
    substitutions.put(APP_GET_BUSINESS_ID_VALUE,generateAppBusinessMethod());

    // apply the substitutions necessary to the template
    String results = Util.applySubstitutions(substitutions, base_app_template);

    // does it already exist?
    if (!overwrite && Util.fileExists(app_filename)) {

      System.out.println("Application base file <"+app_filename+"> already exists, skipping code generation");
      return;

    }

    // write out the results
    Util.writeFile(app_filename, results);

    // status message
    System.out.println("Application base file <"+app_filename+"> created");

  }


/**
 * Generates the Base BO stub for a sub directory.  The
 * name of the directory is converted to a Java-ised name
 * by ensuring the first character of the directory name
 * is capitalised, underscores are removed, and characters
 * after the underscore are capitalised.
 *
 * @param dir_name  The name of the sub directory.
 * @param output_dir  The destination base directory for the stub
 * @return The Java-ised name of the directory
 * @exception Exception  If an unrecoverable error occurs.
 */
  private void generateDirBaseStub(String dir_name, String output_dir) throws Exception {

    // ensure the directory is created
    Util.makeDirectory(output_dir);

    // build the filename for this stub
    String file_name = output_dir + File.separator + Util.toWIC(dir_name) + "BO.java";

    // does this file already exist?
    if (!overwrite && Util.fileExists(file_name)) {

      // move along, nothing to do here...
      System.out.println("Directory base file <"+file_name+"> already exists, skipping stub generation");
      return;

    }

    // extract the difference between this output dir & the base output dir &
    // replace the file separator with dot to get a relative package name
    String relative_package = output_dir.substring(this.output_dir.length()+1).replace(File.separatorChar,'.');

    // put the package name in the substitutions...
    substitutions.put(PACKAGE,base_package_name+".business."+relative_package);

    // create the Java-ised name for substitution
    substitutions.put(CLASS_JNAME, Util.toWIC(dir_name));

    // determine the class to extend
    if (relative_package.equals(dir_name)) {

      // extend the base application class
      substitutions.put(EXTEND_JNAME, Util.toWIC(application_name));

    } else {

      // extend the directory's class above
      
      // tokenise the relative package name to get directory part names.
      StringTokenizer strtok = new StringTokenizer(relative_package, ".");

      String parent_dir = "";

      // got through the tokens...
      while (strtok.hasMoreTokens()) {

        // get it.
        String this_one = strtok.nextToken();

        // if there's still more
        if (strtok.hasMoreTokens()) {

          // keep this one - we want the second last directory part name
          parent_dir = this_one;

        }

      }

      substitutions.put(EXTEND_JNAME, Util.toWIC(parent_dir));

    }

    // for directory base stubs, there are no access/commit methods by default
    substitutions.put(BEGIN_ACCESS_METHODS, "");
    substitutions.put(END_ACCESS_METHODS, "");
    substitutions.put(ACCESS_METHODS, "");
    substitutions.put(BEGIN_COMMIT_METHODS, "");
    substitutions.put(END_COMMIT_METHODS, "");
    substitutions.put(COMMIT_METHODS, "");
    substitutions.put(BEGIN_GET_ACCESS_METHODS, "");
    substitutions.put(END_GET_ACCESS_METHODS, "");
    substitutions.put(GET_ACCESS_METHODS, "");
    substitutions.put(BEGIN_BUSINESS_METHODS, "");
    substitutions.put(END_BUSINESS_METHODS, "");
    substitutions.put(BUSINESS_METHODS, "");

    // apply the necessary substitutions to a copy of the template
    String results = Util.applySubstitutions(substitutions, new String(stubs_template));

    // write the template out to file
    Util.writeFile(file_name, results);

    System.out.println("Directory base file <"+file_name+"> created");

// TODO - add filename details to list file.

  }


  // flag to indicate if 'else' is required in the getIdValue code...
  private boolean first_id_flag = true;


/**
 * Generates the BO stubs.
 *
 * @param cfg  The Config containing the necessary details
 * @param pocfg  The PO Config section relating to this BO
 * @param output_dir  The destination base directory for the stubs
 * @param dir_base_name  The name of the directory base BO stub
 * @exception Exception  If an unrecoverable error occurs.
 */
  private void generateStubs(KeywordValueTable cfg, KeywordValueTable pocfg, String output_dir, String dir_base_name) throws Exception {

    // current directory java name - used for 'extends' in classes
    String dir_java_name = Util.toWIC(dir_base_name);

    // get the keys for this config
    String[] keys = cfg.keys();

    // go through these keys...
    for (int i=0; i< keys.length; i++) {

      // DBG
      // System.err.println("keys["+i+"] is: "+keys[i]);

      // get the object for this key
      Object keyval = cfg.get(keys[i]);

      // is this in fact a subsection?
      if (keyval instanceof KeywordValueTable) {

        // yes, create the base stub for this section
        generateDirBaseStub(keys[i], output_dir + File.separator + keys[i]); 

        KeywordValueTable kvt = (KeywordValueTable) keyval;

        // get the base name of this table
        String kvt_str = kvt.toString();

        // get the PO config that matches 
        KeywordValueTable pokvt = pocfg.getSection(keys[i]);

        // recurse, with the appropriate subsection & output
        generateStubs(kvt, pokvt, output_dir + File.separator + keys[i], keys[i]); 

      } else {

        // leaf node - possibly generate the code

        // convert the object to string name
        String file_name = output_dir + File.separator + ((String) keys[i]) + "BO.java";

        // does this file already exist?
        if (overwrite || !Util.fileExists(file_name)) {

          // code generation time...
          System.out.println("Generating stub code for File <"+file_name+">");

          // are we in the base directory?
          if (dir_base_name.equals("")) {

            // yes, put the package name in the substitutions as the base value
            substitutions.put(PACKAGE,base_package_name+".business");

          } else {
 
            // no, so extract the difference between this output dir & the base output dir &
            // replace the file separator with dot to get a relative package name
            String relative_package = dir_base_name.replace(File.separatorChar,'.');

            // put the package name in the substitutions...
            substitutions.put(PACKAGE,base_package_name+".business."+relative_package);

          }

          // create the Java-ised name for substitution
          substitutions.put(CLASS_JNAME, Util.toWIC((String) keys[i]));

          // fix what this class extends
          if (dir_base_name.equals("")) { 

            // base directory, therefore use the application base class
            substitutions.put(EXTEND_JNAME, Util.toWIC(application_name));

          } else {

            // sub directory, so use this directory's base class
            substitutions.put(EXTEND_JNAME, dir_java_name);

          }

          // DBG
          // System.err.println("PO subsection is: "+pocfg.toString());

          // get the ACCESSES section of this PO
          KeywordValueTable access_cfg = pocfg.getSection(keys[i]+"."+ACCESSES);

          // any Access Methods?
          if (access_cfg != null) {

            // yes, generate the code
            String access_methods = generateAccessMethods(access_cfg);
            String access_defs = generateAccessDefs(access_cfg);

            // set substitutions
            substitutions.put(ACCESS_METHODS, access_methods);
            substitutions.put(ACCESS_DEFS, access_defs);
            substitutions.put(BEGIN_ACCESS_METHODS, "\n/*\n * Start of Access Methods\n*/\n");
            substitutions.put(END_ACCESS_METHODS, "\n/*\n * End of Access Methods\n*/\n\n");

          } else {

            substitutions.put(BEGIN_ACCESS_METHODS, "/*");
            substitutions.put(ACCESS_DEFS, "");
            substitutions.put(ACCESS_METHODS, "  No Access Method(s) Generated!");
            substitutions.put(END_ACCESS_METHODS, "*/");

          }

          // get the COMMITS section of this PO
          KeywordValueTable commit_cfg = pocfg.getSection(keys[i]+"."+COMMITS);

          // any Commit Methods?
          if (commit_cfg != null) {

            // yes, generate the code
            String commit_methods = generateCommitMethods(commit_cfg);

            // set substitutions
            substitutions.put(COMMIT_METHODS, commit_methods);
            substitutions.put(BEGIN_COMMIT_METHODS, "\n/*\n * Start of Commit Methods\n*/\n");
            substitutions.put(END_COMMIT_METHODS, "\n/*\n * End of Commit Methods\n*/\n\n");

          } else {

            substitutions.put(BEGIN_COMMIT_METHODS, "/*");
            substitutions.put(COMMIT_METHODS, "  No Commit Method(s) Generated!");
            substitutions.put(END_COMMIT_METHODS, "*/");

          }

          // get the IDS section of this PO
          KeywordValueTable ids_cfg = pocfg.getSection(keys[i]+"."+IDS);
          String getAccessMethods = "";
          String businessMethods = "";

          // any IDS to process?
          if (ids_cfg != null) {

            first_id_flag = true;
     
            // yes, generate the code
            if (access_cfg != null)
              getAccessMethods = generateGetAccessMethods(ids_cfg, access_cfg);
            businessMethods = generateBusinessMethods(ids_cfg);

            // set substitutions
            substitutions.put(GET_ACCESS_METHODS, getAccessMethods);
            substitutions.put(BUSINESS_METHODS, businessMethods);

          }

          // any getIdValue methods generated?
          if (getAccessMethods.length() == 0) {

            // no, indicate so in code
            substitutions.put(BEGIN_GET_ACCESS_METHODS, "/*");
            substitutions.put(GET_ACCESS_METHODS, "  No getIdValue Method(s) Generated!");
            substitutions.put(END_GET_ACCESS_METHODS, "*/");

          } else {

            // yes, put start/end markers
            substitutions.put(BEGIN_GET_ACCESS_METHODS, "\n/*\n * Start of getIdValue Methods\n*/\n");
            substitutions.put(END_GET_ACCESS_METHODS, "\n/*\n * End of getIdValue Methods\n*/\n\n");

          }

          // any getBusinessIdValue methods generated?
          if (businessMethods.length() == 0) {

            // no, indicate so in code
            substitutions.put(BEGIN_BUSINESS_METHODS, "/*");
            substitutions.put(BUSINESS_METHODS, "  No getBusinessIdValue Method Generated!");
            substitutions.put(END_BUSINESS_METHODS, "*/");

          } else {

            // yes, put start/end markers
            substitutions.put(BEGIN_BUSINESS_METHODS, "\n/*\n * Start of getBusinessIdValue Method\n*/\n");
            substitutions.put(END_BUSINESS_METHODS, "\n/*\n * End of getBusinessIdValue Method\n*/\n\n");

          }

          // apply the necessary substitutions to a copy of the template
          String results = Util.applySubstitutions(substitutions, new String(stubs_template));

          // write the template out to file
          Util.writeFile(file_name, results);

// TODO - add filename details to list file.

        } else {

          // move along, nothing to do here...
          System.out.println("File <"+file_name+"> already exists, skipping stub generation");

        }

      }

    }

  }


/**
 * Generates the getIdValue id list.  Uses IDS that have
 * a DATAFROM value of ACCESS.
 *
 * @param ids_cfg  The Config containing the necessary details
 * @param access_cfg  The Config containing the necessary details
 * @exception Exception  If an unrecoverable error occurs.
 */
  private String generateGetAccessMethods(KeywordValueTable ids_cfg, KeywordValueTable access_cfg) throws Exception {

    // The results...
    String results = "";
    StringBuffer int_idlist = new StringBuffer();
    StringBuffer oid_idlist = new StringBuffer();

    // get the keys for this config
    String[] keys = ids_cfg.keys();

    boolean apply_subs = false;

    // go through these keys...
    for (int i=0; i< keys.length; i++) {

      // get the section for this id
      //KeywordValueTable id_kvt = ids_cfg.getSection(keys[i]);
      KeywordValueTable id_kvt = KeywordValueFiles.getFilledSection(app_cfg,ids_cfg,keys[i]);
     
      // get the DATAFROM for this key
      String datafrom = id_kvt.getString(DATAFROM, "none");

      // is it from the db?
      if (datafrom.equalsIgnoreCase(DATAFROM_DATABASE)) {

        // first time through?
        if (!first_id_flag) {

          // no, need to add the else condition
          substitutions.put(ELSE, " else ");

        } else {

          substitutions.put(ELSE, " ");

        }

        first_id_flag = false;

        int_idlist.append(generateIdList(keys[i], id_kvt, access_cfg, false));
        oid_idlist.append(generateIdList(keys[i], id_kvt, access_cfg, true));

      }

    }

    // any 'int' ids created?
    if (int_idlist != null && int_idlist.length() > 0) {

      // keep the id list created
      substitutions.put(INT_ID_LIST, int_idlist.toString());

      apply_subs = true;

    }

    // only need to apply substitutions if some ids created!
    if (oid_idlist != null && oid_idlist.length() > 0) {

      // keep the id list created
      substitutions.put(OID_ID_LIST, oid_idlist.toString());

      apply_subs = true;

    }

    // has the flag been set to allow substitutions to be applied?
    if (apply_subs) {

      // yes, apply substitutions
      results = Util.applySubstitutions(substitutions, new String(getaccess_template));

    }

    // return the results
    return results;

  }


/**
 * Generates the code for this id for the getIdValue that takes an
 * int argument.
 *
 * @param id  The id being processed.
 * @param id_kvt  The Config containing the necessary details
 * @param access_cfg  The Config containing the necessary details
 * @param oid_flag  True if generating code for the oid string getIdValue method
 * @exception Exception  If an unrecoverable error occurs.
 */
  private String generateIdList(String id, KeywordValueTable id_kvt, KeywordValueTable access_cfg, boolean oid_flag) throws Exception {

    // The results...
    String results = "";

    // get the relevant access for this id
    String access_name = id_kvt.getString(ACCESS, null);

    // get the "fieldpath" for this id
    String fieldpath = id_kvt.getString(FIELDPATH, null);

    // DBG
    // System.out.println("Fieldpath is: "+fieldpath);
     
    // any db field to use?
    if (fieldpath == null) {

      // no, so this id just has to be handled by a human.
      StringBuffer special_results = new StringBuffer();
      if (!first_id_flag) {
        special_results.append(substitutions.get(ELSE));
      }
      special_results.append("if (id.equalsIgnoreCase(\""+id+"\")) {\n /* TODO: HANDLE THIS ID!!! */\n} ");
      return special_results.toString();

    }

    // get the table name for this access
    String table_name = access_cfg.getString(access_name+".TABLE", null);

    if (table_name == null)
      throw new Exception("Bad Config setting! TABLE not set for this access: "+access_name);

    // create the tableBO name
    char first_char = Character.toLowerCase(table_name.charAt(0));
    String table_bo_name = first_char+table_name.substring(1)+"BO";

    // setup substitutions
    substitutions.put(ID_NAME, id);

    StringBuffer getcall_str = new StringBuffer();
    StringBuffer query_str = new StringBuffer();

    getcall_str.append(table_bo_name);
    getcall_str.append(".getIdValue(");

    // generating code for the getIdValue that takes the oid string?
    if (oid_flag) {

      getcall_str.append("oidstr, id, format");
      query_str.append(" "); // make the set query empty - not needed

    } else {

      getcall_str.append("index, id, format");
      query_str.append(table_bo_name);
      query_str.append(".setQueryResults(");
      query_str.append(access_name);
      query_str.append(");");

    }

    getcall_str.append(")");

    // the call that will retrieve the field
    substitutions.put(GET_CALL, getcall_str.toString());
    substitutions.put(SET_QUERY_RESULTS, query_str.toString());

    // apply substitutions
    results = Util.applySubstitutions(substitutions, new String(id_template));

    // return the results
    return results;

  }


/**
 * Generates the getBusinessIdValue method.  Uses IDS that have
 * a DATAFROM value of BUSINESS.
 *
 * @param ids_cfg  The Config containing the necessary details
 * @exception Exception  If an unrecoverable error occurs.
 */
  private String generateBusinessMethods(KeywordValueTable ids_cfg) throws Exception {

    // The results...
    String results = "";
    StringBuffer idlist = new StringBuffer();

    boolean first_value = true;

    // get the keys for this config
    String[] keys = ids_cfg.keys();

    // go through these keys...
    for (int i=0; i< keys.length; i++) {

      // get the section for this id
      //KeywordValueTable id_kvt = ids_cfg.getSection(keys[i]);
      KeywordValueTable id_kvt = KeywordValueFiles.getFilledSection(app_cfg,ids_cfg,keys[i]);
     
      // get the DATAFROM for this key
      String datafrom = id_kvt.getString(DATAFROM, "none");

      // is it from business method?
      if (datafrom.equalsIgnoreCase(DATAFROM_BUSINESS)) {

        // first time through?
        if (!first_value) {

          // no, need to add the else condition
          idlist.append(" else ");

        }

        first_value = false;

        idlist.append("    if (id.equalsIgnoreCase(\"");
        idlist.append(keys[i]);
        idlist.append("\")) {\n\n      id_found = true;\n\n      /* TODO: Handle this id */\n\n    }");

      }

    }

    // set up the return value
    substitutions.put(BUSINESS_ID_RETURN,"return super.getBusinessIdValue(id,format);");

    // only need to apply substitutions if some ids created!
    if (idlist.length() > 0) {

      // keep the id list created
      substitutions.put(ID_LIST, idlist.toString());

      // apply substitutions
      results = Util.applySubstitutions(substitutions, new String(business_template));

    }

    // return the results
    return results;

  }


/**
 * Generates the access methods.
 *
 * @param access_cfg  The Config containing the necessary details
 * @exception Exception  If an unrecoverable error occurs.
 */
  private String generateAccessMethods(KeywordValueTable access_cfg) throws Exception {

    // The results...
    StringBuffer results = new StringBuffer();

    // get the keys for this config
    String[] keys = access_cfg.keys();

    // go through these keys...
    for (int i=0; i< keys.length; i++) {

      // DBG
      // System.err.println("Access keys["+i+"] is: "+keys[i]);

      // get the section for this key
      KeywordValueTable access_section = access_cfg.getSection(keys[i]);

      // the name of this access method
      substitutions.put(ACCESS_NAME, keys[i]);

      // clear the parameters in case there's none
      substitutions.put(ACCESS_PARAMS, "");
      substitutions.put(ACCESS_JAVADOC_PARAMS, "");

      // get the number of parameters for this access method
      int nr_params = Integer.parseInt(access_section.getString(QUERYARGCOUNT,"0"));

      // setup the parameters
      StringBuffer aparams = new StringBuffer();
      StringBuffer ajparams = new StringBuffer();
      for (int j=0; j<nr_params; j++) {

        // if we're past the first parameter
        if (j>0) {

          // need to put in separators
          aparams.append(", ");

        }

        // get this parameter's field name
        String query_arg = "QUERYARG"+j;
        String field_name = access_section.getString(query_arg);

        aparams.append("String ");
        aparams.append(field_name);

        ajparams.append("\n * @param ");
        ajparams.append(field_name);
        ajparams.append(" Parameter supplied by PO");

      }

      // any parameters generated?
      if (aparams.length() > 0)
        substitutions.put(ACCESS_PARAMS, aparams.toString());

      // any parameters generated (for javadoc)?
      if (ajparams.length() > 0)
        substitutions.put(ACCESS_JAVADOC_PARAMS, ajparams.toString());

      // apply substitutions
      String sub_results = Util.applySubstitutions(substitutions, new String(access_template));

      // DBG
      // System.err.println("subs applied to access template: "+sub_results);

      // add these to the overall results...
      results.append(sub_results);

    }

    // return the results
    return results.toString();

  }


/**
 * Generates the access definitions.  These are the various DOs that
 * get created by the access methods.
 *
 * @param access_cfg  The Config containing the necessary details
 * @exception Exception  If an unrecoverable error occurs.
 */
  private String generateAccessDefs(KeywordValueTable access_cfg) throws Exception {

    // The results...
    StringBuffer results = new StringBuffer();

    // get the keys for this config
    String[] keys = access_cfg.keys();

    // go through these keys...
    for (int i=0; i< keys.length; i++) {

      // DBG
      // System.err.println("Access keys["+i+"] is: "+keys[i]);

      // get the section for this key
      KeywordValueTable access_section = access_cfg.getSection(keys[i]);

      // get the applicable table details, if any
      String table_name = access_section.getString(ACCESS_TABLE,null);

      // any table details?
      if (table_name != null) {

        // yes - need to create definition
        results.append("  private ");
        results.append(table_name);
        results.append("DO[] ");
        results.append(keys[i]);
        results.append(" = null;\n");

      }

    }

    // return the results
    return results.toString();

  }


/**
 * Generates the commit methods.
 *
 * @param commit_cfg  The Config containing the necessary details
 * @exception Exception  If an unrecoverable error occurs.
 */
  private String generateCommitMethods(KeywordValueTable commit_cfg) throws Exception {

    // The results...
    StringBuffer results = new StringBuffer();

    // get the keys for this config
    String[] keys = commit_cfg.keys();

    // go through these keys...
    for (int i=0; i< keys.length; i++) {

      // get the section for this key
      KeywordValueTable commit_section = commit_cfg.getSection(keys[i]);

      // the name of this access method
      substitutions.put(COMMIT_NAME, keys[i]);

      // clear the parameters in case there's none
      substitutions.put(COMMIT_PARAMS, "");
      substitutions.put(COMMIT_JAVADOC_PARAMS, "");

      // get the commit mehtod's parameters
      String[] args = Util.commitArgs(app_cfg, commit_section);

      // setup the parameters
      StringBuffer cparams = new StringBuffer();
      StringBuffer cjparams = new StringBuffer();
      for (int j=0; j<args.length; j++) {

        // if we're past the first parameter
        if (j>0) {

          // need to put in separators
          cparams.append(", ");

        }

        cparams.append("String ");
        cparams.append(args[j]);

        cjparams.append("\n * @param ");
        cjparams.append(args[j]);
        cjparams.append(" Parameter supplied by PO");

      }

      // any parameters generated?
      if (cparams.length() > 0)
        substitutions.put(COMMIT_PARAMS, cparams.toString());

      // any parameters generated (for javadoc)?
      if (cjparams.length() > 0)
        substitutions.put(COMMIT_JAVADOC_PARAMS, cjparams.toString());

      // apply substitutions
      String sub_results = Util.applySubstitutions(substitutions, new String(commit_template));

      // DBG
      // System.err.println("subs applied to access template: "+sub_results);

      // add these to the overall results...
      results.append(sub_results);

    }

    // return the results
    return results.toString();

  }



/**
 * Generates the stub code for the BDO-BO type classes.
 * This involves finding all the IDS in the application
 * that have a DATAFROM=DATABASE entry, plus a FIELDPATH entry.
 *
 * @param output_dir  The destination base directory for the stubs
 * @exception Exception  If an unrecoverable error occurs.
 */
  private void generateBDOStubs(String output_dir) throws Exception {

    String code = "";
    String file_name = "";
    String results = "";
    String old_value = "";
    String po_name = "";
    String id_name = "";
    String access_name = "";
    String table_name = "";
    String id_section_basename = "";
    String[] stubs = null;

    // hashtable to determine if a particular table/id combo has been used before
    Hashtable int_table_ids = new Hashtable(100);
    Hashtable oid_table_ids = new Hashtable(100);

    // get all the IDS where DATAFROM=DATABASE
    String[] id_names = Util.getPOSectionNamesMatching(app_cfg, "DATAFROM", "DATABASE");

    // status message to let the user know whats happening
    System.out.println("\nStarting to generate BDO - BO classes, "+id_names.length+" ids to process.\n");

    // go through all the IDS found...
    for (int i=0; i< id_names.length; i++) {

      // DBG
      System.out.println("Processing ID: "+id_names[i]);

      StringBuffer int_idlist = new StringBuffer();
      StringBuffer oid_idlist = new StringBuffer();

      // get the various names for this ID
      po_name = id_names[i].substring(0,id_names[i].lastIndexOf(".IDS"));
      id_section_basename = id_names[i].substring(0,id_names[i].lastIndexOf(".IDS")+4);
      id_name = id_names[i].substring(id_names[i].lastIndexOf(".IDS")+5);

      // DBG
      // System.out.println("Full ID name: "+id_names[i]+", PO name: "+po_name+", section basename: "+id_section_basename+", id name: "+id_name);

      // get the IDS section
      KeywordValueTable ids_cfg = app_cfg.getSection(id_section_basename);

      // DBG
      // System.out.println("Now have IDS section");

      // get the section for this id
      KeywordValueTable id_kvt = KeywordValueFiles.getFilledSection(app_cfg,ids_cfg,id_name);

      // DBG
      // System.out.println("Now have filled id section");

      // get the access name for this id
      access_name = id_kvt.getString("ACCESS");

      // DBG
      System.out.println("Access name for id is: "+access_name);

      // get the access section for the PO/access
      KeywordValueTable access_cfg = app_cfg.getSection(po_name+"."+ACCESSES);

      // get the table for this id's access
      table_name = access_cfg.getString(access_name+".TABLE");

      // DBG
      System.out.println("Table name for access is: "+table_name);

      // get the current code for this table
      stubs = (String[]) bdo_code.get(table_name);
      int_idlist.append(stubs[0]);
      oid_idlist.append(stubs[1]);

      // set the first id flag if there's no code already (either entry in array is ok to test)
      first_id_flag = !ProdUtil.stringHasContent(stubs[0]);
     
      // first time through?
      if (!first_id_flag) {

        // no, need to add the else condition
        substitutions.put(ELSE, " else ");

      } else {

        substitutions.put(ELSE, " ");

      }

      // generate the stub for this id
      int_idlist.append(generateBDOIdList(id_name, id_kvt, access_cfg, false, int_table_ids));
      oid_idlist.append(generateBDOIdList(id_name, id_kvt, access_cfg, true, oid_table_ids));

      // combine what we've got with the generated code and hold onto it
      String[] tmp_array = new String[2];
      tmp_array[0] = int_idlist.toString();
      tmp_array[1] = oid_idlist.toString();
      bdo_code.put(table_name, tmp_array);

    }

    // go through the BDO hashtable and write out the code
    for (int i=0; i< table_names.length; i++) {

      // get this entry
      stubs = (String[]) bdo_code.get(table_names[i]);

      // set the table name for the code
      substitutions.put("TABLE_NAME", table_names[i]);

      // any getIdValue methods generated?
      if (!ProdUtil.stringHasContent(stubs[0])) {

        // no, indicate so in code
        substitutions.put(BEGIN_GET_ACCESS_METHODS, "/*");
        substitutions.put(GET_ACCESS_METHODS, "  No getIdValue Method(s) Generated!");
        substitutions.put(END_GET_ACCESS_METHODS, "*/");

      } else {

        // keep the id list created
        substitutions.put(INT_ID_LIST, stubs[0]);

        // keep the id list created
        substitutions.put(OID_ID_LIST, stubs[1]);

        code = Util.applySubstitutions(substitutions, new String(bdogetaccess_template));

        // yes, put start/end markers
        substitutions.put(BEGIN_GET_ACCESS_METHODS, "\n/*\n * Start of getIdValue Methods\n*/\n");
        substitutions.put(END_GET_ACCESS_METHODS, "\n/*\n * End of getIdValue Methods\n*/\n\n");

        // set substitutions
        substitutions.put(GET_ACCESS_METHODS, code);

      }

      // apply the necessary substitutions to a copy of the template
      results = Util.applySubstitutions(substitutions, new String(bdostubs_template));

      // any code to write out?
      if (!ProdUtil.stringHasContent(results)) {

        // no, move on
        System.out.println("No code to produce for File <"+file_name+">, moving to next file");
        continue;

      }

      // create the file name for this code
      file_name = output_dir + File.separator + table_names[i]+"BO.java";

      // does this file already exist?
      if (overwrite || !Util.fileExists(file_name)) {

        // code generation time...
        System.out.println("Generating stub code for File <"+file_name+">");

        // write the template out to file
        Util.writeFile(file_name, results);

      } else {

        // move along, nothing to do here...
        System.out.println("File <"+file_name+"> already exists, skipping stub generation");

      }

    }

  }


/**
 * Generates the code for this id for the getIdValue that takes an
 * int argument for use in the BDO-BO class.
 *
 * @param id  The id being processed.
 * @param id_kvt  The Config containing the necessary details
 * @param access_cfg  The Config containing the necessary details
 * @param oid_flag  True if generating code for the oid string getIdValue method
 * @param ht  The duplicate id hashtable
 * @exception Exception  If an unrecoverable error occurs.
 */
  private String generateBDOIdList(String id, KeywordValueTable id_kvt, KeywordValueTable access_cfg, boolean oid_flag, Hashtable ht) throws Exception {

    // The results...
    String results = "";

    // get the relevant access for this id
    String access_name = id_kvt.getString(ACCESS, null);

    // get the "fieldpath" for this id
    String fieldpath = id_kvt.getString(FIELDPATH, null);

    // DBG
    // System.out.println("Fieldpath is: "+fieldpath);
     
    // any db field to use?
    if (fieldpath == null) {

      // no, so don't generate anything for this field
      return "";

    }

    // tokenise the fieldpath on dot to get components
    StringTokenizer st = new StringTokenizer(fieldpath, ".");

    // get the table name for this access
    String table_name = access_cfg.getString(access_name+".TABLE", null);

    if (table_name == null)
      throw new Exception("Bad Config setting! TABLE not set for this access: "+access_name);

    // create the hashtable key to see if we've handled this table/id before
    String table_id_key = table_name+"."+id;

    if (ht.containsKey(table_id_key)) {

      // we've done this one before - check the field path
      String ht_fieldpath = (String) ht.get(table_id_key);

      if (!ht_fieldpath.equals(fieldpath)) {

        throw new Exception("The id: "+id+", in table: "+table_name+" is defined more than once with different fieldpaths: <"+fieldpath+">, <"+ht_fieldpath+">");

      } else {

        // field paths are ok, no code to generate - get out of here
        return "";

      }

    }

    // save the id's fieldpath in the hashtable
    ht.put(table_id_key, fieldpath);

    // setup substitutions
    substitutions.put(ID_NAME, id);
    //substitutions.put(ID_TABLE_NAME, table_name);

    StringBuffer getcall_str = new StringBuffer();

    // generating code for the getIdValue that takes the oid string?
    if (oid_flag) {

      // yes  - clear out the stuff that's only relevant for the 'int index' code.
      substitutions.put(INT_INDEX_SECTION, "");

      getcall_str.append("DO");

    } else {

      // no, keep the 'int' special code intact
      substitutions.put(INT_INDEX_SECTION, Util.applySubstitutions(substitutions, new String(int_index_template)));

      getcall_str.append("query_results[index]");

    }

    String path_str = null;
    String field_table = null;
    String true_table = null;

    while (st.hasMoreTokens()) {

      // keep the previous 'fieldname'
      field_table = path_str;

      // get the token - if it's the last, this gives use the fieldname
      path_str = st.nextToken();

      // add this 'field'
      getcall_str.append(".get");
      
      if (path_str.equalsIgnoreCase(DB_HANDLE)) {

        getcall_str.append("OId().toBigDecimal()");

      } else {

        getcall_str.append(path_str);
        getcall_str.append("()");

      }

      // try to keep track of the real table for the field
      if (true_table == null)
        true_table = table_name;

      if (field_table != null) {

        // DBG
        // System.out.println("true table DO lookup: "+DO_TABLE+"."+true_table+".COLUMN."+field_table+".REFERENCES");

        true_table = app_cfg.getString(DO_TABLE+"."+true_table+".COLUMN."+field_table+".REFERENCES");

      }

    }

    if (path_str.equalsIgnoreCase(DB_HANDLE)) {

      // handle reference
      substitutions.put(EXCEPTION_TYPE, EXCEPTION);

    } else {

      // normal field reference
      substitutions.put(EXCEPTION_TYPE, DATAOBJECT);

    }

    // the call that will retrieve the field
    substitutions.put(GET_CALL, getcall_str.toString());

    // DBG
    // System.out.println("table_name is: "+table_name);
    // System.out.println("path_str is: "+path_str);
    // System.out.println("true_table is: "+true_table);

    // get the DODS type for this field
    String dodsType = null;

    // do we have the true table for the field?
    if (true_table == null) {

      // no, just use the access table and the field name
      dodsType = app_cfg.getString(DO_TABLE+"."+table_name+".COLUMN."+path_str+".DODS_TYPE");

      if (dodsType == null) {

        throw new Exception("Can't find DODS Type for table: "+table_name+", column: "+path_str);

      }

    } else {

      // yes, use it
      dodsType = app_cfg.getString(DO_TABLE+"."+true_table+".COLUMN."+path_str+".DODS_TYPE");

      if (dodsType == null) {

        throw new Exception("Can't find DODS Type for table: "+true_table+", column: "+path_str);

      }

    }


    // DBG
    // System.out.println("dodsType is: "+dodsType);

    // get the Java type for this DODS type
    String javaType = dodsTypes.getJavaType(dodsType);

    // DBG
    // System.out.println("javaType is: "+javaType);

    // definition for the 'result' variable
    substitutions.put(RESULT_DEF, javaType);

    // apply substitutions
    results = Util.applySubstitutions(substitutions, new String(bdoid_template));

    // return the results
    return results;

  }


/**
 * Generates the getBusinessIdValue method for the application base.  
 * Uses IDS that have a DATAFROM value of BUSINESS in the global IDS
 * section.
 *
 * @exception Exception  If an unrecoverable error occurs.
 */
  private String generateAppBusinessMethod() throws Exception {

    // The results...
    String results = "";
    StringBuffer idlist = new StringBuffer();

    boolean first_value = true;

    // get the IDS config section
    KeywordValueTable ids_cfg = app_cfg.getSection(IDS);

    // get the keys for this config
    String[] keys = ids_cfg.keys();

    // go through these keys...
    for (int i=0; i< keys.length; i++) {

      // get the section for this id
      KeywordValueTable id_kvt = ids_cfg.getSection(keys[i]);
     
      // get the DATAFROM for this key
      String datafrom = id_kvt.getString(DATAFROM, "none");

      // is it from business method?
      if (datafrom.equalsIgnoreCase(DATAFROM_BUSINESS)) {

        // first time through?
        if (!first_value) {

          // no, need to add the else condition
          idlist.append(" else ");

        }

        first_value = false;

        idlist.append("    if (id.equalsIgnoreCase(\"");
        idlist.append(keys[i]);
        idlist.append("\")) {\n\n      /* TODO: Handle this id */\n\n    }");

      }

    }

    // only need to apply substitutions if some ids created!
    if (idlist.length() > 0) {

      // keep the id list created
      substitutions.put(ID_LIST, idlist.toString());

    } else {

      // clear the id list tag
      substitutions.put(ID_LIST, "");

    }

    // apply substitutions
    results = Util.applySubstitutions(substitutions, new String(business_template));

    // return the results
    return results;

  }


}

