package com.pisoftware.platypus.pond;

import java.io.*;
import java.lang.*;
import java.util.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import com.lutris.util.*;
//import com.lutris.classloader.MultiClassLoader;
import com.lutris.xml.xmlc.html.HTMLObject;
import com.lutris.xml.xmlc.*;
import com.lutris.appserver.server.ApplicationException;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.Enhydra;
import com.lutris.logging.*;
import com.pisoftware.platypus.util.*; 
import com.pisoftware.platypus.production.*; 

/**
 * Manages the code generation for a single "access" to the database.
 * 
 * <P>
 * 
 * Currently (30Jun99), there is no need for a final 'tidying up' call to this class.
 **/

/*
 *  Copyright (C) 1999 Plugged In Software Pty Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Plugged In Software
 *  PO Box 4130
 *  St. Lucia South Queensland
 *  Australia  4067
 *  info@PIsoftware.com
 */

public class Access{

  /** Name of this access. **/
  String accessName ; 

  /** Configuration info for this access. **/
  KeywordValueTable accessData ; 

  /** Configuration info for PO containing this access. **/
  KeywordValueTable poData ; 

  /** Global configuration info for this app. **/
  KeywordValueTable confData ; 

  /** True if the access should find <EM>exactly</EM> one record. **/
  boolean expectsUnique ; 

  /** DOM class for POG **/
  CodeTemplateHTML po ; 

  /** 
   * Place-marker node: insert code, for getting a field and setting a
   * value in PO's DOM, into this node.
   **/
  Node fieldCallsNode ; 

  /** 
   * Place-marker node: insert code, for removing cloned nodes (from
   * DOM to be served), into this node.
   **/
  Node removeSectionsNode = null ; 

  /** 
   * Current dynamic node object. This is set (and cleared) for each
   * new dynamic node which is filled by this access. **/
  DynamicNode dynamicNode ; 

  
  /**
   * Declare variable(s) to hold this field from database
   **/
  void declareGetFieldVariable( String idName, String fieldVarName ) throws ApplicationException { 
    String decls = "" ; 
    decls += 
      "\n" + 
      "  /** ID " + idName + " from DB query on " + accessName + " **/\n" +
      "  " + dynamicNode.javatypeName + " " + fieldVarName + " ; \n" ;
    GenUtil.appendTextTo( po.getElementPOAccessVarsSkeletons(), decls ); 
  }


// OLD CODE Sep99   /**
// OLD CODE Sep99    * Write code to get the field from the business object.
// OLD CODE Sep99    **/
// OLD CODE Sep99   // TODOONEDAY: Delete this
// OLD CODE Sep99   String businessGetFieldQuery( String fieldName, String fieldVarName ) throws ApplicationException, KeywordValueException {
// OLD CODE Sep99     return
// OLD CODE Sep99       "\n" + 
// OLD CODE Sep99       "        ////  In access " + accessName + ", processing field " + fieldName + "\n" +
// OLD CODE Sep99       "        " + fieldVarName + " = bo." + fieldName + "( one" + accessName + " ); \n" +  
// OLD CODE Sep99       "        " + fieldVarName + " = ProdUtil.cleanString( " + fieldVarName + ", " + dynamicNode.nbspName + " ); \n" ;  
// OLD CODE Sep99   }


// OLD CODE Sep99   /**
// OLD CODE Sep99    * Write code to get the field from the database.
// OLD CODE Sep99    **/
// OLD CODE Sep99   // TODOONEDAY: Delete this
// OLD CODE Sep99   String databaseGetFieldQuery( String fieldName, String fieldVarName ) throws ApplicationException, KeywordValueException {
// OLD CODE Sep99     String getCodeText ; 
// OLD CODE Sep99     String code = "" ; // The generated code
// OLD CODE Sep99     getCodeText = ".get" + fieldName + "()" ; 
// OLD CODE Sep99     code += 
// OLD CODE Sep99       "\n" + 
// OLD CODE Sep99       "        ////  In access " + accessName + ", processing field " + fieldName + "\n" +
// OLD CODE Sep99       "        // TODO: If we're querying through intermediate tables, the .getXxxxYyyy() method name is probably incomplete\n" ; 
// OLD CODE Sep99     if( "CHECKBOX".equalsIgnoreCase( dynamicNode.typeName ) ){ 
// OLD CODE Sep99       // CHECKBOX INPUT tag. Data layer returns a boolean. 
// OLD CODE Sep99       code += 
// OLD CODE Sep99         "        " + fieldVarName + " = one" + accessName + getCodeText + "; \n" ; 
// OLD CODE Sep99     }else{
// OLD CODE Sep99       // Default case   
// OLD CODE Sep99       code += 
// OLD CODE Sep99         "        " + fieldVarName + " = one" + accessName + getCodeText + "; \n" +  
// OLD CODE Sep99         "        " + fieldVarName + " = ProdUtil.cleanString( " + fieldVarName + ", " + dynamicNode.nbspName + " ); \n" ;  
// OLD CODE Sep99     }
// OLD CODE Sep99     return code ; 
// OLD CODE Sep99   }


  /**
   *
   **/
  String getStringFieldQuery( String idName, String fieldVarName, String recordSelector ) throws ApplicationException, KeywordValueException {
    return
      "\n" + 
      "        ////  In access " + accessName + ", processing ID " + idName + "\n" +
      "        " + fieldVarName + " = bo.getIdValue( " + recordSelector + ", \"" + idName + "\", " + dynamicNode.formatExpr() + " ); \n" +  
      "        " + fieldVarName + " = ProdUtil.cleanString( " + fieldVarName + ", " + dynamicNode.nbspName + " ); \n" ;  
  }


  /**
   *
   **/
  String getBooleanFieldQuery( String idName, String fieldVarName, String recordSelector ) throws ApplicationException, KeywordValueException {
    return
      "\n" + 
      "        ////  In access " + accessName + ", processing ID " + idName + "\n" +
      "        " + fieldVarName + " = bo.isIdValue( " + recordSelector + ", \"" + idName + "\" ); \n" ;  
  }


  /**
   *
   **/
  public void processGetFieldCall( String idName, KeywordValueTable idData ) throws ApplicationException, KeywordValueException { 
    String code ; // The code generated for processing a field
    String fieldVarName ; // Name of variable (in generated code) containing DB field value.
    String recordSelector ; // Expression in generated code -- might be int or OIdStr String.
    String displayAsText ; // Copy this value to another location (Text) in HTML!
    String javaType ; 
    String indent ; 
    //// Temporary variables
    String tmpStr ; 

    //////  Initialise
    dynamicNode = new DynamicNode( poData, idName, idData ); 
    displayAsText = idData.getString( "DISPLAYASTEXT", null ); 
    javaType = idData.getString( "JAVATYPE", "String" ); 
    // Changed 1Sep99: fieldVarName = "field" + ProdUtil.replaceChar( fieldPath, '.', '_' );
    fieldVarName = "field" + idName ;
    indent = ProdUtil.indentString( 8 ); 

    //////  Write code to declare the fieldXxxx string variable.
    declareGetFieldVariable( idName, fieldVarName ); 

    //////  Write code to get the field from the database.
    if( expectsUnique ){
      recordSelector = accessData.getString( "ISUNIQUE", accessName ); 
      if( "TRUE".equalsIgnoreCase( recordSelector ) ){ 
        // Not a very useful record selector! 
        String dbTable ; 
        dbTable = accessData.getString( "TABLE", "NO_TABLE_IN_ACCESS_CONF_" ); 
        recordSelector = "/*TODO: Check record selector*/ qs" + dbTable + "Id" ; 
      }
    }else{
      recordSelector = "i" + accessName ; 
    }
    if( "String".equalsIgnoreCase( javaType ) ){ 
      code = getStringFieldQuery( idName, fieldVarName, recordSelector ); 
    }else if( "boolean".equalsIgnoreCase( javaType ) ){ 
      code = getBooleanFieldQuery( idName, fieldVarName, recordSelector ); 
    }else{
      throw new ApplicationException( "In ID '" + idName + "', have illegal JAVATYPE value '" + javaType + "'" ); 
    }

    //////  Write code to put the field value into the prototype DOM node.
    code += dynamicNode.insertValueIntoDOM( fieldVarName, indent ); 

    //////  Write code to put the field value into another (Text) node, if needed.
    if( null != displayAsText ){ 
        if( "NEW".equalsIgnoreCase( displayAsText ) ){ 
            // Create a new Text node to insert
            code += indent + "ProdUtil.insertSiblingBefore( document.createTextNode( " + fieldVarName + " ), proto" + idName + " ); \n" ;
        }else{ 
            // Insert into an existing Text node, using getFirstText().
            // Variable tmpStr is the method to find the node to insert the text in.
            if( "BEFORE".equalsIgnoreCase( displayAsText ) ){ 
                tmpStr = "getPreviousSibling" ; 
            }else if( "AFTER".equalsIgnoreCase( displayAsText ) ){ 
                tmpStr = "getNextSibling" ; 
            }else if( "ABOVE".equalsIgnoreCase( displayAsText ) ){ 
                tmpStr = "getParentNode" ; 
            }else{
                throw new ApplicationException( "ID " + idName + " has unexpected DISPLAYASTEXT value '" + displayAsText + "'" ); 
            }
            code += indent + "XMLCUtil.getFirstText( proto" + idName + "." + tmpStr + "() ).setData( " + fieldVarName + " ); \n" ; 
        }
    }

    //////  Insert, into DOM, the code generated for processing a field
    GenUtil.appendTextTo( fieldCallsNode, code ); 

    //////  Tidy up
    dynamicNode = null ; 
  }


  /**
   * Build the argument list for the access method call. That is, the
   * call which retrieves records from the database into the business
   * layer.
   **/
 public String buildAccessArgList() throws ApplicationException { 
    StringBuffer argList ; // The generated argument list
    int argCount ; 
    // Temporary variables 
    String pairValue ; 
    int i ; 

    try{ 
      pairValue = accessData.getString( "QUERYARGCOUNT" ); 
    }catch( KeywordValueException kve ){ 
      throw new ApplicationException( "Problem reading query argument count in " + accessName ); 
    }
    argCount = new Integer( pairValue ).intValue(); 

    //////  Build arg list
    argList = new StringBuffer( "" ); 
    for( i=0 ; i<argCount ; i++ ){ 
      try{ 
        pairValue = accessData.getString( "QUERYARG" + i ); 
      }catch( KeywordValueException kve ){ 
        throw new ApplicationException( "Problem reading query argument " + i + " (of " + argCount + ") in " + accessName ); 
      }
      if( i > 0 ) argList.append( ", " ); 
      argList.append( pairValue ); 
    }

    //////  Check for possible error in QUERYARGCOUNT
    try{
        pairValue = accessData.getString( "QUERYARG" + argCount, null ); 
    }catch ( KeywordValueException kve ){ 
      // This really shouldn't happen!
      System.err.println( "Problem in config files for QUERYARG. Report to programmer." ); 
    }
    if( null != pairValue ){ 
      System.err.println( "WARNING: QUERYARG" + argCount + " exists in config info, but is not used." ); 
    }

    //////  Tidy up and leave
    return argList.toString() ; 
  }


  /** 
   * Generates code to conditionally remove a cloned node from the DOM tree.
   * @exception ApplicationException 
   **/
  void removeClone( Wrapper removeCall, String idName, String accessName ) throws ApplicationException { 
    String code ; 
    code = 
      "        // TODO: Code for getting values (if any) - move from inside the 'then' clause to before the 'if()'.\n" + 
      "        // TODO: Check the if() condition! Perhaps ( bo.isIdValue( \"" + idName + "\" ) )?\n" + 
      "        if( ProdUtil.stringHasContent( field" + idName + " ) ){\n" + 
      "        }else{\n" + 
      "          " + removeCall.wrap( "(Node)(idnodes" + accessName + ".get( \"" + idName + "\" ))" ) + "; \n" + 
      "        }\n" ; 
    ProdUtil.testForNull( removeSectionsNode, "Don't expect this to be called for a unique access! (BTW: " + expectsUnique + ")" ); 
    GenUtil.appendTextTo( removeSectionsNode, code );     
  }


  /** 
   * Guesses what the previous displayed PO might have been. Not reliable!
   * @exception ApplicationException
   **/
  String guessFromPO() throws ApplicationException { 
    String invokedBy ; 
    String fromPO ; 

    try{       
      if( poData.containsKey( "INVOKEDBY" ) ){ 
        invokedBy = poData.getString( "INVOKEDBY" ); 
        if( invokedBy.indexOf( "," ) < 0 ){ 
          // Only one PO in list
          fromPO = ProdUtil.replaceChar( invokedBy, '.', '/' ); 
        }else{
          // Many POs can invoke this PO
          fromPO = "TODO_ComputePOAmong:" + invokedBy ; 
        }
      }else{
        fromPO = "TODO_ComputeFromPO" ; 
      }
    }catch( KeywordValueException kve ){
        throw new ApplicationException( kve ); 
    }
    return fromPO ; 
  }


  /** 
   *
   * @exception ApplicationException
   **/
  void fillAccessCallTemplate( KeywordValueTable accessData ) throws ApplicationException { 
    // This has got to be done in prototype, because we have access methods for these nodes.
    String recordId ; // Name of node copied for each record in access
    String redirectList ; // Code for list redirection URL. Null ==> use generic guess.
    String thisPODir ; // Name of directory containing this PO (relative to presentation/)
    // UNUSED (15Oct99)? String thisPO ; // Name of this PO (including directories, excluding .po)
    String fromPO ; // Name of previous displayed PO (including directories, excluding .po)
    // Temporary variable
    String tmpString ; 

    //////  Initialise 
    try{ 
      recordId = accessData.getString( "PROTOTYPE", null ); 
      if( null == recordId ){ 
        if( expectsUnique ){ 
          recordId = " TODO: Delete! (Not needed for unique accesses!) " ; 
        }else{ 
          throw new ApplicationException( "In access " + accessName + ", missing PROTOTYPE" ); 
        }
      }
      redirectList = poData.getString( "REDIRECTLIST", null ); 
      if( null == redirectList ) redirectList = poData.getString( "REDIRECTLIST", null ); 
      if( null == redirectList ){ 
        if( expectsUnique ){ 
          redirectList = " TODO: Delete! (Not needed for unique accesses!) " ; 
        }else{ 
          // TODOONEDAY: throw the exception instead of generating crappy code.
          redirectList = " TODO: REDIRECTLIST missing from config file!" ; 
          // throw new ApplicationException( "In access " + accessName + ", missing REDIRECTLIST" ); 
        }
      }
      thisPODir = poData.getString( "DIRECTORY", null ); 
      // UNUSED (15Oct99)? thisPO = ( ".".equals( thisPODir ) ) ? "" : thisPODir + "/" ; 
      fromPO = guessFromPO(); 
    }catch( KeywordValueException kve ){ 
      throw new ApplicationException( "Problem getting config info for access " + accessName ); 
    }

    //////
    po.setTextAccessName2( accessName ); 
    po.setTextAccessName4( accessName ); 
    po.setTextAccessName5( accessName ); 
    po.setTextAccessName6( accessName ); 
    po.setTextAccessName8( accessName ); 
    po.setTextAccessName9( accessName ); 
    po.setTextAccessName10( accessName ); 
    po.setTextAccessName11( accessName ); 
    po.setTextAccessName12( accessName ); 
    po.setTextAccessName13( accessName ); 
    po.setTextAccessName14( accessName ); 
    po.setTextAccessName21( accessName ); 
    po.setTextAccessName25( accessName ); 
    po.setTextAccessName26( accessName ); 
    po.setTextAccessName28( accessName ); 
    po.setTextAccessName29( accessName ); 
    po.setTextAccessName30( accessName ); 
    po.setTextAccessName33( accessName ); 
    po.setTextAccessName34( accessName ); 
    po.setTextAccessName35( accessName ); 
    po.setTextAccessName41( accessName ); 
    po.setTextAccessName42( accessName ); 
    po.setTextAccessName44( accessName ); 
    po.setTextAccessName45( accessName ); 
    po.setTextAccessName46( accessName ); 
    po.setTextAccessName47( accessName ); 
    po.setTextAccessName48( accessName ); 
    po.setTextIdName4( recordId ); 
    po.setTextIdName5( recordId ); 
    po.setTextIdName7( recordId ); 
    po.setTextIdName8( recordId ); 
    po.setTextPOAccessFromPO( fromPO ); 
    po.setTextPODirectory( thisPODir ); 

    ////// 
    if( null != redirectList ){ 
      po.setTextPORedirectListURL( redirectList ); 
      ProdUtil.removeThisNode( po.getElementPORedirectURLDecl() ); 
    }else{
      po.setTextPORedirectListURL( "tmpURLString" ); 
    }
  }


  /** 
   * @param po DOM model containing generated code
   * @param accessName name of this access
   * @exception ApplicationException
   **/
  public Access( CodeTemplateHTML po, KeywordValueTable confData, String poName, KeywordValueTable poData, String accessName, KeywordValueTable accessData ) throws ApplicationException { 
    Node protoPOAccessVars ; 
    Node protoPOAccessCall ; 
    Node filledPOAccessVars ; 
    Node filledPOAccessCall ; 
    boolean stripedRows ; // True implies generate code for alternating colours in table
    // Temporary variable
    String tmpString ; 

    //////  Initialise 
    try{ 
      // Values of ISUNIQUE, other than "FALSE", are used as expressions in generated code.
      tmpString = accessData.getString( "ISUNIQUE", "FALSE" ); 
      expectsUnique = ! "FALSE".equalsIgnoreCase( tmpString ); 
      tmpString = accessData.getString( "STRIPED", "FALSE" ); 
      stripedRows = (new Boolean( tmpString )).booleanValue(); 
    }catch( KeywordValueException kve ){ 
      throw new ApplicationException( "Problem getting config info for access " + accessName ); 
    }
    this.accessData = accessData ; 
    this.accessName = accessName ; 
    this.poData = poData ; 
    this.confData = confData ; 
    this.po = po ; 
    protoPOAccessVars = po.getElementPOAccessVarsTemplate(); 
    protoPOAccessCall = po.getElementPOAccessCallTemplate(); 

    //////  Fill in values in prototype code
    fillAccessCallTemplate( accessData ); 

    //////  Build argument list for access call.
    po.setTextAccessArgList( buildAccessArgList() ); 

    //////  Write out access declarations in PO skeleton
    filledPOAccessVars = protoPOAccessVars.cloneNode( true ); 
    if( expectsUnique ){ 
      // Remove multi-record processing from access variables.
      Hashtable idHash ; 
      idHash = ProdUtil.idHashtable( (HTMLElement) filledPOAccessVars ); 
      ProdUtil.removeThisNode( (Node)(idHash.get( "POAccessVarsLoopIndex" ) ) ); 
    }
    po.getElementPOAccessVarsSkeletons().appendChild( filledPOAccessVars ); 

    //////  Write out access loop and calls in PO skeleton
    if( ! stripedRows ) ProdUtil.removeThisNode( po.getElementPOAccessStripedCode() ); 
    filledPOAccessCall = protoPOAccessCall.cloneNode( true ); 
    if( expectsUnique ){ 
      // Remove multi-record processing from access call.
      Hashtable idHash ; 
      idHash = ProdUtil.idHashtable( (HTMLElement) filledPOAccessCall ); 
      ProdUtil.removeThisNode( (Node)(idHash.get( "POAccessCallHeadTemplate" ) ) ); 
      ProdUtil.removeThisNode( (Node)(idHash.get( "POAccessCallTailTemplate" ) ) ); 
    }else{
      removeSectionsNode = GenUtil.getSpanNode( filledPOAccessCall, "RemoveClonedNodes" ); 
    }
    fieldCallsNode = GenUtil.getSpanNode( filledPOAccessCall, "POAccessFieldCalls" ); 
    po.getElementPOFillDynamicSkeletons().appendChild( filledPOAccessCall ); 
  }
}
