package com.pisoftware.platypus.pond;

import java.io.*;
import java.lang.*;
import java.util.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import com.lutris.util.*;
import com.lutris.xml.xmlc.html.HTMLObject;
import com.lutris.xml.xmlc.*;
import com.lutris.appserver.server.ApplicationException;
import com.lutris.appserver.server.Enhydra;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.logging.*;
import com.pisoftware.platypus.util.*; 
import com.pisoftware.platypus.production.*; 

// TODOONEDAY: [After we have a good class for building up code (like
// an expanded Wrapper class)] The code for processing a single ID
// node should probably be broken out into a separate class. However,
// both the new class and the existing class would need to use the
// variable 'conf' (the string buffer where the config is built up)
// and some of the methods for operating on it. So 'conf' and its
// methods would have to be broken out to yet another class. But this
// would be best left until we have a good code builder class.

/**
 * 
 **/

/*
 *  Copyright (C) 1999 Plugged In Software Pty Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Plugged In Software
 *  PO Box 4130
 *  St. Lucia South Queensland
 *  Australia  4067
 *  info@PIsoftware.com
 */

public class ConfigGenerator {

  /** 
   * Configuration file content is built in this buffer before being
   * written to file.
   **/
  StringBuffer conf ; 

  /** Configuration information (so far) for content of application pages. **/
  KeywordValueFiles confData ; 

  /** Name of application, in lower case **/
  String appNameCGI ; 

  /** Name of presentation object, capitalised **/
  String poNameCap ; 

  /** Name of presentation object, in lower case **/
  String poName ; 

  /** Path to class from SRC directory.  <BR> The
   * <CODE>presentation/</CODE> part may be omitted.
   **/
  String POPathCGI ; 

  /** Name of PO, computed from CGI variable. Excludes package-name / directory. **/
  String poBaseName ; 

  /** Path to presentation object **/
  String poPath ; 

  /** 
   * Path to presentation object, relative to presentation directory,
   * except that "." is used to separate directories instead of "/",
   * and it has a trailing ".". Contains "" if the PO is in the
   * presentation directory.
   **/
  String poPathDot ; 

  /** Name of directory to put the config file into **/
  String ConfDirCGI ; 

  /** Name of directory to find dev.conf in. **/
  String RootDirCGI ; 

  /** Document at root of DOM data structure **/
  Document document ; 

  /** Fully qualified class name for DOM class for this PO. **/
  String domQualifiedClassName ; 

  /** Directory containing presentation object, EXCLUDING the
   *  application level! For example, "presentation/Welcome" rather
   *  than "platypus/presentation/Welcome".
   **/
  String poPackageDir ; 

  /** Java package containing presentation object **/
  String poPackageName ; 

  /** Java package containing business object for this PO **/
  String poBusinessPackageName ; 

  /** 
   * If this is a list page, and might redirect, dynamic node
   * <CODE>redirectIdAttr</CODE> probably contains the URL to redirect
   * to. 
   **/
  String redirectIdAttr ; 



  /** Add a line to the conf buffer. **/
  void confAppendLn( String str ){
    conf.append( str + "\n" ); 
  }


  /** Add a name-value pair, in the POs section, to the conf buffer. **/
  void confAppendPair( String name, String value ){
    boolean quoteValue = false ; 
    int i ; 
    if( value.length() == 0 ){ 
      quoteValue = true ; 
    }else if( Character.isLetter( value.charAt( 0 ) ) ){ 
      for( i=1 ; i<value.length(); i++ ){ 
        if( ! Character.isLetterOrDigit( value.charAt( i ) ) ){ 
          quoteValue = true ; 
          break ; 
        }
      }
    }else{
      quoteValue = true ; 
    }
    if( value.indexOf( '\n' ) >= 0 ){ 
      // An embedded newline will cause a syntax error in the config file.
      conf.append( "# TODO: Check this pair for a newline embedded in the value.\n" ); 
    }
    if( quoteValue ) value = "\"" + value + "\"" ; 
    conf.append( "PO." + poPathDot + poNameCap + "." + name + " = " + value + "\n" ); 
  }


  /** 
   * Adds a name-value pair, in the POs section, to the conf
   * buffer. Optionally, it prepends a comment / heading above the
   * pair.
   **/
  void confAppendPairLines( String comment, int hashMargin, String name, String value ){ 
    int i ; 
    for( i=0 ; i<hashMargin ; i++ ) confAppendLn( "#" ); 
    if( null != comment ) confAppendLn( comment ); 
    for( i=0 ; i<hashMargin ; i++ ) confAppendLn( "#" ); 
    confAppendPair( name, value ); 
  }


  /** 
   * Adds a name-value pair, in the POs section, to the conf buffer,
   * unless that name already occurs in the global conf data.
   **/
  void confPerhapsAppendPairLines( String comment, int hashMargin, String name, String value ) 
    throws KeywordValueException { 
    if( ! confData.containsKey( name ) ){ 
      confAppendPairLines( comment, hashMargin, name, value ); 
    }
  }


  /** Add the header to the conf file **/
  void buildConfHeader(){ 
    confAppendLn( "################################################################################" ); 
    confAppendLn( "#" ); 
    confAppendLn( "#\tConfiguration information for presentation object \t" + poNameCap ); 
    confAppendLn( "#" ); 
    confAppendLn( "" ); 
    confAppendLn( "# Generated by Platypus version " + VersionInfo.version() + ". TODO: Hand edit this line. (So 'patch' won't change the version number!)" ); 
    confAppendLn( "" ); 
    confAppendLn( "# Directory containing PO. Relative to presentation directory." ); 
    confAppendPair( "DIRECTORY", poPath ); 
    confAppendLn( "" ); 
    confAppendLn( "# Java class containing DOM model for PO" ); 
    confAppendPair( "DOMCLASS", domQualifiedClassName ); 
    confAppendLn( "" ); 
    confAppendLn( "# TODO: Complete INVOKEDBY. (Only delete if PO doesn't read from query string.)" ); 
    confAppendLn( "PO." + poPathDot + poNameCap + ".INVOKEDBY = \"\"" ); 
    confAppendLn( "" ); 
    confAppendLn( "" ); 
  }


  /** Add the footer to the conf file **/
  void buildConfFooter(){ 
    if( null != redirectIdAttr ){ 
      confAppendLn( "" ); 
      confAppendLn( "# TODO: Check REDIRECTLIST value, and move to top of file" ); 
      confAppendPair( "REDIRECTLIST", "field" + redirectIdAttr ); 
    }else{
      confAppendLn( "" ); 
      confAppendLn( "# TODO: Delete, or complete and uncomment: PO." + poPathDot + poNameCap + ".REDIRECTLIST = " ); 
    }
    confAppendLn( "#" ); 
    confAppendLn( "#  End of information for " + poNameCap ); 
    confAppendLn( "#" ); 
    confAppendLn( "################################################################################" ); 
    confAppendLn( "" ); 
  }


  /**
   * Estimates whether to use nbsp if the field's string is empty.
   **/
  private boolean useNbspForNode( Node node ){ 
    Node plainParentNode ; // Parent node -- skipping Anchor tags

    //////  Return false if this is a FORM element node
    if( node instanceof HTMLInputElement ) return false ; 
    if( node instanceof HTMLSelectElement ) return false ; 
    if( node instanceof HTMLTextAreaElement ) return false ; 
    if( node instanceof HTMLAnchorElement ) return false ; 
    //////  Return true if this is a table cell, or its parent is.
    if( node instanceof HTMLTableCellElement ) return true ; 

    //////  Check the parent node (skipping some types of element).
    plainParentNode = node ; 
    while( true ){ // This is a really ugly loop.
      plainParentNode = plainParentNode.getParentNode(); 
      if( plainParentNode instanceof HTMLAnchorElement ) continue ; 
      break ; 
    }
    if( plainParentNode instanceof HTMLTableCellElement ) return true ; 

    //// Return false if we've got no idea.
    return false ; 
  }


  /** Process all nodes with ID attributes **/
  private void processIDNodes() throws KeywordValueException, HttpPresentationException { 
    HTMLElement el ; 
    ListNodesWithID nodes ; 
    String tagName ; // Type of tag. Eg, SPAN, TABLE, TR.
    String idAttr ; 
    String nameAttr ; 
    String typeAttr ; 
    String hrefAttr ; 
    String useNbsp ; // "TRUE" implies use "&nbsp;" rather than "" in served HTML.
    String htmlLocn ; // Value for config name HTMLLOCN
    String dataFrom ; // Value for config name DATAFROM
    String accessName ; // Guessed name for enclosing Access (if any)
    //// Temporary variables
    String tmpStr ; 

    nodes = new ListNodesWithID( document ); 
    while( nodes.hasMoreElements() ){ 

      //////  Initialise
      el = (HTMLElement) nodes.nextNode() ; 
      idAttr = el.getAttribute( "id" ); 
      tagName = el.getTagName().toUpperCase(); 
      nameAttr = el.getAttribute( "name" ); 
      typeAttr = el.getAttribute( "type" ); 
      hrefAttr = el.getAttribute( "href" ); 
      accessName = GenUtil.findAccessName( el ); 

      //////  Write IDS.idAttr section into config file

      //////  Write header
      // Includes TAG, REFERS, FORMAT values.
      confAppendLn( "" ); 
      confAppendPairLines( "#########            tag containing ID attribute   " + idAttr, 1, 
                           "IDS." + idAttr + ".TAG", tagName ); 
      confAppendPairLines( null, 0, "IDS." + idAttr + ".REFERS", "IDS." + idAttr ); 
      confAppendPairLines( null, 0, "IDS." + idAttr + ".FORMAT", "" ); 

      //////  
      if( ProdUtil.stringHasContent( nameAttr ) ){
          confAppendPairLines( null /*"# NAME attribute of tag " + idAttr*/, 0, 
                               "IDS." + idAttr + ".NAME", nameAttr ); 
      }
      if( ProdUtil.stringHasContent( typeAttr ) ){ 
          confAppendPairLines( null, 0, "IDS." + idAttr + ".TYPE", typeAttr ); 
      }
      if( ProdUtil.stringHasContent( hrefAttr ) ){ 
          confAppendPairLines( null, 0, "IDS." + idAttr + ".HREF", hrefAttr ); 
          if( null == redirectIdAttr ) redirectIdAttr = idAttr ; 
      }
      if( "SELECT".equalsIgnoreCase( tagName ) ){
        confPerhapsAppendPairLines( "# TODO: Is PULLDOWNKEY correct for ID " + idAttr, 0, 
                                    "IDS." + idAttr + ".PULLDOWNKEY", idAttr ); 
      }

      //////  
      if( null != accessName ){ 
        dataFrom = "DATABASE" ; 
      }else if( "SELECT".equalsIgnoreCase( tagName ) ){
        dataFrom = "PULLDOWN" ; 
      }else{
        dataFrom = "BYHAND" ; 
      }
      confPerhapsAppendPairLines( null, 0, "IDS." + idAttr + ".DATAFROM", dataFrom ); 

      //////  
      // TODOONEDAY: The following 'switch' on tagName could be tidied --
      // perhaps a lot absorbed into a Hashtable?
      if( "SELECT".equalsIgnoreCase( tagName ) ){
        htmlLocn = "PULLDOWN" ; 
      }else if( "INPUT".equalsIgnoreCase( tagName ) ){ 
        if( "RADIO".equalsIgnoreCase( typeAttr ) ){ 
          htmlLocn = "ATTR_CHECKED" ; 
        }else if( "CHECKBOX".equalsIgnoreCase( typeAttr ) ){ 
          htmlLocn = "ATTR_CHECKED" ; 
        }else{ 
          htmlLocn = "ATTR_VALUE" ; 
        }
      }else if( "A".equalsIgnoreCase( tagName ) ){ 
        htmlLocn = "ATTR_HREF" ; 
      }else if( "TEXTAREA".equalsIgnoreCase( tagName ) ){ 
        htmlLocn = "FIRSTTEXT" ; 
      }else if( "TD".equalsIgnoreCase( tagName ) ){ 
        htmlLocn = "FIRSTTEXT" ; 
      }else if( "SPAN".equalsIgnoreCase( tagName ) ){ 
        htmlLocn = "FIRSTTEXT" ; 
      }else if( "TH".equalsIgnoreCase( tagName ) ){ 
        htmlLocn = "FIRSTTEXT" ; 
      }else if( "TR".equalsIgnoreCase( tagName ) ){ 
        htmlLocn = "NOVALUE" ; 
      }else if( "TABLE".equalsIgnoreCase( tagName ) ){ 
        htmlLocn = "NOVALUE" ; 
      }else{ 
        // Default location for value in HTML
        htmlLocn = "FIRSTTEXT" ; 
      }

      //////  
      confAppendPairLines( null, 0, "IDS." + idAttr + ".HTMLLOCN", htmlLocn ); 
      if( ! "SELECT".equalsIgnoreCase( tagName ) ){
        useNbsp = useNbspForNode( el ) ? "TRUE" : "FALSE" ; 
      }else{
        useNbsp = "FALSE" ; 
      }

      //////  
      confAppendPairLines( null, 0, "IDS." + idAttr + ".NBSP", useNbsp ); 
      // OLD CODE 29Sep99 if( tagName.equalsIgnoreCase( "TD" ) ){ 
      // OLD CODE 29Sep99     confAppendPairLines( null, 0, "IDS." + idAttr + ".STRIPED", "TRUE" ); 
      // OLD CODE 29Sep99 }

      //////  Write a ACCESS and FIELDPATH pair (perhaps)
      if( null != accessName ){ 
        tmpStr = "IDS." + idAttr + ".ACCESS" ; 
        confAppendPairLines( null, 0, tmpStr, accessName ); 
        tmpStr = "IDS." + idAttr + ".FIELDPATH" ; 
        confPerhapsAppendPairLines( null, 0, tmpStr, idAttr ); 
      }else{ 
        // Perhaps put in place-holder for ACCESS value.
        if( "DATABASE".equalsIgnoreCase( confData.getString( "IDS." + idAttr + ".DATAFROM", "" ) ) ){ 
          confAppendPairLines( null, 0, "IDS." + idAttr + ".ACCESS", "TODO-:-what-is-access?" ); 
        }
      }

      //////  Write a DISPLAYASTEXT pair, if appropriate
      if( ("INPUT".equalsIgnoreCase( tagName )) && ("HIDDEN".equalsIgnoreCase( typeAttr )) ){ 
        tmpStr = "IDS." + idAttr + ".DISPLAYASTEXT" ; 
        confAppendPairLines( "# TODO: Do we want a DisplayAsText value?", 0, tmpStr, "NEW" ); 
      } 

      //////  Perhaps, write ACCESSES.idAttr section into config file
      if( GenUtil.guessIsAccess( el ) ){ 
        String isUnique = "FALSE" ; 
        if( "TABLE".equalsIgnoreCase( tagName ) ) isUnique = "TRUE" ; 
        confAppendLn( "" ); 
        confAppendLn( "# TODO: ConfigGenerator guessed an ACCESS was wanted. Delete if not required." ); 
        confAppendPairLines( "#########            Access for ID attribute   " + idAttr, 1, 
                             "ACCESSES." + idAttr + ".PROTOTYPE", idAttr ); 
        tmpStr = "ACCESSES." + idAttr + ".TABLE" ; 
        confAppendPairLines( "# TODO: Is this TABLE value correct?", 0, tmpStr, idAttr ); 
        confAppendPairLines( null, 0, "ACCESSES." + idAttr + ".ISUNIQUE", isUnique ); 
        confAppendPairLines( null, 0, "ACCESSES." + idAttr + ".STRIPED", "TRUE" ); 
        // if( null != redirectIdAttr ){ 
        //   confAppendPairLines( "# TODO: Check REDIRECTLIST value", 0, "ACCESSES." + idAttr + ".REDIRECTLIST", "field" + redirectIdAttr ); 
        // }else{
        //   confAppendPairLines( null, 0, "ACCESSES." + idAttr + ".REDIRECTLIST", "TODO: complete or delete REDIRECTLIST value" ); 
        // }
      }
    }
  }


  /** 
   * Finds all NAME attributes which occur more than once in a
   * FORM. These should belong to RADIO sets and CHECKBOX sets.
   * @return Hashtable indexed by duplicated names. Value of each is
   * (Integer)0.
   **/
  Hashtable findDuplicateNames( HTMLFormElement form ){ 
    ListFormElementNodes nodes = new ListFormElementNodes( form ); 
    Hashtable names ; // List of NAME attributes found in FORM.
    Hashtable duplicateNames ; // List of NAME attibutes duplicated in FORM.

    //////  Used in loop
    Node node ; 
    HTMLElement el ; 
    String nameAttr ; // NAME attribute of input

    names = new Hashtable(); 
    duplicateNames = new Hashtable(); 
    while( nodes.hasMoreElements() ){ 
      node = nodes.nextNode(); 
      el = (HTMLElement) node ; 
      nameAttr = el.getAttribute( "name" ); 
      if( null != names.get( nameAttr ) ){ 
        // Name is duplciated in FORM
        duplicateNames.put( nameAttr, new Integer( 0 ) ); 
      }
      names.put( nameAttr, "Don't hobbits have funny names, Glorfingle?" ); 
    }

    return duplicateNames ; 
  }


  void processOptionsInSelectNode( HTMLSelectElement selectNode, String selectPrefix ){ 
    HTMLCollection options ; 
    int optionCount ; 
    String selectNameAttr ; 
    //// Used in loop
    int i ; 
    HTMLOptionElement optionElement ; 
    String valueAttr ; 
    String associatedText ; 

    selectNameAttr = selectNode.getAttribute( "name" );
    options = selectNode.getOptions(); 
    optionCount = options.getLength(); 
    for( i=0 ; i<optionCount ; i++ ){ 
      optionElement = (HTMLOptionElement) options.item( i ); 
      valueAttr = optionElement.getAttribute( "value" ); 
      if( null != valueAttr ){ 
          confAppendPairLines( null /*"# VALUE of OPTION element " + i + " in SELECT " + selectNameAttr*/, 0, 
                             selectPrefix + ".OPTIONS.option" + i + ".VALUE" ,valueAttr ); 
      }
      associatedText = optionElement.getText(); 
      if( null != associatedText ){ 
          confAppendPairLines( null /*"# Text for OPTION element " + i + " in SELECT " + selectNameAttr*/, 0, 
                             selectPrefix + ".OPTIONS.option" + i + ".TEXT" ,associatedText ); 
      }
      confAppendLn( "" ); 
    }
  }            


  /** Process a FORM's element nodes, ie all INPUT, SELECT and TEXTAREA nodes in the FORM. **/
  void processFormElementNodes( HTMLFormElement form, String poNameCap ){ 
    ListFormElementNodes nodes = new ListFormElementNodes( form ); 
    String formNameAttr ; // NAME attribute of form
    // Note: In Hashtable duplicateNames, each element is an Integer,
    // being the index of the next input in the set.
    Hashtable duplicateNames ; // Stores NAME attributes which occur multiple times.
    // Config info useful if this form drives an access in another PO.
    StringBuffer queryArgs ; // Potential arguments to access.
    ////  Used in loop
    Node node ; 
    HTMLElement el ; 
    String tagName ; // Type of tag. Eg, INPUT, TEXTAREA, SELECT
    String typeAttr ; // Name of DOM type for tag
    String tagOrType ; // typeAttr (for INPUT tags) or tagName (for other FORM elements).
    String nameAttr ; // NAME attribute of input. Set to _... if value is empty or attribute absent.
    String valueAttr ; // VALUE attribute of input
    String idAttr ; // ID attribute of input
    Integer buttonNumber ; // Index of button in set. First button is index 0. 
    // Note: tagPrefix and namePrefix are usually the same. But they
    // differ for RADIO sets and (if appropriate) CHECKBOX sets.
    String tagPrefix ; // Prefix for section describing a tag. 
    String namePrefix ; // Prefix for section describing an INPUT name. 
    //// Temporary variables
    String tmpStr ; 

    //////  Initialise FORM node variables
    formNameAttr = form.getAttribute( "name" ); 
    if( ! ProdUtil.stringHasContent( formNameAttr ) ) formNameAttr = "" ; 
    duplicateNames = findDuplicateNames( form ); 
    queryArgs = new StringBuffer( "# PO.PageName.ACCESSES.AccessName.QUERYARGCOUNT = 9999\n" ); 

    ////// Process each FORM element in turn
    while( nodes.hasMoreElements() ){ 

      //////  Initialise INPUT node variables
      node = nodes.nextNode(); 
      el = (HTMLElement) node ; 
      tagName = el.getTagName().toUpperCase(); 
      nameAttr = el.getAttribute( "name" ); 
      if( 0 == nameAttr.length() ){
        nameAttr =  (null != el.getAttributeNode( "name" ) ) ? "_novalue" :  "_noattr" ; 
      }
      typeAttr = el.getAttribute( "type" ).toUpperCase(); 
      idAttr = el.getAttribute( "id" ); 
      valueAttr = el.getAttribute( "value" ); 
      tagOrType = ("INPUT".equals( tagName )) ? typeAttr : tagName ; 
      namePrefix = "FORMS." + formNameAttr + ".ELEMENTS." + nameAttr ; 
      buttonNumber = (Integer) duplicateNames.get( nameAttr ); 
      if( null == buttonNumber ){ 
        // Name attribute is unique in FORM
        tagPrefix = namePrefix ; 
      }else{ 
        // Name attribute denotes a RADIO or CHECKBOX set
        tagPrefix = namePrefix + ".b" + buttonNumber.toString() ; 
      }

      //////  Check for various error conditions
      if( 0 == nameAttr.length() ){ 
        // If there's no NAME attribute, just skip this INPUT
        // TODO: Actually, if its an TYPE=IMAGE, it doesn't need a
        // name attribute! Handle this case, here and below.
        continue ; 
        // NOTE: Below is old code for this condition -- gives a warning.
        //String message ; 
        //message = "MORTAL ERROR (live for now, die later): INPUT node " ; 
        //message += ( 0 == nameAttr.length() ) ? "(with no ID)" : "ID='"+ idAttr +"'" ;
        //message += " has no NAME attribute." ; 
        //System.err.println( message ); 
      }

      //////  Extend 'queryArgs' 
      // OLD CODE if( !("IMAGE".equalsIgnoreCase( typeAttr ) ) && !("SUBMIT".equalsIgnoreCase( typeAttr ) ) ){ 
      if( !("IMAGE".equalsIgnoreCase( typeAttr ) ) ){ 
        queryArgs.append( "# PO.PageName.ACCESSES.AccessName.QUERYARG = qs" + nameAttr + "\n" ); 
      }

      //////  Write out info about element(s) with this NAME
      if( null == buttonNumber ){ 
        // Usual case: NAME is unique in FORM
        confAppendLn( "" ); 
        confAppendPairLines( "##### FORM element containing NAME attribute   " + nameAttr, 0, 
                             namePrefix + ".TYPE" ,tagOrType ); 
        // confAppendLn( "" ); 
      }else{ 
        // Special case: NAME is duplicated in FORM
        if( 0 == buttonNumber.intValue() ){ 
          // This is the first occurence of the NAME
          confAppendLn( "" ); 
          confAppendPairLines( "##### FORM element starting " + tagName + " set NAME   " + nameAttr, 0,
                               namePrefix + ".TYPE" ,tagOrType ); 
          // confAppendLn( "" ); 
        }
      }

      //////  Handle special case of CHECKBOXes -- maybe part of a set
      if( "CHECKBOX".equalsIgnoreCase( tagOrType ) ){ 
        // Special case: CHECKBOX element.
        if( (null == buttonNumber) || (0 == buttonNumber.intValue()) ){ 
          // First occurence of this NAME
          confAppendPairLines( null /*"# Does this CHECKBOX form part of a set?"*/, 0, 
                               namePrefix + ".SET", (null!=buttonNumber) ? "TRUE" : "FALSE" ); 
          // confAppendLn( "" ); 
        }
      }

      // NOT USED? (1Oct99)  //////  Handle special case of TYPE HIDDEN -- write DISPLAYASTEXT pair.
      // NOT USED? (1Oct99)  if( "HIDDEN".equalsIgnoreCase( tagOrType ) ){ 
      // NOT USED? (1Oct99)    // Special case: HIDDEN type.
      // NOT USED? (1Oct99)    tmpStr = namePrefix + ".DISPLAYASTEXT" ; 
      // NOT USED? (1Oct99)    confAppendPairLines( "# TODO: Do we want " + tmpStr, 0, tmpStr, "NEW" ); 
      // NOT USED? (1Oct99)    // confAppendLn( "" ); 
      // NOT USED? (1Oct99)  }

      //////  Handle special case of SELECTs -- write out contents of OPTIONs
      if( "SELECT".equalsIgnoreCase( tagOrType ) ){ 
        processOptionsInSelectNode( (HTMLSelectElement) node, tagPrefix ); 
      }

      //////  Write out info about this element
      if( ProdUtil.stringHasContent( idAttr ) ){
        confAppendPairLines( null /*"# ID attribute of form element " + nameAttr*/, 0, 
                             tagPrefix + ".ID" ,idAttr ); 
        // confAppendLn( "" ); 
      }
      if( ProdUtil.stringHasContent( valueAttr ) ){
        confAppendPairLines( null /*"# VALUE attribute of form element " + nameAttr*/, 0, 
                             tagPrefix + ".VALUE" ,valueAttr ); 
        // confAppendLn( "" ); 
      }

      //////  Increment the index of duplicated node (if required)
      if( null != buttonNumber ){ 
        // OK, so it's required. Increment index and store it in hashtable. 
        duplicateNames.put( nameAttr, new Integer( 1 + buttonNumber.intValue() ) ); 
      }
          
    }

    //////  Write out queryArgs suggestion for next POs Access.
    conf.append( "\n" + queryArgs.toString() + "\n" ); 
    
  }

  /** 
   * Process FORM nodes and embedded FORM elements.
   **/
  private void processFormNodes(){ 
    ListFormNodes nodes ; 
    //// Used in loop
    Node node ; 
    HTMLElement el ; 
    String nameAttr ; // NAME attribute of form
    String actionAttr ; // Action attribute of form

    nodes = new ListFormNodes( document ); 
    while( nodes.hasMoreElements() ){ 

      //////  Initialise variables for node
      node = nodes.nextNode(); 
      el = (HTMLElement) node ; 
      nameAttr = el.getAttribute( "name" ); 
      actionAttr = el.getAttribute( "action" ); 

      //////  Describe FORM
      confAppendLn( "" ); 
      confAppendLn( "# " ); 
      confAppendLn( "#######             FORM containing NAME attribute   " + nameAttr ); 
      confAppendLn( "# " ); 
      confAppendLn( "" ); 
      if( ProdUtil.stringHasContent( actionAttr ) ){
        confAppendPairLines( "# ACTION attribute of form " + nameAttr, 0, 
                             "FORMS." + nameAttr + ".ACTION", actionAttr ); 
        // confAppendLn( "" ); 
      }

      //////  Process FORM elements (and prepare arg decls).
      processFormElementNodes( (HTMLFormElement) node, poNameCap ); 

    }
  }


  /** 
   * Builds and appends to current ID node lists, and builds lists for
   * ID node subtrees.  Also writes out the lists at the end of an ID
   * node. Note that an ID node does not get included in its own
   * lists.
   * @param node Current node, which might be added to current lists
   * @param idList Current list of ID nodes
   * @param inputList Current list of FORM ELEMENT nodes
   * @param thisIdName If non-null, indicates that this is an ID node
   **/
  private void buildHasListsForNode( Node node, StringBuffer idList, StringBuffer inputList, String thisIdName ){ 
    NodeList children ; 
    int childI, childCount ; 
    //// Used in loop
    Node child ; 
    NamedNodeMap childAttrs ; 
    Attr idAttr, nameAttr ; 

    //////  Get children
    children = node.getChildNodes(); 
    childCount = children.getLength(); 

    //////  Process each child
    for( childI=0 ; childI<childCount ; childI++ ){ 

      //////  Initialise
      child = children.item(childI); 
      childAttrs = child.getAttributes(); 
      idAttr = null ; 
      if( null != childAttrs ){ 
        idAttr = (Attr) childAttrs.getNamedItem( "id" ); 
        if( null == idAttr ) idAttr = (Attr) childAttrs.getNamedItem( "ID" ); 
      }
      nameAttr = null ; 
      if( null != childAttrs ){ 
        nameAttr = (Attr) childAttrs.getNamedItem( "name" ); 
        if( null == nameAttr ) nameAttr = (Attr) childAttrs.getNamedItem( "NAME" ); 
      }
      // OLD CODE idAttr = (null!=childAttrs) ? (Attr) childAttrs.getNamedItem( "id" ) : null ;  
      // OLD CODE nameAttr = (null!=childAttrs) ? (Attr) childAttrs.getNamedItem( "name" ) : null ;  

      //////  Add this child node (not subtree) to our lists
      if( null != idAttr ){ 
        if( idList.length() > 0 ) idList.append( ',' ); 
        idList.append( idAttr.getValue() ); 
        // DBG
        // System.err.println( "So far in " + thisIdName + " we have: " + idList.toString() ); 
      }
      if( (null!=nameAttr) && ListFormElementNodes.isFormElement( child ) ){ 
        if( inputList.length() > 0 ) inputList.append( ',' ); 
        inputList.append( nameAttr.getValue() ); 
      }

      //////  Process tree under this child
      if( null != idAttr ){ 
        buildHasListsForNode( child, new StringBuffer(), new StringBuffer(), idAttr.getValue() ); 
      }else{
        buildHasListsForNode( child, idList, inputList, null ); 
      }
    }

    //////  Write lists for this node, if needed
    if( null != thisIdName ){ 
      if( idList.length() > 0 ){ 
        confAppendLn( "" ); 
        confAppendPairLines( null, 0, "IDS." + thisIdName + ".HASIDS", idList.toString() ); 
      }
      if( inputList.length() > 0 ){ 
        confAppendLn( "" ); 
        confAppendPairLines( null, 0, "IDS." + thisIdName + ".HASINPUTS", inputList.toString() ); 
      }
    }
  }


  /** Create lists of ID and FORM ELEMENT nodes contained inside ID nodes.  **/
  private void processHasLists() throws HttpPresentationException { 
    confAppendLn( "" ); 
    confAppendLn( "#" ); 
    confAppendLn( "#######  Tree structure lists" ); 
    confAppendLn( "#" ); 
    buildHasListsForNode( document.getDocumentElement(), new StringBuffer(), new StringBuffer(), null ); 
  }


  /** Read the command line arguments. **/
  void parseArgs( String [] args ){ 
    // Parse args, return a usage statement if they do not
    // appear to be correct.
    if ( args.length < 3 ) {
      System.out.println("  Usage:  java platypus.pond.ConfigGenerator <AppName> <POPath> <ConfDir>");
      System.exit(1);
    }
    appNameCGI = args[0];
    POPathCGI = args[1];
    ConfDirCGI = args[2];
    RootDirCGI = args[3];
    if( POPathCGI.startsWith( "presentation/" ) ){ 
      // We need to remove the 'presentation/' part of the name. It is added elsewhere.
      POPathCGI = POPathCGI.substring( "presentation/".length() ); 
    }
  }


  /** Compute package and directory related names **/
  void computePackageNames() throws Exception { 
    int tmpSlashIndex ; // Used in separating the directory path from the basename of the PO. 
    // First, extract directory and base names from full pathname.
    tmpSlashIndex = POPathCGI.lastIndexOf( "/" ); 
    if( tmpSlashIndex == -1 ){ 
      // No directory component in given path name
      poPath = "." ; 
      poPathDot = "" ; 
      poBaseName = POPathCGI ; 
    }else{ 
      // Extract directory part of given path name
      poPath = POPathCGI.substring( 0, tmpSlashIndex ); 
      poPathDot = ProdUtil.replaceChar( poPath, '/', '.' ) + "." ; 
      poBaseName = POPathCGI.substring( tmpSlashIndex+1 ); 
    }
    // Second, derive a whole lotta names
    poNameCap = poBaseName ; // Name of presentation object (capitalised)
    //appNameCap = appNameCGI.substring( 0, 1 ).toUpperCase() + appNameCGI.substring( 1 ) ; 
    poName = poNameCap.substring( 0, 1 ).toLowerCase() + poNameCap.substring( 1 ); 
    poPackageName = appNameCGI + ".presentation" ; 
    poBusinessPackageName = appNameCGI + ".business" ; 
    poPackageDir = "presentation" ; 
    if( ! ".".equals( poPath ) ){
      poPackageName += "." + poPath ; 
      poBusinessPackageName += "." + poPath ; 
      poPackageDir += "/" + poPath ; 
    }
    domQualifiedClassName = poPackageName + "." + poNameCap + "HTML" ; 
    if( poPackageName.indexOf( "/" ) >= 0 ){
      throw new Exception( "Programmer oversight: must change '/' to '.'" ); 
    }
  }


  public static void main (String[] args) throws Exception {
    ConfigGenerator configGenerator = new ConfigGenerator(); 
    configGenerator.mainInstance( args ); 
  }


  void mainInstance( String [] args ) throws Exception { 
    //// Temporary / intermediate variables
    java.lang.Class domClass ; 
    HTMLObject domInstance ; 

    //////  Initialise
    parseArgs( args ); 
    computePackageNames(); 
    redirectIdAttr = null ; 
    confData = new KeywordValueFiles( RootDirCGI + File.separator + "dev.conf", false ); 
    
    //////  Get DOM object for PO we're looking at
    domClass = Class.forName( domQualifiedClassName ); 
    domInstance = (HTMLObject) domClass.newInstance(); 
    document = domInstance.getDocumentElement().getOwnerDocument(); 
    
    //////  Build and write configuration file
    conf = new StringBuffer(); 
    buildConfHeader(); 
    processIDNodes();
    processFormNodes();
    confAppendLn( "" ); 
    processHasLists(); 
    confAppendLn( "#" ); 
    confAppendLn( "#######  Miscellaneous area" ); 
    confAppendLn( "#" ); 
    confAppendLn( "" ); 
    buildConfFooter(); 
    Util.writeFile( ConfDirCGI + "/" + poNameCap + ".conf", conf.toString() ); 
  }

}
