package com.pisoftware.platypus.pond;

import java.io.*;
import java.lang.*;
import java.util.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import com.lutris.util.*;
//import com.lutris.classloader.MultiClassLoader;
import com.lutris.xml.xmlc.html.HTMLObject;
import com.lutris.xml.xmlc.*;
import com.lutris.appserver.server.ApplicationException;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.Enhydra;
import com.lutris.logging.*;
import com.pisoftware.platypus.util.*; 
import com.pisoftware.platypus.production.*; 

/**
 * Manages the code generation for a single dynamic node.
 **/

/*
 *  Copyright (C) 1999 Plugged In Software Pty Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Plugged In Software
 *  PO Box 4130
 *  St. Lucia South Queensland
 *  Australia  4067
 *  info@PIsoftware.com
 */

public class DynamicNode{

  /** Configuration section describing PO containing this dynamic node. **/
  KeywordValueTable poData ; 

  /** Configuration section describing this dynamic node. **/
  KeywordValueTable idData ; 

  /** ID attribute associated with current KeywordValueTable section. **/
  String idName ; 

  /** Namevalue extracted from KeywordValueTable section. **/
  String tagName ; 

  /** Namevalue extracted from KeywordValueTable section. **/
  String javatypeName ; 

  /** Namevalue extracted from KeywordValueTable section. **/
  String htmllocnName ; 

  /** Namevalue extracted from KeywordValueTable section. **/
  String typeName ; 

  /** Namevalue extracted from KeywordValueTable section. **/
  String datafromName ; 

  /** Namevalue extracted from KeywordValueTable section, cast to lower case. **/
  String nbspName ; 

  /** Namevalue extracted from KeywordValueTable section. **/
  String hrefName ; 

  /** Namevalue extracted from KeywordValueTable section. **/
  String pulldownkeyName ; 

  /** Namevalue extracted from KeywordValueTable section. **/
  String formatName ; 


  /**
   * Returns the expression needed for the 'format' (ie 3rd) argument
   * of getIdValue() in the generated code.
   **/
  // Note: the return is used as an expression, so a constant string
  // must be enclosed in quotes.
  String formatExpr() throws ApplicationException {
    String expr ; 
    if( "DRILL".equals( formatName ) ){ 
      // Handle drill downs.
      String nextPOURL ; // URL of next PO in workflow (includes .po)
      String accessTableName ; // Javarised name of the table containing this URL.
      String accessName ; // ACCESS field of idData.
      KeywordValueTable accessData ; 
      //// Compute URL of next PO in workflow
      if( ProdUtil.stringHasContent( hrefName ) ){ 
        int lastDot = hrefName.lastIndexOf( '.' ) ;           
        if( lastDot >= 0 ){ 
          nextPOURL = hrefName.substring( 0, lastDot ) + ".po" ;
        }else{ 
          nextPOURL = hrefName + ".po" ;
        }
      }else{ 
        nextPOURL = "NEXTPAGE.po" ; 
      }
      //// Compute Javarised name of the table
      try{ 
        accessName = idData.getString( "ACCESS", null ); 
      }catch( KeywordValueException kve ){ 
        // This shouldn't happen!
        throw new ApplicationException( "Huh! How'd this happen?", kve ); 
      }
      if( ! ProdUtil.stringHasContent( accessName ) ){ 
        // No access field!
        accessTableName = "Object" ; 
      }else{ 
        try{ 
          accessData = poData.getSection( "ACCESSES." + accessName ); 
          accessTableName = accessData.getString( "TABLE", "MISSING_TABLE_IN_ACCESS_" + accessName ); 
        }catch( KeywordValueException kve ){ 
          // No access section!
          accessTableName = "MISSING_ACCESS_" + accessName ; 
        }
      }
      //// Build the expression
      expr = 
        "FormatUtil.DRILL + \"" + nextPOURL + "?" + accessTableName + "Id=\"" +
        "/*TODO: Check drill down URL*/ " ;
    }else{ 
      // Normal case.
      if( ProdUtil.stringHasContent( formatName ) && ! formatName.equalsIgnoreCase( "NONE" ) ){ 
        expr = "FormatUtil." + formatName ; 
      }else{
        expr = "\"\"" ; 
      }
    }
    return expr ; 
  }


  /**
   * Write code to put the value into the prototype DOM node.
   **/
  String insertValueIntoDOM( String valueExpr, String indent ){
    String code ; // The generated code
    if( htmllocnName.equalsIgnoreCase( "PULLDOWN" ) ){ 
      // TODOONEDAY: Need to set this up so it handles static Pulldowns as well (see Pond.java). [Later Sep99] Why?
      // SELECT tag
      code = 
        indent + "ProdUtil.drainSelect( proto" + idName + ", \"" + pulldownkeyName + "\" ); // TODO: Really delete all OPTIONS?\n" + 
        indent + "ProdUtil.fillSelect( proto" + idName + ", \"" + pulldownkeyName + "\", bo.getPulldownItems( \"" + pulldownkeyName + "\" ), " + valueExpr + " ); \n" ;
    }else if( htmllocnName.equalsIgnoreCase( "ATTR_CHECKED" ) ){ 
      // Use CHECKED attribute (probably a RADIO or CHECKBOX tag) if VALUE is what we want
      code = 
        indent + "if( proto" + idName + ".getAttribute( \"value\" ).equals( " + valueExpr + " ) ) proto" + idName + ".setAttributeNode( ProdUtil.getCheckedAttributeTemplate() ); \n" ; 
    }else if( htmllocnName.equalsIgnoreCase( "ATTR_VALUE" ) ){ 
      code = 
        indent + "proto" + idName + ".setAttribute( \"value\", " + valueExpr + " ); \n" ; 
    }else if( htmllocnName.equalsIgnoreCase( "FIRSTTEXT" ) ){ 
      code = 
        indent + "XMLCUtil.getFirstText( proto" + idName + " ).setData( " + valueExpr + " ); \n" ; 
    }else if( htmllocnName.equalsIgnoreCase( "ATTR_HREF" ) ){ 
      // Anchor tag (I hope!) -- the field value goes into the HREF
      // attribute, after its protocol. Offer the programmer code to
      // delete the node if the field is empty, and other code to
      // replace the node with its children.
      String hrefProtocol = ProdUtil.protocolOf( hrefName ); 
      code = 
        indent + "proto" + idName + ".setAttribute( \"href\", \"" + hrefProtocol + ":\" + " + valueExpr + " ); // TODO: Is the URL protocol field correct? \n" + 
        indent + "if( ! ProdUtil.stringHasContent( " + valueExpr + " ) ){  // Handle missing value\n" + 
        indent + "  // TODO: Delete one of the following node-removal lines.\n" + 
        indent + "  ProdUtil.removeThisNode( proto" + idName + "); // Removes the node *and* its children.\n" + 
        indent + "  ProdUtil.replaceNodeWithChildren( proto" + idName + "); // Removes the node but *not* its children.\n" + 
        indent + "} \n" ; 
    }else if( htmllocnName.equalsIgnoreCase( "NOVALUE" ) ){ 
      code = indent + "// NOTE: Config file indicates no value for node ID = " + idName + "\n" ; 
    }else{
      code = indent + "// TODO: Don't know how to insert value for node ID = " + idName + "\n" ; 
    }
    return code ; 
  }


  public DynamicNode( KeywordValueTable poData, String idName, KeywordValueTable idData ) throws KeywordValueException { 
    this.poData = poData ; 
    this.idName = idName ; 
    this.idData = idData ; 
    tagName = idData.getString( "TAG", null ); 
    javatypeName = idData.getString( "JAVATYPE", "String" ); 
    htmllocnName = idData.getString( "HTMLLOCN", null ); 
    typeName = idData.getString( "TYPE", null ); 
    pulldownkeyName = idData.getString( "PULLDOWNKEY", null ); 
    formatName = idData.getString( "FORMAT", "" ); 
    if( ! ProdUtil.stringHasContent( formatName ) ) formatName = "NONE" ; 
    datafromName = idData.getString( "DATAFROM", null ); 
    hrefName =  idData.getString( "HREF", null ); 
    nbspName = idData.getString( "NBSP", null ); 
    if( null != nbspName ) nbspName = nbspName.toLowerCase(); 
  }
}
