package com.pisoftware.platypus.pond;

import java.io.*;
import java.net.*;
import java.util.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import com.lutris.util.*;
import com.lutris.xml.xmlc.*;
import com.lutris.appserver.server.sql.*;
import com.lutris.appserver.server.ApplicationException;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;
import com.pisoftware.platypus.util.*; 
import com.pisoftware.platypus.production.*; 

/** 
 * This class contains utilities used while generating code, but not
 * used when the code is in production.
 **/


/*
 *  Copyright (C) 1999 Plugged In Software Pty Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Plugged In Software
 *  PO Box 4130
 *  St. Lucia South Queensland
 *  Australia  4067
 *  info@PIsoftware.com
 */

public class GenUtil{

  /** Maps a tag name (eg TR) to a DOM type name (eg HTMLTableRowElement) **/
  static Hashtable tagMap = null ; 


  /** 
   * Returns a string, the concatentation of all Text nodes in <CODE>node</CODE>'s tree. 
   **/
  static public String toText( Node node ){ 
    StringBuffer buffer ; 
    buffer = new StringBuffer( 4096 ); 
    toText( node, buffer ); 
    return buffer.toString(); 
  }


  /** 
   * Utility for <CODE>toText( Node )</CODE>. 
   * @param buffer text is accumulated in this buffer
   * @return <CODE>null</CODE>
   **/
  static public String toText( Node node, StringBuffer buffer ){ 
    NodeList childNodes ; 
    Node child ; 
    int i ; 
    Text textNode ; 
    childNodes = node.getChildNodes(); 
    for( i=0 ; i<childNodes.getLength() ; i++ ){ 
      child = childNodes.item( i ); 
      if( child instanceof Text ){ 
        textNode = (Text) child ; 
        buffer.append( textNode.getData() ); 
      }else{
        toText( child, buffer ); 
      }
    }
    return null ; 
  }


  /** 
   * Appends a text node after the last child of <CODE>node</CODE>. If
   * <CODE>text</CODE> is the empty string, nothing is appended (this
   * may change in future).
   * @exception ApplicationException if <CODE>text</CODE> is null.
   **/
  public static void appendTextTo( Node node, String text ) throws ApplicationException { 
    Text textNode ; 
    if( null == text ) throw new ApplicationException( "Can't append null text" ); 
    if( text.length() == 0 ) return ; 
    textNode = node.getOwnerDocument().createTextNode( text ); 
    node.appendChild( textNode ); 
  }


  /** 
   * Finds a SPAN node with ID attribute <CODE>spanId</CODE> in the
   * tree under <CODE>rootNode</CODE>. This is useful if
   * <CODE>rootNode</CODE> is a clone.
   * @exception ApplicationException if there is no such SPAN node.
   **/
  static Node getSpanNode( Node rootNode, String spanId ) throws ApplicationException { 
    ListNodesWithID nodes ; 
    Node node ; 
    NamedNodeMap map ; 
    Node idAttr ; 
    nodes = new ListNodesWithID( rootNode ); 
    while( nodes.hasMoreElements() ){ 
      node = (Node) nodes.nextElement(); 
      map = node.getAttributes(); 
      if( null == map ) continue ; 
      idAttr = map.getNamedItem( "id" ) ; // OpenXML
      if( null == idAttr ) idAttr = map.getNamedItem( "ID" ) ; // Docuverse
      if( null == idAttr ) continue ; 
      if( spanId.equalsIgnoreCase( idAttr.getNodeValue() ) ) return node ; 
    }
    throw new ApplicationException( "Couldn't find a node with ID='" + spanId + "'" ); 
  }


  /** Maps a tag name (eg TR) to a DOM type name (eg HTMLTableRowElement) **/
  public static String tagToDOMType( String tagName ){ 
    String type ; 
    String tagNameCap ; 
    //////  Initialise
    tagName = tagName.toUpperCase(); 
    tagNameCap = tagName.substring( 0, 1 ).toUpperCase() + tagName.substring( 1 ).toLowerCase() ; 
    //////  Initialise tagMap, if necessary
    if( null == tagMap ){ 
      tagMap = new Hashtable(); 
      tagMap.put( "TR".toUpperCase(), "HTMLTableRowElement" ); 
      tagMap.put( "TH".toUpperCase(), "HTMLElement" ); // XMLC rejects HTMLTableCellElement
      tagMap.put( "TD".toUpperCase(), "HTMLTableCellElement" ); 
      tagMap.put( "IMG".toUpperCase(), "HTMLImageElement" ); 
      tagMap.put( "TEXTAREA".toUpperCase(), "HTMLTextAreaElement" ); 
      tagMap.put( "A".toUpperCase(), "HTMLAnchorElement" ); 
      tagMap.put( "FRAMESET".toUpperCase(), "HTMLFrameSetElement" ); 
      tagMap.put( "BR".toUpperCase(), "HTMLBRElement" ); 
      tagMap.put( "P".toUpperCase(), "HTMLParagraphElement" ); 
      tagMap.put( "SPAN".toUpperCase(), "HTMLElement" ); 
    }
    //////  Map tag name to type
    type = (String) tagMap.get( tagName ); 
    if( null == type ){ 
      type = "HTML" + tagNameCap + "Element" ; 
    }
    //////  Leave
    return type ; 
  }


  /** 
   * Quick and dirty guess as to whether <CODE>urlString</CODE> specifies a protocol.
   **/
  static public boolean guessURLHasProtocol( String urlString ){ 
    int colonSlashSlash ; 
    colonSlashSlash = urlString.indexOf( "://" ); 
    if( colonSlashSlash > 0 ) return true ; 
    return false ; 
  }


  /** Guess whether this node is the template for an Access. **/
  static boolean guessIsAccess( Node node ){ 
    HTMLElement el ; 
    String tagName ; 
    
    if( ! (node instanceof HTMLElement) ) return false ; 
    el = (HTMLElement) node ; 
    tagName = el.getTagName().toUpperCase(); 
    if( "TR".equalsIgnoreCase( tagName ) ){ 
      return ListNodesWithID.hasIDAttr( node ); 
    }else if( "TABLE".equalsIgnoreCase( tagName ) ){ 
      return ListNodesWithID.hasIDAttr( node ); 
    }else{ 
      return false ; 
    }
  } 


  /** 
   * Finds an ancestor node which <EM>might</EM> be the template for
   * an Access. It relies on what is, at best, a heuristic guess.
   * @return the template node, or null if none found.
   **/
  static public Node findAccessTemplate( Node node ){ 
    node = node.getParentNode(); 
    while( (null!=node) && ! guessIsAccess( node ) ){ 
      node = node.getParentNode(); 
    }
    return node ; 
  }


  /** 
   * Finds the name of the enclosing Access.
   * Assumes the Access template node is an HTMLElement.
   * @see findAccessTemplate
   * @return value of Access template's ID attribute, or null. 
   * @throws ClassCastException if the template node isn't an HTMLElement.
   **/
  static public String findAccessName( Node node ){ 
    Node templateNode ; 
    HTMLElement templateEl ; 
    String idAttr ; 
    templateNode = findAccessTemplate( node ); 
    if( null == templateNode ) return null ; 
    templateEl = (HTMLElement) templateNode ; 
    idAttr = templateEl.getAttribute( "id" ); 
    if( null == idAttr ) idAttr = templateEl.getAttribute( "ID" ); 
    return idAttr ; 
  }
}
