package com.pisoftware.platypus.production;

import java.io.*;
import java.lang.*;
import java.util.*;
import java.net.URLEncoder;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import com.lutris.util.*;
import com.lutris.xml.xmlc.html.HTMLObject;
import com.lutris.xml.xmlc.*;
import com.lutris.appserver.server.sql.*;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.*;
import com.lutris.logging.*;
import com.pisoftware.platypus.production.*;
//import com.lutris.dods.builder.sourceGenerators.Query.*;

/** 
 * This class contains utilities used when the code is in
 * production. They might also be used while the code is being
 * generated.
 *
 *<P>
 *
 * <B>WARNING</B>: This class uses ListDOMNodes, and its subclasses. So, it
 * must be compiled after these classes.
 **/


/*
 *  Copyright (C) 1999 Plugged In Software Pty Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Plugged In Software
 *  PO Box 4130
 *  St. Lucia South Queensland
 *  Australia  4067
 *  info@PIsoftware.com
 */


public class ProdUtil{

  /** Contains a small DOM tree, with templates to incorporate into served pages. **/
  public static TemplateHTML templateDOM = new TemplateHTML() ; 

  /** Used to flag sections of DOM for deleting. **/
  public static Comment deleteMeComment = null ; 

  /** Contents of <CODE>deleteMeComment</CODE> **/
  public static String deleteMeString = null ; 

  /** Document of DOM class for PO being served. **/
  public static Document document = null ; 

  /** Returns an OPTION node (cloned from Template.html) **/
  static public HTMLOptionElement getOptionTemplate(){ 
    return (HTMLOptionElement) templateDOM.getElementOptionTemplate().cloneNode( true ); 
  }

  /** Returns an Attr with no value **/
  static public Attr getCheckedAttributeTemplate(){ 
    return (Attr) templateDOM.getElementCheckboxTemplate().getAttributeNode( "checked" ).cloneNode( true ); 
  }


  /** Fills an OPTION node from a PulldownItem. **/
  static public void fillOptionElement( HTMLSelectElement selectNode, HTMLOptionElement optionNode, PulldownItem pulldownItem ){ 
    String optionValue ; // Value attribute of option (used in loop body)
    String optionText ; // Text displayed by option (used in loop body)

    optionValue = pulldownItem.getValue() ; 
    optionNode.setAttribute( "value", optionValue ); 
    optionText = pulldownItem.getDisplayValue() ; 
    XMLCUtil.getFirstText( optionNode ).setData( optionText ); 
    //DBG
    ProdUtil.log( "Inserting option with value '" + optionValue + "' and text '" + optionText + "'" ); 
  }


  /** 
   * Deletes all options in the SELECT dropbox. 
   * @param selectNode, SELECT node to drain of options.
   * @exception ApplicationException.
   **/
  static public void drainSelect( HTMLSelectElement selectNode, String pulldownName ){ 
    
    //////  Remove any existing option elements in the select element. 
    while( selectNode.hasChildNodes() ){ 
      selectNode.removeChild( selectNode.getFirstChild() ); 
    }
  }


  /** 
   * Finds an OPTION by value attribute, within a SELECT.
   * @param selectNode 
   * @param soughtValue
   * @return OPTION node with the sought value, or null if no OPTIONS match.
   **/
  static public HTMLOptionElement getOptionByValue( HTMLSelectElement selectNode, String soughtValue ){ 
    HTMLCollection optionCollection ; 
    HTMLOptionElement optionNode ; 
    String valueAttr ; 
    int i ; 
    optionCollection = selectNode.getOptions(); 
    for( i=0 ; i<optionCollection.getLength() ; i++ ){ 
      optionNode = (HTMLOptionElement) optionCollection.item( i ); 
      valueAttr = optionNode.getAttribute( "value" ); 
      if( soughtValue.equals( valueAttr ) ) return optionNode ; 
    }
    return null ; 
  }


  /** 
   * Fills the SELECT dropbox. If the defaultValue or defaultDisplay
   * is non-null, the option with that value (if any) will be
   * selected. Parameter <CODE>defaultValue</CODE> checks the
   * <CODE>value</CODE> attribute, whereas <CODE>defaultDisplay</CODE>
   * checks the displayed text of the option. Only one of these two
   * parameters should be non-null. If the defaultValue and
   * defaultDisplay are both null, the PulldownItem.isDefault() method
   * will determine the selected option (if any).
   * 
   * @param selectNode, SELECT node to fill with options.
   * @param pulldownName, Passed to business layer, to indicate what goes in this pulldown.
   * @param pulldownItems, list of items to instantiate OPTION tags.
   * @param defaultValue, if non-null, value selected when page is served.
   * @param defaultDisplay, if non-null, value selected when page is served.
   * @exception ApplicationException.
   **/
  // WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING
  // Discussion on the Enhydra mailing list (XMLC and "<option
  // selected>", 2 Jul 99, markd@lutris.com) states that "The
  // implementation of .HTMLOptionElement.setDefaultSelected() is not
  // complete in the OpenXML DOM." MarkD's next email is also pertinent.
  static public void fillSelect( HTMLSelectElement selectNode, String pulldownName, PulldownItem [] pulldownItems, String defaultValue, String defaultDisplay ) 
    throws ApplicationException { 
    HTMLOptionElement protoOption ; // Prototype option node. Never added to SELECT -- only its clones are.
    Text optionTextNode ; // Text node child of optionNode
    PulldownItem pulldownItem ; 
    int selectedItem ; 
    // Used in loop body ...
    int i  ; 
    boolean selectThisOption ; 
    HTMLOptionElement newOption ; // Option being built (used in loop body)

    //////  Initialisation
    selectNode.setMultiple( false ); 
    protoOption = getOptionTemplate(); 
    // DBG
    ProdUtil.log( "In fillSelect " + (null==pulldownName?"":(" for " + pulldownName)) + " with defaultValue '" + defaultValue + "' and with defaultDisplay '" + defaultDisplay + "'" ); 

    //////  Build and insert options
    if( (null!=pulldownItems) && (0<pulldownItems.length) ){ 
      //////  Have one or more options from database. Build SELECT as normal.
      selectedItem = -1 ; // Flag value.
      for( i=0 ; i<pulldownItems.length ; i++ ){ 

        //////  First, the easy bit: get the data, insert the option, put its values in.
        pulldownItem = pulldownItems[i] ; 
        newOption = (HTMLOptionElement) protoOption.cloneNode( true );
        newOption.removeAttribute( "id" );
        fillOptionElement( selectNode, newOption, pulldownItem ); 

        //////  Now, the hard stuff: selecting an option!
        selectThisOption = false ; 
        if( null != defaultValue ){
          // DBG
          ProdUtil.log( "Comparing defaultValue '" + defaultValue + "' with '" + pulldownItem.getValue() + "'" ); 
          selectThisOption = defaultValue.equals( pulldownItem.getValue() ) ; 
        }else if( null != defaultDisplay ){
          // DBG
          ProdUtil.log( "Comparing defaultDisplay '" + defaultDisplay + "' with '" + pulldownItem.getDisplayValue() + "'" ); 
          selectThisOption = defaultDisplay.equals( pulldownItem.getDisplayValue() ) ; 
        }else{ 
          // DBG
          ProdUtil.log( "Static default for this option is " + pulldownItem.isDefault() ); 
          selectThisOption = pulldownItem.isDefault() ; 
        }
        selectNode.appendChild( newOption ); 
        //if( selectThisOption ) selectedItem = i ; 
        newOption.setDefaultSelected( selectThisOption ); 
        newOption.removeAttribute( "selected" ); 
        if( selectThisOption ) newOption.setAttribute( "selected", "selected" ); 
      }
    }else{ 
      //////  Didn't get any pulldown items back from database.
      ProdUtil.log( "Pulldown request" + (null==pulldownName?"":(" for " + pulldownName)) + " got no values back from database." ); 
      newOption = (HTMLOptionElement) protoOption.cloneNode( true );
      newOption.setAttribute( "value", "" ); 
      XMLCUtil.getFirstText( newOption ).setData( "-" ); 
      selectNode.appendChild( newOption ); 
    }
    // Note: HTMLElementSelect.setSelectedIndex() doesn't do what we want :-(
    // OLD CODE (Jun99) if( 0 <= selectedItem ) selectNode.setSelectedIndex( selectedItem ); 
  }


  /** 
   * Overloaded method.
   * @param selectNode, SELECT node to fill with options.
   * @param pulldownName, Passed to business layer, to indicate what goes in this pulldown.
   * @param pulldownItems, list of items to instantiate OPTION tags.
   * @param defaultValue, if non-null, value selected when page is served.
   * @exception ApplicationException.
   **/
  static public void fillSelect( HTMLSelectElement selectNode, String pulldownName, PulldownItem [] pulldownItems, String defaultValue ) 
    throws ApplicationException { 
    fillSelect( selectNode, pulldownName, pulldownItems, defaultValue, null ); 
  }


  /** 
   * Overloaded method. 
   * @param selectNode SELECT node to fill with options.
   * @param pulldownItems list of items to instantiate OPTION tags.
   * @param defaultValue if non-null, value selected when page is served.
   * @exception ApplicationException.
   **/
  static public void fillSelect( HTMLSelectElement selectNode, PulldownItem [] pulldownItems, String defaultValue ) 
    throws ApplicationException { 
    fillSelect( selectNode, null, pulldownItems, defaultValue ); 
  }


  /** 
   * Overloaded method. 
   * <BR>
   * This variant always uses the PulldownItem.isDefault()
   * method to determine the selected option.
   * 
   * @param selectNode SELECT node to fill with options.
   * @param pulldownName Passed to business layer, to indicate what goes in this pulldown.
   * @param pulldownItems list of items to instantiate OPTION tags.
   * @exception ApplicationException.
   **/
  static public void fillSelect( HTMLSelectElement selectNode, String pulldownName, PulldownItem [] pulldownItems ) 
    throws ApplicationException { 
    fillSelect( selectNode, pulldownName, pulldownItems, null ); 
  }


  /** 
   * Overloaded method. 
   * <BR>
   * This variant always uses the PulldownItem.isDefault()
   * method to determine the selected option.
   * @param selectNode SELECT node to fill with options.
   * @param pulldownItems list of items to instantiate OPTION tags.
   * @exception ApplicationException.
   **/
  static public void fillSelect( HTMLSelectElement selectNode, PulldownItem [] pulldownItems ) 
    throws ApplicationException { 
    fillSelect( selectNode, null, pulldownItems, null ); 
  }


  /** 
   * Removes a node from the document, by removing it from its parent's children.
   * <BR> 
   * If this node has no parent (ie, isn't part of a document), the method is a no-op.
   * @param thisNode, Node to be removed from document. 
 **/
  static public void removeThisNode( Node thisNode ){ 
    // Note: Please don't apply this method to a Document or DocumentFragment node!
    Node parentNode ; 
    parentNode = thisNode.getParentNode() ; 
    if( null != parentNode ) parentNode.removeChild( thisNode ); 
  }


  /** 
   * Removes a node and its context from the document. It searches up
   * the DOM tree until it finds a node of class <CODE>removeClass</CODE>. It then
   * removes that node. If no such node is found, the method is a
   * no-op.
   * 
   * @param thisNode, Node to be removed from document. 
 **/
  static public void removeThisNodeAndContext( Node thisNode, java.lang.Class removeClass ){ 
    while( (null != thisNode) && ! removeClass.isInstance( thisNode ) ){ 
      thisNode = thisNode.getParentNode(); 
    }
    if( null != thisNode ){ 
      // DBG
      // ProdUtil.log( "About to remove a " + ((HTMLElement)thisNode).getTagName() ); 
      removeThisNode( thisNode ); 
    }else{
      // DBG
      // ProdUtil.log( "Did not remove a " + ((HTMLElement)thisNode).getTagName() ); 
    }
  }


  /** 
   * Removes this node from the DOM without removing its children. The
   * children become children of its parent node.
   **/
  // TODOONEDAY: (Sep99) test this method!
  static public void replaceNodeWithChildren( Node node ){ 
    NodeList childNodes ; 
    Node child ; 
    Node parent ; 
    Node afterNode ; 
    int i ; 
    Text textNode ; 
    childNodes = node.getChildNodes(); 
    parent = node.getParentNode(); 
    afterNode = node.getNextSibling(); 
    for( i=0 ; i<childNodes.getLength() ; i++ ){ 
      child = childNodes.item( i ); 
      node.removeChild( child ); 
      parent.insertBefore( child, afterNode ); 
    }
    parent.removeChild( node ); 
  }


  /** 
   * Extracts the protocol, if any, from an HREF attribute. If there's
   * no ':', it returns a string which couldn't be the protocol.
   **/
  static public String protocolOf( String hrefAttr ){ 
    int colonIndex ; 
    colonIndex = hrefAttr.indexOf( ":" ); 
    if( colonIndex >= 0 ){ 
      return hrefAttr.substring( 0, colonIndex-1 ); 
    }else{ 
      return "UNKNOWN-PROTOCOL" ; 
    }
  }


  /** 
   * Puts a name value pair into a query string, replacing any existing value fo that name. 
   * <BR>
   * Assumes that the query string is already URL encoded. 
   * @param queryString, Original value of query string.
   * @param name, Name part of name value pair, <B>not</B> URL encoded.
   * @param name, Value part of name value pair, <B>not</B> URL encoded.
   * @returns modified query string.
   **/
  static public String setValueInQueryString( String queryString, String name, String value ){ 
    int nameStart, valueStart, afterStart ; 
    int ampersandIndex ; 
    int queryStringLength = queryString.length() ; 
    String queryStringBeforePair, queryStringAfterPair, newQueryString ; 
    String pairString ; 

    //////  Find query string up to name.
    if( queryString.startsWith( name + "=" ) ){ 
      nameStart = 0 ; 
      queryStringBeforePair = "" ; 
    }else{ 
      ampersandIndex = queryString.indexOf( "&" + name + "=" ); 
      if( ampersandIndex >= 0 ){ 
        nameStart = ampersandIndex + 1 ; 
        queryStringBeforePair = queryString.substring( 0, nameStart-1 ); 
      }else{ 
        nameStart = queryStringLength ; 
        queryStringBeforePair = queryString ; 
      }
    }

    //////  Find query string after name/value pair.
    if( nameStart < queryStringLength ){ 
      ampersandIndex = queryString.indexOf( "&", nameStart ); 
      if( ampersandIndex >= 0 ){ 
        afterStart = ampersandIndex + 1 ; 
        queryStringAfterPair = queryString.substring( afterStart ); 
      }else{ 
        afterStart = queryStringLength ; 
        queryStringAfterPair = "" ; 
      }
    }else{ 
      afterStart = queryStringLength ; 
      queryStringAfterPair = "" ; 
    }

    //////  Build new query string
    pairString = URLEncoder.encode( name ) + "=" + URLEncoder.encode( value ); 
    if( queryStringBeforePair.length() > 0 ) pairString = "&" + pairString ; 
    if( queryStringAfterPair.length() > 0 ) pairString = pairString + "&" ; 
    newQueryString = queryStringBeforePair + pairString + queryStringAfterPair ; 

    //////  Tidy up and leave
    return newQueryString ; 
  }


  /** 
   * Detects whether a string has any content.  
   * 
   * <BR> 
   * 
   * <B>WARNING</B>: JDBC has a bug such that it returns a single
   * space instead of the empty string.  This method currently (Sep99)
   * returns false if the argument is a single space, as well as for
   * empty and null strings.
   *
   * <B>NOTE</B>: disabled hack for above.  The JDBC driver in question
   * was for Sybase.
   * 
   * @param str potentially null or empty string.
   * @return true if the string is non-null and has length greater than zero 
   **/
  static public boolean stringHasContent( String str ){ 
      if( null == str ) return false ; 
      if( 0 == str.length() ) return false ; 
      // if( " ".equals( str ) ) return false ; // Nasty hack 'coz of JDBC bug -- it returns " "
      return true ; 
  }


  /** 
   * Overloaded method.
   * @param str potentially null or empty string.
   * @return true if the string is non-null and has length greater than zero 
   **/
  static public boolean stringHasContent( StringBuffer str ){ 
      if( null == str ) return false ; 
      return stringHasContent( str.toString() ); 
  }


  /** 
   * Developers convenience method for method <CODE>stringHasContent()</CODE>.
   * This method returns the same value, but logs what the string value was.
   *
   * @see ProdUtil#stringHasContent
   **/
  static public boolean stringHasContentLOG( String str ){ 
    String logString ; 
    logString = "Call to stringHasContentLOG() with " ; 
    if( null == str ){ 
      logString += "a null string." ; 
    }else{ 
      logString += "string '" + str + "'" ; 
    }
    ProdUtil.log( logString ); 
    return stringHasContent( str ); 
  }


  /** 
   * Check to see whether a name occurs in a KeywordValueTable. 
   * @exception HttpPresentationException if KeywordValueTable.get() threw a KeywordValueTable exception. **/
  static public boolean nameInQueryString( KeywordValueTable args, String name ) throws HttpPresentationException {
    try{ 
      // KeywordValueTable.get() returns null if name not in table. 
      return args.containsKey( name ); 
    }catch( KeywordValueException kve ){ 
      throw new HttpPresentationException( kve );
    }
  }


  /** 
   * Gets the String value for a name from a KeywordValueTable. 
   *
   * <BR> 
   *
   * @return null if no name found. 
   * @exception HttpPresentationException if KeywordValueTable.get() threw a KeywordValueTable exception. **/
  static public String getFromQueryString( KeywordValueTable args, String name ) throws HttpPresentationException {
    try{ 
      // KeywordValueTable.get() returns null if name not in table. 
      return (String) args.get( name ); 
    }catch( KeywordValueException kve ){ 
      throw new HttpPresentationException( "Problem with '" + name + "'.", kve );
    }
  }


  /** 
   * Gets a String array value for a name from a KeywordValueTable. 
   * @return null if no name found. 
   * @exception ApplicationException if KeywordValueTable.get() threw a KeywordValueTable exception. **/
  static public String[] getArrayFromQueryString( KeywordValueTable args, String name ) throws ApplicationException {
    Object queryObject ; 
    String [] list ; 
    try{ 
      queryObject = args.get( name ); 
    }catch( KeywordValueException kve ){ 
      // Exception thrown "If the keyword is not syntactically legal
      // or is a non-leaf element of the keyword is not a section. "
      throw new ApplicationException( "Problem with '" + name + "' on query string. Look up JavaDoc on Class com.lutris.util.KeywordValueTable!", kve );
    }
    if( null == queryObject ){ 
      return null ; 
    }else if( queryObject instanceof String ){
      list = new String [1] ; 
      list[0] = (String) queryObject ; 
      return list ; 
    }else if( queryObject instanceof String[] ){
      return (String[]) queryObject ; 
    }else{ 
      throw new ApplicationException( "Problem with '" + name + "' on query string. Look up JavaDoc on Class com.lutris.util.KeywordValueTable!" );
    }
  }


  /** 
   * Gets section of sessionData reserved for presentation layer. 
   * If necessary, it creates and initialises the presentation layer section. 
   * @exception HttpPresentationException
   **/
    // TODOONEDAY: Replace with a call to ensureSectionData(). Don't
    // forget special initialisation, like setting the "nextId" field.
  static public KeywordValueTable presentationData( KeywordValueTable sessionData ) throws HttpPresentationException { 
    KeywordValueTable psd ; // section of session data used by presentation layer.
    try{ 

      //////  Try to get "presentation" section of sessionData.
      psd = (KeywordValueTable) sessionData.getSection( "presentation" ); 
      
      //////  Create "presentation" section, if necessary
      if( null == psd ){ 
        sessionData.set( "presentation.nextId", new Integer( 1 ) ); 
        psd = (KeywordValueTable) sessionData.getSection( "presentation" ); 
      }

      //////  Tidy up and leave 
      return psd ; 

    }catch( KeywordValueException kve ){ 
      throw new HttpPresentationException( "Problem getting / setting session data.", kve ); 
    }
  }


  /** 
   * Returns a section of a KeywordValueTable. If necessary, it
   * creates the section.
   * @exception HttpPresentationException
   **/
  static public KeywordValueTable ensureSectionData( String sectionName, KeywordValueTable outerData ) throws HttpPresentationException { 
    KeywordValueTable innerData ; // desired subsection 
    try{ 

      //////  Create  section, if necessary
      if( ! outerData.containsKey( sectionName ) ){ 
        // A gruesome hack, I know, but I can't figure out newSection().
        outerData.set( sectionName + ".ZZZZZZ", "Now you see me, " ); 
        outerData.getSection( sectionName ).remove( "ZZZZZZ" ); // Now you don't
      }

      //////  Get the section
      innerData = (KeywordValueTable) outerData.getSection( sectionName ); 
      
      //////  Tidy up and leave 
      return innerData ; 

    }catch( KeywordValueException kve ){ 
      throw new HttpPresentationException( "Problem getting / setting section.", kve ); 
    }
  }


  /** 
   * Returns a unique name for a served page. It changes each time a page is served!
   * @exception HttpPresentationException
   **/
  static public String getUniquePageString( KeywordValueTable psd ) throws HttpPresentationException { 
    Integer idNum ; 
    try{ 

      idNum = (Integer) psd.get( "nextId" ); 
      psd.set( "nextId", new Integer ( idNum.intValue() + 1 ) ); 
      return "served" + idNum ; 

    }catch( KeywordValueException kve ){ 
      throw new HttpPresentationException( "Problem getting / setting session data.", kve ); 
    }
  }    


  /**
   * Map String to ObjectId
   **/
  static public ObjectId toObjectId( String idString ) throws HttpPresentationException {
    try{ 
      return new ObjectId( idString ); 
    } catch( ObjectIdException oie ){ 
      throw new HttpPresentationException( "Failed to construct a DB ObjectId for '" + idString + "'", oie ); 
    }
  }


  /**
   * Throws an exception if it's passed a null object.
   **/
  static public void testForNull( Object object, String message ){ 
    if( null == object ){ 
      throw new NullPointerException( message ); 
    }
  }


  /**
   * Throws an exception if it's passed a null object.
   **/
  static public void testForNull( Object object ){ 
    testForNull( object, "Got a null where I didn't expect one. \nProbably straight from a database query." ); 
  }


  /**
   * Returns a string for a double. 
   * <BR> 
   * @param zeroString If non-null, return this string if the double is zero.
   **/
  static public String doubleToString( double x, String zeroString ){ 
    String str ; 
    str = "" + x ; 
    if( (null!=zeroString) && (0.0 == x) ) str = zeroString ; 
    return str ; 
  }


  /**
   * Returns a string for a date. 
   *
   * <BR> 
   *
   * This does <EM>not</EM> subsume the functionality of method
   * <CODE>cleanString()</CODE>. If <CODE>date</CODE> is
   * <CODE>null</CODE>, then <CODE>null</CODE> is returned.
   **/
  static public String toString( Date date ){ 
    if( null == date ){ 
      return null ; 
    }else{
      return date.toString(); 
    }
  }


  /**
   * Returns a "Yes" or "No" for a boolean. 
   * @param b  Value to display
   * @deprecated  Use booleanToDisplay(b)
   **/
  static public String toDisplayString( boolean b ){ 
      return b ? "Yes" : "No" ; 
  }


  /** 
   * Appends <CODE>one</CODE> to <CODE>combined</CODE>, if there's
   * any content in <CODE>one</CODE>.
   **/
  static public StringBuffer appendStringPerhaps( StringBuffer combined, String one, String sep ){ 
    if( ! stringHasContent( one ) ){
      return combined ; 
    }else if( 0 == combined.length() ){ 
      return combined.append( one );
    }else{ 
      return combined.append( sep + one ); 
    }
  }


  /** 
   * Sets the class attribute in the node. 
   * 
   * <BR>
   * 
   * Commonly used in TableRows to get alternating colours. 
   */
  public static void setClassAttributeModulo( HTMLElement el, String classPrefix, int i, int max ){ 
    el.setAttribute( "class", classPrefix + ((i%max)+1) ); 
  }
  
  
  /** 
   * Overloaded method.
   * 
   * <BR>
   * 
   * This variant alternates between CLASS="LINE1" and CLASS="LINE2".
   */
  public static void setClassAttributeModulo( HTMLElement el, int i ){ 
    setClassAttributeModulo( el, "LINE", i , 2 ); 
  }


  /** 
   * Replaces a character with another throughout a string. The method
   * returns a new string ; it does not change the original string.
   **/
  static public String replaceChar( String str, char remove, char insert ){ 
    StringBuffer tmp ; 
    int i ; 
    tmp = new StringBuffer( str ); 
    for( i=0 ; i<tmp.length() ; i++ ){ 
      if( remove == tmp.charAt( i ) ) tmp.setCharAt( i, insert ); 
    }
    return tmp.toString(); 
  }


  /** 
   * Inserts <CODE>insert</CODE> after every <CODE>pattern</CODE> in
   * <CODE>str</CODE>. Returns null if any argument is null. (Note:
   * behaviour for null <CODE>insert</CODE> and <CODE>pattern</CODE>
   * arguments may change in future.)
   **/
  static public String insertAfterPattern( String str, String pattern, String insert ){ 
    StringBuffer tmp ; 
    int i ;
    final int insertLen = insert.length(); 
    final int patternLen = pattern.length(); 
    if( null == str ) return null ; 
    if( null == pattern ) return null ; 
    if( null == insert ) return null ; 
    tmp = new StringBuffer( str ); 
    i = tmp.toString().indexOf( pattern ); 
    while( i >= 0 ){ 
      tmp.insert( i+patternLen, insert ); 
      i = tmp.toString().indexOf( pattern, i+patternLen+insertLen ); 
    }
    return tmp.toString(); 
  }


// <BR> broken under OpenXML :-(   /** 
// <BR> broken under OpenXML :-(    * Inserts "<CODE>&lt;BR&gt;</CODE>" after every linefeed in <CODE>str</CODE>
// <BR> broken under OpenXML :-(    **/
// <BR> broken under OpenXML :-(   static public String insertAfterPattern( String str ){ 
// <BR> broken under OpenXML :-(     return insertAfterPattern( str, "\n", "<BR>" );
// <BR> broken under OpenXML :-(   }


  /**
   * Inserts node <CODE>insert</CODE> before node <CODE>existing</CODE>, as a sibling. 
   **/
  static public void insertSiblingBefore( Node insert, Node existing ){
    Node parent ; 
    parent = existing.getParentNode(); 
    parent.insertBefore( insert, existing );
  }


  /**
   * Inserts node <CODE>insert</CODE> after node <CODE>existing</CODE>, as a sibling. 
   **/
  static public void insertSiblingAfter( Node insert, Node existing ){
    Node parent ; 
    Node siblingAfter ; 
    parent = existing.getParentNode(); 
    siblingAfter = existing.getNextSibling(); 
    parent.insertBefore( insert, siblingAfter );
  }


  /** 
   * Instantiates the deleteMeComment and deleteMeString variables.
   * @see #deleteMeComment
   * @see #deleteMeString
   **/
  static public void instantiateDeleteMeComment(){ 
    deleteMeComment = (Comment) templateDOM.getElementDeleteMeContainer().getFirstChild(); 
    deleteMeString = deleteMeComment.getData(); 
  }


  /** 
   * Deletes <CODE>deleteMe</CODE> comments in the DOM tree, and
   * perhaps their surrounds. <BR> The surrounds are deleted if
   * <CODE>deleteClass</CODE> is non null. In this case, the tree
   * rooted at the nearest ancestor of class <CODE>deleteClass</CODE>
   * is deleted.
   * @param deleteClass if not null, specifies class of node to be deleted
   * @see #deleteMeComment
   **/
  static public void removeDeleteMeComments( Node node, java.lang.Class deleteClass ){ 
    ListCommentContentNodes deleteUs ; 
    Node deleteMe ; 
    if( null == deleteMeComment ) instantiateDeleteMeComment(); 
    deleteUs = new ListCommentContentNodes( node, deleteMeString ); 
    while( deleteUs.hasMoreElements() ){ 
      deleteMe = (Node) deleteUs.nextElement() ; 
      if( null == deleteClass ){ 
          // Just delete this comment node.
          removeThisNode( deleteMe ); 
      }else{
        // Delete the nearest ancestor of class deleteClass
        while( (null!=deleteMe) && ! deleteClass.isInstance( deleteMe ) ){ 
            deleteMe = deleteMe.getParentNode(); 
        }
        if( null != deleteMe ) removeThisNode( deleteMe ); 
      }
    }
  }


  /** 
   * Overloaded method: This variant only deletes the comments, not their surrounds.
   **/
  static public void removeDeleteMeComments( Node node ){ 
    removeDeleteMeComments( node, null ); 
  }


  /** 
   * Inserts a clone of the <CODE>deleteMe</CODE> comment before the given node.
   * @see #deleteMeComment
   **/
  static public void flagWithDeleteMe( Node node ){ 
    Node parent ; 
    parent = node.getParentNode(); 
    parent.insertBefore( deleteMeComment.cloneNode( true ), node ); 
  }


  /**
   * Overloaded method: This variant only inserts the comment if <CODE>str</CODE> has content. 
   * @see #flagWithDeleteMe
   **/
  static public void flagWithDeleteMe( Node node, String str ){ 
    if( ! stringHasContent( str ) ) flagWithDeleteMe( node ); 
  }


  /**
   * Overloaded method: This variant only inserts the comment if <CODE>str</CODE> has content. 
   * @see #flagWithDeleteMe
   **/
  static public void flagWithDeleteMe( Node node, StringBuffer str ){ 
    flagWithDeleteMe( node, str.toString() ); 
  }


  /** 
   * Extracts the rest of the first line in <CODE>protoCode</CODE> containing
   * <CODE>pattern</CODE>. 
   * @return As above. If <CODE>pattern</CODE> does not occur, returns null.
   **/
  static public StringBuffer getProtoCode( StringBuffer protoCode, String pattern ){
    String code = protoCode.toString(); 
    int i, j ; 
    i = code.indexOf( pattern ); 
    if( i < 0 ) return null ; 
    i += pattern.length(); 
    // 'i' is start of section to return
    j = code.indexOf( '\n', i ); 
    // 'j' is end of line to return
    if( j >= 0 ){ 
      return new StringBuffer( code.substring( i, j ) ); 
    }else{
      return new StringBuffer( code.substring( i ) ); 
    }
  }


  /** 
   * Copies <CODE>protoLine</CODE>, and inserts <CODE>insert</CODE>
   * into the copy, after all instances of <CODE>pattern</CODE>. 
   **/
  static public String fillProtoCode( StringBuffer protoLine, String pattern, String insert ){
    StringBuffer filledLine = new StringBuffer( protoLine.toString() ); 
    int startOfPattern ; 
    int insertLocn ; 
    startOfPattern = filledLine.toString().indexOf( pattern ) ; 
    while( startOfPattern >= 0 ){ 
      insertLocn = startOfPattern + pattern.length(); 
      filledLine.insert( insertLocn, insert );             
      startOfPattern = filledLine.toString().indexOf( pattern, insertLocn + insert.length() ) ; 
    }
    return filledLine.toString(); 
  }


  /** 
   * Inserts <CODE>insert</CODE> into <CODE>theCode</CODE>, after
   * the first line containing <CODE>pattern</CODE>.
   **/
  static public void insertInCode( StringBuffer theCode, String pattern, String insert ){ 
    int i ; 
    i = theCode.toString().indexOf( pattern ); 
    if( i < 0 ) return ; 
    i = theCode.toString().indexOf( '\n', i ); 
    if( i < 0 ){ 
      theCode.append( "\n" + insert ); 
    }else{ 
      theCode.insert( i, "\n" + insert ); 
    }
  }


  /** 
   * Returns a Hashtable mapping to all the nodes with ID attributes. The keys are the ID strings. 
   **/
  static public Hashtable idHashtable( HTMLElement node ){
    ListNodesWithID list ; 
    HTMLElement idNode ; 
    String idAttr ; 
    Hashtable hash ; 

    hash = new Hashtable(); 
    list = new ListNodesWithID( node ); 
    while( list.hasMoreElements() ){ 
      idNode = (HTMLElement) list.nextElement(); 
      idAttr = idNode.getAttribute( "id" ); 
      if( null == idAttr ) idAttr = idNode.getAttribute( "ID" ); 
      hash.put( idAttr, idNode ); 
    }
    return hash ; 
  }

  /**
   * Method log() writes it's argument to where-ever the logging goes. 
   * <P>
   * Under the multiserver, it goes to stderr in the xterm where the
   * multiserver was invoked, and to the output/xxxx.log file.
   **/
  static public void log( String message ){ 
    LogChannel lc = Enhydra.getLogChannel();

    if (lc != null){
      lc.write( Logger.INFO, message );
    }else{
      System.err.println( message ); 
    }
  }


  /**
   * Method debug() writes it's argument to where-ever the logging goes. 
   * <P>
   * Under the multiserver, it goes to stderr in the xterm where the
   * multiserver was invoked, and to the output/xxxx.log file.
   **/
  static public void debug( String message ){ 
    LogChannel lc = Enhydra.getLogChannel();

    if (lc != null){
      lc.write( Logger.DEBUG, message );
    }else{
      System.err.println( message ); 
    }
  }


  /**
   * Method error() writes it's argument to where-ever the logging goes. 
   * <P>
   * Under the multiserver, it goes to stderr in the xterm where the
   * multiserver was invoked, and to the output/xxxx.log file.
   **/
  static public void error( String message ){ 
    LogChannel lc = Enhydra.getLogChannel();

    if (lc != null){
      lc.write( Logger.ERROR, message );
    }else{
      System.err.println( message ); 
    }
  }


  /**
   * Method cleanString(String) chops trailing whitespace from a given
   * String, and if it is null sets it to the empty string.
   **/
  static public String cleanString( String input ){

    return cleanString( input, false);

  }

  /**
   * Method cleanString(String, boolean) chops trailing whitespace
   * from a given String, and if it is null sets it to
   * defaultValue. If defaultValue is null, the empty string is used.
   **/
  static public String cleanString( String input, String defaultValue ){

    // Convert null Strings to empty.
    if( null == input ) input = "" ;

    // Create a StringBuffer to operate on.
    StringBuffer output = new StringBuffer( input );

    // Chop trailing spaces from the String.
    while ( ((output.length() - 1 ) >= 0) && (' ' == input.charAt(output.length() - 1 )) ) {
      output.setLength(output.length() - 1);
    }

    // Convert empty Strings to HTML non-breaking space, if required.
    if( ( 0 == output.length() ) && ( null != defaultValue ) ) {
      output = new StringBuffer(defaultValue) ;
    }

    // Return the cleaned String.
    return output.toString();
  }


  /**
   * This variant replaces a null <CODE>input</CODE> with
   * "<CODE>&nbsp;</CODE>".
   **/
  static public String cleanString( String input, boolean setNbsp ){

    return cleanString( input, setNbsp? "\u00A0" : "" ); 

  }


  /**
   * Finds the HTML TITLE Node for the given document.
   * Returns null if the TITLE is not found.
   **/
  static public HTMLTitleElement getTitleElement ( Document document ) throws HttpPresentationException {
  
    // Find and return the HTML TITLE Node in a document, or null otherwise.
  
    Node htmlNode;           // The HTML Node for the current document.
    Node headNode;           // The HEAD Node for the current document.
    Node titleNode;          // The HTML TITLE Node for the current document.
    HTMLTitleElement titleElement;  // The HTMLTitleElement version of 
                                    // titleNode.
  
    // Find the document's HTML Node.
    htmlNode = document.getDocumentElement();
  
    // Find the document's HEAD node.
    headNode = getChildOfType( htmlNode, org.w3c.dom.html.HTMLHeadElement.class );
    if ( null == headNode ) {
      // If the HTML HEAD node isn't there, there won't be a TITLE either!
      return null;
    }
  
    // Look for a TITLE node.
    titleNode = getChildOfType( headNode, org.w3c.dom.html.HTMLTitleElement.class );

    try {
      titleElement = (HTMLTitleElement)titleNode;
    } catch ( ClassCastException cce ) {
      throw new HttpPresentationException( "Could not convert an HTML TITLE Node into an HTMLTitleElement!  This simply should not happen :(", cce );
    }

    // titleNode will be null if it is not found.
    return titleElement;
  
  }


  /**
   * Finds the HTML BODY Node for the given document.
   * Returns null if the BODY is not found.
   **/
  static public HTMLBodyElement getBodyElement ( Document document ) throws HttpPresentationException {
  
    // Find and return the HTML BODY Node in a document, or null otherwise.
  
    Node htmlNode;           // The HTML Node for the current document.
    Node headNode;           // The HEAD Node for the current document.
    Node bodyNode;          // The HTML BODY Node for the current document.
    HTMLBodyElement bodyElement;  // The HTMLBodyElement version of 
                                    // bodyNode.
  
    // Find the document's HTML Node.
    htmlNode = document.getDocumentElement();
  
    // Find the document's BODY node.
    bodyNode = getChildOfType( htmlNode, org.w3c.dom.html.HTMLBodyElement.class );

    try {
      bodyElement = (HTMLBodyElement)bodyNode;
    } catch ( ClassCastException cce ) {
      throw new HttpPresentationException( "Could not convert an HTML BODY Node into an HTMLBodyElement!  This simply should not happen :(", cce );
    }

    // bodyNode will be null if it is not found.
    return bodyElement;
  
  }


  /**
   * Finds the first child of a given Node that matches the Class given.
   * Returns null if no matches are found.
   **/
  static public Node getChildOfType( Node node_, java.lang.Class comparison_ ) {
  
    NodeList tmpNodeList;    // The children of node_.  Used
                             // as a temporary storage of Nodes.
    Node tmpNode;            // A temporary working Node, used for 
                             // Object comparisons.
  
    // Find the children of node_.
    tmpNodeList = node_.getChildNodes();
  
    // Try to find a node of the same type as the one to compare to.
    for ( int i=0 ; i < tmpNodeList.getLength() ; i++ ) {
  
      tmpNode = tmpNodeList.item( i );
      //if ( tmpNode.getClass().isInstance( comparison_ ) ) {
  
      if ( comparison_.isInstance( tmpNode ) ) {
  
        // Yay!  We found one!
        return tmpNode;
      }
  
    }
  
    // If we haven't returned by now, we didn't find a Node of the right type.
    return null;
  
  }


  /**
   * Checks to see if the DEBUG flag is turned on in the session data. 
   * @param debugData should be session data section "presentation.DEBUG" 
   **/
  static public boolean sessionDebugFlag( KeywordValueTable debugData ) throws ApplicationException {
    String tmpString ; 
    try{ 
      tmpString = debugData.getString( "DEBUG", "FALSE" ); 
    }catch( KeywordValueException kve ){ 
      throw new ApplicationException( kve ); 
    }
    return new Boolean( tmpString ).booleanValue(); 
  }


  /**
   * Checks to see if the DEBUG flag is turned on in the session data. 
   * @param debugData should be session data section "presentation.DEBUG" 
   **/
  static public boolean booleanFromData( String name, KeywordValueTable debugData ) throws KeywordValueException {
    String tmpString ; 
    tmpString = debugData.getString( name ); 
    return new Boolean( tmpString ).booleanValue(); 
  }


/**
 * Finds all FORM nodes in rootNode, and sets the name value pairs in
 * each one from a well-known-location in the presentation data.
 * 
 * @param rootNode 
 * @param nvPairs contains name value pairs to be used
 **/
  static public void restoreFormsFromPresentationData( Node rootNode, KeywordValueTable presentationData ){ 
    KeywordValueTable nvPairs ; 
    try{
      nvPairs = presentationData.getSection( "RESTOREQUERYSTRING" ); 
    }catch( KeywordValueException kve ){ 
      nvPairs = null ; 
    }
    if( null == nvPairs ){ 
      ProdUtil.log( "Failed to find RESTOREQUERYSTRING in presentationData" ); 
      return ; 
    }
    restoreForms( rootNode, nvPairs ); 
    // try{
    // }catch( KeywordValueException kve ){ 
    //   ProdUtil.log( "Failed to restore from query string: " + nvPairs.toString() ); 
    // }
  }


// NOT USED (10/9/99) /**
// NOT USED (10/9/99)  * Finds all FORM nodes in rootNode, and sets the name value pairs in
// NOT USED (10/9/99)  * each one from a query string.
// NOT USED (10/9/99)  * 
// NOT USED (10/9/99)  * @param rootNode 
// NOT USED (10/9/99)  * @param nvPairs contains name value pairs to be used
// NOT USED (10/9/99)  **/
// NOT USED (10/9/99)   static public void restoreForms( Node rootNode, String queryString ){ 
// NOT USED (10/9/99)     KeywordValueTable nvPairs ; 
// NOT USED (10/9/99)     try{
// NOT USED (10/9/99)       nvPairs = HttpUtil.buildQueryTable( queryString ); 
// NOT USED (10/9/99)       restoreForms( rootNode, nvPairs ); 
// NOT USED (10/9/99)     }catch( KeywordValueException kve ){ 
// NOT USED (10/9/99)       ProdUtil.log( "Failed to restore from query string: " + queryString ); 
// NOT USED (10/9/99)     }
// NOT USED (10/9/99)   }


/**
 * Finds all FORM nodes in rootNode, and sets the name value pairs in
 * each one from a KeywordValueTable.
 * 
 * @param rootNode 
 * @param nvPairs contains name value pairs to be used
 **/
  static public void restoreForms( Node rootNode, KeywordValueTable nvPairs ){ 
    ListFormNodes formNodes ; 

    // Set classes document variable, if not yet done.
    if( null == document ) document = rootNode.getOwnerDocument(); 

    // Now, on with the job ...
    formNodes = new ListFormNodes( rootNode ); 
    while( formNodes.hasMoreElements() ){ 
      restoreForm( (HTMLFormElement)(formNodes.nextElement()), nvPairs ); 
    }
  }


/**
 * Sets the name value pairs in a FORM from a KeywordValueTable.
 * 
 * @param formNode FORM to be set
 * @param nvPairs contains name value pairs to be used
 **/
  static public void restoreForm( HTMLFormElement formNode, KeywordValueTable nvPairs ){ 
    ListFormElementNodes inputNodes ; // Includes other form elements, eg SELECT
    HTMLElement elementNode ; 
    HTMLInputElement inputElement ; 
    HTMLTextAreaElement textAreaElement ; 
    HTMLSelectElement selectElement ; 
    HTMLOptionElement optionElement ; 
    String nameAttr ; 
    String pairValue ; 

    inputNodes = new ListFormElementNodes( formNode ); 
    while( inputNodes.hasMoreElements() ){ 
      elementNode = (HTMLElement)(inputNodes.nextElement()); 
      testForNull( elementNode, "elementNode is null" ); 

      //////  Check the names and values exist where they should, and get them.
      nameAttr = elementNode.getAttribute( "name" ); 
      if( null == nameAttr ) continue ; 
      // DBG
      System.err.println( "Found NAME='" + nameAttr + "'" ); 
      try{ 
        pairValue = nvPairs.getString( nameAttr, null ); 
      }catch( KeywordValueException kve ){
        continue ; 
      }
      if( null == pairValue ) continue ; 
      // DBG
      System.err.println( "In KeywordValueTable, " + nameAttr + "='" + pairValue + "'" ); 
      pairValue = cleanString( pairValue, false ); 
  
      //////  Insert value into form element.
      if( elementNode instanceof HTMLInputElement ){
        System.err.println( "In restoreForm(), reached A" ); // DBG TODO delete
        elementNode.setAttribute( "value", pairValue ); 
      }else if( elementNode instanceof HTMLTextAreaElement ){
        System.err.println( "In restoreForm(), reached B" ); // DBG TODO delete
        Text firstText ; 
        firstText = XMLCUtil.getFirstText( elementNode ); 
        System.err.println( "In restoreForm(), reached C" ); // DBG TODO delete
        if( null == firstText ){ 
          // Create text node with desired content, then insert it. 
        System.err.println( "In restoreForm(), reached D" ); // DBG TODO delete
          firstText = document.createTextNode( pairValue ); 
          elementNode.appendChild( firstText ); 
        }else{ 
          // Set content in existing text node.
        System.err.println( "In restoreForm(), reached E" ); // DBG TODO delete
          firstText.setData( pairValue ); 
        }
      }else if( elementNode instanceof HTMLSelectElement ){
        System.err.println( "In restoreForm(), reached F" ); // DBG TODO delete
        // This section programmed by evolution -- keep changing
        // until it works. Some accurate doco on selecting options
        // would be *much* appreciated. Rant rant rant ...
        HTMLCollection optionCollection ; 
        HTMLOptionElement optionNode ; 
        String valueAttr ; 
        int i ; 
        optionCollection = ((HTMLSelectElement)elementNode).getOptions(); 
        for( i=0 ; i<optionCollection.getLength() ; i++ ){ 
          optionNode = (HTMLOptionElement) optionCollection.item( i ); 
          valueAttr = optionNode.getAttribute( "value" ); 
          optionNode.removeAttribute( "selected" ); 
          optionNode.setDefaultSelected( pairValue.equals( valueAttr ) ); 
          if( pairValue.equals( valueAttr ) ) optionNode.setAttribute( "selected", "selected" ); 
        }
        // OLD CODE optionElement = ProdUtil.getOptionByValue( (HTMLSelectElement) elementNode, pairValue ); 
        // OLD CODE // DBG 
        // OLD CODE ProdUtil.log( "Setting option: " + optionElement.toString() ); 
        // OLD CODE optionElement.setDefaultSelected( true ); 
      }else{
        System.err.println( "In restoreForm(), reached G" ); // DBG TODO delete
        System.err.println( "Unknown 'FORM element' type: " + elementNode.getClass().getName() );
      }
    }
  }


  /**
   * Returns the first Text node in rootNode. If there isn't one, it
   * inserts one as the first child, with the empty string for content.
   * @return Text node, newly created if necessary.
   **/
  static public Text getFirstText( Node rootNode ){ 
    Text textNode ; 
    try{
      textNode = XMLCUtil.getFirstText( rootNode ); 
    }catch( NullPointerException npe ){ 
        testForNull( rootNode, "AAA" ); 
        testForNull( rootNode.getOwnerDocument(), "BBB" ); 
      textNode = rootNode.getOwnerDocument().createTextNode( "" ); 
      rootNode.insertBefore( textNode, rootNode.getFirstChild() ); 
    }
    return textNode ; 
  }


  /**
   * Removes all the ID attributes from nodes in the tree
   **/
  static public void removeIDAttributes( Node rootNode ){ 
    ListNodesWithID list ; 
    Node node ; 
    HTMLElement element ; 

    list = new ListNodesWithID( rootNode ); 
    while( list.hasMoreElements() ){ 
      node = (Node) (list.nextElement()); 
      if( node instanceof HTMLElement ){ 
        element = (HTMLElement) node ; 
        element.removeAttribute( "id" ); 
      }
    }
  }


  /**
   * Removes all the ID attributes from nodes in the tree
   **/
  static public void removeIDAttributes( Document document ){ 
    removeIDAttributes( document.getDocumentElement() ); 
  }


  /**
   * Translates a relative URL to be relative to the application root directory
   * (used for ClientPageRedirectExceptions)
   **/
  static public String URLAppAbsoluteDir ( String relativeURL, String appSubDir ) {

    if ( relativeURL.startsWith("/") ) {
      return relativeURL;
    }

    return "/" + appSubDir + "/" + relativeURL;

  }


  /** 
   * Returns a string of spaces, to be used for indentation of generated code.
   * @param count Number of spaces to indent by
   **/
  static public String indentString( int count ){ 
     if( count > 50 ){ 
       // Yeah -- like this will ever happen ... 
       return indentString( 50 ) + indentString( count - 50 );
       // ... famous last words :-)
     }else{
       return "                                                         ".substring( 0, count );      
     }
  }    


  /**
   * Translates a com.lutris.KeywordValueTable into a hierarchical String representation
   **/
  public String KVTToString ( KeywordValueTable kvt ) {
    return KVTToString ( kvt, 0 );
  }

  /**
   * Translates a com.lutris.KeywordValueTable into a hierarchical String representation
   * (used for more complete error reporting - neither the KeywordValueTable toString()
   *  or toHtml() methods do exactly this :)
   **/
  public String KVTToString ( KeywordValueTable kvt, int indentLevel ) {

    StringBuffer sb = new StringBuffer();

    try {
      String[] keys = kvt.keys();
      for ( int i=0 ; i< keys.length ; i++ ) {
        if ( kvt.get( keys[i] ) instanceof KeywordValueTable ) {
          sb.append( indentString(indentLevel * 2) + keys[i] + ":\n" );
          sb.append( KVTToString ( (KeywordValueTable)kvt.get( keys[i] ), indentLevel+1 ) );
        } else {
          sb.append( indentString(indentLevel * 2) + keys[i] + "=" + kvt.getString( keys[i] ) + "\n" );
        }
      }

    } catch ( Exception e ) {
      System.out.println( "ERROR in method KVTToString: " + e.toString() );
    }

    return sb.toString();

  }

}

