package com.pisoftware.platypus.production;

import java.io.*;
import java.lang.*;
import java.util.*;
import java.net.URLEncoder;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import com.lutris.util.*;
import com.lutris.xml.xmlc.html.HTMLObject;
import com.lutris.xml.xmlc.*;
import com.lutris.appserver.server.sql.*;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.ApplicationException;
import com.lutris.appserver.server.Enhydra;
import com.lutris.logging.*;
import com.pisoftware.platypus.production.*;

/** 
 * <CODE>UserExceptions</CODE> are thrown for errors the user ought to
 * be able to handle. For example, a user error should be thrown if
 * the users attempts to access or delete a record that doesn't exist.
 * 
 * <BR>
 * 
 * <CODE>UserExceptions</CODE> do not go to the Enhydra error handling
 * page. Instead, they are caught by the BASE manipulation class, and
 * handled in <CODE>AppTemplate.java</CODE>. (Note: the config files
 * can specify other handlers for individual POs.)
 * 
 **/


/*
 *  Copyright (C) 1999 Plugged In Software Pty Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Plugged In Software
 *  PO Box 4130
 *  St. Lucia South Queensland
 *  Australia  4067
 *  info@PIsoftware.com
 */



public class UserException extends ApplicationException{

  /** Message to be given to user **/
  public String message ; 

  /** URL the user may go back to. **/
  public String backURL ; 


  public UserException( String message ){ 
    this( message, (String)null );
  }


  public UserException( String message, Exception e ){ 
    this( message, null, e );
  }


  public UserException( String message, String backURL ){ 
    super( message ); 
    this.message = message ; 
    this.backURL = backURL ; 
  }


  public UserException( String message, String backURL, Exception e ){ 
    super( e ); 
    this.message = message ; 
    this.backURL = backURL ; 
  }
}
