package AU.net.plugged.sort.event;

import java.util.*;

// Class:       SortEvent

/**
 *
 *  Version: 1.0<p>
 *
 *  Author: <A HREF="MAILTO:cryan@plugged.net.au">Chris Ryan</A><p>
 *
 * Company: <blocquote>Plugged In Software<br>
 *             P.O. Box 4130<br>
 *             St Lucia South<br>
 *             Brisbane Queensland 4067<br>
 *             Australia<br>
 *             <a href="http://www.plugged.net.au/">http://www.plugged.net.au</a><br>
 *
 *<p>
 *  Date:       15 September 1997
 *<p>
 *<dl><dt>Purpose:
 *      <dd>Notifies listeners of sort event(s).
 *                </dl>
 *
 *<dl><dt> Modification
 *  History:<dd>
 *</dl>
*/

/*
 *  Copyright (C) 1999 Plugged In Software Pty Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Plugged In Software
 *  PO Box 4130
 *  St. Lucia South Queensland
 *  Australia  4067
 *  info@PIsoftware.com
 */

public class SortEvent extends EventObject {


  /**
  *  The vector being sorted.
  */
  protected Vector _sort_vector = null;


  /**
  *    The type of event that occured.  Use the constant(s) provided.
  */
  protected int _event_type;


  /**
  * The event type is sort completed.
  */
  public final static int SORT_COMPLETED = 0;


  /**
  *    Creates an event, which can be passed to a SortListener
  *
  * @param source The object that generated this event
  * @param id     The type of event that these sort_vector belong to
  * @param sort_vector  A Vector containing String objects of the filenames
  */
  public SortEvent(Object source, int id, Vector sort_vector) {

    super(source);

    // Keep the values passed
    _sort_vector = sort_vector;
    _event_type = id;

  }


  /**
  *    Sets the type of event that occured
  * @param type Type of event that occured - see constants
  */
  public void setEventType(int type) {

    _event_type = type;

  }


  /**
  *    Returns the type of event that occured
  * @return Returns a constant from above 
  */
  public int getEventType() {

    return _event_type;

  }


  /**
  *    Sets the vector to be sorted
  * @param sort_vector A vector containing objects to be sorted
  */
  public void setSortVector(Vector sort_vector) {

    _sort_vector = sort_vector;

  }


  /**
  *    Gets the vector to be sorted
  * @return Returns the sort vector.
  */
  public Vector getSortVector() {

    return _sort_vector;

  }



  // The String gets changed to the current version by RCS
  private static final String version_string = "$Revision: 1.2 $";

  /**
  *    Returns the RCS version information for this class.
  *
  * @return Returns the RCS Revision string for this class
  */
  public static String getVersionString() {

    return version_string;

  }

}
