package com.pisoftware.platypus.squirt;

import  com.lutris.appserver.server.session.*;
import  com.lutris.util.*;
import  com.lutris.dods.builder.engine.*;
import  java.io.*;
import  java.util.*;
import  java.math.*;
import  java.net.*;
import  java.text.*;
import  com.pisoftware.platypus.util.*;


/**
 *  Author: <A HREF="MAILTO:cryan@plugged.net.au">Chris Ryan</A>, 
 *          <A HREF="MAILTO:guy@plugged.net.au">Guy Smith</A>, 
 *<P>
 * Company: Plugged In Software<BR>
 *          P.O. Box 4130<BR>
 *          St Lucia South<BR>
 *          Brisbane Queensland 4067<BR>
 *          Australia<BR>
 * <A HREF="http://www.plugged.net.au/">http://www.plugged.net.au</A><BR>
 *<P>
 *    Date: 26 July 1999
 *<P>
 * Purpose: This application reads an input file containing
 *          SQL statements and generates the necessary code to
 *          interface to the DO layer.  Refer to the squirt.txt
 *          document in the 'doc' directory.
 *
 */

/*
 *  Copyright (C) 1999 Plugged In Software Pty Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Plugged In Software
 *  PO Box 4130
 *  St. Lucia South Queensland
 *  Australia  4067
 *  info@PIsoftware.com
 */

public class Squirt {

/**
 * The application global config filename - with path
 */
  private String application_config = null;


/**
 * Handle to the application wide config file
 */
  private KeywordValueTable app_cfg = null;


/**
 * The starting output directory for the stubs, which
 * will probably be <appname>/<appname>/business.
 */
  private String output_dir = null;


/**
 * CORE is a required key in the global conf that contains
 * the QUERY subsection.
 */
  private static final String CORE = "CORE";


 /** 
  * QUERY is a required key in the global conf that contains
  * the queries within the application.  
  */
  private static final String QUERY = "QUERY";
  private static final String DO_TABLE = "DO.TABLE";


/**
 * The custom query template filename & internal holder
 */
  private String custom_application_template_file = null;
  private String custom_app_template = null;


/**
 * The custom query where clause template filename & internal holder
 */
  private String where_clause_template_file = null;
  private String where_clause_template = null;


/**
 * The easy query template filename & internal holder
 */
  private String easy_application_template_file = null;
  private String easy_app_template = null;


/**
 * The objectid query template filename & internal holder
 */
  private String objectid_application_template_file = null;
  private String objectid_app_template = null;


/**
 * The hashtable containing the strings for substitution.
 */
  private Hashtable substitutions = new Hashtable(100);


/**
 * The following strings are used as the keys to the substitution
 * hashtable.
 */
  private final static String SUB_DATE = "DATE";   // current date
  private final static String SUB_APPLICATION = "APP_NAME";  // application name
  private final static String SUB_JNAME = "APP_JNAME";  // application name, Java-ised


/**
 * Indicates if a file should be overwritten.
 */
  private boolean overwrite = false;


/**
 * Constructor - keeps values passed.
 * 
 * @param application_config   The application global config file name
 * @param output_dir  The destination base directory for the stubs
 */
  public Squirt(String application_config, String output_dir) {

    this.application_config = application_config;
    this.output_dir = output_dir;

  }


/**
 * The arguments expected to the program are:
 *
 * 1. The application global config file name
 * 2. Output directory name for the generated code
 */
  public static void main( String argv[] ) throws Exception {

    if (argv.length != 2 && argv.length != 3) {

      System.err.println("Not enough parameters!\n");
      System.err.println("Usage: Squirt <app_conf> <output_dir>\n");
      System.err.println("<app_conf> is the application global config file");
      System.err.println("<output_dir> is the output directory name for the generated code");
      System.err.println("[OVERWRITE] optional, must be in caps.  Overwrites any existing files.  USE WITH CAUTION!");
      System.exit(1);

    }

    Squirt squirt = new Squirt(argv[0], argv[1]);

    // set overwrite flag, if necessary
    if (argv.length == 3) {

      if (argv[2].equals("OVERWRITE")) {

        System.out.println("OVERWRITING files...");
        squirt.overwrite = true;

      } else {

        System.err.println("Unknown third parameter: "+argv[2]);
        System.exit(1);

      }

    }

    // get the Squirt base directory from the system properties.  
    // If not found, then current directory is assumed
    String bos_base_dir = System.getProperty("squirt.location", ".");

    // create the template filenames
    squirt.custom_application_template_file = bos_base_dir + File.separator + "templates" + File.separator + "custom.template";
    squirt.where_clause_template_file = bos_base_dir + File.separator + "templates" + File.separator + "where.template";
    squirt.easy_application_template_file = bos_base_dir + File.separator + "templates" + File.separator + "easy.template";
    squirt.objectid_application_template_file = bos_base_dir + File.separator + "templates" + File.separator + "objectid.template";

    // read the templates
    squirt.custom_app_template = squirt.readTemplate(squirt.custom_application_template_file);
    squirt.where_clause_template = squirt.readTemplate(squirt.where_clause_template_file);
    squirt.easy_app_template = squirt.readTemplate(squirt.easy_application_template_file);
    squirt.objectid_app_template = squirt.readTemplate(squirt.objectid_application_template_file);

    // kick the process off...
    squirt.process();

  }


/**
 * Reads a template file and holds onto the contents in a String.
 *
 * @param template_file  The template file to read. 
 */
  public String readTemplate(String template_file) throws Exception {

    return Util.readFile(template_file);

  }


/**
 * Processes the configuration keyword value table to
 * generate the necessary stub files.
 */
  public void setupSubstitutions( String queryname, KeywordValueTable onequerycfg ) throws Exception {
    String querytype ; // Type of query: custom, easy, objectid.
    String dbtablename ; // Database table name
    String jtablename ; // Java-ised table name
    String [] parameters ; // Parameters from configuration info.
    String [] paramtypes ; // Paramtypes from configuration info.
    String [] null_allowed ; // null_allowed from configuration info.
    String [] paramdecls ; // Element-wise merge of parameters and paramtypes.
    StringBuffer setQueryCalls ; // Builds SET_QUERY_PARAMETERS substitution.
    int i ; 
    String tmpStr ; 
    String [] tmpStrArray ; 

    querytype = onequerycfg.getString( "TYPE" ); 
    jtablename = onequerycfg.getString( "DO" ); 

    if (jtablename == null) {

      throw new Exception("Mandatory DO option not set for: "+queryname);

    }

    dbtablename = app_cfg.getString( "DO.TABLE." + jtablename + ".SQL_NAME" ); 

    if (dbtablename == null) {

      throw new Exception("Unknown table name: "+jtablename);

    }

    // Get the parameters & their details from the config file
    parameters = (String[]) onequerycfg.get( "PARAMETERS", new String[0]); 
    paramtypes = (String[]) onequerycfg.get( "PARAMTYPES", new String[0]); 
    null_allowed = (String[]) onequerycfg.get( "NULL_ALLOWED", new String[0]); 

    // basic check - make sure the number of parameters & types match
    if( parameters.length != paramtypes.length ){ 
      tmpStr = "In QUERY." + queryname + ", " ; 
      tmpStr += "PARAMETERS and PARAMTYPES have different item counts" ; 
      tmpStr += " ( " + parameters.length + " and " + paramtypes.length + " )" ; 
      throw new Exception( tmpStr ); 
    }

    // basic check - if null_allowed specified, the number of entries must match the number of parameters
    if( null_allowed.length != 0 && parameters.length != null_allowed.length ){ 
      tmpStr = "In QUERY." + queryname + ", " ; 
      tmpStr += "PARAMETERS and NULL_ALLOWED have different item counts" ; 
      tmpStr += " ( " + parameters.length + " and " + null_allowed.length + " )" ; 
      throw new Exception( tmpStr ); 
    }


    // create the parameters for the method & the EASY type setQueryXXX calls.
    paramdecls = new String[parameters.length] ; 
    setQueryCalls = new StringBuffer(); 
    boolean null_flag = false;

    for( i=0 ; i<parameters.length ; i++ ){
      paramdecls[i] = paramtypes[i] + " " + parameters[i] ; 

      if (i < null_allowed.length) {

        null_flag = null_allowed[i].equalsIgnoreCase("INCLUDE");

      } else {

        null_flag = false;

      } 

      if (!null_flag) {

        // TODO - type checking for correct null/empty object tests

        setQueryCalls.append( "if (ProdUtil.stringHasContent(" );
        setQueryCalls.append( parameters[i] );
        setQueryCalls.append( " )) { \n\n" ); 

      }

      setQueryCalls.append( "        query.setQuery" );
      setQueryCalls.append( parameters[i] );
      setQueryCalls.append( "( " );
      setQueryCalls.append( parameters[i] );
      setQueryCalls.append( " ); \n\n" ); 

      if (!null_flag)
        setQueryCalls.append( "      } \n\n" ); 

    }

    //     The query name.
    substitutions.put( "QUERYNAME", queryname ); 

    //     Use QUERY.queryname.DO
    substitutions.put( "TABLE_JNAME", jtablename ); 

    //     Use QUERY.queryname.UNIQUE
    String unique =  onequerycfg.getString( "UNIQUE", "FALSE" );

    if (unique.equalsIgnoreCase("false")) {
      substitutions.put( "REQUIRE_UNIQUE_INSTANCE", "" );
    } else {
      substitutions.put( "REQUIRE_UNIQUE_INSTANCE", "query.requireUniqueInstance();" );
    }

    //   The method parameters are built from QUERY.queryname.PARAMETERS[]
    // and QUERY.queryname.PARAMTYPES[] options.
    substitutions.put( "PARAMETERS", Util.expandStrings( "", ", ", "", paramdecls ) ); 

    //   The Javadoc "@params" are built from QUERY.queryname.PARAMETERS[]
    // option.
    substitutions.put( "JAVADOC", Util.expandStrings( "@param ", "", "\n", paramdecls ) ); 

    //   Process the "CUSTOM" type.  See Squirt docs for more details.
    if( "CUSTOM".equalsIgnoreCase( querytype ) ){ 

      StringBuffer where_clause = new StringBuffer();

      // get the SQL section for this query
      KeywordValueTable sqlcfg = onequerycfg.getSection( "SQL" ); 

      // SELECT & FROM fields currently ignored...

      // process each clause...
      int clause_nr = 1;
      KeywordValueTable clausecfg = null;
      Hashtable bwc_substitutions = new Hashtable();
      String where_code = "";
      String first_pass = "";
      int[] affected_parameters = null;

      while ((clausecfg = sqlcfg.getSection("WHERE.clause"+clause_nr)) != null) {

        // setup the operator, if any
        bwc_substitutions.put("OPERATOR", clausecfg.getString("OPERATOR", " "));

        // get the where clause code
        where_code = clausecfg.getString("CODE");

        // find which parameter(s) are affected by this code
        affected_parameters = findParameters(where_code, parameters);

        // set the null/empty parameter test as needed
        bwc_substitutions.put("NULL_TEST", getNullTest(parameters, null_allowed, affected_parameters));

        // set the base substitution for the clause code
        bwc_substitutions.put("CLAUSE_CODE", "sqlbuf.append(\""+where_code+"\");\n");

        // apply the base substitutions 
        first_pass = Util.applySubstitutions( bwc_substitutions, new String(where_clause_template) );

        for( i=0 ; i<affected_parameters.length ; i++ ) 
          bwc_substitutions.put( parameters[affected_parameters[i]], "\");\n      sqlbuf.append("+parameters[affected_parameters[i]]+");\n      sqlbuf.append(\"" );

        // re-apply subsitutions for the parameters and append to current where clause
        where_clause.append(Util.applySubstitutions( bwc_substitutions, first_pass));
    
        // move to the next clause
        clause_nr++;

      }
    
      substitutions.put( "BUILD_WHERE_CLAUSE", where_clause.toString() ); 

    } else {

      substitutions.remove( "BUILD_WHERE_CLAUSE" ); 

    }

    //   This is used in the "EASY" type.  For each parameter the following
    // code is generated:
    //   
    //   setQuery<parameter>(<parameter>);
    //
    //   Note that the <parameter> must match the field name in the
    // database.
    if( "EASY".equalsIgnoreCase( querytype ) ){ 
      substitutions.put( "SET_QUERY_PARAMETERS", setQueryCalls.toString() ); 
    }else{
      substitutions.remove( "SET_QUERY_PARAMETERS" ); 
    }

    //   This is used in the "EASY" type.  For each parameter the 
    // following code is generated:
    //
    //   addOrderBy<parameter>();
    if( "EASY".equalsIgnoreCase( querytype ) ){ 
      tmpStrArray = (String[]) onequerycfg.get( "ORDERBY", new String[0]); 
      substitutions.put( "SET_QUERY_ORDERBY", Util.expandStrings( "query.addOrderBy", "", "(); \n", tmpStrArray ) ); 
    }else{
      substitutions.remove( "SET_QUERY_ORDERBY" ); 
    }

  }


/**
 * Builds the 'if' statement test to for checking
 * the parameter(s) for null/empty.  If all parameters
 * are allowed to be null, "true" is returned to
 * make the null test ignored.
 *
 * @param parameters  The parameter names
 * @param null_allowed  Indicates if a parameter can be null/empty
 * @param affected_parameters  The list of parameters to check.
 * @return see comment.
 */
  public String getNullTest(String[] parameters, String[] null_allowed, int[] affected_parameters) {

    StringBuffer result = new StringBuffer();
    boolean first_statement = true;
    
    for (int i=0; i< affected_parameters.length; i++) {

      // nulls *not* allowed for this parameter?
      if (null_allowed.length == 0 || null_allowed[affected_parameters[i]].equalsIgnoreCase("EXCLUDE")) {

        // first time through?
        if (!first_statement) {

          // no, need to 'and' the tests...
          result.append(" && ");

        }

        result.append("ProdUtil.stringHasContent(");
        result.append(parameters[affected_parameters[i]]);
        result.append(")");

        first_statement = false;

      }

    }

    // anything to return?
    if (result.length() == 0) {

      // no, ensure the 'if' test is ignored
      return "true";

    } else {

      // yes, return what we've created
      return result.toString();

    }

  }


/**
 * Examines the code passed to see if any of the parameters
 * exist in it.  Any parameters found are returned in
 * an array.
 *
 * @param code  The code to examine for potential parameters
 * @param parameters  The parameters to look for
 * @return see comment.
 */
  public int[] findParameters(String code, String[] parameters) {

    int[] tmp_results = new int[parameters.length];
    int[] results = new int[0];
    int index = 0;

    // for each parameter...
    for (int i=0; i<parameters.length; i++) {

      // is it in the code?
      if (code.indexOf("<"+parameters[i]+">") >= 0) {

        // yes, keep this parameter
        tmp_results[index++] = i;

      }

    }

    // any parameters?
    if (index > 0) {

      results = new int[index];

      System.arraycopy(tmp_results, 0, results, 0, index);

    }
    
    return results;

  }


/**
 * Processes the configuration keyword value table to
 * generate the necessary stub files.
 */
  public void process() throws Exception {

    // Get the data from the config file
    KeywordValueFiles cfg = new KeywordValueFiles(application_config,false);

    // ensure all sections are read in now
    cfg.readAllSections(true);

    // keep the application config
    app_cfg = cfg;

    // DBG
    // System.err.println("Config toString: "+cfg.toString());

    // setup the substitution hashtable

    // current date
    substitutions.put(SUB_DATE, (DateFormat.getDateInstance()).format(new Date()).toString());

    // end setup of substitution hashtable

    KeywordValueTable querycfg = cfg.getSection(QUERY);

    if (querycfg == null) {

      System.err.println("Can't find the QUERY section!");
      System.exit(1);

    }

    // make the output directory
    Util.makeDirectory(output_dir);

    // Generate the stubs
    generateStubs(querycfg, output_dir);

    // The End

  }


/**
 * Generates the Query stubs and writes the code to a file
 * in the directory suppiled.
 *
 * @param querycfg  The QUERY Config section relating to this BO
 * @param output_dir  The destination base directory for the stubs
 * @exception Exception  If an unrecoverable error occurs.
 */
  private void generateStubs(KeywordValueTable querycfg, String output_dir) throws Exception {

    // get the keys for this config
    String[] keys = querycfg.keys();

    // go through these keys...
    for (int i=0; i< keys.length; i++) {

      // DBG
      // System.err.println("keys["+i+"] is: "+keys[i]);

      // get the object for this key
      String queryname = (String) keys[i] ; 

      // convert the object to string name
      String file_name = output_dir + File.separator + ((String) queryname) + ".squirt";

      // does this file already exist?
      if (overwrite || !Util.fileExists(file_name)) {

        // code generation time...
        System.out.println("Generating stub code for File <"+file_name+">");

        // DBG
        // System.err.println("QUERY subsection is: "+querycfg.toString());

        KeywordValueTable onequerycfg = querycfg.getSection( queryname ); 

        // Build query specific substitutions.
        setupSubstitutions( queryname, onequerycfg ); 

        // apply the necessary substitutions to a copy of the template
        String querytype ; // Type of query: custom, easy, objectid.
        String stubs_template ; 
        querytype = onequerycfg.getString( "TYPE" ); 
        if( "EASY".equalsIgnoreCase( querytype ) ){ 
          stubs_template = new String( easy_app_template ); 
        }else if( "CUSTOM".equalsIgnoreCase( querytype ) ){ 
          stubs_template = new String( custom_app_template ); 
        }else if( "OBJECTID".equalsIgnoreCase( querytype ) ){ 
          stubs_template = new String( objectid_app_template ); 
        }else{
          throw new Exception( "Unexpected value: QUERY." + queryname + ".TYPE = " + querytype ); 
        }
        String results = Util.applySubstitutions( substitutions, stubs_template );

        // write the template out to file
        Util.writeFile(file_name, results);

      } else {

        // move along, nothing to do here...
        System.out.println("File <"+file_name+"> already exists, skipping stub generation");

      }

    }

  }


}
