package com.pisoftware.platypus.util; 

import java.lang.*; 
import java.util.*; 

/** 
 * Class MainArgs processes the command line arguments passed to the main() method. 
 **/


/*
 *  Copyright (C) 1999 Plugged In Software Pty Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Plugged In Software
 *  PO Box 4130
 *  St. Lucia South Queensland
 *  Australia  4067
 *  info@PIsoftware.com
 */

public class MainArgs{ 

  /** Arguments as passed to main() **/
  String [] args ; 

  /** Minimum expected number of (non-option) arguments. **/
  int minExpected ; 

  /** Maximum expected number of (non-option) arguments. **/
  int maxExpected ; 

  /** List of expected options **/
  Hashtable allowedOptions ; 


  public MainArgs( String [] args ){ 
    this.args = args ; 
    minExpected = 0 ; 
    maxExpected = 1000000 ; // If this is too small, well, I didn't expect it!
    allowedOptions = new Hashtable(); 
  }


  /** Set the expected number of (non-option) arguments. **/
  public void setExpectedCount( int min, int max ){ 
    minExpected = min ; 
    maxExpected = max ; 
  }


  /** Set the expected number of (non-option) arguments. **/
  public void setExpectedCount( int expected ){ 
    minExpected = expected ; 
    maxExpected = expected ; 
  }


  /** Allow the indicated option. **/
  public void setOption( String option ){ 
    allowedOptions.put( option, "" ); 
  }


  /** Returns true if the indicated argument is an option. **/
  public boolean isOption( int argIndex ){ 
    if( args[argIndex].startsWith( "-" ) ) return true ; 
    return false ; 
  }


  /** Returns an argument. The first argument on the command line
      after the class name, excluding options, is argument index
      0. **/
  public String getPlainArg( int argIndex ){ 
    int i ; 
    i = -1 ; 
    do{
      i++ ; 
      while( isOption( i ) ) i++ ; 
      argIndex -- ; 
    }while( argIndex >= 0 ); 
    return args[i] ;
  }


  /** Returns true if the arguments contain the given option **/
  public boolean hasOption( String soughtOption ) throws Exception { 
    int i ; 
    if( null == allowedOptions.get( soughtOption ) ){ 
      throw new Exception( "Hey! Option " + soughtOption + " hasn't been explicitly allowed!" ); 
    }
    for( i=0 ; i<args.length ; i++ ){ 
      if( soughtOption.equals( args[i] ) ) return true ; 
    }
    return false ; 
  }


  /** Checks that the arguments fit the configuration given by the set() methods. **/
  public boolean valid(){
    int plainArgCount = 0 ; 
    int i ; 
    for( i=0 ; i<args.length ; i++ ){ 
      if( isOption( i ) ){ 
        if( null == allowedOptions.get( args[i] ) ) return false ;
      }else{
        plainArgCount ++ ; 
      }
    }
    if( plainArgCount > maxExpected ) return false ; 
    if( plainArgCount < minExpected ) return false ; 
    return true ; 
  }
           
}
