package com.pisoftware.platypus.util;

import java.io.*;
import java.util.*;
import java.net.*;
import java.text.*;
import com.lutris.util.*;
import com.lutris.appserver.server.ApplicationException;
import com.pisoftware.platypus.production.*; 
import com.pisoftware.quicksort.QuickSort; 


/**
 *  Author: <A HREF="MAILTO:cryan@plugged.net.au">Chris Ryan</A>
 *<P>
 * Company: Plugged In Software<BR>
 *          P.O. Box 4130<BR>
 *          St Lucia South<BR>
 *          Brisbane Queensland 4067<BR>
 *          Australia<BR> * <A HREF="http://www.plugged.net.au/">http://www.plugged.net.au</A><BR> 
 *<P>
 *    Date: 26 July 1999
 *<P>
 * Purpose: Provides various static convenience methods for use
 *          throughout the platypus packages.
 *
 */

/*
 *  Copyright (C) 1999 Plugged In Software Pty Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Plugged In Software
 *  PO Box 4130
 *  St. Lucia South Queensland
 *  Australia  4067
 *  info@PIsoftware.com
 */

public class Util {



/**
 * Makes the directory if it doesn't already exist
 *
 * @param dir_name  The directory to create
 * @exception Exception  If an unrecoverable error occurs.
 */
  static public void makeDirectory(String dir_name) throws Exception {

    File dirFile = new File(dir_name);

    dirFile.mkdirs();

  }


/**
 * Determines if a file exists.
 *
 * @param file_name  The file name, with path, to check
 * @return True if file exists, false otherwise
 * @exception Exception  If an unrecoverable error occurs.
 */
  static public boolean fileExists(String file_name) throws Exception {

    File file = new File(file_name);

    return file.exists();

  }


/**
 * Word Initial Capital (WIC) on a string.  The
 * first character in each word is capitalised.  It
 * is assumed the underscores & white space are
 * word separators.  For underscores, the character
 * is also removed from the string.
 *
 * @param data_str  The string to process
 * @return see comment.
 */
  static public String toWIC(String data_str) {

    // ensure no nulls
    if (data_str == null)
      return null;

    StringBuffer result = new StringBuffer(data_str.length());

    // flag to indicate if next character is to be uppercase
    boolean upper_flag = true;

    for (int i=0; i < data_str.length(); i++) {

      char ch = data_str.charAt(i);
      boolean whitespace = Character.isWhitespace(ch);
      boolean underscore = (ch == '_');

      // is this character whitespace or underscore?
      if (whitespace || underscore) {

        // ensure next character is converted to uppercase
        upper_flag = true;

        // don't add the underscore character to the results
        if (!underscore) {

          result.append(ch);

        }

      } else {

        // do we have to capitalise this character?
        if (upper_flag) {

          // yes
          result.append(Character.toUpperCase(ch));

          // reset the uppercase flag
          upper_flag = false;

        } else {

          // keep as normal
          result.append(ch);

        }

      }

    }

    // return our result
    return result.toString();

  }


/**
 *
 * Reads the contents of the file, which is then returned as a String
 *
 * @param infile                   The file to read
 * @return see comment.
 */
  static public String readFile(String infile) throws IOException, FileNotFoundException {

    // Reader for the file
    FileReader reader = new FileReader(infile);

    // get the contents of the reader
    String filebuffer = getContents(reader);

    // close the stream
    reader.close();

    // return the contents
    return filebuffer;

  } // readFile


private static final int MAX_BUFFER_SIZE = 60000;

/**
 * Reads the contents of the reader, which is then returned as a String
 *
 * @param reader  The Reader to read from.
 * @return see comment.
 */
  static public String getContents(Reader reader) throws IOException {

    // buffer for I/O
    char[] inbuffer = new char[MAX_BUFFER_SIZE];

    // String buffer for the contents
    StringBuffer stringbuffer = new StringBuffer();

    // counter of bytes read
    int nr_read = 0;

    // while there's more to read...
    while ((nr_read = reader.read(inbuffer,0,MAX_BUFFER_SIZE)) != -1) {

      // DBG
      //System.out.println("Read from file: "+nr_read+" bytes");

      // add the bytes read to the buffer
      stringbuffer.append(inbuffer,0,nr_read);

    }

    // DBG
    //System.out.println("String length: "+stringbuffer.length()+" bytes");

    // return the contents
    return stringbuffer.toString();

  } // getContents


/**
 *
 * Writes the contents to the file
 *
 * @param outfile  The file to write to
 * @param content  The content to write out
 */
  static public void writeFile(String outfile, String content) throws IOException {

    // Writer for the file
    FileWriter writer = new FileWriter(outfile);

    // write the contents to the file
    writer.write(content);

    writer.write("\n");

    // close the stream
    writer.close();

  } // writeFile



  /** 
   * Expands an array of strings into a single string. 
   * 
   * @param before, Prefix to each array element
   * @param sep, Goes between suffix of previous element, and prefix of next.
   * @param after, Suffix to each array element
   * @param strings, Array of strings to be expanded
   * @param startFrom Index of first element to use
   **/
  // Maybe this functionality is offered by some standard Java method. 
  static public String expandStrings( String before, String sep, String after, String [] strings, int startFrom ){ 
    String fullString ; 
    int i ; 
    if( 0 < strings.length ){ 
      fullString = before + strings[startFrom] + after ; 
      for( i=startFrom+1 ; i<strings.length ; i++ ) fullString += sep + before + strings[i] + after ;
    }else{ 
      fullString = "" ; 
    }
    return fullString ; 
  }


  /** 
   * Parameter <CODE>startFrom</CODE> defaults to <CODE>0</CODE>
   **/
  static public String expandStrings( String before, String sep, String after, String [] strings ){
    return expandStrings( before, sep, after, strings, 0 ); 
  }


  /** 
   * Expands an array of strings into a single string. 
   * 
   * @param before, Prefix to each array element
   * @param strings, Array of strings to be expanded
   * @param after, Suffix to each array element
   **/
  // Maybe this functionality is offered by some standard Java method. 
  static public String expandStrings( String before, String [] strings, String after ){ 
    return expandStrings( before, "", after, strings, 0 ); 
  }


  /** 
   * Takes a list of items in a string, and returns an array of the items. 
   * 
   * @param listString, Contains list of items
   * @param sep, character which separates items in listString
   **/
  // Maybe this functionality is offered by some standard Java method. 
  static public String [] getStringList( String listString, char sep ){
    Vector strings ; 
    String [] stringArray ; 
    int fieldStart, afterField ; // Bounds of a field in listString. 
    strings = new Vector(); 
    fieldStart = 0 ; 
    afterField = 0 ; 
    while( (listString.length() > 0) && (afterField >= 0) ){ 
      afterField = listString.indexOf( sep, fieldStart ); 
      if( afterField >= 0 ){ 
        strings.addElement( listString.substring( fieldStart, afterField ) ); 
      }else{ 
        strings.addElement( listString.substring( fieldStart ) ); 
      }
      fieldStart = afterField + 1 ; 
    }
    stringArray = new String[strings.size()] ; 
    strings.copyInto( stringArray ) ; 
    return stringArray ; 
  }


  /** 
   * Returns true if the string is in the list. An exact match is required. 
   **/
  static public boolean stringInList( String str, String [] list ){ 
    int i ; 
    if( null == list ) return false ; 
    for( i=0 ; i<list.length ; i++ ){ 
      if( list[i].equals( str ) ) return true ; 
    }
    return false ; 
  }


/**
 * Applies the substitutions to the template passed
 *
 * @param subs  Maps substitution keys to their values.
 * @param template  The template to process
 * @return The template modified by subs
 * @exception Exception  If an unrecoverable error occurs.
 */
 static public String applySubstitutions( Hashtable subs, String template) {

    StringBuffer results = new StringBuffer();

    // tokenize the template into lines
    StringTokenizer strtok = new StringTokenizer(template, "\n", true);

    // process each line...
    while (strtok.hasMoreTokens()) {

      // get this line
      String current_line = strtok.nextToken();

      // current line length done once per token for efficiency
      int current_line_length = current_line.length();

      // position/search indicators to find tags
      int search_start = 0;
      int ltpos = -1;
      int gtpos = -1;
      int testltpos = -1;

      // find potential start(s) of tags requiring substitution
      while ((ltpos = current_line.indexOf('<', search_start)) != -1) {

        // get the end of the tag
        gtpos = current_line.indexOf('>', ltpos);

        // any end of tag?
        if (gtpos == -1) {

          // no, get out of here...
          break;

        }

        // get start of 'next' tag
        testltpos = current_line.indexOf('<', ltpos+1);

        // is it before the end of the first tag?  (May indicate < operator being used)
        if (testltpos < gtpos && testltpos > 0) {

          results.append(current_line.substring(search_start,testltpos));

          // reset the search start and try again
          search_start = testltpos;
          continue;

        }

        // append the stuff before tag to the results
        results.append(current_line.substring(search_start, ltpos));

        // get the tag name found
        String tag_name = current_line.substring(ltpos+1,gtpos);

        // is the tag one we've got a substitution for?
        if (subs.containsKey(tag_name)) {

          // yes, put in the substitution value into the results
          results.append((String) subs.get(tag_name));

        } else {

          // not a recognised tag, just put it into the results
          results.append(current_line.substring(ltpos,gtpos+1));

        }

        // move the next search to the correct position
        search_start = gtpos + 1;

        // ensure this doesn't cause a length problem
        if (search_start > current_line_length) {

          // no more to process
          search_start = -1;

          // get out of here
          break;

        }

      }

      // append whatever's left of the current line to the results
      if (search_start != -1)
        results.append(current_line.substring(search_start));

    }

    // return the results generated
    return results.toString();

  }


  /** 
   * Returns a KeywordValueTable section name. Specifically, given the
   * full name of section within a PO, this method returns the section
   * name of the PO.
   *
   *<BR> 
   *
   * Note: in pathological cases, this method will return the
   * wrong name. Specifically, if the firstSubsectionName string occurs
   * earlier in the name, a truncated name may be returned.
   * @param fullSectionName Full name of subsection
   * @param name of the subsection in the PO section
   **/
  static public String getPOSectionName( String fullSectionName, String firstSubsectionName ) throws Exception { 
    int offset ; 
    offset = fullSectionName.indexOf( "." + firstSubsectionName ); 
    if( offset<0 ){
      throw new Exception( "Subsection "+ firstSubsectionName + " not found in "+ fullSectionName  ); 
    }
    return fullSectionName.substring( 0, offset ); 
  }


  /** 
   * Returns an array of argument names for the database
   * <CODE>add</CODE> and <CODE>update</CODE> methods provided by the
   * business object. If its an update method, the database index is
   * the first argument in the array. All arguments are assumed to be
   * of type <CODE>String</CODE> (As at 5Aug99. This may change in the
   * future).
   * @param devData Configuration info for entire application
   * @param commitData Configuration info for this add/update.
   * @return Array of argument names, as described above.
   * @exception Exception
   **/
  // TODOONEDAY: Actually, there are several pieces of data this could
  // return. It could be usefully turned into a class. Such a class
  // could also have methods returning the DBIndex, and whether its a
  // add/update.
  static public String [] commitArgs( KeywordValueTable devData, KeywordValueTable commitData ) throws Exception { 
    String [] formList ; // Section names (in root) of FORMS
    Hashtable nameSet ; // Keys are list of INPUT NAMES in these FORMS. Duplicates are removed.
    String dbIndex ; // Value of DBINDEX in commitData.
    String [] sortedNames ; // Same as namesSet, but sorted, and DBINDEX removed.
    //// Used in FORM loop
    KeywordValueTable elementsData ; 
    String poSectionName ; // Name of section of PO containing FORM.
    String [] formNames ; // NAMES of INPUTS in FORM
    //// Temporary variables
    String tmpString ; 
    String [] tmpNames ; 
    int iForm, iName ; 

    //////  Check sanity of arguments (ie, not transposed!)
    if( ! devData.containsKey( "PO" ) ){ 
      if( commitData.containsKey( "PO" ) ){ 
        throw new Exception( "Arguments reversed? commitData has PO section." ); 
      }else{ 
        throw new Exception( "devData has no PO section." ); 
      }
    }

    //////  Get list of FORMS
    tmpString = commitData.getString( "FORMS" ); 
    formList = Util.getStringList( tmpString, ',' ); 

    //////  Get NAMES of INPUTS in FORMS
    nameSet = new Hashtable(); 
    for( iForm=0 ; iForm<formList.length ; iForm++ ){ 
      tmpString = formList[iForm] + ".ELEMENTS" ; 
      //// Ensure FORM's PO is read into devData
      if( devData instanceof KeywordValueFiles ){ 
        poSectionName = getPOSectionName( tmpString, "FORMS" ); 
        ((KeywordValueFiles)devData).readSection( poSectionName ); 
      }
      //// Get section containing elements of FORM
      elementsData = devData.getSection( tmpString ); 
      if( null == elementsData ){ 
        System.err.println( "ERROR: In COMMIT, section " + tmpString + " is referenced but not found." ); 
        continue ; // Just skip this FORM
      }
      //// Copy the FORMS' INPUTS' NAMES into formNames
      formNames = elementsData.keys(); 
      for( iName=0 ; iName<formNames.length ; iName++ ){ 
        nameSet.put( formNames[iName], tmpString ); 
      }
    }

    //////  Exclude names and DB index, add extras, as appropriate
    //// Exclude DB Index
    dbIndex = commitData.getString( "DBINDEX", null ); 
    if( null != dbIndex ){
      if( ! nameSet.containsKey( dbIndex ) ){ 
        System.err.println( "WARNING from GenUtil.commitArgs(): DB index " + dbIndex + " is not in FORM arguments" ); 
      }
      nameSet.remove( dbIndex ); 
    }
    //// Explicit exclusions
    tmpString = commitData.getString( "EXCLUDE", "" ); 
    tmpNames = Util.getStringList( tmpString, ',' ); 
    for( iName=0 ; iName<tmpNames.length ; iName++ ) nameSet.remove( tmpNames[iName] ); 
    //// Explicit inclusions
    tmpString = commitData.getString( "EXTRA", "" ); 
    tmpNames = Util.getStringList( tmpString, ',' ); 
    for( iName=0 ; iName<tmpNames.length ; iName++ ) nameSet.put( tmpNames[iName], "EXTRA" ); 

    //////  Sort names (except DB index)
    sortedNames = Util.sortStrings( nameSet.keys() ); 

    //////  Build return array, Tidy up and leave
    if( null == dbIndex ){ 
      return sortedNames ; 
    }else{
      // Must prepend DBINDEX
      String [] tmp ; 
      tmp = new String[sortedNames.length+1] ; 
      tmp[0] = dbIndex ;
      System.arraycopy( sortedNames, 0, tmp, 1, sortedNames.length ); 
      return tmp ; 
    }
  }


  /** 
   * Finds all ID sections containing the given name value pair.
   * NOTE: Defaults specified by a REFERS value are
   * included. 
   *
   * <BR>The value test is case insensitive. The name test is case
   * sensitive.
   * @param confData
   * @param soughtName
   * @param soughtValue
   * @return array of all the (fully qualified) names of the found sections
   **/
  // TODO: This method needs two utility methods, and passes a lot of
  // state between them. It's ripe for breaking out into a class of
  // its own.
  // TODOONEDAY: get rid of -- "Chris doesn't want this :-( " code
  public static String [] getPOSectionNamesMatching( KeywordValueTable confData, String soughtName, String soughtValue ) throws KeywordValueException, ApplicationException { 
    KeywordValueTable poData ; // PO subsection in outer section
    // Chris doesn't want this :-( KeywordValueTable idsData ; // IDS subsection in outer section
    Vector sectionNameVec ; 
    String [] sectionNameArray ; 

    //////  Initialise
    sectionNameVec = new Vector(); 
    poData = confData.getSection( "PO" ); 
    // Chris doesn't want this :-( idsData = confData.getSection( "IDS" ); 

    //////  Find all IDS sections
    // Chris doesn't want this :-( getPOSNMIDS( sectionNameVec, "IDS.", confData, idsData, soughtName, soughtValue ); 
    getPOSNM( sectionNameVec, "PO.", confData, poData, soughtName, soughtValue ); 

    //////  Tidy up and leave.
    sectionNameArray = new String[ sectionNameVec.size() ]; 
    sectionNameVec.copyInto( sectionNameArray ); 
    return sectionNameArray ; 
  }


  /** 
   * Tests whether a (qualified) name represents a directory in hte
   * presentation layer, or a presentation object. Should return
   * FALSE for "PO.Welcome" and "PO.reports.Sales". Should return
   * TRUE for "PO.reports".
   * @return as described above.
   * @throws KeywordValueException
   * @throws ApplicationException
   **/
  public static boolean isNameDirectory( KeywordValueTable confData, String name )
    throws KeywordValueException, ApplicationException { 
    Object kvtObject ; 
    kvtObject = confData.get( "CORE." + name ); 
    if( kvtObject instanceof KeywordValueTable ){
      return true ; 
    }else if( kvtObject instanceof String ){
      return false ; 
    }else{
      throw new ApplicationException( "Unknown class of object in CORE." + name ); 
    }
  }


  /** 
   * Utility method for getPOSectionNamesMatching() 
   * @param poData If null, returns immediately.
   * @see getPOSectionNamesMatching
   **/
  public static void getPOSNM( Vector sectionNames, String prefix, KeywordValueTable confData, KeywordValueTable poData, String soughtName, String soughtValue )
    throws KeywordValueException, ApplicationException { 
    String [] keys ; 
    // OLD CODE String [] directories ; 
    KeywordValueTable directoryData ; 
    KeywordValueTable idsData ; 
    // Temporary variables
    String tmpStr ; 
    int i ; 
    String name ; 

    //////  Initialise
    if( null == poData ) return ; 
    keys = poData.keys(); 
    // OLD CODE tmpStr = poData.getString( "DIRECTORIES", "" ); 
    // OLD CODE directories = Util.getStringList( tmpStr, ',' );

    //////  Loop over entries: POs and directories
    for( i=0 ; i<keys.length ; i++ ){ 
      name = keys[i] ; 
      // OLD CODE if( "DIRECTORIES".equals( name ) ){ 
        // Skip this element
      // OLD CODE }else if( Util.stringInList( name, directories ) ){ 
      if( isNameDirectory( confData, prefix + name ) ){
        // Process directory
        directoryData = poData.getSection( name ); 
        getPOSNM( sectionNames, prefix + name + '.', confData, directoryData, soughtName, soughtValue ); 
      }else{ 
        // Process PO. 
        idsData = poData.getSection( name + ".IDS" ); 
        getPOSNMIDS( sectionNames, prefix + name + ".IDS.", confData, idsData, soughtName, soughtValue ); 
      }
    }
  }


  /** 
   * Utility method for getPOSNM() 
   * @param idsData If null, returns immediately.
   * @see getPOSNM
   **/
  public static void getPOSNMIDS( Vector sectionNames, String prefix, KeywordValueTable confData, KeywordValueTable idsData, String soughtName, String soughtValue )
    throws KeywordValueException { 
    String [] keys ; 
    KeywordValueTable idData ; 
    // Temporary variables
    String tmpStr ; 
    int i ; 
    String idName ; 
    String foundValue ; 

    //////  Initialise
    if( null == idsData ) return ; 
    keys = idsData.keys(); 

    //////  Loop over ID entries
    for( i=0 ; i<keys.length ; i++ ){ 
      idName = keys[i] ; 
      idData = KeywordValueFiles.getFilledSection( confData, idsData, idName ); 
      if( idData.containsKey( soughtName ) ){ 
        foundValue = idData.getString( soughtName, null ); 
        if( (null != foundValue) && (foundValue.equalsIgnoreCase( soughtValue )) ){ 
          sectionNames.addElement( prefix + idName ); 
          // DBG 
          ProdUtil.log( "Section " + prefix + idName + " contains NV pair '" + soughtName + "' = '" + foundValue + "'" ); 
        }
      }
    }
  }


  /** 
   * Sorts a list of strings.
   * @return String array, in alphabetic order
   **/
  static public String [] sortStrings( Vector vec ){ 
    String [] sortedStrings ; 
    QuickSort.quickSort( vec, QuickSort.STRING ); 
    sortedStrings = new String[ vec.size() ] ; 
    vec.copyInto( sortedStrings ); 
    return sortedStrings ; 
  }


  /** 
   * Sorts a list of strings.
   **/
  static public String [] sortStrings( Enumeration before ){ 
    Vector vec = new Vector(); 
    while( before.hasMoreElements() ){ 
      vec.addElement( before.nextElement() ); 
    }
    return sortStrings( vec ); 
  }


  /** 
   * Sorts a list of strings.
   **/
  static public String [] sortStrings( String [] before ){ 
    Vector vec = new Vector(); 
    int i ; 
    for( i=0 ; i<before.length ; i++ ){ 
      vec.addElement( before[i] ); 
    }
    return sortStrings( vec ); 
  }
}
