#-----------------------------------------------------------------------------
# Enhydra Java Application Server
# Copyright 1997-1999 Lutris Technologies, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in
#    the documentation and/or other materials provided with the distribution.
# 3. All advertising materials mentioning features or use of this software
#    must display the following acknowledgement:
#      This product includes Enhydra software developed by Lutris
#      Technologies, Inc. and its contributors.
# 4. Neither the name of Lutris Technologies nor the names of its contributors
#    may be used to endorse or promote products derived from this software
#    without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#-----------------------------------------------------------------------------
# $Id: stdrules.mk,v 1.26 1999/10/07 05:23:11 guy Exp $
#-----------------------------------------------------------------------------
#
# stdrules.mk
#  Standard rules for use for Enhydra server applications.  It contains rules
#  for compiling generic Java modules and packages.  This requires GNU make,
#  as conditionals are used.  This is normally included from a top-level file,
#  config.mk, which is included by each of the make files.
#
#
#  The first section of this file defines variables that contain the
#  locations of various devsys components.  They default to standard values.
#  to overide these values, set the macros after *BEFORE* this file has been
#  included.  This file explictly checks with conditionals; this works around
#  certain definition order problems. The variables are:
#
#    o JDKDIR - Location of Sun Java Development Kit (JDK).
#    o JDKVERSION - Major version of the JDK.  Currently this should be
#	either "1.1" or "1.2".
#
#    o WAIDIR -  Directory containing the Netscape WAI classes.
#      Only required if you using WAI connectivity to a Netscape server.
#
#    o JAVACCDIR - Location of Sun Javacc Kit.
#
#    o SWINGDIR -  Directory containing the JFC (Java Foundation
#		   Classes) otherwise known as "swing".
#
#    o JDAPDIR -  Directory containing the JDAP classes.
#
#    o XTRA_CLASSPATH -- Extra elements to add to the front of the class path
#      If defined, this must be a legal elements to insert infront of a
#      CLASSPATH environment.  This must include a trailing separator as it
#      will be prepended to the standard CLASSPATH.  Note that the separator
#      is `:' on Unix and `\;' on Windows.  See the setting of CLASSPATH
#      in this file for examples of using make conditionals to detect Windows
#      or Unix.
#
#
#  Individual Makefiles define the following variables to use the targets in
#  this Makefile:
#
#    o ROOT -- The relative path to the directory containing the config.mk
#      file.  For example, "../..".  This is required.
#
#    o CLASSES -- The classes, if any, to compile for the current subdirectory.
#      Do not use the .java or .class extension.
#
#    o JOLT_CLASSES -- The jolt POs, if any, to compile for the current
#      subdirectory. Do not include the .jhtml extension.
#      NOTE:  make clean will remove any generate .java files with the same
#      base name.  This is necessary to clean up any files left around
#      after running joltc -k or killing joltc before it completes.  These
#      files will be seen make make and converted to
#
#    o RMI_CLASSES -- The remote object implementation classes, if any, to run
#      the RMI compiler on for the current subdirectory.  Do not use the .java
#      or .class extension.
#
#    o SUBDIRS -- Subdirectories, if any, to recursively build.
#
#    o PACKAGEDIR -- If CLASSES is not empty, the directory, relative to the
#      install root, where the resulting class files will be written by javac.
#      This is needed in order to allow "make" to check dependencies.
#
#    o PRESENTATION_OBJECTS -- [Deprecated] List of presentation
#      objects, each of which results in a xxxxHTML DOM class and a
#      xxxx manipulation class being generated. See PO_UNUSED &
#      friends for the variables now used for this purpose.
#
#    o PO_UNUSED, PO_DOM, PO_CONF, PO_JAVA, PO_CLASS, PO_DONE --
#      Between them, these variables list all the presentation objects
#      (POs) in the directory. The individual variables here control
#      how far the Makefiles take each PO through the development
#      process. PO_UNUSED: Makefiles do nothing - development has not
#      yet begun on this PO. PO_DOM: Makefiles create the xxxxHTML DOM
#      class. PO_CONF: Makefiles create (or update) the POs config
#      file. PO_JAVA: Makefiles generate skeleton Java in the view
#      directory, and copy it into the src directory *if* there isn't
#      already one there. PO_CLASS: Makefiles compile the Java, ready
#      to run. PO_DONE: Makefiles do nothing -- development is
#      complete for this PO. Note: each stage subsumes it predecessors
#      -- for example, PO_JAVA will also compile the DOM class and
#      generate config files if these are out of date.
#
#    o HTML_CLASSES - List of classes to be generated from HTML using XMLC.
#      These classes *must* be named in the form xxxxHTML and be generated
#      from a HTML file named xxxx.html in HTML_DIR.  The names in this 
#      variable must not include the .class extension.
#
#    o HTML_DIR - Directory containing HTML files to compile with XMLC.  It
#      should be relative to the current directory or $(ROOT).
#
#    o HTML_XMLC_OPTS_FILE - Name of .xmlc options file to use when compiling
#      HTML files with XMLC.  Maybe left unspecified or empty.
#
#    o XMLC_HTML_OPTS - Options to pass to XMLC when compiling HTML files.
#      Maybe left unspecified or empty.
#
#    o M4_FILES - List of m4 files which create prototype html
#      files. (No longer used? 7Oct99)
#
#    o GEN_DIR - Directory for generated "source" code: $(ROOT)/$(APP_DIR)
#
#    o SRC_DIR - Directory containing source code: $(ROOT)/src
#
#    o VIEW_DIR - Directory containing generated files used in
#      development: $(ROOT)/view
#
#    o CLASS_DIR - Directory containing class files: $(ROOT)/classes/$(APP_DIR)
#
#    o 
#
#    o APP_NAME - Java package name of this application. Eg, timesheet.
#
#    o APP_DIR - Directory containing Java classes for this
#      applciation. (This is the same as APP_NAME for applications,
#      but is necessary for compiling Platypus.)
#
#
##### 

# TODOONEDAY: JAR_INSTALL and COPY_TO_CLASS are very similar. Can they
# be merged? [LATER, GMS] Hmm, Lutris's JAR_INSTALL copies things if
# they're different to what's already there. My COPY_TO_CLASS just
# uses Make's time dependencies. Lutris's way has the advantage that
# it will copy older files over (you may want to do this if you move
# an entire images directory into place, or you use soft links in your
# source directory). My way has the advantage of being quicker.


#----------------------------------------------------------------------------- 
# Copyright (C) 1997 Lutris Technologies, Inc.            All Rights Reserved.
#----------------------------------------------------------------------------- 
# $Id: stdrules.mk,v 1.26 1999/10/07 05:23:11 guy Exp $
#----------------------------------------------------------------------------- 


#----------------------------------------------------------------------------- 
# The full path to the MSDOS shell. This is necessary on windows systems
# because some java utilities (e.g. javacc) are launched by .bat files.
#
ifeq ($(OS),Windows_NT)
    MSDOS = cmd
else
    MSDOS = /windows/command.com /E:4096
endif



#-----------------------------------------------------------------------------
# Default directories for various components.  Many are relative to
# $(ENHYDRA_DIR), which should be set in the file that includes this.
#----------------------------------------------------------------------------- 


#----------------------------------------------------------------------------- 
# The full path to the Enhydra directory and classes.  ENHYDRA_DIR is
# normally set by the config.mk file that includes this file.
#
ifeq ($(ENHYDRA_DIR),)
    ENHYDRA_DIR = /usr/local/enhydra
endif
ifeq ($(ENHYDRA_CLASSES),)
    ENHYDRA_CLASSES = $(ENHYDRA_DIR)/lib/enhydra.jar
endif

#
# Jolt HTML compiler.
#
ifeq ($(JOLTC),)
    JOLTC = $(ENHYDRA_DIR)/bin/joltc 
endif

#
# XMLC compiler.
#
ifeq ($(XMLC),)
    XMLC = $(ENHYDRA_DIR)/bin/xmlc 
endif

#
# Command to run XMLC.  Need to append options and input file.
#
ifeq ($(XMLC_CMD),)
   XMLC_CMD = @mkdir -p $(CLASS_OUTPUT); CLASSPATH=$(CLASSPATH) ; PATH=$(PATH) ; export CLASSPATH PATH ; set -x ; $(XMLC) -d $(CLASS_OUTPUT)
endif

#
# Command to run ConfigGenerator.  Needs some arguments appended. 
#
ifeq ($(CONG_CMD),)
   CONG_CMD = CLASSPATH=$(CLASSPATH) ; PATH=$(PATH) ; export CLASSPATH PATH ; set -x ; $(PLATYPUSDIR)/bin/cong
endif

#
# Command to run diff & patch.  Needs some arguments appended. 
#
ifeq ($(DIFFPATCH_CMD),)
   DIFFPATCH_CMD = PATH=$(PATH) ; export PATH ; set -x ; $(PLATYPUSDIR)/bin/diffpatch
endif

#
# Command to run Pond.  Needs some arguments appended. 
#
ifeq ($(POND_CMD),)
   POND_CMD = CLASSPATH=$(CLASSPATH) ; PATH=$(PATH) ; export CLASSPATH PATH ; set -x ; $(PLATYPUSDIR)/bin/pond
endif

#----------------------------------------------------------------------------- 
# The directory containing the Sun Java Development Kit (JDK) and
# various components.  JDKARCH is need only if native methods are being
# compiled.  This is that name used as for platform-specific directories in
# JDK. JDKVERSION is the major version number which is currently wither
# 1.1 or 1.2.
#
ifeq ($(JDKDIR),)
    ifneq ($(OSTYPE),cygwin32)
        JDKDIR  = /usr/local/java
    else
        JDKDIR  = /usr/java1.2
    endif
endif
ifeq ($(JDKARCH),)
    ifneq ($(OSTYPE),cygwin32)
        JDKARCH = solaris
    else
        JDKARCH = win
    endif
endif
ifeq ($(JDKVERSION),)
    JDKVERSION = 1.2
endif

#
# The full path to the JDK devsys classes archive
#
ifeq ($(JDK_CLASSES),)
    JDK_CLASSES=$(JDKDIR)/lib/classes.zip
endif

#
# Directory that contains the JDK programs.
#
ifeq ($(JDKBIN),)
    JDKBIN = $(JDKDIR)/bin
endif
PATH = $(JDKBIN):$(shell echo $${PATH})

#
# Java compiler command.
#
ifeq ($(JAVAC),)
    JAVAC = $(JDKBIN)/javac
endif

#
# Compile-time flags for the Java compiler.
#
ifeq ($(JCFLAGS),)
    JCFLAGS = -g
endif

#
#  Java native method header generator command.
#
ifeq ($(JAVAH),)
    JAVAH = $(JDKBIN)/javah
endif

#
# JavaDoc builder command.
#
ifeq ($(JAVADOC),)
    JAVADOC = $(JDKBIN)/javadoc
endif

#
# Memory argument to pass to the javadoc command.
#
ifeq ($(JDOCMEM),)
    JDOCMEM=-J-mx32m
endif

#
# The name of the RMI compiler.
#
ifeq ($(RMIC),)
    RMIC = $(JDKBIN)/rmic
endif

#
# jar command.
#
ifeq ($(JAR),)
    JAR = $(JDKBIN)/jar
endif

#
# The name of the java interpreter.
#
ifeq ($(JAVA),)
    JAVA = $(JDKBIN)/java
endif
ifeq ($(JOPTS),)
    JOPTS =
endif

#----------------------------------------------------------------------------- 
# The full path to the JFC library, swing.
#
ifeq ($(SWINGDIR),)
    ifneq ($(OSTYPE),cygwin32)
	SWINGDIR = /usr/local/swing
    else
	SWINGDIR = /usr/local/swing
    endif
endif
ifeq ($(SWING_CLASSES),)
    ifneq ($(OSTYPE),cygwin32)
        SWING_CLASSES = $(SWINGDIR)/swingall.jar
    else
        SWING_CLASSES = $(SWINGDIR)/swingall.jar
    endif
endif

#----------------------------------------------------------------------------- 
# The full path to the Java LDAP library, jdap.
#
ifeq ($(JDAPDIR),)
    ifneq ($(OSTYPE),cygwin32)
	JDAPDIR = /usr/local/jdap1.0
    else
	JDAPDIR = /usr/local/jdap1.0
    endif
endif
ifeq ($(JDAP_CLASSES),)
    ifneq ($(OSTYPE),cygwin32)
        JDAP_CLASSES = $(JDAPDIR)/ldapjdk.jar:$(JDAPDIR)/ldapfilt.jar:$(JDAPDIR)/vjscdk.jar
    else
        JDAP_CLASSES = $(JDAPDIR)/ldapjdk.jar\;$(JDAPDIR)/ldapfilt.jar\;$(JDAPDIR)/vjscdk.jar
    endif
endif

#----------------------------------------------------------------------------- 
# The directory containing the Sun Javacc kit.
#
ifeq ($(JAVACCDIR),)
    ifneq ($(OSTYPE),cygwin32)
        JAVACCDIR  = /usr/local/javacc
    else
        JAVACCDIR  = /usr/local/javacc
    endif
endif
ifeq ($(JAVACC_CLASSES),)
    ifneq ($(OSTYPE),cygwin32)
        JAVACC_CLASSES = $(JAVACCDIR)/JavaCC.zip
    else
        JAVACC_CLASSES = $(JAVACCDIR)/JavaCC.zip
    endif
endif
ifeq ($(JJTREE),)
    ifneq ($(OSTYPE),cygwin32)
	JJTREE = $(JAVACCDIR)/bin/jjtree
    else
	#
        # This seems to break things, and is not like any of the other
        # rules. So I changed it to match the others. If this breaks the
        # NT build, switch back to this line.
        # JJTREE = $(MSDOS) /c "\usr\local\javacc0.8.2\bin\jjtree.bat"
        # 
	JJTREE = $(JAVACCDIR)/bin/jjtree
    endif
endif
ifeq ($(JAVACC),)
    ifneq ($(OSTYPE),cygwin32)
	JAVACC = $(JAVACCDIR)/bin/javacc
    else
	#
        # This seems to break things, and is not like any of the other
        # rules. So I changed it to match the others. If this breaks the
        # NT build, switch back to this line.
	# JAVACC = $(MSDOS) /c "\usr\local\javacc0.8.2\bin\javacc.bat"
	#
        JAVACC = $(JAVACCDIR)/bin/javacc
    endif
endif


#----------------------------------------------------------------------------- 
# The full path to the directory containing the Netscape WAI classes
# (only required if you using WAI connectivity to a Netscape server).
#
ifeq ($(WAIDIR),)
    ifneq ($(OSTYPE),cygwin32)
        WAIDIR = /usr/local/nshome
    else
        WAIDIR = /usr/local/nshome
    endif
endif
ifeq ($(WAI_CLASSES),)
    ifneq ($(OSTYPE),cygwin32)
        WAI_CLASSES = $(WAIDIR)/nisb.zip:$(WAIDIR)/WAI.zip
    else
        WAI_CLASSES = $(WAIDIR)/nisb.zip\;$(WAIDIR)/WAI.zip
    endif
endif


#-----------------------------------------------------------------------------
# Definitions of components of the source tree and destination tree.
#----------------------------------------------------------------------------- 

#
# The directory that contains all generated files.  This is an image of
# what gets installed on a production system.
#
ifeq ($(OUTPUT),)
    OUTPUT = $(ROOT)/output
endif

#
# The directory into which compiled Java classes are to be placed.
# The java compiler automatically arranges the classes correctly
# below this directory according to package name.
#
# GMS 25Jul99: This contains the root of the classes, as should be put
# on CLASSPATH, etc.
#
ifeq ($(CLASS_OUTPUT),)
    CLASS_OUTPUT = $(ROOT)/classes
endif

#
# Where scripts command files go.
#
ifeq ($(BIN_OUTPUT),)
    BIN_OUTPUT = $(OUTPUT)/bin
endif

#
# Where jar/zip files go.
#
ifeq ($(LIB_OUTPUT),)
    LIB_OUTPUT = $(OUTPUT)/lib
endif

#
# Where Java Native Interface DLL's are placed.
#
ifeq ($(NATIVE_OUTPUT),)
    NATIVE_OUTPUT = $(OUTPUT)/lib
endif

#
# Where Java Native Interface generated include files go.
#
ifeq ($(INCLUDE_OUTPUT),)
    INCLUDE_OUTPUT = $(OUTPUT)/include
endif

#  
# Where Java Native Interface include files are placed.
#  
ifeq ($(JNI_INCLUDES),)
    JNI_INCLUDES =  -I$(JDKDIR)/include -I$(JDKDIR)/include/$(JDKARCH) \
		    -I$(INCLUDE_OUTPUT)
endif

#  
# Where javadoc is written.
#
ifeq ($(JAVADOCDIR),)
    JAVADOCDIR = $(ROOT)/jdoc
endif

#  
# Default name of jar file without the .jar extension.
#
ifeq ($(JARNAME),)
    JARNAME = lbsApp
endif

#
# Class Path to use for all Java compiles and execution.
# On windows systems this should be a quoted semi-colon separated list, with
# a `\' to protect the `;' from shell evaluation.  $(ROOT)ROOT 
#
ifneq ($(OSTYPE),cygwin32)
    CLASSPATH = $(XTRA_CLASSPATH):$(CLASS_OUTPUT):$(ROOT):$(ENHYDRA_CLASSES):$(JDK_CLASSES):$(WAI_CLASSES):$(JDAP_CLASSES):$(SWING_CLASSES):$(JAVACC_CLASSES)
else
    CLASSPATH = $(XTRA_CLASSPATH)\;$(CLASS_OUTPUT)\;$(ROOT)\;$(ENHYDRA_CLASSES)\;$(JDK_CLASSES)\;$(WAI_CLASSES)\;$(JDAP_CLASSES)\;$(SWING_CLASSES)\;$(JAVACC_CLASSES)
endif

#
# The '.' seperated package name derived from PACKAGEDIR
#
PACKAGE = $(shell echo $(PACKAGEDIR) | tr / .)

#
# Package output directory.
#
PACKAGE_OUTPUT = $(CLASS_OUTPUT)/$(PACKAGEDIR)

# 
# Presentation object lists
#
PO_JAVA_UP = $(PO_JAVA) $(PO_CLASS)
PO_CONF_UP = $(PO_CONF) $(PO_JAVA_UP)
PO_DOM_UP = $(PO_DOM) $(PO_CONF_UP)


###############################################################################
# Default "all" rule if none is specified in the subdir make file.
###############################################################################
build_all:	write_header check_sanity do_pre_misc_build java_pass1 java_pass2 java_pass3
#	@echo "Done  build_all  	in $(REL_DIR)"

build_1:	write_header check_sanity do_pre_misc_build java_pass1
#	@echo "Done  build_1  	in $(REL_DIR)"

build_2:	write_header check_sanity do_pre_misc_build java_pass2
#	@echo "Done  build_2  	in $(REL_DIR)"

build_3:	write_header check_sanity do_pre_misc_build java_pass3
#	@echo "Done  build_3  	in $(REL_DIR)"

build_12:	write_header check_sanity do_pre_misc_build java_pass1 java_pass2
#	@echo "Done  build_12  	in $(REL_DIR)"

build_23:	write_header check_sanity do_pre_misc_build java_pass2 java_pass3
#	@echo "Done  build_23  	in $(REL_DIR)"

all:		build_all
#	@echo "Done  all  	in $(REL_DIR)"

all_conf:	write_header check_sanity conf_pass
#	@echo "Done  all  	in $(REL_DIR)"

rules:
	@echo ""
	@echo "public make rules"
	@echo "================="
	@echo "all	-	Build all source."
	@echo "build_all -	Same as \"all\" rule"
	@echo "javadoc	-	Build all the javadoc in the source files."
	@echo "jar	-	Build the jar file."
	@echo "world	-	Build all source. Can be called from anywhere"
	@echo "			within the source tree."
	@echo "clean	-	Clean the source tree."
	@echo "clean_world -	Clean the source tree. Can be called from"
	@echo "			anywhere within the source tree."
	@echo ""


###############################################################################
# Pre Java miscellaneous build rules
###############################################################################
ifneq ($(PRE_MISC_BUILD),)
do_pre_misc_build:
	@for x in $(PRE_MISC_BUILD) ; do \
		$(MAKE) $$x || exit 1; \
	done
else
do_pre_misc_build:
endif


###############################################################################
#				Java Build Rules
###############################################################################

#
# Performs a Java target build pass at the current directory level.
# 
java_pass1: do_pre_misc_build do_copy_to_class do_m4_targets do_xmlc_html_targets dojhtml_targets do_dom_targets do_conf_targets jpass_subdirs1
java_pass2: do_po_java_targets jpass_subdirs2
java_pass3: dojpass_targets dormi_targets donative_targets do_po_class_targets domisc_targets jpass_subdirs3

#
# Default rule to build Makefile in GEN directory
#
do_makefile: $(GEN_DIR)/$(REL_DIR)/Makefile
#	@echo "Done  do_makefile  	in $(REL_DIR)"

#
# Default rule to copy files into the CLASSES directory
#
do_copy_to_class: $(COPY_TO_CLASS:%=$(CLASS_DIR)/$(REL_DIR)/%)
#	@echo "Done  do_copy_to_class  	in $(REL_DIR)"

#
# Default rule to build prototype HTML (No longer used? 7Oct99)
#
do_m4_targets: $(M4_FILES:%=$(GEN_DIR)/$(REL_DIR)/%.html)
#	@echo "Done  do_m4_targets  	in $(REL_DIR)"

#
# Default rule to build Java class targets. 17may99: Now excludes HTML_CLASSES and JOLT_CLASSES.
# 
dojpass_targets: $(CLASSES:%=$(PACKAGE_OUTPUT)/%.class)
#	@echo "Done  dojpass_targets  	in $(REL_DIR)"

#
# Default rule to build presentation objects. (Added by Guy Smith, 16 July 1999.)
# If there are any *.rej files afterwards, they are listed.
# 
do_conf_targets: $(PO_CONF_UP:%=$(VIEW_DIR)/$(REL_DIR)/%.conf) $(PO_CONF_UP:%=$(SRC_DIR)/$(REL_DIR)/%.conf) 
	@if [ "`echo *.rej`" != '*.rej' ] ; then \
	  ( cd $(ROOT) ; ls -lt src/$(REL_DIR)/*.rej ) ; \
	fi 
#	@echo "Done  do_conf_targets  	in $(REL_DIR)"


#
# Default rule to build presentation objects. (Added by Guy Smith, 17 May 1999.)
#
# Modified by GMS 27Jul99: Builds and compiles BASE java, Ensures hand-crafted java exists.
# 
do_po_java_targets: $(PO_JAVA_UP:%=$(GEN_DIR)/$(REL_DIR)/%BASE.java) $(PO_JAVA_UP:%=$(SRC_DIR)/$(REL_DIR)/%.java)
#	@echo "Done  do_po_java_targets  	in $(REL_DIR)"

#
# Default rule to build presentation objects. (Added by Guy Smith, 17 May 1999.)
#
# Modified by GMS 27Jul99: Compiles hand-crafted java file.
# 
do_po_class_targets: $(PO_JAVA_UP:%=$(PACKAGE_OUTPUT)/%BASE.class) $(PO_CLASS:%=$(PACKAGE_OUTPUT)/%.class)
#	@echo "Done  do_po_class_targets  	in $(REL_DIR)"


#
# Default rule to build DOM java classes. (Added by Guy Smith, 17 May 1999.)
# Extended by Guy Smith, 9 July 99, to process DOM_CLASSES
# 
do_dom_targets: $(PO_DOM_UP:%=clean_dom_java_source_%) $(DOM_OBJECTS:%=clean_dom_java_source_%) $(PO_DOM_UP:%=$(PACKAGE_OUTPUT)/%HTML.class) $(DOM_OBJECTS:%=$(PACKAGE_OUTPUT)/%HTML.class)
#	@echo "Done  do_dom_targets  	in $(REL_DIR)"

#
# Warn about XxxxHTML.java files accidentally left over by XMLC. 
#
clean_dom_java_source_%:
	/bin/rm -f $(SRC_DIR)/$(REL_DIR)/$*HTML.java

#
# Default rule to build JOLT source targets.
#
dojhtml_targets: $(JOLT_CLASSES:%=%.jhtml)
#	@echo "Done  dojhtml_targets  	in $(REL_DIR)"

#
# Default rule to build classes from HTML using XMLC
#
do_xmlc_html_targets: $(HTML_CLASSES:%=$(CLASS_DIR)/$(REL_DIR)/%.class)
#	@echo "Done  do_xmlc_html_targets  	in $(REL_DIR)" 

#
# Default rule to build rmi skeleton and stub classes.
#
dormi_targets: $(RMI_CLASSES:%=$(PACKAGE_OUTPUT)/%.class) $(RMI_CLASSES:%=$(PACKAGE_OUTPUT)/%_Skel.class)
#	@echo "Done  dormi_targets  	in $(REL_DIR)"

#
# Default rule to build native class targets.
# FIX: this should not loop
ifneq ($(NATIVE),)
donative_targets:
	@mkdir -p $(NATIVE_OUTPUT)
	@for x in $(NATIVE); do \
		$(MAKE) $(NATIVE_OUTPUT)/lib$$x.so || exit 1 ; \
	done
else
donative_targets:
endif

#
# Default rule to install MISC files.
# FIX: this should not loop
#
domisc_targets: domisc_build domisc_install
#	@echo "Done  domisc_targets  	in $(REL_DIR)"

ifneq ($(MISC_BUILD),)
domisc_build:
	@for x in $(MISC_BUILD) ; do \
		$(MAKE) $$x || exit 1; \
	done
else
domisc_build:
endif

ifneq ($(MISC_INSTALL),)
domisc_install:
	@mkdir -p $(OUTPUT)/$(PACKAGEDIR)
	@for f in $(MISC_INSTALL) ; do \
		ff=`basename $$f`; \
		if cmp $$f $(OUTPUT)/$(PACKAGEDIR)/$$ff > /dev/null 2>&1; \
		then \
			: ; \
		else \
			echo Installing $$f; \
			cp `basename $$f` $(OUTPUT)/$(PACKAGEDIR); \
		fi; \
	done
#	@echo "Done  domisc_install  	in $(REL_DIR)"
else
domisc_install:
#	@echo "Done  empty domisc_install  	in $(REL_DIR)"
endif


#
# Copy files into the classes directory, so they will be put in the jar file.
#
dojar_targets: dojar_install

ifneq ($(JAR_INSTALL),)
dojar_install:
	@mkdir -p $(PACKAGE_OUTPUT)
	@for f in $(JAR_INSTALL) ; do \
		ff=`basename $$f`; \
		if cmp $$f $(PACKAGE_OUTPUT)/$$ff > /dev/null 2>&1; \
		then \
			: ; \
		else \
			echo Adding to JAR classes: $$f; \
			cp $$f $(PACKAGE_OUTPUT); \
		fi; \
	done
#	@echo "Done  dojar_install  	in $(REL_DIR)"
else
dojar_install:
#	@echo "Done  empty dojar_install  	in $(REL_DIR)"
endif



#
# Default rule to run the *first* Java target build pass on any subdirectories
# listed in SUBDIRS. The build is performed depth-first, so that
# subdirectories are output before targets at the current level are output.
#
ifneq ($(SUBDIRS),)
conf_pass_subdirs:
	@if [ -n "$$MAKE_PATH" ] ; then \
	    PARENT_PATH="$$MAKE_PATH" ; \
	else  \
	    PARENT_PATH="." ; \
	fi ; \
	export PARENT_PATH ; \
	for d in $(SUBDIRS) ; \
	    do \
		( cd $$d || exit 1 ; \
		MAKE_PATH="$$PARENT_PATH/$$d" ; \
		export MAKE_PATH ; \
		echo "<< Start  CONF_BUILD pass 1 in: $$MAKE_PATH >>" ; \
		$(MAKE) conf_pass || exit 1 ; \
		) ; \
	    done
else
conf_pass_subdirs:
endif



#
# Default rule to run the *first* Java target build pass on any subdirectories
# listed in SUBDIRS. The build is performed depth-first, so that
# subdirectories are output before targets at the current level are output.
#
ifneq ($(SUBDIRS),)
jpass_subdirs1:
	@if [ -n "$$MAKE_PATH" ] ; then \
	    PARENT_PATH="$$MAKE_PATH" ; \
	else  \
	    PARENT_PATH="." ; \
	fi ; \
	export PARENT_PATH ; \
	for d in $(SUBDIRS) ; \
	    do \
		( cd $$d || exit 1 ; \
		MAKE_PATH="$$PARENT_PATH/$$d" ; \
		export MAKE_PATH ; \
		echo "<< Start  JAVA_BUILD pass 1 in: $$MAKE_PATH >>" ; \
		$(MAKE) java_pass1 || exit 1 ; \
		) ; \
	    done
else
jpass_subdirs1:
endif


#
# Default rule to run the *second* Java target build pass on any subdirectories
# listed in SUBDIRS. The build is performed depth-first, so that
# subdirectories are output before targets at the current level are output.
#
ifneq ($(SUBDIRS),)
jpass_subdirs2:
	@if [ -n "$$MAKE_PATH" ] ; then \
	    PARENT_PATH="$$MAKE_PATH" ; \
	else  \
	    PARENT_PATH="." ; \
	fi ; \
	export PARENT_PATH ; \
	for d in $(SUBDIRS) ; \
	    do \
		( cd $$d || exit 1 ; \
		MAKE_PATH="$$PARENT_PATH/$$d" ; \
		export MAKE_PATH ; \
		echo "<< Start  JAVA_BUILD pass 2 in: $$MAKE_PATH >>" ; \
		$(MAKE) java_pass2 || exit 1 ; \
		) ; \
	    done
else
jpass_subdirs2:
endif


#
# Default rule to run the *second* Java target build pass on any subdirectories
# listed in SUBDIRS. The build is performed depth-first, so that
# subdirectories are output before targets at the current level are output.
#
ifneq ($(SUBDIRS),)
jpass_subdirs3:
	@if [ -n "$$MAKE_PATH" ] ; then \
	    PARENT_PATH="$$MAKE_PATH" ; \
	else  \
	    PARENT_PATH="." ; \
	fi ; \
	export PARENT_PATH ; \
	for d in $(SUBDIRS) ; \
	    do \
		( cd $$d || exit 1 ; \
		MAKE_PATH="$$PARENT_PATH/$$d" ; \
		export MAKE_PATH ; \
		echo "<< Start  JAVA_BUILD pass 3 in: $$MAKE_PATH >>" ; \
		$(MAKE) java_pass3 || exit 1 ; \
		) ; \
	    done
else
jpass_subdirs3:
endif

###############################################################################
#			  JavaDoc Documentation Rule
###############################################################################
#
#    At any point in the tree, you can type "make javadoc", and you will
# end up with a directory "jdoc". Normally this is run at the top of the
# tree, but for testing you can run it in a subdirectory. 
#

#
# Invoke javadoc with a list of packages that were discovered.
#
javadoc: jdclean jddir jdinit javadoc_pass 
	$(JAVADOC) $(JDOCMEM) $(JAVADOCFLAGS) -d $(JAVADOCDIR) \
	    -sourcepath "$(ROOT)/javadoc_tmp:$(CLASSPATH)" `cat $(JAVADOCDIR)/package.list`
	rm -r $(ROOT)/javadoc_tmp

#
# Get rid of the old doc.
#
jdclean:
	rm -rf $(JAVADOCDIR)

#
# Get the jdoc directory created and initialized.
# In Java 1.2 they removed the images and html file, so if they exist
# in the JDK, then copy them. If the don't that's fine.
#
jddir:
	@[ -d $(JAVADOCDIR) ] || mkdir -p $(JAVADOCDIR)
	@[ ! -f $(JDKDIR)/docs/api/API_users_guide.html ]  || \
	    make $(JAVADOCDIR)/API_users_guide.html
	@[ ! -d $(JDKDIR)/docs/api/images ]  || \
	    make $(JAVADOCDIR)/images

#
# Create the javadoc directory, if it doesn't already exist.
#
$(JAVADOCDIR)/API_users_guide.html:
	@[ -d $(JAVADOCDIR) ] || mkdir -p $(JAVADOCDIR)
	cp -rf $(JDKDIR)/docs/api/API_users_guide.html $(JAVADOCDIR)
	chmod u+w $(JAVADOCDIR)/API_users_guide.html

$(JAVADOCDIR)/images:
	@[ -d $(JAVADOCDIR)/images ] || mkdir -p $(JAVADOCDIR)/images
	cp -rf $(JDKDIR)/docs/api/images/ $(JAVADOCDIR)
	chmod -R u+w $(JAVADOCDIR)/images

#
# Builds JavaDoc documentation from the source tree.
#
javadoc_pass:	javadoc_subdirs javadoc_targets

#
# Default rule to descend the source tree and build all JavaDoc HTML
# documentation from the source code.
#
javadoc_subdirs:
	@subs="$(SUBDIRS)" ; \
	if [ -n "$$MAKE_PATH" ] ; then \
	    PARENT_PATH="$$MAKE_PATH" ; \
	else  \
	    PARENT_PATH="." ; \
	fi ; \
	export PARENT_PATH ; \
	for d in $$subs ; \
	    do \
		( cd $$d || exit 1; \
		MAKE_PATH="$$PARENT_PATH/$$d" ; \
		export MAKE_PATH ; \
		echo "<< Start  JAVA_DOC pass in: $$MAKE_PATH >>" ; \
		$(MAKE) javadoc_pass JAVADOCDIR=../$(JAVADOCDIR) || exit 1 ; \
		echo "<< Finish JAVA_DOC pass in: $$MAKE_PATH >>" ; \
		) ; \
	    done

#
# Default rule to build JavaDoc targets. Directories containing no
# CLASSES or PRESENTATION_OBJECTS cannot contain javadoc and are thus skipped.
#
javadoc_targets:
	@PACKAGEDIR=$(PACKAGEDIR) ; \
	export PACKAGEDIR ; \
	if [ -n "$$PACKAGEDIR" ] ; then \
	    if [ -n "$(CLASSES)" -o -n "$(PO_JAVA_UP)" ] ; then \
		PKGNAME=`echo "$$PACKAGEDIR" | sed 's/\//./g'` ; \
		echo "Including package '$$PKGNAME' in documentation..." ; \
		echo "$$PKGNAME" >> $(JAVADOCDIR)/package.list ; \
		mkdir -p $(ROOT)/javadoc_tmp/$$PACKAGEDIR ; \
		cp *.java $(ROOT)/javadoc_tmp/$$PACKAGEDIR || echo "No java files in `pwd`" ; \
		cp $(GEN_DIR)/$(REL_DIR)/*.java $(ROOT)/javadoc_tmp/$$PACKAGEDIR || echo "No java files in $(GEN_DIR)/$(REL_DIR)" ; \
	    fi ; \
	fi

#
# Clean the generated javadoc tree.
#
clean_javadoc:
	rm -rf $(JAVADOCDIR)

#
# Initialize the package list prior to building JavaDoc.
#
jdinit:
	rm -f $(JAVADOCDIR)/package.list


###############################################################################
#				 General build rules
###############################################################################

#
# At any directory level, "make world" builds from the root of the tree.
#
world:
	@ MAKE_PATH='.' ; export MAKE_PATH ; \
	echo "<< Start  Global build: "`date`" >>" ; \
	cd $(ROOT) || exit 1 ; \
	$(MAKE) build_all || exit 1 ; \
	echo "<< Finish Global build: "`date`" >>"

#
# Default Clean-up rule to remove compiled classes.
#
clean::	clean_subdirs clean_current clean_misc

#
# Cleans up any subdirectories listed in SUBDIRS.
#
clean_subdirs:
	@subs="$(SUBDIRS)" ; \
	if [ -n "$$MAKE_PATH" ] ; then \
	    PARENT_PATH="$$MAKE_PATH" ; \
	else  \
	    PARENT_PATH="." ; \
	fi ; \
	export PARENT_PATH ; \
	for d in $$subs ; \
	    do \
		( cd $$d || exit 1 ; \
		MAKE_PATH="$$PARENT_PATH/$$d" ; \
		export MAKE_PATH ; \
		echo "<< Start  CLEANUP pass in: $$MAKE_PATH >>" ; \
		$(MAKE) clean || exit 1 ; \
		echo "<< Finish CLEANUP pass in: $$MAKE_PATH >>" ; \
		) ; \
	    done

#
# Cleans the current directory level.  Conditionals are because some rm
# commands complain if there are no arguments.
#
clean_current:
ifneq ($(PACKAGEDIR),)
	-rm -rf $(PACKAGE_OUTPUT)/*.class
endif
ifneq ($(JOLT_CLASSES),)
	-rm -rf $(JOLT_CLASSES:%=./%.java)
endif
ifneq ($(HTML_CLASSES),)
	-rm -rf $(HTML_CLASSES:%=./%.java)
endif
ifneq ($(MISC_INSTALL),)
	-rm -rf $(MISC_INSTALL:%=$(OUTPUT)/$(PACKAGEDIR)/%)
endif

#
# Cleans up miscellaneous files such as emac history files and core images.
#
clean_misc:
	@rm -f *~ core


#
# Does a global cleanup -- removes the entire "output" tree and
# generated javadoc leaving the tree in pristine condition.
#
clean_world:	clean_javadoc
	-rm -rf $(OUTPUT) $(CLASS_OUTPUT)


###############################################################################
#			 Pattern-matching rules
###############################################################################


#
# Pattern-matching rule to copy html files between directories
#
$(CLASS_DIR)/$(REL_DIR)/%.html: %.html
	@mkdir -p $(CLASS_DIR)/$(REL_DIR)
	set -x ; \
	cp $< $@
	@mkdir -p $(VIEW_DIR)/$(REL_DIR)
	set -x ; \
	cp $< $(VIEW_DIR)/$(REL_DIR)


#
# Pattern-matching rule to copy js files between directories
#
$(CLASS_DIR)/$(REL_DIR)/%.js: %.js
	@mkdir -p $(CLASS_DIR)/$(REL_DIR)
	set -x ; \
	cp $< $@
	@mkdir -p $(VIEW_DIR)/$(REL_DIR)
	set -x ; \
	cp $< $(VIEW_DIR)/$(REL_DIR)


#
# Pattern-matching rule to copy gif files between directories
# Copies gif files to both $(CLASS_DIR) and $(VIEW_DIR)
#
$(CLASS_DIR)/$(REL_DIR)/%.gif: %.gif
	@mkdir -p $(CLASS_DIR)/$(REL_DIR)
	set -x ; \
	cp $< $@
	@mkdir -p $(VIEW_DIR)/$(REL_DIR)
	set -x ; \
	cp $< $(VIEW_DIR)/$(REL_DIR)


#
# Pattern-matching rule to copy jpg files between directories
# Copies jpg files to both $(CLASS_DIR) and $(VIEW_DIR)
#
$(CLASS_DIR)/$(REL_DIR)/%.jpg: %.jpg
	@mkdir -p $(CLASS_DIR)/$(REL_DIR)
	set -x ; \
	cp $< $@
	@mkdir -p $(VIEW_DIR)/$(REL_DIR)
	set -x ; \
	cp $< $(VIEW_DIR)/$(REL_DIR)

#
# Pattern-matching rule to copy png files between directories
# Copies png files to both $(CLASS_DIR) and $(VIEW_DIR)
#
$(CLASS_DIR)/$(REL_DIR)/%.png: %.png
	@mkdir -p $(CLASS_DIR)/$(REL_DIR)
	set -x ; \
	cp $< $@
	@mkdir -p $(VIEW_DIR)/$(REL_DIR)
	set -x ; \
	cp $< $(VIEW_DIR)/$(REL_DIR)

#
# Pattern-matching rule to copy cascading stylesheet files between directories
#
$(CLASS_DIR)/$(REL_DIR)/%.css: %.css
	@mkdir -p $(CLASS_DIR)/$(REL_DIR)
	set -x ; \
	cp $< $@
	@mkdir -p $(VIEW_DIR)/$(REL_DIR)
	set -x ; \
	cp $< $(VIEW_DIR)/$(REL_DIR)

#
# Pattern-matching rule to build java classes and output them directly
# to the $(OUTPUT) directory
#
$(PACKAGE_OUTPUT)/%.class: %.java
	@mkdir -p $(CLASS_DIR)/$(REL_DIR)
	@CLASSPATH=$(CLASSPATH) ; PATH=$(PATH) ; export CLASSPATH PATH  ; \
	set -x ; \
	$(JAVAC) -d $(CLASS_OUTPUT) $(JCFLAGS) $< || exit 1

#
# Pattern-matching rule to build java classes and output them directly
# to the $(OUTPUT) directory. Modified by Guy Smith on 17 May 1999 to
# get source form the $(GEN_DIR) directory. This rule should come *after* 
# the rule for compiling from the src directory, so that the src directory 
# can override any code thats in the autogenerated area.
#
$(PACKAGE_OUTPUT)/%.class: $(GEN_DIR)/$(REL_DIR)/%.java
	@mkdir -p $(CLASS_DIR)/$(REL_DIR)
	@CLASSPATH=$(CLASSPATH) ; PATH=$(PATH) ; export CLASSPATH PATH  ; \
	set -x ; \
	cd $(GEN_DIR)/$(REL_DIR) ; \
	$(JAVAC) -d $(CLASS_OUTPUT) $(JCFLAGS) $< || exit 1

#
# Pattern-matching rule to build skeleton config file from java HTML classes. 
# Created by Guy Smith on 1 July 1999.
# Modified by Guy Smith on 18 Aug 1999 to handle diff/patch merges.
#
$(VIEW_DIR)/$(REL_DIR)/%.conf: $(PACKAGE_OUTPUT)/%HTML.class 
	@mkdir -p $(GEN_DIR)/$(REL_DIR)
	@mkdir -p $(VIEW_DIR)/$(REL_DIR) 
	$(CONG_CMD) $(APP_NAME) $(REL_DIR)/$* $(VIEW_DIR)/$(REL_DIR) $(ROOT)
	$(DIFFPATCH_CMD) $(APP_NAME) $(GEN_DIR) $(REL_DIR) $(SRC_DIR) $(VIEW_DIR) $*

#
# Pattern-matching rule to put config files into SRC directory.
# Note: this should not overwrite an existing file!
# Note: the view/....cp file is used as the common base for diff/patch merges.
#
$(SRC_DIR)/$(REL_DIR)/%.conf: 
	if [ ! -f $@ ] ; then \
	  echo "File $(SRC_DIR)/$(REL_DIR)/$*.conf should have been created when building $(VIEW_DIR)/$(REL_DIR)/$*.conf" ; \
	  exit 1 ; \
	fi 

#
# Pattern-matching rule to build java BASE manipulation class source from java HTML classes. 
# Note: this creates skeleton for hand-coded manipulation class source.
# Created by Guy Smith on 17 May 1999.
#
$(GEN_DIR)/$(REL_DIR)/%BASE.java: $(PACKAGE_OUTPUT)/%HTML.class $(SRC_DIR)/$(REL_DIR)/%.conf
	      @mkdir -p $(GEN_DIR)/$(REL_DIR)
	      @mkdir -p $(VIEW_DIR)/$(REL_DIR)
	      $(POND_CMD) $(APP_NAME) $(REL_DIR)/$* $(VIEW_DIR)/$(REL_DIR) $(GEN_DIR)/$(REL_DIR) 
	      cp $(GEN_DIR)/$(REL_DIR)/$*BASE.java $(VIEW_DIR)/$(REL_DIR)/$*BASE.java 

#
# Pattern-matching rule to put skeleton java into src directory
# Note: this should not overwrite an existing file!
# Note: This should not have a dependency on the conf file -- we don't want an m4 change to flow through.
#
$(SRC_DIR)/$(REL_DIR)/%.java: 
	if [ ! -f $@ ] ; then \
	  set -x ;  \
	  cp $(VIEW_DIR)/$(REL_DIR)/$*.java . ; \
	fi

#
# Pattern-matching rule to put skeleton java into view directory
# Useful if some idiot (aka me) deletes the java in the src directory.
#
$(VIEW_DIR)/$(REL_DIR)/%.java: $(GEN_DIR)/$(REL_DIR)/%BASE.java

#
# Pattern-matching rule to build html files from m4 and output them directly
# to GEN_DIR
#
# Guy added the Common.m4 prerequisite. 
#
$(GEN_DIR)/$(REL_DIR)/%.html: %.m4 $(ROOT)/src/presentation/include/Common.m4 
	@mkdir -p $(GEN_DIR)/$(REL_DIR)
	@mkdir -p $(VIEW_DIR)/$(REL_DIR)
	set -x ; \
	m4 -P $< > $(GEN_DIR)/$(REL_DIR)/$*.html || exit 1
	[ ! -f $(GEN_DIR)/$(REL_DIR)/$*.html ] || cp $(GEN_DIR)/$(REL_DIR)/$*.html $(VIEW_DIR)/$(REL_DIR)/$*.html


#
# Rule for building RMI stub and skeleton classes.
#
$(PACKAGE_OUTPUT)/%_Skel.class: $(PACKAGE_OUTPUT)/%.class
	@mkdir -p $(CLASS_DIR)/$(REL_DIR)
	@CLASSPATH=$(CLASSPATH) ; PATH=$(PATH) ; export CLASSPATH PATH ; \
	set -x ; \
	$(RMIC) -d $(CLASS_OUTPUT) $(PACKAGE).$(<F:%.class=%) || exit 1


#
# Pattern-matching rule to build java native class methods.
# to the $(OUTPUT) directory
#
$(NATIVE_OUTPUT)/lib%.so: $(OUTPUT)/$(PACKAGEDIR)/%.class
	@mkdir -p $(INCLUDE_OUTPUT) $(NATIVE_OUTPUT)
	@class=`basename $< .class` ; \
	includes="-I$(JDKDIR)/include -I$(JDKDIR)/include/solaris" ; \
	includes="$$includes -I$(INCLUDE_OUTPUT)" ; \
	CLASSPATH=$(CLASSPATH) ; PATH=$(PATH) ; export CLASSPATH PATH ; \
	set -x ; \
	$(JAVAH) -jni -o $(INCLUDE_OUTPUT)/$$class.h $(PACKAGE).$$class || exit 1 ; \
	$(CC) -G -o $@ $$includes $$class.c || exit 1 ; \

#
# Compile JoltHTML.
#
$(PACKAGE_OUTPUT)/%.class: %.jhtml
	@mkdir -p $(CLASS_DIR)/$(REL_DIR)
	@CLASSPATH=$(CLASSPATH) ; PATH=$(PATH) ; export CLASSPATH PATH ; \
	set -x ; \
	$(JOLTC) $(JOLTCFLAGS) -d $(CLASS_OUTPUT) $< $(PACKAGE)

#
# Compile HTML with XMLC. Added by Guy Smith on 30 Sep 1999 to handle framesets in the src directory.
#
# Note: XMLC drops the HTML.java intermediate in the current directory!
#
# Special rule to handle framesets. Should precede the more general rule. 
#
# Note: we assume all frameset files finish with 'Frameset.m4'
#
$(PACKAGE_OUTPUT)/%FramesetHTML.class: $(SRC_DIR)/$(REL_DIR)/%Frameset.html 
	$(XMLC_CMD) -html:frameset -class $(PACKAGE).$*FramesetHTML $(XMLC_HTML_OPTS) $(HTML_XMLC_OPTS_FILE) $(SRC_DIR)/$(REL_DIR)/$*Frameset.html
	mkdir -p $(VIEW_DIR)/$(REL_DIR)
	set -x ; [ ! -f $*FramesetHTML.java ] || mv $*FramesetHTML.java $(VIEW_DIR)/$(REL_DIR)/$*FramesetHTML.java 

#
# Compile HTML with XMLC. Added by Guy Smith on 17 May 1999 to handle the HTML generated from m4.
#
# Note: XMLC drops the HTML.java intermediate in the current directory!
#
# Special rule to handle framesets. Should precede the more general rule. 
#
# Note: we assume all frameset files finish with 'Frameset.m4'
#
$(PACKAGE_OUTPUT)/%FramesetHTML.class: $(GEN_DIR)/$(REL_DIR)/%Frameset.html 
	$(XMLC_CMD) -html:frameset -class $(PACKAGE).$*FramesetHTML $(XMLC_HTML_OPTS) $(HTML_XMLC_OPTS_FILE) $(GEN_DIR)/$(REL_DIR)/$*Frameset.html
	mkdir -p $(VIEW_DIR)/$(REL_DIR)
	set -x ; [ ! -f $*FramesetHTML.java ] || mv $*FramesetHTML.java $(VIEW_DIR)/$(REL_DIR)/$*FramesetHTML.java 

#
# Compile HTML in SRC_DIR with XMLC
#
# GMS: This has some ugly stuff, 'coz XMLC won't regenerate its
# HTML.java file if one already exists!  The '-keep' option should be
# used with great care!
#
$(PACKAGE_OUTPUT)/%HTML.class: $(HTML_DIR)/%.html 
	mkdir -p $(VIEW_DIR)/$(REL_DIR)
	[ ! -f $*HTML.java ] || ( set -x ; rm $*HTML.java )
	$(XMLC_CMD) -keep -class $(PACKAGE).$*HTML $(XMLC_HTML_OPTS) $(HTML_XMLC_OPTS_FILE) $(HTML_DIR)/$*.html
	[ ! -f $*HTML.java ] || ( set -x ; mv $*HTML.java $(VIEW_DIR)/$(REL_DIR)/$*HTML.java )

#
# Compile HTML in GEN_DIR with XMLC. 
#
# Note: XMLC drops the HTML.java intermediate in the current directory!
#
# GMS: This has some ugly stuff, 'coz XMLC won't regenerate its
# HTML.java file if one already exists!  The '-keep' option should be
# used with great care!
#
$(PACKAGE_OUTPUT)/%HTML.class: $(GEN_DIR)/$(REL_DIR)/%.html 
	mkdir -p $(VIEW_DIR)/$(REL_DIR)
	[ ! -f $*HTML.java ] || ( set -x ; rm $*HTML.java )
	$(XMLC_CMD) -keep -class $(PACKAGE).$*HTML $(XMLC_HTML_OPTS) $(HTML_XMLC_OPTS_FILE) $(GEN_DIR)/$(REL_DIR)/$*.html
	[ ! -f $*HTML.java ] || ( set -x ; mv $*HTML.java $(VIEW_DIR)/$(REL_DIR)/$*HTML.java )

#
# Check for missing parts of the build environment.  If NO_LUTRIS_CHECK
# is defined, this doesn't check for things that the lutris internal tree
# is going to build. WAI_CHECK& JDAP_CHECK must be set to "YES" to
# check for these libraries.
#
check_sanity:
	@[ -d $(JDKDIR) -o -h $(JDKDIR) ] || (echo "ERROR: Unable to find JDK. You must set JDKDIR to point to your installation of Sun's Java Development Kit, in the files config.mk or /usr/local/enhydra/lib/stdrules.mk" ; echo)
	@[ -d $(JDKDIR) -o -h $(JDKDIR) ]
	@#[ -f $(JDK_CLASSES) ] || (echo "ERROR: Unable to find classes.zip in the JDK. You must set JDKDIR to point to your installation of Sun's Java Development Kit, in the files config.mk or /usr/local/enhydra/lib/stdrules.mk" ; echo)
	@#[ -f $(JDK_CLASSES) ] 
	@[ -f $(JAVAC) ] || (echo "ERROR: Unable to find javac in the JDK. You must set JDKDIR to point to your installation of Sun's Java Development Kit, in the files config.mk or /usr/local/enhydra/lib/stdrules.mk" ; echo)
	@[ -f $(JAVAC) ]
	@[ -f $(JAVA) ] || (echo "ERROR: Unable to find java interpreter in the JDK. You must set JDKDIR to point to your installation of Sun's Java Development Kit, in the files config.mk or /usr/local/enhydra/lib/stdrule.mk" ; echo)
	@[ -f $(JAVA) ]
	@[ -d $(SWINGDIR) -o -h $(SWINGDIR) ] || (echo "WARNING: Unable to find SWING (JFC). To fix, set SWINGDIR to point to your installation of swing, in the files config.mk or /usr/local/enhydra/lib/stdrules.mk" ; echo)
ifeq ($(WAI_CHECK),YES)
	@[ -d $(WAIDIR) -o -h $(WAIDIR) ] || (echo "WARNING: Unable to find WAI, so you will not be able to use that feature. To use WAI, set WAIDIR to point to your installation of Netscape's Web Application Interface, in the files config.mk or /usr/local/enhydra/lib/stdrules.mk" ; echo)
endif
ifeq ($(JDAP_CHECK),YES)
	@[ -d $(JDAPDIR) -o -h $(JDAPDIR) ] || (echo "WARNING: Unable to find JDAP, so you will not be able to use that feature. To use JDAP, set JDAPDIR to point to your installation of Netscape's Java lightweight Directory Access Protocol, in the files config.mk or /usr/local/enhydra/lib/stdrules.mk" ; echo)
endif
ifeq ($(NO_LUTRIS_CHECK),)
	@[ -d $(ENHYDRA_DIR) -o -h $(ENHYDRA_DIR) ] || (echo "ERROR: Unable to find the Enhydra directory, normally at /usr/local/enhydra. You must set ENHYDRA_DIR to point to the Enhydra distribution directory in your file config.mk." ; echo)
	@[ -d $(ENHYDRA_DIR) -o -h $(ENHYDRA_DIR) ]
	@[ -f $(ENHYDRA_CLASSES) -o -d $(ENHYDRA_CLASSES) ] || (echo "ERROR: Unable to find either enhydra.jar or the enhydra classes directory. You must set ENHYDRA_DIR to point to the Enhydra distribution directory in your file config.mk." ; echo)
	@[ -f $(ENHYDRA_CLASSES) -o -d $(ENHYDRA_CLASSES) ] 
	@[ -f $(JOLTC) ] || (echo "ERROR: Unable to find joltc. You must set ENHYDRA_DIR to point to the Enhydra distribution directory in your file config.mk." ; echo)
	@[ -f $(JOLTC) ]
endif

write_header:
	@echo ; echo "Build Started `date`"

jar:
	@mkdir -p $(LIB_OUTPUT)
	@jardir=`(cd $(LIB_OUTPUT) && pwd)` ;\
	cd $(CLASS_OUTPUT) ;\
	dirs=`echo *` ;\
	echo "<< Building JAR file $(JARNAME): "`date`" >>" ;\
	CLASSPATH=$(CLASSPATH) ; PATH=$(PATH) ; export CLASSPATH PATH ; \
	set -x ; \
	$(JAR) cf $$jardir/$(JARNAME).jar $$dirs

