package timesheet.business.maintenance;

import com.lutris.appserver.server.*;
import com.lutris.appserver.server.sql.*;
import com.lutris.appserver.server.httpPresentation.*;
import timesheet.*;
import timesheet.data.*;
import timesheet.business.*;
import java.util.*;
import com.lutris.dods.builder.sourceGenerators.Query.*;
import com.pisoftware.platypus.production.*;

/**
 * Date Created: Oct 15, 1999
 *<P>
 * Purpose: This class is a BO for the AddPersonCommit PO.  
 *
 */
public class AddPersonCommitBO extends MaintenanceBO {


/**
 * Constructor.  Creates the object, and initialises the appropriate variables, etc.
 *
 * @param comms  Used to get the session data required
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public AddPersonCommitBO(HttpPresentationComms comms) throws ApplicationException {

    super(comms);

  }
    
   
/**   
 * Returns the list of pulldown items for the name passed.
 *  
 * @param pulldown_name  The Pulldown name to query for
 * @return an array of PulldownDOs containing the list of values for this pulldown name, null or empty array if none found.
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */   
  public PulldownItem[] getPulldownItems(String pulldown_name) throws ApplicationException {

    /*
     * Add your code here if you want to override/add pulldown item(s).
     */
  
    return super.getPulldownItems(pulldown_name);

  }


/*

  No Access Method(s) Generated!
*/


/*
 * Start of Commit Methods
*/


/**
 * commitAddPerson: Commit method for use by the PO layer.
 * 
 * @param AuthorisationLevel Parameter supplied by PO
 * @param CompanyId Parameter supplied by PO
 * @param Email Parameter supplied by PO
 * @param Fax Parameter supplied by PO
 * @param FirstName Parameter supplied by PO
 * @param Phone Parameter supplied by PO
 * @param PublicPassword Parameter supplied by PO
 * @param Surname Parameter supplied by PO
 * @param Title Parameter supplied by PO
 * @param Type Parameter supplied by PO
 * @param UserName Parameter supplied by PO
 * @return The ObjectId, as a String, for the record added/updated
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public String commitAddPerson(String AuthorisationLevel, String CompanyId, String Email, String Fax, String FirstName, String Phone, String PublicPassword, String Surname, String Title, String Type, String UserName) throws ApplicationException {

    /*
     * Insert your code in here to do the add/update.
     */

    // see if the person already exists
    PersonDO[] exist_results = personBO.getExistingPersons(FirstName, Surname, CompanyId);

    if (exist_results != null && exist_results.length > 0) {

      // person already exists, get out of here

      // get the company details for this person
      CompanyDO companyDO = companyBO.findCompany(CompanyId);

      try {

        throw new UserDuplicateRecordException("This person already exists!\nFirstName: "+FirstName+", Surname: "+Surname+", and Company: "+companyDO.getName());

      } catch (DataObjectException doe) {

        throw new UserDuplicateRecordException("This person already exists!\nFirstName: "+FirstName+", Surname: "+Surname+", and some company I can't determine at this point: "+CompanyId);

      }

    }

    PersonDO newPerson = personBO.createPerson();
    String PersonId = null;

    try {

      PersonId = newPerson.getHandle();

    } catch (Exception e) {

      throw new ApplicationException("Unable to get Object Id of new person!", e);

    }

    // use the newly created id in the edit update method
    return commitEditPerson(PersonId, AuthorisationLevel, CompanyId, Email, Fax, FirstName, Phone, PublicPassword, Surname, Title, Type, UserName);

  }


/**
 * commitEditPerson: Commit method for use by the PO layer.
 * 
 * @param PersonId Parameter supplied by PO
 * @param AuthorisationLevel Parameter supplied by PO
 * @param CompanyId Parameter supplied by PO
 * @param Email Parameter supplied by PO
 * @param Fax Parameter supplied by PO
 * @param FirstName Parameter supplied by PO
 * @param Phone Parameter supplied by PO
 * @param PublicPassword Parameter supplied by PO
 * @param Surname Parameter supplied by PO
 * @param Title Parameter supplied by PO
 * @param Type Parameter supplied by PO
 * @param UserName Parameter supplied by PO
 * @return The ObjectId, as a String, for the record added/updated
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public String commitEditPerson(String PersonId, String AuthorisationLevel, String CompanyId, String Email, String Fax, String FirstName, String Phone, String PublicPassword, String Surname, String Title, String Type, String UserName) throws ApplicationException {

    /*
     * Insert your code in here to do the add/update.
     */

    // find this person, if possible
    PersonDO personDO = personBO.findPerson(PersonId);

    // get the company details for this person
    CompanyDO companyDO = companyBO.findCompany(CompanyId);

    /*
     * Put in your data error checking code here - for this tutorial we
     * can't be bothered.  :-)
     */

    // update the database - this will throw an exception if it fails.
    personBO.updatePerson(personDO, companyDO, AuthorisationLevel, Email, Fax, FirstName, Phone, PublicPassword, Surname, Title, Type, UserName);

    // the id passed (the record id) is our return value
    return PersonId;

  }



/*
 * End of Commit Methods
*/



/*
  No getIdValue Method(s) Generated!
*/

/*
  No getBusinessIdValue Method Generated!
*/

}

