package timesheet.business;

import timesheet.*;
import timesheet.data.*;
import java.util.*;
import com.lutris.appserver.server.*;
import com.lutris.appserver.server.sql.*;
import com.lutris.dods.builder.sourceGenerators.Query.*;
import java.sql.*;
import java.math.*;
import com.pisoftware.platypus.production.*;


/**
 * Date Created: Oct 22, 1999
 * <P>
 * Purpose:  This class provides the business logic associated with
 *           a TimeEntry.  It should probably be used by other BOs only.
 *
 */
public class TimeEntryBO extends TimeEntryBDO {


/**
 * Constructor.  Creates the object, and initialises the
 * appropriate variables, etc.
 * 
 * @exception Exception thrown by BDO constructor.  :-(
 */
  public TimeEntryBO() throws Exception {

    super();

  }



/*
 * Start of getIdValue Methods
*/

  private TimeEntryDO[] query_results = null;


/**
 * Sets the query results to be used in the getIdValue
 * method calls.
 *
 * @param results  The results to use
 */
  public void setQueryResults(TimeEntryDO[] results) {

    this.query_results = results;

  }


/**
 * Gets the current query results being used in the getIdValue
 * method calls.
 *
 * @return see comment
 */
  public TimeEntryDO[] getQueryResults() {

    return this.query_results;

  }


/**
 * Gets information from the DO layer for the ID passed.
 * Note that a previous ACCESS method must have been
 * called for this function to work.
 *
 * @param index  The record to use
 * @param id  The id who's value is to be returned.
 * @param format  Determines what formatting is required for the id's value
 * @return A String representing the value retrieved.
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public String getIdValue(int index, String id, String format) throws ApplicationException {

    if (query_results == null) {

      throw new ApplicationException("No query results to obtain data from!");

    }

    String result_str = "";
    boolean id_found = false;

      if (id.equalsIgnoreCase("DayDate")) {

    id_found = true;

    
    // ensure we've got something to get data from
    if (query_results == null || query_results.length == 0)
      throw new ApplicationException("Unable to obtain data for DayDate.  No data available from query");

    if (index < 0 || index > query_results.length -1)
      throw new ApplicationException("Invalid index to obtain data from query: "+index);


    java.sql.Date result;

    try {

      result = query_results[index].getEntryDate();

    } catch (DataObjectException doe) {

      throw new ApplicationException("Unable to obtain data for DayDate", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("ProjectName")) {

    id_found = true;

    
    // ensure we've got something to get data from
    if (query_results == null || query_results.length == 0)
      throw new ApplicationException("Unable to obtain data for ProjectName.  No data available from query");

    if (index < 0 || index > query_results.length -1)
      throw new ApplicationException("Invalid index to obtain data from query: "+index);


    String result;

    try {

      result = query_results[index].getProjectId().getName();

    } catch (DataObjectException doe) {

      throw new ApplicationException("Unable to obtain data for ProjectName", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("ActivityName")) {

    id_found = true;

    
    // ensure we've got something to get data from
    if (query_results == null || query_results.length == 0)
      throw new ApplicationException("Unable to obtain data for ActivityName.  No data available from query");

    if (index < 0 || index > query_results.length -1)
      throw new ApplicationException("Invalid index to obtain data from query: "+index);


    String result;

    try {

      result = query_results[index].getActivityId().getActivityName();

    } catch (DataObjectException doe) {

      throw new ApplicationException("Unable to obtain data for ActivityName", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("Details")) {

    id_found = true;

    
    // ensure we've got something to get data from
    if (query_results == null || query_results.length == 0)
      throw new ApplicationException("Unable to obtain data for Details.  No data available from query");

    if (index < 0 || index > query_results.length -1)
      throw new ApplicationException("Invalid index to obtain data from query: "+index);


    String result;

    try {

      result = query_results[index].getDetails();

    } catch (DataObjectException doe) {

      throw new ApplicationException("Unable to obtain data for Details", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("NrHours")) {

    id_found = true;

    
    // ensure we've got something to get data from
    if (query_results == null || query_results.length == 0)
      throw new ApplicationException("Unable to obtain data for NrHours.  No data available from query");

    if (index < 0 || index > query_results.length -1)
      throw new ApplicationException("Invalid index to obtain data from query: "+index);


    double result;

    try {

      result = query_results[index].getNrHours();

    } catch (DataObjectException doe) {

      throw new ApplicationException("Unable to obtain data for NrHours", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("DayDate1")) {

    id_found = true;

    
    // ensure we've got something to get data from
    if (query_results == null || query_results.length == 0)
      throw new ApplicationException("Unable to obtain data for DayDate1.  No data available from query");

    if (index < 0 || index > query_results.length -1)
      throw new ApplicationException("Invalid index to obtain data from query: "+index);


    java.sql.Date result;

    try {

      result = query_results[index].getEntryDate();

    } catch (DataObjectException doe) {

      throw new ApplicationException("Unable to obtain data for DayDate1", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("TimeEntryId")) {

    id_found = true;

    
    // ensure we've got something to get data from
    if (query_results == null || query_results.length == 0)
      throw new ApplicationException("Unable to obtain data for TimeEntryId.  No data available from query");

    if (index < 0 || index > query_results.length -1)
      throw new ApplicationException("Invalid index to obtain data from query: "+index);


    java.math.BigDecimal result;

    try {

      result = query_results[index].getOId().toBigDecimal();

    } catch (Exception doe) {

      throw new ApplicationException("Unable to obtain data for TimeEntryId", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("ActivityId")) {

    id_found = true;

    
    // ensure we've got something to get data from
    if (query_results == null || query_results.length == 0)
      throw new ApplicationException("Unable to obtain data for ActivityId.  No data available from query");

    if (index < 0 || index > query_results.length -1)
      throw new ApplicationException("Invalid index to obtain data from query: "+index);


    java.math.BigDecimal result;

    try {

      result = query_results[index].getActivityId().getOId().toBigDecimal();

    } catch (Exception doe) {

      throw new ApplicationException("Unable to obtain data for ActivityId", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("ProjectId")) {

    id_found = true;

    
    // ensure we've got something to get data from
    if (query_results == null || query_results.length == 0)
      throw new ApplicationException("Unable to obtain data for ProjectId.  No data available from query");

    if (index < 0 || index > query_results.length -1)
      throw new ApplicationException("Invalid index to obtain data from query: "+index);


    java.math.BigDecimal result;

    try {

      result = query_results[index].getProjectId().getOId().toBigDecimal();

    } catch (Exception doe) {

      throw new ApplicationException("Unable to obtain data for ProjectId", doe);

    }

    result_str = FormatUtil.format(result, format);

  }


    // was the id found?
    if (!id_found) {

      // no, return a value with the details passed
      return "ID Value passed: "+id+", index is: "+index+", format is: "+format;

    } else {

      // return the formatted result
      return result_str;

    }

  }


  // psuedo-cache to speed access to the db data.
  private java.math.BigDecimal oid = null;


/**
 * Gets information from the DO layer for the ID passed.
 *
 * @param oidstr  The database oid for the record
 * @param id  The id who's value is to be returned.
 * @param format  Determines what formatting is required for the id's value
 * @return A String representing the value retrieved.
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public String getIdValue(String oidstr, String id, String format) throws ApplicationException {

    String result_str = "";
    boolean id_found = false;

    try {

      // are we accesing the same oid again?
      if (DO == null || !oidstr.equals(DO.getHandle().toString())) {

        // no - get this oid's DO from the db.

        oid = new java.math.BigDecimal(oidstr);
        DO = TimeEntryDO.createExisting(oid);

      }

    } catch (Exception e) {

      throw new ApplicationException("Unable to obtain value for id: "+id+" using this ObjectId: "+oidstr, e);

    }

      if (id.equalsIgnoreCase("DayDate")) {

    id_found = true;

    

    java.sql.Date result;

    try {

      result = DO.getEntryDate();

    } catch (DataObjectException doe) {

      throw new ApplicationException("Unable to obtain data for DayDate", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("ProjectName")) {

    id_found = true;

    

    String result;

    try {

      result = DO.getProjectId().getName();

    } catch (DataObjectException doe) {

      throw new ApplicationException("Unable to obtain data for ProjectName", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("ActivityName")) {

    id_found = true;

    

    String result;

    try {

      result = DO.getActivityId().getActivityName();

    } catch (DataObjectException doe) {

      throw new ApplicationException("Unable to obtain data for ActivityName", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("Details")) {

    id_found = true;

    

    String result;

    try {

      result = DO.getDetails();

    } catch (DataObjectException doe) {

      throw new ApplicationException("Unable to obtain data for Details", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("NrHours")) {

    id_found = true;

    

    double result;

    try {

      result = DO.getNrHours();

    } catch (DataObjectException doe) {

      throw new ApplicationException("Unable to obtain data for NrHours", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("DayDate1")) {

    id_found = true;

    

    java.sql.Date result;

    try {

      result = DO.getEntryDate();

    } catch (DataObjectException doe) {

      throw new ApplicationException("Unable to obtain data for DayDate1", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("TimeEntryId")) {

    id_found = true;

    

    java.math.BigDecimal result;

    try {

      result = DO.getOId().toBigDecimal();

    } catch (Exception doe) {

      throw new ApplicationException("Unable to obtain data for TimeEntryId", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("ActivityId")) {

    id_found = true;

    

    java.math.BigDecimal result;

    try {

      result = DO.getActivityId().getOId().toBigDecimal();

    } catch (Exception doe) {

      throw new ApplicationException("Unable to obtain data for ActivityId", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("ProjectId")) {

    id_found = true;

    

    java.math.BigDecimal result;

    try {

      result = DO.getProjectId().getOId().toBigDecimal();

    } catch (Exception doe) {

      throw new ApplicationException("Unable to obtain data for ProjectId", doe);

    }

    result_str = FormatUtil.format(result, format);

  }


    // was the id found?
    if (!id_found) {

      // no, return a value with the details passed
      return "ID Value passed: "+id+", oidstr is: "+oidstr+", format is: "+format;

    } else {

      // return the formatted result
      return result_str;

    }

  }



/*
 * End of getIdValue Methods
*/




}

