package timesheet.business.entry;

import com.lutris.appserver.server.*;
import com.lutris.appserver.server.sql.*;
import com.lutris.appserver.server.httpPresentation.*;
import timesheet.*;
import timesheet.data.*;
import timesheet.business.*;
import java.util.*;
import com.lutris.dods.builder.sourceGenerators.Query.*;
import com.pisoftware.platypus.production.*;

/**
 * Date Created: Oct 18, 1999
 *<P>
 * Purpose: This class is a BO for the AddOneItemForm PO.  
 *
 */
public class AddOneItemFormBO extends EntryBO {


/**
 * Constructor.  Creates the object, and initialises the appropriate variables, etc.
 *
 * @param comms  Used to get the session data required
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public AddOneItemFormBO(HttpPresentationComms comms) throws ApplicationException {

    super(comms);

  }
    
   
/**   
 * Returns the list of pulldown items for the name passed.
 *  
 * @param pulldown_name  The Pulldown name to query for
 * @return an array of PulldownDOs containing the list of values for this pulldown name, null or empty array if none found.
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */   
  public PulldownItem[] getPulldownItems(String pulldown_name) throws ApplicationException {

    /*
     * Add your code here if you want to override/add pulldown item(s).
     */
  
    return super.getPulldownItems(pulldown_name);

  }



/*
 * Start of Access Methods
*/

  private TimeEntryDO[] TimeEntryTable = null;



/**
 * getTimeEntryTable: Query method for use by the PO layer.
 * 
 * @param qsEntryDate Parameter supplied by PO
 * @return The number of records found.
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public int getTimeEntryTable(String qsEntryDate) throws ApplicationException {

    /*
     * Insert your code to do your query.  Put the result into
     * the variable TimeEntryTable as defined above.  getIdValue
     * will use the TimeEntryTable where appropriate.
     *
     */
   
    String entryDate = FormatUtil.displayDateToSQL(qsEntryDate, null);

    // any problems with the date?
    if (!ProdUtil.stringHasContent(entryDate)) {

      throw new ApplicationException("Entry Date is invalid: <"+qsEntryDate+">");

    }

    TimeEntryTable = timeEntryBO.getTimeEntryForDate(operator_id, entryDate);
   
    // determine the number of records found
    return (TimeEntryTable == null) ? 0 : TimeEntryTable.length;  

  }



/*
 * End of Access Methods
*/



/*
  No Commit Method(s) Generated!
*/


/*
 * Start of getIdValue Methods
*/

  private Hashtable hours = new Hashtable(1);
  private boolean hash_calculated = false;



/**
 * Gets information from the DO layer for the ID passed.
 * Note that a previous ACCESS method must have been
 * called for this function to work.
 *
 * @param index  The record to use
 * @param id  The id who's value is to be returned.
 * @param format  Determines what formatting is required for the id's value
 * @return A String representing the value retrieved.
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public String getIdValue(int index, String id, String format) throws ApplicationException {

    String result_str = "";
    boolean id_found = false;

     if (id.equalsIgnoreCase("DayHours")) {

    /*
     * This id is used to determine the number of hours in
     * entries for this day.  The code below was copied from
     * the reports area (ReportPeriodResults).
     */

    // have we fix up the hashtable yet?
    if (!hash_calculated) {

      // set the query results to use
      timeEntryBO.setQueryResults(TimeEntryTable);

      // build the hashtable
      timeEntryBO.buildHoursHash(hours);

      // set the flag so this gets done only once
      hash_calculated = true;

    }

    // get this entry's date
    String entry_date = timeEntryBO.getIdValue(index, "DayDate", null);

    // get the result from the hash table.
    String nr_hours = (String) hours.get(entry_date);

    // format it whatever way was requested.
    return FormatUtil.format(nr_hours, format);

}  else  if (id.equalsIgnoreCase("DayDate")) {

    id_found = true;

    timeEntryBO.setQueryResults(TimeEntryTable);

    result_str = timeEntryBO.getIdValue(index, id, format);

  }
 else  if (id.equalsIgnoreCase("ProjectName")) {

    id_found = true;

    timeEntryBO.setQueryResults(TimeEntryTable);

    result_str = timeEntryBO.getIdValue(index, id, format);

  }
 else  if (id.equalsIgnoreCase("ActivityName")) {

    id_found = true;

    timeEntryBO.setQueryResults(TimeEntryTable);

    result_str = timeEntryBO.getIdValue(index, id, format);

  }
 else  if (id.equalsIgnoreCase("Details")) {

    id_found = true;

    timeEntryBO.setQueryResults(TimeEntryTable);

    result_str = timeEntryBO.getIdValue(index, id, format);

  }
 else  if (id.equalsIgnoreCase("NrHours")) {

    id_found = true;

    timeEntryBO.setQueryResults(TimeEntryTable);

    result_str = timeEntryBO.getIdValue(index, id, format);

  }


    // did we get some data?
    if (!id_found) {

      // no - try our parent, it might know better...
      result_str = super.getIdValue(index, id, format);

    }

    return result_str;

  }


/**
 * Gets information from the DO layer for the ID passed.
 *
 * @param oidstr  The database oid for the record
 * @param id  The id who's value is to be returned.
 * @param format  Determines what formatting is required for the id's value
 * @return A String representing the value retrieved.
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public String getIdValue(String oidstr, String id, String format) throws ApplicationException {

    String result_str = "";
    boolean id_found = false;

     if (id.equalsIgnoreCase("DayHours")) {
// not used - ignore...
}  else  if (id.equalsIgnoreCase("DayDate")) {

    id_found = true;

     

    result_str = timeEntryBO.getIdValue(oidstr, id, format);

  }
 else  if (id.equalsIgnoreCase("ProjectName")) {

    id_found = true;

     

    result_str = timeEntryBO.getIdValue(oidstr, id, format);

  }
 else  if (id.equalsIgnoreCase("ActivityName")) {

    id_found = true;

     

    result_str = timeEntryBO.getIdValue(oidstr, id, format);

  }
 else  if (id.equalsIgnoreCase("Details")) {

    id_found = true;

     

    result_str = timeEntryBO.getIdValue(oidstr, id, format);

  }
 else  if (id.equalsIgnoreCase("NrHours")) {

    id_found = true;

     

    result_str = timeEntryBO.getIdValue(oidstr, id, format);

  }


    // did we get some data?
    if (!id_found) {

      // no - try our parent, it might know better...
      result_str = super.getIdValue(oidstr, id, format);

    }

    return result_str;

  }


/*
 * End of getIdValue Methods
*/



/*
  No getBusinessIdValue Method Generated!
*/

}

