package timesheet.business.entry;

import com.lutris.appserver.server.*;
import com.lutris.appserver.server.sql.*;
import com.lutris.appserver.server.httpPresentation.*;
import timesheet.*;
import timesheet.data.*;
import timesheet.business.*;
import java.util.*;
import com.lutris.dods.builder.sourceGenerators.Query.*;
import com.pisoftware.platypus.production.*;

/**
 * Date Created: Oct 18, 1999
 *<P>
 * Purpose: This class is a BO for the EditWorkItemsResults PO.  
 *
 */
public class EditWorkItemsResultsBO extends EntryBO {


/**
 * Constructor.  Creates the object, and initialises the appropriate variables, etc.
 *
 * @param comms  Used to get the session data required
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public EditWorkItemsResultsBO(HttpPresentationComms comms) throws ApplicationException {

    super(comms);

  }
    
   
/**   
 * Returns the list of pulldown items for the name passed.
 *  
 * @param pulldown_name  The Pulldown name to query for
 * @return an array of PulldownDOs containing the list of values for this pulldown name, null or empty array if none found.
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */   
  public PulldownItem[] getPulldownItems(String pulldown_name) throws ApplicationException {

    /*
     * Add your code here if you want to override/add pulldown item(s).
     */
  
    return super.getPulldownItems(pulldown_name);

  }



/*
 * Start of Access Methods
*/

  private TimeEntryDO[] TimeEntryTable = null;



/**
 * getTimeEntryTable: Query method for use by the PO layer.
 * 
 * @param qsFromDate Parameter supplied by PO
 * @param qsToDate Parameter supplied by PO
 * @return The number of records found.
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public int getTimeEntryTable(String qsFromDate, String qsToDate) throws ApplicationException {

    /*
     * Insert your code to do your query.  Put the result into
     * the variable TimeEntryTable as defined above.  getIdValue
     * will use the TimeEntryTable where appropriate.
     *
     */
   
    String fromDate = FormatUtil.displayDateToSQL(qsFromDate, null);
    String toDate = FormatUtil.displayDateToSQL(qsToDate, null);

    // any problems with the dates?
    if (!ProdUtil.stringHasContent(fromDate)) {

      throw new ApplicationException("From Date is invalid: <"+qsFromDate+">");

    }

    if (!ProdUtil.stringHasContent(toDate)) {

      throw new ApplicationException("To Date is invalid: <"+qsToDate+">");

    }

    TimeEntryTable = timeEntryBO.getTimeEntryRecords(operator_id, fromDate, toDate);
   
    // determine the number of records found
    return (TimeEntryTable == null) ? 0 : TimeEntryTable.length;  

  }



/*
 * End of Access Methods
*/




/*
 * Start of Commit Methods
*/


/**
 * commitEditWorkItems: Commit method for use by the PO layer.
 * 
 * @param TimeEntryId Parameter supplied by PO
 * @param Activity Parameter supplied by PO
 * @param Details Parameter supplied by PO
 * @param Hours Parameter supplied by PO
 * @param Project Parameter supplied by PO
 * @return The ObjectId, as a String, for the record added/updated
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public String commitEditWorkItems(String TimeEntryId, String Activity, String Details, String Hours, String Project) throws ApplicationException {

    /*
     * Insert your code in here to do the add/update.
     */

    // find this Time Entry
    TimeEntryDO timeEntryDO = timeEntryBO.findTimeEntry(TimeEntryId);

    // ensure the hours are ok
    double nrHours = 0.0;

    try {

      Double tmpdouble = new Double(Hours);

      nrHours = tmpdouble.doubleValue();

    } catch (NumberFormatException nfe) {

      throw new ApplicationException("The number of hours entered is not a valid number: <"+Hours+">",nfe);

    }

    // check & convert the date
    java.sql.Date entryDate = null;

    try {

      entryDate = timeEntryDO.getEntryDate();

    } catch (Exception de) {

      throw new ApplicationException("Unable to retrieve entry date!", de);

    }

    // find this activity
    ActivityDO activityDO = activityBO.findActivity(Activity);

    // find this project
    ProjectDO projectDO = projectBO.findProject(Project);

    // update the new record
    timeEntryBO.updateTimeEntry(timeEntryDO, activityDO, Details, entryDate, nrHours, projectDO, operatorDO);

    return TimeEntryId;

  }



/*
 * End of Commit Methods
*/




/*
 * Start of getIdValue Methods
*/

  private Hashtable hours = new Hashtable(20);
  private boolean hash_calculated = false;



/**
 * Gets information from the DO layer for the ID passed.
 * Note that a previous ACCESS method must have been
 * called for this function to work.
 *
 * @param index  The record to use
 * @param id  The id who's value is to be returned.
 * @param format  Determines what formatting is required for the id's value
 * @return A String representing the value retrieved.
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public String getIdValue(int index, String id, String format) throws ApplicationException {

    String result_str = "";
    boolean id_found = false;

     if (id.equalsIgnoreCase("DayHours")) {

    /*
     * This id is used to determine the number of hours in
     * entries for this day.  The code below was copied from
     * the reports area (ReportPeriodResults).
     */

    // have we fix up the hashtable yet?
    if (!hash_calculated) {

      // set the query results to use
      timeEntryBO.setQueryResults(TimeEntryTable);

      // build the hashtable
      timeEntryBO.buildHoursHash(hours);

      // set the flag so this gets done only once
      hash_calculated = true;

    }

    // get this entry's date
    String entry_date = timeEntryBO.getIdValue(index, "DayDate", null);

    // get the result from the hash table.
    String nr_hours = (String) hours.get(entry_date);

    // format it whatever way was requested.
    return FormatUtil.format(nr_hours, format);

}  else  if (id.equalsIgnoreCase("DayDate")) {

    id_found = true;

    timeEntryBO.setQueryResults(TimeEntryTable);

    result_str = timeEntryBO.getIdValue(index, id, format);

  }
 else  if (id.equalsIgnoreCase("ProjectName")) {

    id_found = true;

    timeEntryBO.setQueryResults(TimeEntryTable);

    result_str = timeEntryBO.getIdValue(index, id, format);

  }
 else if (id.equalsIgnoreCase("DayHeader")) {

   /*
    * This id is used by the presentation layer to determine if
    * a new day section is to be created in the report.
    * If we determine this to be the case, then all
    * we have to return is a non-empty/null string. The
    * presentation layer then formats a new day header
    * as it sees appropriate.
    *
    * Note that the results of the query are in date order.
    */

   // check for bad index values
   if (index < 0 || index >= TimeEntryTable.length) {

     throw new UserException("Bad index value when examining Time Entry results: "+index);

   }

   // if this is the first result...
   if (index == 0) {

     return "New Day";

   }

   // see if this entry's date is different to the previous one
   String curr_entry_date = timeEntryBO.getIdValue(index,"DayDate",null);
   String prev_entry_date = timeEntryBO.getIdValue(index-1,"DayDate",null);

   if (!curr_entry_date.equals(prev_entry_date)) {

     // different days - new header required.
     return "New Day";

   }

   // same day - no header required.
   return null;
 
}  else  if (id.equalsIgnoreCase("ActivityName")) {

    id_found = true;

    timeEntryBO.setQueryResults(TimeEntryTable);

    result_str = timeEntryBO.getIdValue(index, id, format);

  }
 else  if (id.equalsIgnoreCase("Details")) {

    id_found = true;

    timeEntryBO.setQueryResults(TimeEntryTable);

    result_str = timeEntryBO.getIdValue(index, id, format);

  }
 else  if (id.equalsIgnoreCase("NrHours")) {

    id_found = true;

    timeEntryBO.setQueryResults(TimeEntryTable);

    result_str = timeEntryBO.getIdValue(index, id, format);

  }


    // did we get some data?
    if (!id_found) {

      // no - try our parent, it might know better...
      result_str = super.getIdValue(index, id, format);

    }

    return result_str;

  }


/**
 * Gets information from the DO layer for the ID passed.
 *
 * @param oidstr  The database oid for the record
 * @param id  The id who's value is to be returned.
 * @param format  Determines what formatting is required for the id's value
 * @return A String representing the value retrieved.
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public String getIdValue(String oidstr, String id, String format) throws ApplicationException {

    String result_str = "";
    boolean id_found = false;

     if (id.equalsIgnoreCase("DayHours")) {
// not used - ignore...
}  else  if (id.equalsIgnoreCase("DayDate")) {

    id_found = true;

     

    result_str = timeEntryBO.getIdValue(oidstr, id, format);

  }
 else  if (id.equalsIgnoreCase("ProjectName")) {

    id_found = true;

     

    result_str = timeEntryBO.getIdValue(oidstr, id, format);

  }
 else if (id.equalsIgnoreCase("DayHeader")) {
 // not used - ignore...
}  else  if (id.equalsIgnoreCase("ActivityName")) {

    id_found = true;

     

    result_str = timeEntryBO.getIdValue(oidstr, id, format);

  }
 else  if (id.equalsIgnoreCase("Details")) {

    id_found = true;

     

    result_str = timeEntryBO.getIdValue(oidstr, id, format);

  }
 else  if (id.equalsIgnoreCase("NrHours")) {

    id_found = true;

     

    result_str = timeEntryBO.getIdValue(oidstr, id, format);

  }


    // did we get some data?
    if (!id_found) {

      // no - try our parent, it might know better...
      result_str = super.getIdValue(oidstr, id, format);

    }

    return result_str;

  }


/*
 * End of getIdValue Methods
*/



/*
  No getBusinessIdValue Method Generated!
*/

}

