package timesheet.business.entry;

import com.lutris.appserver.server.*;
import com.lutris.appserver.server.sql.*;
import com.lutris.appserver.server.httpPresentation.*;
import timesheet.*;
import timesheet.data.*;
import timesheet.business.*;
import java.util.*;
import com.lutris.dods.builder.sourceGenerators.Query.*;
import com.pisoftware.platypus.production.*;

/**
 * Date Created: Oct 18, 1999
 *<P>
 * Purpose: This class is a BO for the Entry PO.  
 *
 */
public class EntryBO extends TimesheetBO {


/**
 * Constructor.  Creates the object, and initialises the appropriate variables, etc.
 *
 * @param comms  Used to get the session data required
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public EntryBO(HttpPresentationComms comms) throws ApplicationException {

    super(comms);

  }
    
   
/**   
 * Returns the list of pulldown items for the name passed.
 *  
 * @param pulldown_name  The Pulldown name to query for
 * @return an array of PulldownDOs containing the list of values for this pulldown name, null or empty array if none found.
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */   
  public PulldownItem[] getPulldownItems(String pulldown_name) throws ApplicationException {

    /*
     * Add your code here if you want to override/add pulldown item(s).
     */

    
    PulldownItem[] pi = null;
  
    if (pulldown_name.equalsIgnoreCase("projectid")) {

      try {

        // get all the valid projects
        ProjectDO[] projects = projectBO.getAllProjects();

        pi = new PulldownItem[projects.length];

        for (int i=0; i<projects.length; i++) {

          pi[i] = new PulldownItem(pulldown_name, projects[i].getHandle(), projects[i].getName(), false);

        }

      } catch (Exception e) {

        // exceptions should only be DatabaseManagerException or DataObjectException
        throw new ApplicationException("Unable to create pulldown for: "+pulldown_name, e);

      }

      return pi;

    } else if (pulldown_name.equalsIgnoreCase("activityid")) {

      try {

        // get all the valid activities
        ActivityDO[] activities = activityBO.getAllActivities();

        pi = new PulldownItem[activities.length];

        for (int i=0; i<activities.length; i++) {

          pi[i] = new PulldownItem(pulldown_name, activities[i].getHandle(), activities[i].getActivityName(), false);

        }

      } catch (Exception e) {

        // exceptions should only be DatabaseManagerException or DataObjectException
        throw new ApplicationException("Unable to create pulldown for: "+pulldown_name, e);

      }

      return pi;

    }
  
    // try our parent if we get here...
    return super.getPulldownItems(pulldown_name);

  }



















}

