package timesheet.presentation; 

import java.io.*;
import java.lang.*;
import java.util.*;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import com.lutris.util.*;
import com.lutris.xml.xmlc.html.HTMLObject;
import com.lutris.xml.xmlc.*;
import com.lutris.appserver.server.sql.*;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.ApplicationException;
import com.lutris.appserver.server.Enhydra;
import com.lutris.logging.*;
import com.pisoftware.platypus.production.*; 

/** 
 **/

public class AppTemplate{

  /**
   * Handles <CODE>UserException</CODE>. Replaces contents of current
   * document with a short page, filled out with data from the
   * UserException.
   **/
  static public void processUserException( Document document, UserException ue ) throws HttpPresentationException { 
    AppTemplateHTML template = new AppTemplateHTML(); 
    HTMLBodyElement body ; 

    //////  Fill in components of UserException page
    template.setTextUserExceptionMessage( ue.message );
    if( ProdUtil.stringHasContent( ue.backURL ) ){ 
      template.getElementBackURL().setAttribute( "href", ue.backURL ); 
    }else{
      ProdUtil.removeThisNodeAndContext( template.getElementBackURL(), HTMLParagraphElement.class ); 
    }

    //////  Replace contents of normal page with UserException page
    body = ProdUtil.getBodyElement( document ); 
    while( body.hasChildNodes() ) ProdUtil.removeThisNode( body.getFirstChild() ); 
    body.appendChild( template.getElementUserExceptionTemplate() ); 
  }


  /**
   * This method is part of the AdminNotice module, which has not yet
   * been fully incorporated into Platypus.
   **/
  static public void perhapsGenerateAdminNotification( String currentAdminNotice, Document document, KeywordValueTable presentationData ) throws HttpPresentationException {
    String lastServedNotice ; 
    String code ; // Code to put at start of HTML body.
    HTMLBodyElement body ; 
    HTMLScriptElement protoScript ; 
    Comment jsComment ; 
    HTMLScriptElement newScript ; 
    AppTemplateHTML appTemplate ; 

    //////  Get notice. Return if we don't need to serve a notice.
    if( ! ProdUtil.stringHasContent( currentAdminNotice ) ) return ; 
    try{ 
      lastServedNotice = presentationData.getString( "LASTADMINNOTICE", "" ); 
    }catch( KeywordValueException kve ){
      // OK, so they don't get a notice.
      lastServedNotice = "" ; 
    }
    if( lastServedNotice.equals( currentAdminNotice ) ) return ; 

    //////  Append JavaScript for displaying notice
    appTemplate = new AppTemplateHTML(); 
    code = 
      "\n" + 
      "  alert( \"" + currentAdminNotice + "\" ); \n" + 
      "// " ; 
    protoScript = appTemplate.getElementJavaScript(); 
    jsComment = (Comment) protoScript.getFirstChild(); 
    jsComment.setData( code ); 
    newScript = (HTMLScriptElement) protoScript.cloneNode( true ); 
    body = ProdUtil.getBodyElement( document ); 
    if( null == body ) return ; // Probably in a FRAMESET
    body.insertBefore( newScript, body.getFirstChild() ); 

    //////  Record notice in presentation data.
    try{ 
      presentationData.set( "LASTADMINNOTICE", currentAdminNotice ); 
    }catch( KeywordValueException kve ){
      throw new HttpPresentationException( "Can't set LASTADMINNOTICE in presentation section of session data" ); 
    }
  }
}
