package timesheet.business;

import timesheet.*;
import timesheet.data.*;
import java.util.*;
import com.lutris.appserver.server.*;
import com.lutris.appserver.server.sql.*;
import com.lutris.dods.builder.sourceGenerators.Query.*;
import java.sql.*;
import java.math.*;
import com.pisoftware.platypus.production.*;


/**
 * Date Created: Oct 15, 1999
 * <P>
 * Purpose:  This class provides the business logic associated with
 *           a TimeEntry.  It should probably be used by other BOs only.
 *
 */
public class TimeEntryBO extends TimeEntryBDO {


/**
 * Constructor.  Creates the object, and initialises the
 * appropriate variables, etc.
 * 
 * @exception Exception thrown by BDO constructor.  :-(
 */
  public TimeEntryBO() throws Exception {

    super();

  }



/*
 * Start of getIdValue Methods
*/

  private TimeEntryDO[] query_results = null;


/**
 * Sets the query results to be used in the getIdValue
 * method calls.
 *
 * @param results  The results to use
 */
  public void setQueryResults(TimeEntryDO[] results) {

    this.query_results = results;

  }


/**
 * Gets the current query results being used in the getIdValue
 * method calls.
 *
 * @return see comment
 */
  public TimeEntryDO[] getQueryResults() {

    return this.query_results;

  }


/**
 * Gets information from the DO layer for the ID passed.
 * Note that a previous ACCESS method must have been
 * called for this function to work.
 *
 * @param index  The record to use
 * @param id  The id who's value is to be returned.
 * @param format  Determines what formatting is required for the id's value
 * @return A String representing the value retrieved.
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public String getIdValue(int index, String id, String format) throws ApplicationException {

    if (query_results == null) {

      throw new ApplicationException("No query results to obtain data from!");

    }

    String result_str = "";
    boolean id_found = false;

      if (id.equalsIgnoreCase("DayDate")) {

    id_found = true;

    
    // ensure we've got something to get data from
    if (query_results == null || query_results.length == 0)
      throw new ApplicationException("Unable to obtain data for DayDate.  No data available from query");

    if (index < 0 || index > query_results.length -1)
      throw new ApplicationException("Invalid index to obtain data from query: "+index);


    java.sql.Date result;

    try {

      result = query_results[index].getEntryDate();

    } catch (DataObjectException doe) {

      throw new ApplicationException("Unable to obtain data for DayDate", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("ProjectName")) {

    id_found = true;

    
    // ensure we've got something to get data from
    if (query_results == null || query_results.length == 0)
      throw new ApplicationException("Unable to obtain data for ProjectName.  No data available from query");

    if (index < 0 || index > query_results.length -1)
      throw new ApplicationException("Invalid index to obtain data from query: "+index);


    String result;

    try {

      result = query_results[index].getProjectId().getName();

    } catch (DataObjectException doe) {

      throw new ApplicationException("Unable to obtain data for ProjectName", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("ActivityName")) {

    id_found = true;

    
    // ensure we've got something to get data from
    if (query_results == null || query_results.length == 0)
      throw new ApplicationException("Unable to obtain data for ActivityName.  No data available from query");

    if (index < 0 || index > query_results.length -1)
      throw new ApplicationException("Invalid index to obtain data from query: "+index);


    String result;

    try {

      result = query_results[index].getActivityId().getActivityName();

    } catch (DataObjectException doe) {

      throw new ApplicationException("Unable to obtain data for ActivityName", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("Details")) {

    id_found = true;

    
    // ensure we've got something to get data from
    if (query_results == null || query_results.length == 0)
      throw new ApplicationException("Unable to obtain data for Details.  No data available from query");

    if (index < 0 || index > query_results.length -1)
      throw new ApplicationException("Invalid index to obtain data from query: "+index);


    String result;

    try {

      result = query_results[index].getDetails();

    } catch (DataObjectException doe) {

      throw new ApplicationException("Unable to obtain data for Details", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("NrHours")) {

    id_found = true;

    
    // ensure we've got something to get data from
    if (query_results == null || query_results.length == 0)
      throw new ApplicationException("Unable to obtain data for NrHours.  No data available from query");

    if (index < 0 || index > query_results.length -1)
      throw new ApplicationException("Invalid index to obtain data from query: "+index);


    double result;

    try {

      result = query_results[index].getNrHours();

    } catch (DataObjectException doe) {

      throw new ApplicationException("Unable to obtain data for NrHours", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("DayDate1")) {

    id_found = true;

    
    // ensure we've got something to get data from
    if (query_results == null || query_results.length == 0)
      throw new ApplicationException("Unable to obtain data for DayDate1.  No data available from query");

    if (index < 0 || index > query_results.length -1)
      throw new ApplicationException("Invalid index to obtain data from query: "+index);


    java.sql.Date result;

    try {

      result = query_results[index].getEntryDate();

    } catch (DataObjectException doe) {

      throw new ApplicationException("Unable to obtain data for DayDate1", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("TimeEntryId")) {

    id_found = true;

    
    // ensure we've got something to get data from
    if (query_results == null || query_results.length == 0)
      throw new ApplicationException("Unable to obtain data for TimeEntryId.  No data available from query");

    if (index < 0 || index > query_results.length -1)
      throw new ApplicationException("Invalid index to obtain data from query: "+index);


    java.math.BigDecimal result;

    try {

      result = query_results[index].getOId().toBigDecimal();

    } catch (Exception doe) {

      throw new ApplicationException("Unable to obtain data for TimeEntryId", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("ActivityId")) {

    id_found = true;

    
    // ensure we've got something to get data from
    if (query_results == null || query_results.length == 0)
      throw new ApplicationException("Unable to obtain data for ActivityId.  No data available from query");

    if (index < 0 || index > query_results.length -1)
      throw new ApplicationException("Invalid index to obtain data from query: "+index);


    java.math.BigDecimal result;

    try {

      result = query_results[index].getActivityId().getOId().toBigDecimal();

    } catch (Exception doe) {

      throw new ApplicationException("Unable to obtain data for ActivityId", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("ProjectId")) {

    id_found = true;

    
    // ensure we've got something to get data from
    if (query_results == null || query_results.length == 0)
      throw new ApplicationException("Unable to obtain data for ProjectId.  No data available from query");

    if (index < 0 || index > query_results.length -1)
      throw new ApplicationException("Invalid index to obtain data from query: "+index);


    java.math.BigDecimal result;

    try {

      result = query_results[index].getProjectId().getOId().toBigDecimal();

    } catch (Exception doe) {

      throw new ApplicationException("Unable to obtain data for ProjectId", doe);

    }

    result_str = FormatUtil.format(result, format);

  }


    // was the id found?
    if (!id_found) {

      // no, return a value with the details passed
      return "ID Value passed: "+id+", index is: "+index+", format is: "+format;

    } else {

      // return the formatted result
      return result_str;

    }

  }


  // psuedo-cache to speed access to the db data.
  private java.math.BigDecimal oid = null;


/**
 * Gets information from the DO layer for the ID passed.
 *
 * @param oidstr  The database oid for the record
 * @param id  The id who's value is to be returned.
 * @param format  Determines what formatting is required for the id's value
 * @return A String representing the value retrieved.
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public String getIdValue(String oidstr, String id, String format) throws ApplicationException {

    String result_str = "";
    boolean id_found = false;

    try {

      // are we accesing the same oid again?
      if (DO == null || !oidstr.equals(DO.getHandle().toString())) {

        // no - get this oid's DO from the db.

        oid = new java.math.BigDecimal(oidstr);
        DO = TimeEntryDO.createExisting(oid);

      }

    } catch (Exception e) {

      throw new ApplicationException("Unable to obtain value for id: "+id+" using this ObjectId: "+oidstr, e);

    }

      if (id.equalsIgnoreCase("DayDate")) {

    id_found = true;

    

    java.sql.Date result;

    try {

      result = DO.getEntryDate();

    } catch (DataObjectException doe) {

      throw new ApplicationException("Unable to obtain data for DayDate", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("ProjectName")) {

    id_found = true;

    

    String result;

    try {

      result = DO.getProjectId().getName();

    } catch (DataObjectException doe) {

      throw new ApplicationException("Unable to obtain data for ProjectName", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("ActivityName")) {

    id_found = true;

    

    String result;

    try {

      result = DO.getActivityId().getActivityName();

    } catch (DataObjectException doe) {

      throw new ApplicationException("Unable to obtain data for ActivityName", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("Details")) {

    id_found = true;

    

    String result;

    try {

      result = DO.getDetails();

    } catch (DataObjectException doe) {

      throw new ApplicationException("Unable to obtain data for Details", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("NrHours")) {

    id_found = true;

    

    double result;

    try {

      result = DO.getNrHours();

    } catch (DataObjectException doe) {

      throw new ApplicationException("Unable to obtain data for NrHours", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("DayDate1")) {

    id_found = true;

    

    java.sql.Date result;

    try {

      result = DO.getEntryDate();

    } catch (DataObjectException doe) {

      throw new ApplicationException("Unable to obtain data for DayDate1", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("TimeEntryId")) {

    id_found = true;

    

    java.math.BigDecimal result;

    try {

      result = DO.getOId().toBigDecimal();

    } catch (Exception doe) {

      throw new ApplicationException("Unable to obtain data for TimeEntryId", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("ActivityId")) {

    id_found = true;

    

    java.math.BigDecimal result;

    try {

      result = DO.getActivityId().getOId().toBigDecimal();

    } catch (Exception doe) {

      throw new ApplicationException("Unable to obtain data for ActivityId", doe);

    }

    result_str = FormatUtil.format(result, format);

  }
 else  if (id.equalsIgnoreCase("ProjectId")) {

    id_found = true;

    

    java.math.BigDecimal result;

    try {

      result = DO.getProjectId().getOId().toBigDecimal();

    } catch (Exception doe) {

      throw new ApplicationException("Unable to obtain data for ProjectId", doe);

    }

    result_str = FormatUtil.format(result, format);

  }


    // was the id found?
    if (!id_found) {

      // no, return a value with the details passed
      return "ID Value passed: "+id+", oidstr is: "+oidstr+", format is: "+format;

    } else {

      // return the formatted result
      return result_str;

    }

  }

/*
 * End of getIdValue Methods
*/



/**
 * Builds a hashtable containing the total hours for each day
 * for the current query.  The query results must be in data order.
 *
 * @param hours  The hashtable to build
 * @return see comment
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public void buildHoursHash(Hashtable hours) throws ApplicationException {

    if (query_results == null) {

      throw new ApplicationException("No Time Entry query to build hashtable for!");

    }

    try {

      double hour_total = 0;
      java.sql.Date prev_date = query_results[0].getEntryDate();
      java.sql.Date curr_date = null;

      for (int i=0; i<query_results.length; i++) {

        // get this entry's date
        curr_date = query_results[i].getEntryDate();

        // any change?
        if (!curr_date.equals(prev_date)) {

          // yes, save the information in the hashtable
          hours.put(prev_date.toString(),""+hour_total); 

          // reset total
          hour_total = 0;

          // keep this date for next timne round...
          prev_date = curr_date;

        }

        // add to the current total for the day
        hour_total += query_results[i].getNrHours();

      }

      // any leftovers?
      if (hour_total > 0)
        hours.put(curr_date.toString(),""+hour_total); 

    } catch (DataObjectException doe) {

      throw new ApplicationException("Unable to retrieve data while building hashtable!", doe);

    }

  }


/**
 * Updates the TimeEntry table for the data passed.
 */
  public void updateTimeEntry(TimeEntryDO timeEntryDO, ActivityDO activityDO, String Details, java.sql.Date EntryDate, double NrHours, ProjectDO projectDO, PersonDO personDO) throws ApplicationException {

    String project_name = "Unknown";
    String activity_name = "Unknown";
    String entry_date = "Unknown";

    try {

      // values for the exception handler...
      project_name = projectDO.getName();
      activity_name = activityDO.getActivityName();
      entry_date = EntryDate.toString();

      // use the DO passed
      DO = timeEntryDO;

      // set the time entry details
      setPersonId(personDO);
      setActivityId(activityDO);
      setProjectId(projectDO);
      setDetails(Details);
      setEntryDate(EntryDate);
      setNrHours(NrHours);

      // commit this record to the database
      commit();

    } catch (Exception e) {

      throw new ApplicationException("Unable to create update the Time Entry with the date: "+entry_date+", Project: "+project_name+", Activity: "+activity_name+", Details: "+Details+", NrHours: "+NrHours, e);

    }

  }

    // 'cache' for query - createExisting can be a time expensive call...
    private String findTimeEntry_ObjectId = "";
    private TimeEntryDO findTimeEntry_DO = null;


/**
 * findTimeEntry: generated by Squirt!  This query is for
 * finding a record in a table by ObjectId.  The row
 * must be in the table, otherwise an exception will
 * be thrown.
 *
 * @param objectid  The record to retrieve from TimeEntry
 * @return an array of TimeEntryDOs for this query
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public TimeEntryDO findTimeEntry(String objectid) throws ApplicationException {

    // have we just referenced this object id?
    if (objectid.equals(findTimeEntry_ObjectId)) {

      // yes, use our cached version
      return findTimeEntry_DO;

    }

    TimeEntryDO result = null;

    try {

      java.math.BigDecimal bd = new java.math.BigDecimal(objectid);
      result = TimeEntryDO.createExisting(bd);

      // keep details in our cache
      findTimeEntry_ObjectId = objectid;
      findTimeEntry_DO = result;

    } catch (Exception e) {

      throw new ApplicationException("Unable to create existing TimeEntry Data Object", e );

    }

    // return the results found, if any
    return result;

  }



/**
 * getTimeEntryForDate: generated by Squirt!
 *
 * <JAVADOC_PARAMETERS>
 * @return an array of TimeEntryDOs for this query
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public TimeEntryDO[] getTimeEntryForDate(/* String */ String PersonId, /* String */ String EntryDate) throws ApplicationException {

    String WHERE = " WHERE ";
    String sql = null;
    StringBuffer sqlbuf = new StringBuffer();
    TimeEntryDO[] results = null;
    TimeEntryQuery query = new TimeEntryQuery();
    boolean operator_required = false;

    sqlbuf.append(WHERE);
    
    // test for null parameters, if required
    if (ProdUtil.stringHasContent(PersonId)) {

      // need to concatentate this part of the where clause with previous details?
      if (operator_required) {

        sqlbuf.append(' ');
        sqlbuf.append(" ");
        sqlbuf.append(' ');

      }

      sqlbuf.append(" PersonId =  ");
      sqlbuf.append(PersonId);
      sqlbuf.append("  ");


      operator_required = true;

    }


    // test for null parameters, if required
    if (ProdUtil.stringHasContent(EntryDate)) {

      // need to concatentate this part of the where clause with previous details?
      if (operator_required) {

        sqlbuf.append(' ');
        sqlbuf.append("AND");
        sqlbuf.append(' ');

      }

      sqlbuf.append(" EntryDate = ' ");
      sqlbuf.append(EntryDate);
      sqlbuf.append(" ' ");


      operator_required = true;

    }


    // test for null parameters, if required
    if (true) {

      // need to concatentate this part of the where clause with previous details?
      if (operator_required) {

        sqlbuf.append(' ');
        sqlbuf.append(" ");
        sqlbuf.append(' ');

      }

      sqlbuf.append(" ORDER BY EntryDate, ObjectId ");


      operator_required = true;

    }


    sql = sqlbuf.toString();

    // ensure there's something for the where clause
    if (!sql.equalsIgnoreCase(WHERE))
      query.setArbitraryWhereClause(sql);

    

    try {

      results = query.getDOArray();

    } catch (DataObjectException doe) {

      throw new ApplicationException( "Unable to create existing TimeEntry Data Object", doe );

    } catch ( NonUniqueQueryException nuqe ) {

      // ...this exception will only occur if requireUniqueInstance was called.
      throw new ApplicationException( "Non unique result!", nuqe );  

    }

    // return the results found, if any
    return results;

  }



/**
 * getTimeEntryRecords: generated by Squirt!
 *
 * <JAVADOC_PARAMETERS>
 * @return an array of TimeEntryDOs for this query
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public TimeEntryDO[] getTimeEntryRecords(/* String */ String PersonId, /* String */ String FromDate, /* String */ String ToDate) throws ApplicationException {

    String WHERE = " WHERE ";
    String sql = null;
    StringBuffer sqlbuf = new StringBuffer();
    TimeEntryDO[] results = null;
    TimeEntryQuery query = new TimeEntryQuery();
    boolean operator_required = false;

    sqlbuf.append(WHERE);
    
    // test for null parameters, if required
    if (ProdUtil.stringHasContent(PersonId)) {

      // need to concatentate this part of the where clause with previous details?
      if (operator_required) {

        sqlbuf.append(' ');
        sqlbuf.append(" ");
        sqlbuf.append(' ');

      }

      sqlbuf.append(" PersonId =  ");
      sqlbuf.append(PersonId);
      sqlbuf.append("  ");


      operator_required = true;

    }


    // test for null parameters, if required
    if (ProdUtil.stringHasContent(FromDate) && ProdUtil.stringHasContent(ToDate)) {

      // need to concatentate this part of the where clause with previous details?
      if (operator_required) {

        sqlbuf.append(' ');
        sqlbuf.append("AND");
        sqlbuf.append(' ');

      }

      sqlbuf.append(" EntryDate BETWEEN ' ");
      sqlbuf.append(FromDate);
      sqlbuf.append(" ' AND ' ");
      sqlbuf.append(ToDate);
      sqlbuf.append(" ' ");


      operator_required = true;

    }


    // test for null parameters, if required
    if (true) {

      // need to concatentate this part of the where clause with previous details?
      if (operator_required) {

        sqlbuf.append(' ');
        sqlbuf.append(" ");
        sqlbuf.append(' ');

      }

      sqlbuf.append(" ORDER BY EntryDate, ObjectId ");


      operator_required = true;

    }


    sql = sqlbuf.toString();

    // ensure there's something for the where clause
    if (!sql.equalsIgnoreCase(WHERE))
      query.setArbitraryWhereClause(sql);

    

    try {

      results = query.getDOArray();

    } catch (DataObjectException doe) {

      throw new ApplicationException( "Unable to create existing TimeEntry Data Object", doe );

    } catch ( NonUniqueQueryException nuqe ) {

      // ...this exception will only occur if requireUniqueInstance was called.
      throw new ApplicationException( "Non unique result!", nuqe );  

    }

    // return the results found, if any
    return results;

  }



}

