package timesheet.business.entry;

import com.lutris.appserver.server.*;
import com.lutris.appserver.server.sql.*;
import com.lutris.appserver.server.httpPresentation.*;
import timesheet.*;
import timesheet.data.*;
import timesheet.business.*;
import java.util.*;
import com.lutris.dods.builder.sourceGenerators.Query.*;
import com.pisoftware.platypus.production.*;

/**
 * Date Created: Oct 18, 1999
 *<P>
 * Purpose: This class is a BO for the AddOneItemCommit PO.  
 *
 */
public class AddOneItemCommitBO extends EntryBO {


/**
 * Constructor.  Creates the object, and initialises the appropriate variables, etc.
 *
 * @param comms  Used to get the session data required
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public AddOneItemCommitBO(HttpPresentationComms comms) throws ApplicationException {

    super(comms);

  }
    
   
/**   
 * Returns the list of pulldown items for the name passed.
 *  
 * @param pulldown_name  The Pulldown name to query for
 * @return an array of PulldownDOs containing the list of values for this pulldown name, null or empty array if none found.
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */   
  public PulldownItem[] getPulldownItems(String pulldown_name) throws ApplicationException {

    /*
     * Add your code here if you want to override/add pulldown item(s).
     */
  
    return super.getPulldownItems(pulldown_name);

  }


/*

  No Access Method(s) Generated!
*/


/*
 * Start of Commit Methods
*/


/**
 * commitAddOneItem: Commit method for use by the PO layer.
 * 
 * @param ActivityId Parameter supplied by PO
 * @param Details Parameter supplied by PO
 * @param EntryDate Parameter supplied by PO
 * @param NrHours Parameter supplied by PO
 * @param ProjectId Parameter supplied by PO
 * @return The ObjectId, as a String, for the record added/updated
 * @exception ApplicationException All exceptions are caught and redirected to this.
 */
  public String commitAddOneItem(String ActivityId, String Details, String EntryDate, String NrHours, String ProjectId) throws ApplicationException {

    /*
     * Insert your code in here to do the add/update.
     */

    String result_str = "";
    TimeEntryDO timeEntryDO = null;

    try {

      timeEntryDO = TimeEntryDO.createVirgin();
      result_str = timeEntryDO.getHandle();

    } catch (Exception e) {

      throw new ApplicationException("Unable to create new Time Entry record!", e);

    }

    // ensure the hours are ok
    double nrHours = 0.0;

    try {

      Double tmpdouble = new Double(NrHours);

      nrHours = tmpdouble.doubleValue();

    } catch (NumberFormatException nfe) {

      throw new ApplicationException("The number of hours entered is not a valid number: <"+NrHours+">",nfe);

    }

    // check & convert the date
    java.sql.Date entryDate = null;

    try {

      String tmpdate_str = FormatUtil.displayDateToSQL(EntryDate, null);

      entryDate = java.sql.Date.valueOf(tmpdate_str);

    } catch (Exception de) {

      throw new ApplicationException("The date entered is not valid: <"+EntryDate+">", de);

    }

    // find this activity
    ActivityDO activityDO = activityBO.findActivity(ActivityId);

    // find this project
    ProjectDO projectDO = projectBO.findProject(ProjectId);

    // update the new record
    timeEntryBO.updateTimeEntry(timeEntryDO, activityDO, Details, entryDate, nrHours, projectDO, operatorDO);

    // return the object id generated
    return result_str;

  }



/*
 * End of Commit Methods
*/



/*
  No getIdValue Method(s) Generated!
*/

/*
  No getBusinessIdValue Method Generated!
*/

}

