package timesheet.presentation;

import java.util.*;
// import java.net.URLEncoder;
import org.w3c.dom.*;
import org.w3c.dom.html.*;
import com.lutris.util.*;
import com.lutris.xml.xmlc.*;
import com.lutris.appserver.server.ApplicationException;
import com.lutris.appserver.server.sql.*;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;
import timesheet.business.*;
import com.pisoftware.platypus.production.*;

/** 
 * Class Debug manipulates the DOM
 * model created by class Debug. It
 * extends class DebugBASE and uses
 * many methods and variables declared in that class.
 * 
 * TO DO: More detailed doco on what this class does.
 * 
 * The 'fieldXxxx' string variables hold values returned from the database.
 * 
 * The 'qsXxxx string variables hold values obtained from the query string. 
 **/

public class Debug{
 
  /** Enhydra's DOM model for PO we're generating code for **/
  DebugHTML po ;
 
  /** DOM Document at root of presentation DOM data structure **/
  Document document ; 
 
  /** HttpPresentationComms object passed to run() **/
  HttpPresentationComms comms ; 
 
  /** Session data **/
  SessionData sessionData ;
 
  /** Section of session data reserved for presentation layer **/
  KeywordValueTable presentationData ;
 
  /** Section of session data reserved for debug flags. **/
  KeywordValueTable debugData ;
 
  /** Name value pairs from query string **/
  KeywordValueTable args ; 

  /** Unique id number for this invocation of the page. **/
  String servedId ; 

  /** Debugging flag **/
  boolean DEBUG ; 

  /** Variable to hold prototype INPUT node DebugFlag **/
  HTMLInputElement protoDebugFlag ; 

  /**  Business object which supports this presentation object **/
  TimesheetBO bo ; 


  //////  Variables for the query string values
  ////  Form  'Flags' in 'Debug' PO invokes URL  'Debug.html'

  /** Value of 'FromPO' on query string, from Flags form on debug PO **/
  String qsFromPO ; 

  /** Value of 'DebugFlag' on query string, from Flags form on debug PO **/
  String qsDebugFlag ; 


  /**
   * @exception HttpPresentationException
   **/
  // Uses variables like 'comms' and 'sessionData', so is in
  // manipulation class. Otherwise, would belong in the ProdUtil
  // class. One day, when stable, move to ProdUtil.
  HTMLElement startDebugTemplate() throws HttpPresentationException {
    AppTemplateHTML appTemplateHTML = new AppTemplateHTML(); 
    appTemplateHTML.setTextDebugQueryString( HttpUtil.getQueryFromRequest( comms.request ) ); 
    appTemplateHTML.setTextDebugSessionData( sessionData.toString() ); 
    return (HTMLElement) appTemplateHTML.getElementDebugTemplate(); 
  }


  /** 
   * Does the actual manipulation of the DOM data structure. 
   * 
   * It defines the abstract method in DebugBASE.java.  
   * 
   * @exception ApplicationException
   * @exception HttpPresentationException
   **/
  public void manipulate() throws ApplicationException, HttpPresentationException {
    boolean debugFlagForForm ; // Default value of DebugFlag for the form.
    try{ 

      //////  Initialise
      bo = new TimesheetBO( comms ); 
      DEBUG = true ; 
      
      //////  Get values from query string 
      qsFromPO = ProdUtil.getFromQueryString( args, "FromPO" );
      qsDebugFlag = ProdUtil.getFromQueryString( args, "DebugFlag" );
      qsDebugFlag = ProdUtil.stringHasContent( qsDebugFlag ) ? "TRUE" : "FALSE" ; // CHECKBOX INPUT tag 
      
      //////  Fill dynamic nodes
      if( ProdUtil.stringHasContent( qsFromPO ) ){ 
        debugFlagForForm = qsDebugFlag.equalsIgnoreCase( "TRUE" ); 
        debugData.set( "DEBUG", debugFlagForForm ? "TRUE" : "FALSE" ); 
      }else{
        debugFlagForForm = ProdUtil.sessionDebugFlag( debugData ); 
      }
      if( debugFlagForForm ) protoDebugFlag.setAttributeNode( ProdUtil.getCheckedAttributeTemplate() ); 
      
      //////  Generate debugging information, if requested.
      if( DEBUG ){ 
        HTMLElement debugTemplate ; 
        //// Fill in debugging information
        debugTemplate = startDebugTemplate(); // Method is in BASE class.
        //// Insert debugging information into DOM tree.
        HTMLBodyElement bodyElement = ProdUtil.getBodyElement( document ); 
        bodyElement.insertBefore( debugTemplate, null ); 
      }
    }catch( KeywordValueException kve ){
      throw new ApplicationException( kve ); 
    }
  }


  /**
   * @exception HttpPresentationException
   **/
  public void run( HttpPresentationComms comms ) throws HttpPresentationException {

    //////  Initialise
    this.comms = comms ; 
    sessionData = comms.sessionData ;
    presentationData = ProdUtil.presentationData( sessionData ); 
    debugData = ProdUtil.ensureSectionData( "DEBUG", presentationData ); 
    po = new DebugHTML();
    document = po.getDocument();
    servedId = ProdUtil.getUniquePageString( presentationData ); 

    //////  Process query string 
    try{ 
      args = HttpUtil.buildQueryTable( HttpUtil.getQueryFromRequest( comms.request ) );
    }catch( KeywordValueException kve ){ 
      throw new HttpPresentationException( "Couldn't get CGI query string", kve );
    }
    
    //////  Initialise prototype variables
    protoDebugFlag = po.getElementDebugFlag(); 

    //////  Manipulate the DOM data structure
    try{
      manipulate(); 
    // }catch( UserException ue ){
      // AppTemplate.processUserException( document, ue ); 
    }catch( ApplicationException ae ){ 
      throw new HttpPresentationException( ae ); 
    }
    
    //////  Remove ID attributes from nodes
    ProdUtil.removeIDAttributes( document ); 
    
    //////  Write out manipulated DOM as HTML
    comms.response.writeHTML(po.toDocument());
  }
}
