package applicationName.presentation;

import org.w3c.dom.*;
import org.w3c.dom.html.*;
import com.lutris.xml.xmlc.*;
import com.lutris.util.*;
import com.lutris.appserver.server.httpPresentation.*;
import com.lutris.appserver.server.session.*;
import java.lang.*;
import java.util.*;
import java.io.*;
import com.lutris.appserver.server.Enhydra;
import com.lutris.logging.Logger;
import javax.servlet.http.Cookie;
import com.pisoftware.platypus.production.*;
import applicationName.business.*;

/** 
 * Presentation object Errorhandler is never called directly. Rather,
 * it is invoked by Enhydra when the manipulation class for a PO
 * generates an exception and doesn't catch it. The Errorhandler PO
 * reports the error to the operator.
 **/

/*
 *  Copyright (C) 1999 Plugged In Software Pty Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Plugged In Software
 *  PO Box 4130
 *  St. Lucia South Queensland
 *  Australia  4067
 *  info@PIsoftware.com
 */

////////////////////////////////////////////////////////////////////////////////
// TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO //

// Remove all the ID attributes from DOM nodes. 

// The served page includes messages for both the operator and
// programmer. Really, the programmer feedback should go to the logs.
 
////////////////////////////////////////////////////////////////////////////////

public class ErrorHandler implements HttpPresentation {

  // NOT USED /** Business object servicing this PO **/
  // NOT USED private ApplicationNameBO bo ; 

  /** Document created in ErrorHandlerHTML.java **/
  private Document document ; 

  /** Name value pairs from query string **/
  private KeywordValueTable args ; 

  /** Data which persists from PO to PO throughout the session **/
  private SessionData sessionData ; 

  /** Exception invoking this page **/
  Throwable exception ; 

  /** Message from exception **/
  String exceptionMessage ; 

  /** URI string of exception **/
  String uriString ; 

  /** Query string of exception **/
  String queryString ; 

  /** The Writer to use for getting a String representation of
    the stack trace **/
  StringWriter strw;

  /** The PrintWriter to use with the StringWriter, above. **/
  PrintWriter pw;

  /** StringBuffer to hold the stack trace **/
  StringBuffer trace;

  /** An Enumeration of Strings which represent the parameters
    passed to the called PO **/
  Enumeration parameters;

  /** A String representation of the parameters **/
  String paramString = "";

  /** A String[] representation of the parameters' values **/
  String[] paramValues = null;

  /** A String representation of the parameters **/
  String cookieString = "";

  /** A String[] representation of the parameters' values **/
  Cookie[] cookieValues = null;

  /** A loop variable **/
  int i = 0;

  /** Method run() manipulates the DOM model to generate the served HTML. **/
  public void run( HttpPresentationComms comms ) throws Exception {
    //////  Declare variables
    ErrorHandlerHTML po = new ErrorHandlerHTML();
    String reachedStr = null ; 

    try{ 
      //////  Initialise
      reachedStr = "Initialise" ; 
      document = po.getDocument();
      sessionData = comms.sessionData ; 
      exception = comms.exception ; 
      exceptionMessage = exception.getMessage(); 
      if( exceptionMessage == null ) exceptionMessage = "-- exceptionMessage null --" ; 
      // NOT USED bo = new ApplicationNameBO( comms ); 
      
      //////  Process query string. Not all name/value pairs are extracted here.
      // NOT USED reachedStr = "Process query string. Not all name/value pairs are extracted here." ; 
      // NOT USED args = HttpUtil.buildQueryTable( HttpUtil.getQueryFromRequest( comms.request ) ); 
        
      //////  Instantiate prototype HTML
      reachedStr = "Instantiate prototype HTML" ; 
      reachedStr = "Instantiate prototype HTML A" ; 
      
      // Write out the date and time.
      po.setTextDate( (new Date()).toString() );
      reachedStr = "Instantiate prototype HTML B" ; 
      
      po.setTextOperatorMessage( exceptionMessage ); 
      reachedStr = "Instantiate prototype HTML C" ; 
      uriString = comms.request.getRequestURI(); 
      reachedStr = "Instantiate prototype HTML D" ; 
      if( uriString == null ) uriString = "-- uriString null --" ; 
      po.setTextPOName( uriString ); 
      reachedStr = "Instantiate prototype HTML E" ; 
      queryString = comms.request.getQueryString(); 
      reachedStr = "Instantiate prototype HTML F" ; 
      if( queryString == null ) queryString = "-- queryString null --" ; 
      po.setTextQuery( queryString ); 
      reachedStr = "Instantiate prototype HTML G" ; 
      po.setTextProgrammerMessage( exceptionMessage ); 
      reachedStr = "Instantiate prototype HTML G" ; 
      
      // Create the summary table of request and requestor information
      reachedStr = "Create the summary table of request and requestor information" ; 
      if ( null != comms.request.getRequestURI() ) po.setTextURI( comms.request.getRequestURI() );
      if ( null != comms.request.getProtocol() ) po.setTextProtocol( comms.request.getProtocol() );
      if ( null != comms.request.getScheme() ) po.setTextScheme( comms.request.getScheme() );
      if ( null != comms.request.getServerName() ) po.setTextServerName( comms.request.getServerName() );
      po.setTextServerPort( String.valueOf( comms.request.getServerPort() ) );
      if ( null != comms.request.getPresentationObjectPath() ) po.setTextPath( comms.request.getPresentationObjectPath() );
      
      // This will generally not be specified.
      String extrapath = comms.request.getPathTranslated();
      if ( null != extrapath ) po.setTextExtraPath( extrapath );
      
      if ( null != comms.request.getMethod() ) po.setTextMethod( comms.request.getMethod() );
      if ( null != comms.request.getAuthType() ) po.setTextAuthType( comms.request.getAuthType() );
      if ( null != comms.request.getRemoteAddr() ) po.setTextRemoteIP( comms.request.getRemoteAddr() );
      
      // This will generally not be specified.
      String rhost = comms.request.getRemoteHost();
      if ( null != rhost ) po.setTextRemoteHostname( rhost );
      
      // This will generally not be specified.
      String ruser = comms.request.getRemoteUser();
      if ( null != ruser ) po.setTextRemoteUser( ruser );
      
      // Write out the parameters for this request.
      try{ 
        reachedStr = "Write out the parameters for this request. A" ; 
        ProdUtil.log( "Write out the parameters for this request. A" ); 
        if( null == comms ){ 
          reachedStr = "Testing comms for null -- failed!" ; 
          ProdUtil.log( "Testing comms for null -- failed!" ); 
          throw new Exception(); 
        }
        if( null == comms.request ){ 
          reachedStr = "Testing comms.request for null -- failed!" ; 
          ProdUtil.log( "Testing comms.request for null -- failed!" ); 
          throw new Exception(); 
        }
        reachedStr = "Write out the parameters for this request. A" ; 
        ProdUtil.log( "Write out the parameters for this request. A" ); 
        paramString = comms.request.getQueryString(); 
        parameters = comms.request.getParameterNames();
        reachedStr = "Write out the parameters for this request. B" ; 
        ProdUtil.log( "Write out the parameters for this request. B" ); 
        String pname = "";  // Temporary String.
        while ( parameters.hasMoreElements() ) {
          pname = (String)parameters.nextElement();
          paramString += pname + " = " ;
        
          // Get the values for each name (can be more than one).
          paramValues = comms.request.getParameterValues( pname );
          for ( i = 0 ; i < paramValues.length ; i++ ) {
              if ( i > 0 ) paramString += ", ";
              paramString += paramValues[i];
          }
          paramString += "\n";
        }
      }catch( Exception e ){
        paramString = "Warning: problems writing request parameters." ; 
      }
      reachedStr = "Write out the parameters for this request. C" ; 
      ProdUtil.log( "Write out the parameters for this request. C" ); 
      po.setTextParameters ( paramString );
//      po.setTextParameters ( "not computed" );
      
      // Write out list of Cookies.
      reachedStr = "Write out list of Cookies." ; 
      ProdUtil.log( "Write out list of Cookies." ); 
      cookieValues = comms.request.getCookies();
      if ( cookieValues != null ) {
          for ( i = 0; i < cookieValues.length; i++) {
              cookieString += cookieValues[i].getName() + " = " +  cookieValues[i].getValue() + "\n";
          }
      }
      po.setTextCookies ( cookieString );
      
      //////  Write out the session data
      reachedStr = "Write out the session data" ; 
      ProdUtil.log( "Write out the session data" ); 
      po.setTextSessionData( sessionData.toString() ); 
      
      // Write out the stack trace
      reachedStr = "Write out the stack trace" ; 
      ProdUtil.log( "Write out the stack trace" ); 
      strw = new StringWriter();
      pw = new PrintWriter( strw );
      exception.printStackTrace( pw );
      trace = strw.getBuffer();
      po.setTextStackTrace( trace.toString() );
      
      //////  Write stack trace to log
      reachedStr = "Write stack trace to log" ; 
      ProdUtil.log( "Write stack trace to log" ); 
      exception.printStackTrace(); 
      
      //////  Write out, tidy up and leave.
      reachedStr = "Write out, tidy up and leave." ; 
      ProdUtil.log( "Write out, tidy up and leave." ); 
      comms.response.writeHTML(po.toDocument());
    }
    catch( Exception e ){ 
      comms.response.writeHTML(po.toDocument());
      String exceptionMessage ; 
      exceptionMessage = e.toString(); 
      if( exceptionMessage == null ) exceptionMessage = "" ; 
      System.err.println( "Error in ErrorHandler!!!!\n\nReached this stage: " + reachedStr + "\n" + exceptionMessage ); 
    }
  }

}

